@@ -24,6 +24,7 @@ Z* ------------------------------------------------------------------- */
 #include"Setting.h"
 #include"Feedback.h"
 #include"MemoryCache.h"
+#include"Base.h"
 
 #ifndef true
 #define true 1
@@ -33,29 +34,35 @@ Z* ------------------------------------------------------------------- */
 #define false 0
 #endif
 
-static MapType *_MapNew(float range,float *vert,int nVert,
+static MapType *_MapNew(PyMOLGlobals *G,float range,float *vert,int nVert,
                         float *extent,int *flag,int group_id,int block_id);
 
+float MapGetDiv(MapType *I)
+{
+  return I->Div;
+}
 void MapFree(MapType *I)
 {
   if(I)
 	 {
-		CacheFreeP(I->Head,I->group_id,I->block_base + cCache_map_head_offset,false);
-		CacheFreeP(I->Link,I->group_id,I->block_base + cCache_map_link_offset,false);
-		CacheFreeP(I->EHead,I->group_id,I->block_base + cCache_map_ehead_offset,false);
-		CacheFreeP(I->EMask,I->group_id,I->block_base + cCache_map_emask_offset,false);
-		VLACacheFreeP(I->EList,I->group_id,I->block_base + cCache_map_elist_offset,false);
+      PyMOLGlobals *G=I->G;
+		CacheFreeP(G,I->Head,I->group_id,I->block_base + cCache_map_head_offset,false);
+		CacheFreeP(G,I->Link,I->group_id,I->block_base + cCache_map_link_offset,false);
+		CacheFreeP(G,I->EHead,I->group_id,I->block_base + cCache_map_ehead_offset,false);
+		CacheFreeP(G,I->EMask,I->group_id,I->block_base + cCache_map_emask_offset,false);
+		VLACacheFreeP(G,I->EList,I->group_id,I->block_base + cCache_map_elist_offset,false);
 	 }
   OOFreeP(I);
 }
 
 void MapCacheInit(MapCache *M,MapType *I,int group_id,int block_base)
 {
+  PyMOLGlobals *G=I->G;
   int a,*p;
-
+  M->G = G;
   M->block_base = I->block_base;
-  M->Cache = CacheAlloc(int,I->NVert,group_id,block_base + cCache_map_cache_offset);
-  M->CacheLink = CacheAlloc(int,I->NVert,group_id,block_base + cCache_map_cache_link_offset);
+  M->Cache = CacheAlloc(G,int,I->NVert,group_id,block_base + cCache_map_cache_offset);
+  M->CacheLink = CacheAlloc(G,int,I->NVert,group_id,block_base + cCache_map_cache_link_offset);
   M->CacheStart = -1;
   p=M->Cache;
   for(a=0;a<I->NVert;a++)
@@ -96,12 +103,85 @@ void MapCacheReset(MapCache *M)
 
 void MapCacheFree(MapCache *M,int group_id,int block_base)
 {
-  CacheFreeP(M->Cache,group_id,block_base + cCache_map_cache_offset,false);
-  CacheFreeP(M->CacheLink,group_id,block_base + cCache_map_cache_link_offset,false);
+  PyMOLGlobals *G = M->G;
+  CacheFreeP(G,M->Cache,group_id,block_base + cCache_map_cache_offset,false);
+  CacheFreeP(G,M->CacheLink,group_id,block_base + cCache_map_cache_link_offset,false);
 }
 
 #define MapSafety 0.01F
 
+int MapInside(MapType *I,float *v,int *a,int *b,int *c) /* special version for ray-tracing */
+{
+  register int		atmp, btmp, ctmp;
+  register float	iDiv	= I->recipDiv; 
+
+  atmp	= (int)((v[0] - I->Min[0]) * iDiv) + MapBorder;
+  btmp	= (int)((v[1] - I->Min[1]) * iDiv) + MapBorder;
+  ctmp	= (int)((v[2] - I->Min[2]) * iDiv) + MapBorder;
+
+  if(atmp < I->iMin[0])
+    {
+		if((I->iMin[0] - atmp) > 3)
+        return(-1);
+		else 
+        atmp = I->iMin[0]; 
+    }
+  else if(atmp > I->iMax[0]) 
+    { 
+		if((atmp - I->iMax[0]) > 3) 
+        return(-1);
+		else 
+        atmp = I->iMax[0]; 
+    }
+
+
+	if(btmp < I->iMin[1]) 
+	{ 
+		if((I->iMin[1] - btmp) > 3) 
+			return(-1);
+		else 
+			btmp = I->iMin[1];
+	}
+	else if(btmp > I->iMax[1]) 
+	{
+		if((btmp - I->iMax[1]) > 3)
+			return(-1);
+		else 
+			btmp = I->iMax[1];
+	}
+
+   /*    printf("%d %d %d\n",ctmp,I->iMin[2],I->iMax[2]);*/
+	if(ctmp < I->iMin[2]) 
+	{ 
+		if((I->iMin[2] - ctmp) > 3) 
+			return(-1);
+		else 
+			ctmp = I->iMin[2];
+	}
+	else if(ctmp > I->iMax[2]) 
+	{
+		if((ctmp - I->iMax[2]) > 3)
+        return(0); /* just keep searching... */
+		else 
+			ctmp = I->iMax[2];
+	}
+
+   /*   printf("%d %d %d %d %d %d %d %d %d\n",
+          atmp,I->iMin[0],I->iMax[0],
+          btmp,I->iMin[1],I->iMax[1],
+          ctmp,I->iMin[2],I->iMax[2]);
+   */
+
+	if(!*(MapEStart(I,atmp,btmp,ctmp)))
+     return(0);
+		
+	*a		= atmp;
+	*b		= btmp;
+	*c		= ctmp;
+	
+	return(1);  
+}
+
 int MapInsideXY(MapType *I,float *v,int *a,int *b,int *c) /* special version for ray-tracing */
 {
 	int		atmp, btmp, ctmp;
@@ -158,23 +238,24 @@ int MapInsideXY(MapType *I,float *v,int *a,int *b,int *c) /* special version for
 
 void MapSetupExpressXY(MapType *I,int n_vert) /* setup a list of XY neighbors for each square */
 {
+   PyMOLGlobals *G=I->G;
 	int n, a,b,c,flag;
 	register int d,e,i;
 	unsigned int mapSize;
 	int st, dim2;
 	int n_alloc = n_vert * 15; /* emprical est. */
 
-	PRINTFD(FB_Map)
+	PRINTFD(G,FB_Map)
 	" MapSetupExpressXY-Debug: entered.\n"
 	ENDFD;
 	
 	mapSize		= I->Dim[0]*I->Dim[1]*I->Dim[2];
-	I->EHead	= CacheCalloc(int,mapSize,I->group_id,I->block_base + cCache_map_ehead_offset);
-	I->EMask    = CacheCalloc(int,I->Dim[0]*I->Dim[1],
-                             I->group_id,I->block_base + cCache_map_emask_offset);
-	ErrChkPtr(I->EHead);
-	I->EList	= VLACacheMalloc(n_alloc,sizeof(int),5,0,
+	I->EHead	= CacheCalloc(G,int,mapSize,I->group_id,I->block_base + cCache_map_ehead_offset);
+	ErrChkPtr(G,I->EHead);
+	I->EList	= VLACacheMalloc(G,n_alloc,sizeof(int),5,0,
                              I->group_id,I->block_base + cCache_map_elist_offset); 
+	I->EMask    = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
+                             I->group_id,I->block_base + cCache_map_emask_offset);
 	
 	n		= 1;
 	dim2	= I->Dim[2];
@@ -204,7 +285,7 @@ void MapSetupExpressXY(MapType *I,int n_vert) /* setup a list of XY neighbors fo
 							flag	= true;
 							while(i >= 0)
 							{
-								VLACacheCheck(I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+								VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
 								I->EList[n]	= i;
 								n++;
 								i	= MapNext(I,i);
@@ -221,7 +302,7 @@ void MapSetupExpressXY(MapType *I,int n_vert) /* setup a list of XY neighbors fo
 				{
 					*(I->EMask + I->Dim[1]*a + b) = true;
 					*(MapEStart(I,a,b,c))=st;
-					VLACacheCheck(I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+					VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
 					I->EList[n]=-1;
 					n++;
 				}
@@ -230,8 +311,8 @@ void MapSetupExpressXY(MapType *I,int n_vert) /* setup a list of XY neighbors fo
 	}
 		
 	I->NEElem=n;
-	VLACacheSize(I->EList,int,I->NEElem,I->group_id,I->block_base + cCache_map_elist_offset);
-	PRINTFD(FB_Map)
+	VLACacheSize(G,I->EList,int,I->NEElem,I->group_id,I->block_base + cCache_map_elist_offset);
+	PRINTFD(G,FB_Map)
 		" MapSetupExpressXY-Debug: leaving...\n"
 	ENDFD;
 }
@@ -240,6 +321,7 @@ void MapSetupExpressXY(MapType *I,int n_vert) /* setup a list of XY neighbors fo
 
 void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert) /* setup a list of XY neighbors for each square */
 {
+   PyMOLGlobals *G=I->G;
 	int		h, n, a,b,c;
 	int		j,k,dim2;
 	int		d,e;
@@ -247,17 +329,17 @@ void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert) /* setup a list of
 	int		*eBase, *hBase;
 	int      n_alloc = n_vert * 15; /* emprical est. */
 	
-	PRINTFD(FB_Map)
+	PRINTFD(G,FB_Map)
 		" MapSetupExpressXY-Debug: entered.\n"
 	ENDFD;
 	
 	/*mapSize 	= I->Dim[0]*I->Dim[1]*I->Dim[2];*/
-	I->EHead	= CacheCalloc(int, I->Dim[0]*I->Dim[1]*I->Dim[2],
+	I->EHead	= CacheCalloc(G,int, I->Dim[0]*I->Dim[1]*I->Dim[2],
                           I->group_id,I->block_base + cCache_map_ehead_offset);
-	I->EMask    = CacheCalloc(int,I->Dim[0]*I->Dim[1],
+	I->EMask    = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
                              I->group_id,I->block_base + cCache_map_emask_offset);
-	ErrChkPtr(I->EHead);
-	I->EList	= VLACacheMalloc(n_alloc,sizeof(int),5,0,
+	ErrChkPtr(G,I->EHead);
+	I->EList	= VLACacheMalloc(G,n_alloc,sizeof(int),5,0,
                              I->group_id,I->block_base + cCache_map_elist_offset); /* autozero */
 	
 	n		= 1;
@@ -290,7 +372,6 @@ void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert) /* setup a list of
 						for(e = b-1; e <= b+1; e++)
 						{
 							int	i	= *hPtr2;
-							/*i	= *(iPtr + (e * dim2));*/
 							/*i	= *MapFirst(I,d,e,c);*/
 							
 							if(i > -1)
@@ -298,7 +379,7 @@ void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert) /* setup a list of
 								flag = true;
 								while(i > -1) 
 								{
-									VLACacheCheck(I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+									VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
 									I->EList[n]	= i;
 									n++;
 									i = MapNext(I,i);
@@ -315,7 +396,7 @@ void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert) /* setup a list of
 					{
 						*(I->EMask + I->Dim[1]*a + b) = true;
 						*(MapEStart(I,a,b,c))	= st;
-						VLACacheCheck(I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+						VLACacheCheck(G,I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
 						I->EList[n] = -1;
 						n++;
 					}
@@ -332,36 +413,96 @@ void MapSetupExpressXYVert(MapType *I,float *vert,int n_vert) /* setup a list of
 	}
 
 	I->NEElem = n;
-	PRINTFD(FB_Map)
+	PRINTFD(G,FB_Map)
 		" MapSetupExpressXY-Debug: leaving...\n"
 	ENDFD;
 }
 
-void MapSetupExpress(MapType *I) /* setup a list of neighbors for each square */
+void MapSetupExpressPerp(MapType *I, float *vert, float front)
 {
+  PyMOLGlobals *G=I->G;
   int n=0;
   int a,b,c,d,e,f,i;
+
   unsigned int mapSize;
   int st,flag;
 
-  PRINTFD(FB_Map)
+  register int iMin0 = I->iMin[0];
+  register int iMin1 = I->iMin[1];
+  register int iMax0 = I->iMax[0];
+  register int iMax1 = I->iMax[1];
+  register float iDiv	= I->recipDiv;
+  register float min0 = I->Min[0] * iDiv;
+  register float min1 = I->Min[1] * iDiv;
+  register float base0, base1;
+  register float perp_factor,premult,*v0;
+  register int *emask, dim1, *link, *ptr1,*ptr2;
+  
+  PRINTFD(G,FB_Map)
     " MapSetupExpress-Debug: entered.\n"
     ENDFD;
 
   mapSize = I->Dim[0]*I->Dim[1]*I->Dim[2];
-  I->EHead=CacheAlloc(int,mapSize,
+  I->EHead=CacheAlloc(G,int,mapSize,
                  I->group_id,I->block_base + cCache_map_ehead_offset);
-  ErrChkPtr(I->EHead);
-  I->EList=VLACacheMalloc(1000,sizeof(int),5,0,
+  ErrChkPtr(G,I->EHead);
+  I->EList=VLACacheMalloc(G,1000,sizeof(int),5,0,
                      I->group_id,I->block_base + cCache_map_elist_offset);
 
+    I->EMask    = CacheCalloc(G,int,I->Dim[0]*I->Dim[1],
+                            I->group_id,I->block_base + cCache_map_emask_offset);
+  
+  emask = I->EMask;
+  dim1 = I->Dim[1];
+  link = I->Link;
+  premult = -front*iDiv;
+
   n=1;
   for(a=(I->iMin[0]-1);a<=(I->iMax[0]+1);a++)
 	 for(b=(I->iMin[1]-1);b<=(I->iMax[1]+1);b++)
 		for(c=(I->iMin[2]-1);c<=(I->iMax[2]+1);c++)
 		  {
-			 st=n;
-			 flag=false;
+          /* compute a "shadow" mask for all vertices */
+
+          i=*MapFirst(I,a,b,c);
+          while(i>=0) {
+            v0 = vert + 3*i;
+            perp_factor = premult/v0[2];
+            base0 = v0[0] * perp_factor;
+            base1 = v0[1] * perp_factor;
+            
+            d	= (int)(base0 - min0) + MapBorder;
+            e	= (int)(base1 - min1) + MapBorder;
+            
+            if(d < iMin0) {
+              d = iMin0; 
+            } else if(d > iMax0) { 
+              d = iMax0; 
+            }
+            
+            if(e < iMin1) { 
+              e = iMin1;
+            } else if(e > iMax1) {
+              e = iMax1;
+            }
+            i = link[i];
+            ptr2 = (ptr1 = emask + dim1*(d-1) + (e-1));
+            *(ptr2++) = true;
+            *(ptr2++) = true;
+            *(ptr2++) = true;
+            ptr2 = (ptr1 += dim1);
+            *(ptr2++) = true;
+            *(ptr2++) = true;
+            *(ptr2++) = true;
+            ptr2 = (ptr1 += dim1);
+            *(ptr2++) = true;
+            *(ptr2++) = true;
+            *(ptr2++) = true;
+          }
+
+            
+          st=n;
+          flag=false;
 			 for(d=a-1;d<=a+1;d++)
 				for(e=b-1;e<=b+1;e++)
 				  for(f=c-1;f<=c+1;f++)
@@ -370,16 +511,19 @@ void MapSetupExpress(MapType *I) /* setup a list of neighbors for each square */
 						if(i>=0) {
 						  flag=true;
 						  while(i>=0) {
-							 VLACacheCheck(I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+							 VLACacheCheck(G,I->EList,int,n,I->group_id,
+                                    I->block_base + cCache_map_elist_offset);
 							 I->EList[n]=i;
+                      i=link[i];
 							 n++;
-							 i=MapNext(I,i);
 						  }
 						}
 					 }
+
 			 if(flag) {
 				*(MapEStart(I,a,b,c))=st;
-				VLACacheCheck(I->EList,int,n,I->group_id,I->block_base + cCache_map_elist_offset);
+				VLACacheCheck(G,I->EList,int,n,I->group_id,
+                          I->block_base + cCache_map_elist_offset);
 				I->EList[n]=-1;
 				n++;
 			 } else {
@@ -388,8 +532,85 @@ void MapSetupExpress(MapType *I) /* setup a list of neighbors for each square */
 		  }
 
 
-  PRINTFD(FB_Map)
-    " MapSetupExpress-Debug: leaving...\n"
+  PRINTFD(G,FB_Map)
+    " MapSetupExpress-Debug: leaving...n=%d\n",n
+    ENDFD;
+  
+}
+
+void MapSetupExpress(MapType *I) /* setup a list of neighbors for each square */
+{
+  register PyMOLGlobals *G=I->G;
+  register int n=0;
+  register int c,d,e,f,i,cm1,cp2,D1D2=I->D1D2,D2=I->Dim[2];
+  register int mx2=I->iMax[2];
+  register int *link = I->Link;
+  register int st,flag;
+  register int *i_ptr3,*i_ptr4,*i_ptr5;
+  register int *e_list;
+  register int block_offset = I->block_base + cCache_map_elist_offset;
+  register int group_id = I->group_id;
+  int mx0=I->iMax[0],mx1=I->iMax[1],a,am1,ap2,*i_ptr1,b,bm1,bp2,*i_ptr2;
+  unsigned int mapSize;
+
+
+  PRINTFD(G,FB_Map)
+    " MapSetupExpress-Debug: entered.\n"
+    ENDFD;
+
+  mapSize = I->Dim[0]*I->Dim[1]*I->Dim[2];
+  I->EHead=CacheAlloc(G,int,mapSize,group_id,I->block_base + cCache_map_ehead_offset);
+  ErrChkPtr(G,I->EHead);
+  e_list=VLACacheMalloc(G,1000,sizeof(int),5,0,group_id,block_offset);
+
+  n=1;
+  for(a=(I->iMin[0]-1);a<=mx0;a++) {
+    am1 = a-1;
+    ap2 = a+2;
+    i_ptr1 = I->Head + am1 * D1D2;
+	 for(b=(I->iMin[1]-1);b<=mx1;b++) {
+      bm1 = b-1;
+      bp2 = b+2;	
+      i_ptr2 = i_ptr1 + bm1 * D2;
+      for(c=(I->iMin[2]-1);c<=mx2;c++) {
+        st=n;
+        cm1 = c-1;
+        cp2 = c+2;
+        flag=false;
+        i_ptr5 = (i_ptr4 = (i_ptr3 = i_ptr2 + cm1));
+
+        for(d=am1;d<ap2;d++) {
+          for(e=bm1;e<bp2;e++) {
+            for(f=cm1;f<cp2;f++) {
+                /*i=*MapFirst(I,d,e,f);*/
+              if((i= *(i_ptr5++))>=0) {
+                flag=true;
+                do {
+                  VLACacheCheck(G,e_list,int,n,group_id,block_offset);
+                  e_list[n++]=i;
+                  /*i=MapNext(I,i); */
+                } while( (i = link[i]) >= 0);
+              }
+            }
+            i_ptr5 = (i_ptr4 += D2);
+          }
+          i_ptr5 = (i_ptr4 = (i_ptr3 += D1D2));
+        }
+        if(flag) {
+          *(MapEStart(I,a,b,c))=st;
+          VLACacheCheck(G,e_list,int,n,group_id, block_offset);
+          e_list[n]=-1;
+          n++;
+        } else {
+          *(MapEStart(I,a,b,c))=0;
+        }
+      }
+    }
+  }
+  I->EList = e_list;
+
+  PRINTFD(G,FB_Map)
+    " MapSetupExpress-Debug: leaving...n=%d\n",n
     ENDFD;
 
 }
@@ -451,12 +672,12 @@ int MapExclLocus(MapType *I,float *v,int *a,int *b,int *c)
   return(1);
 }
 
-float MapGetSeparation(float range,float *mx,float *mn,float *diagonal)
+float MapGetSeparation(PyMOLGlobals *G,float range,float *mx,float *mn,float *diagonal)
 {
   float maxSize;
   float size,subDiv;
 
-  maxSize = SettingGet(cSetting_hash_max);
+  maxSize = SettingGet(G,cSetting_hash_max);
 
   /* find longest axis */
 
@@ -476,10 +697,10 @@ float MapGetSeparation(float range,float *mx,float *mn,float *diagonal)
   if(subDiv>maxSize ) subDiv = maxSize; /* keep it reasonable - we're talking N^3 here... */
   if(subDiv<1.0) subDiv = 1.0;
 
-  if(Feedback(FB_Map,FB_Debugging)) {
+  if(Feedback(G,FB_Map,FB_Debugging)) {
     PRINTF
       " MapGetSeparation: range %8.3f maxSize %8.3f subDiv %8.3f size %8.3f\n",range,maxSize,subDiv,size
-      ENDF;
+      ENDF(G);
     dump3f(mx,"mx");
     dump3f(mn,"mn");
     dump3f(diagonal,"diagonal");
@@ -488,22 +709,22 @@ float MapGetSeparation(float range,float *mx,float *mn,float *diagonal)
   return(size/subDiv);
 }
 
-MapType *MapNew(float range,float *vert,int nVert,float *extent)
+MapType *MapNew(PyMOLGlobals *G,float range,float *vert,int nVert,float *extent)
 {
-  return(_MapNew(range,vert,nVert,extent,NULL,-1,0));
+  return(_MapNew(G,range,vert,nVert,extent,NULL,-1,0));
 }
 
-MapType *MapNewCached(float range,float *vert,int nVert,float *extent,int group_id,int block_id)
+MapType *MapNewCached(PyMOLGlobals *G,float range,float *vert,int nVert,float *extent,int group_id,int block_id)
 {
-  return(_MapNew(range,vert,nVert,extent,NULL,group_id,block_id));
+  return(_MapNew(G,range,vert,nVert,extent,NULL,group_id,block_id));
 }
 
-MapType *MapNewFlagged(float range,float *vert,int nVert,float *extent,int *flag)
+MapType *MapNewFlagged(PyMOLGlobals *G,float range,float *vert,int nVert,float *extent,int *flag)
 {
-  return(_MapNew(range,vert,nVert,extent,flag,-1,0));
+  return(_MapNew(G,range,vert,nVert,extent,flag,-1,0));
 }
 
-static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *flag,
+static MapType *_MapNew(PyMOLGlobals *G,float range,float *vert,int nVert,float *extent,int *flag,
                         int group_id,int block_base)
 {
   int a,c;
@@ -514,12 +735,12 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
   int firstFlag;
   Vector3f diagonal;
 
-  OOAlloc(MapType);
+  OOAlloc(G,MapType);
 
-  PRINTFD(FB_Map)
+  PRINTFD(G,FB_Map)
     " MapNew-Debug: entered.\n"
     ENDFD;
-
+  I->G=G;
   I->group_id = group_id;
   I->block_base = block_base;
   I->Head = NULL;
@@ -529,8 +750,8 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
   I->EMask = NULL;
   I->NEElem=0;
   
-  I->Link=CacheAlloc(int,nVert,group_id,block_base + cCache_map_link_offset);
-  ErrChkPtr(I->Link);
+  I->Link=CacheAlloc(G,int,nVert,group_id,block_base + cCache_map_link_offset);
+  ErrChkPtr(G,I->Link);
 
   for(a=0;a<nVert;a++)
 	 I->Link[a] = -1;
@@ -546,12 +767,12 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
 	 }
   else
 	 {
-		I->Min[0]=0;
-		I->Max[0]=0;
-		I->Min[1]=0;
-		I->Max[1]=0;
-		I->Min[2]=0;
-		I->Max[2]=0;
+		I->Min[0]=0.0F;
+		I->Max[0]=0.0F;
+		I->Min[1]=0.0F;
+		I->Max[1]=0.0F;
+		I->Min[2]=0.0F;
+		I->Max[2]=0.0F;
 		if(flag) {
 		  firstFlag=true;
 		  v=vert;
@@ -608,7 +829,7 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
     }
   }
      
-  if(Feedback(FB_Map,FB_Debugging)) {
+  if(Feedback(G,FB_Map,FB_Debugging)) {
     printf(" MapSetup: %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n",
            I->Min[0],I->Min[1],I->Min[2],
            I->Max[0],I->Max[1],I->Max[2]);
@@ -630,7 +851,7 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
   }
 
   /* compute final box size */
-  I->Div = MapGetSeparation(range,I->Max,I->Min,diagonal);
+  I->Div = MapGetSeparation(G,range,I->Max,I->Min,diagonal);
   I->recipDiv = 1.0F/(I->Div); /* cache this */
 
   /* add borders to avoid special edge cases */
@@ -638,7 +859,7 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
   I->Dim[1]=(int)((diagonal[1]/I->Div)+1+(2*MapBorder));
   I->Dim[2]=(int)((diagonal[2]/I->Div)+1+(2*MapBorder));
 
-  if(Feedback(FB_Map,FB_Debugging)) {
+  if(Feedback(G,FB_Map,FB_Debugging)) {
     printf(" MapSetup: nVert: %d\n",nVert);
     printf(" MapSetup: I->Div: %8.3f\n",I->Div);
     printf(" MapSetup: %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n",
@@ -660,9 +881,9 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
 
   /* compute size and allocate */
   mapSize = I->Dim[0]*I->Dim[1]*I->Dim[2];
-  I->Head=CacheAlloc(int,mapSize,group_id,block_base + cCache_map_head_offset);
+  I->Head=CacheAlloc(G,int,mapSize,group_id,block_base + cCache_map_head_offset);
   /*printf("%d\n",mapSize);*/
-  ErrChkPtr(I->Head);
+  ErrChkPtr(G,I->Head);
 
   /* initialize */
   /*  for(a=0;a<I->Dim[0];a++)
@@ -675,7 +896,7 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
 
   I->NVert = nVert;
 
-  PRINTFD(FB_Map)
+  PRINTFD(G,FB_Map)
     " MapNew-Debug: creating 3D hash...\n"
     ENDFD;
 
@@ -705,7 +926,7 @@ static MapType *_MapNew(float range,float *vert,int nVert,float *extent,int *fla
 		}
   }
 
-  PRINTFD(FB_Map)
+  PRINTFD(G,FB_Map)
     " MapNew-Debug: leaving...\n"
     ENDFD;
 
