@@ -17,7 +17,10 @@ Z* -------------------------------------------------------------------
 #ifndef _H_Match
 #define _H_Match
 
+#include "PyMOLGlobals.h"
+
 typedef struct {
+  PyMOLGlobals *G;
   float **smat;
   float **mat;
   int *pair;
@@ -25,11 +28,11 @@ typedef struct {
   int na,nb;
 } CMatch;
 
-CMatch *MatchNew(unsigned int na,unsigned int nb);
+CMatch *MatchNew(PyMOLGlobals *G,unsigned int na,unsigned int nb);
 int MatchResidueToCode(CMatch *I,int *vla,int n);
-int MatchMatrixFromFile(CMatch *I,char *fname);
-int MatchPreScore(CMatch *I,int *vla1,int n1,int *vla2,int n2);
+int MatchMatrixFromFile(CMatch *I,char *fname,int quiet);
+int MatchPreScore(CMatch *I,int *vla1,int n1,int *vla2,int n2,int quiet);
 void MatchFree(CMatch *I);
-float MatchAlign(CMatch *I,float gap_penalty,float ext_penalty,int max_gap);
+float MatchAlign(CMatch *I,float gap_penalty,float ext_penalty,int max_gap,int quiet);
 
 #endif
