@@ -25,31 +25,50 @@ Z* -------------------------------------------------------------------
 #include"Feedback.h"
 #include"Setting.h"
 
-/* WARNING - MAJOR GOTCHA!  
 
-   A state of confusion has arisen because of mixed row-major and
-   column-major ordering in this module.  THIS NEEDS TO BE
-   STRAIGHTENED OUT before any further use is made of the following
-   routines.
+int MatrixInvTransformExtentsR44d3f(double *matrix, 
+                              float *old_min, float *old_max,
+                              float *new_min, float *new_max)
+{
+  /* just brute-forcing this for now... */
+  int a;
+  int c;
 
-   The problem is that OpenGL uses one matrix storage format, while C
-   and Python follow the other convention.  The situation
-   arose because some of the routines below are used to emulate OpenGL
-   transformations...(and are in fact based on Mesa code).
-   
-   Proposed Solution: 
+    
+  double inp_min[3],inp_max[3];
+  double out_min[3],out_max[3];
+  double inp_tst[3],out_tst[3];
 
-   (1) Assume the C-PYTHON convention of fast-column (row-major?)
-   ordering - and specifically rename routines which don't conform as
-   fast row (column-major?) "CM44f" matrices.  Clean up and resolve
-   problems with existing code that call these routines.
-   
-   (2) Move all of the 4x4 transformation code into Vector.h and rename this
-   module Algebra.h/.c (for linear algebra and fitting).
+  if(!matrix) 
+    return 0;
 
-*/
+  copy3f3d(old_min,inp_min);
+  copy3f3d(old_max,inp_max);
+  
+  for(c=0;c<8;c++) {
+    inp_tst[0] = c&0x1 ? inp_min[0] : inp_max[0];
+    inp_tst[1] = c&0x2 ? inp_min[1] : inp_max[1];
+    inp_tst[2] = c&0x4 ? inp_min[2] : inp_max[2];
 
-int MatrixInvert44f( const float *m, float *out )
+    inverse_transform44d3d(matrix,inp_tst,out_tst);
+    if(!c) {
+      copy3d(out_tst,out_max);
+      copy3d(out_tst,out_min);
+    }
+    for(a=0;a<3;a++) {
+      if(out_min[a]>out_tst[a]) out_min[a] = out_tst[a];
+      if(out_max[a]<out_tst[a]) out_max[a] = out_tst[a];
+    }
+  }
+  copy3d3f(out_min,new_min);
+  copy3d3f(out_max,new_max);
+  return 1;
+}
+
+#if 0
+/* currently unused */
+
+int MatrixInvertC44f( const float *m, float *out )
 {
 
   /* This routine included in PyMOL under the terms of the 
@@ -175,6 +194,7 @@ int MatrixInvert44f( const float *m, float *out )
 #undef MAT
 #undef SWAP_ROWS
 }
+#endif
 
 /*========================================================================*/
 int *MatrixFilter(float cutoff,int window,int n_pass,int nv,float *v1,float *v2)
@@ -340,65 +360,42 @@ int *MatrixFilter(float cutoff,int window,int n_pass,int nv,float *v1,float *v2)
 }
 
 /*========================================================================*/
-void MatrixDump44f(float *m,char *prefix)
+void MatrixTransformTTTfN3f( unsigned int n, float *q, float *m, float *p )
 {
-  if(prefix) {
-    PRINTF "%s %12.5f %12.5f %12.5f %12.5f\n",prefix,m[ 0],m[ 1],m[ 2],m[ 3] ENDF;
-    PRINTF "%s %12.5f %12.5f %12.5f %12.5f\n",prefix,m[ 4],m[ 5],m[ 6],m[ 7] ENDF;
-    PRINTF "%s %12.5f %12.5f %12.5f %12.5f\n",prefix,m[ 8],m[ 9],m[10],m[11] ENDF;
-    PRINTF "%s %12.5f %12.5f %12.5f %12.5f\n",prefix,m[12],m[13],m[14],m[15] ENDF;
-  } else {
-    PRINTF "%12.5f %12.5f %12.5f %12.5f\n",m[ 0],m[ 1],m[ 2],m[ 3] ENDF;
-    PRINTF "%12.5f %12.5f %12.5f %12.5f\n",m[ 4],m[ 5],m[ 6],m[ 7] ENDF;
-    PRINTF "%12.5f %12.5f %12.5f %12.5f\n",m[ 8],m[ 9],m[10],m[11] ENDF;
-    PRINTF "%12.5f %12.5f %12.5f %12.5f\n",m[12],m[13],m[14],m[15] ENDF;
-  }
-}
-/*========================================================================*/
-void MatrixApplyTTTfn3f( unsigned int n, float *q, const float m[16], float *p )
-/* NOTE: THIS DOESN'T EXPECT A NORMAL 4x4 TRANFORMATION MATRIX...
- * instead, it expects a 4x4 matrix with a pre-translation stored in column 3(0-3) (rows 0-2)
- * and a post-translation stored in row 3(0-3) (cols 0-2), as well as the normal 3x3 */
-{
-  float m0 = m[0],  m4 = m[4],  m8 = m[8],  m12 = m[12];
-  float m1 = m[1],  m5 = m[5],  m9 = m[9],  m13 = m[13];
-  float m2 = m[2],  m6 = m[6],  m10 = m[10],  m14 = m[14];
-  float m3 = m[3],  m7 = m[7],  m11 = m[11];
-  float p0,p1,p2;
+  const register float m0 = m[0],  m4 = m[4],  m8 = m[8],  m12 = m[12];
+  const register float m1 = m[1],  m5 = m[5],  m9 = m[9],  m13 = m[13];
+  const register float m2 = m[2],  m6 = m[6],  m10 = m[10],  m14 = m[14];
+  const register float m3 = m[3],  m7 = m[7],  m11 = m[11];
+  register float p0,p1,p2;
   while(n--) {
-	p0=*(p++) + m3;
-	p1=*(p++) + m7;
-	p2=*(p++) + m11;
-	*(q++) = m0 * p0 + m4  * p1 + m8 * p2 + m12;
-	*(q++) = m1 * p0 + m5  * p1 + m9 * p2 + m13;
-	*(q++) = m2 * p0 + m6 * p1 + m10 * p2 + m14;
+	p0=*(p++) + m12;
+	p1=*(p++) + m13;
+	p2=*(p++) + m14;
+	*(q++) = m0 * p0 + m1 * p1 + m2  * p2 + m3;
+	*(q++) = m4 * p0 + m5 * p1 + m6  * p2 + m7;
+	*(q++) = m8 * p0 + m9 * p1 + m10 * p2 + m11;
   }
 }
 /*========================================================================*/
-void MatrixLoadIdentity44f(float *m)
+void MatrixTransformR44fN3f( unsigned int n, float *q, float *m, float *p )
 {
-   m[0]=1.0F;
-   m[1]=0.0F;
-   m[2]=0.0F;
-   m[3]=0.0F;
-   
-   m[4]=0.0F;   
-   m[5]=1.0F;
-   m[6]=0.0F;
-   m[7]=0.0F;
-   
-   m[8]=0.0F;
-   m[9]=0.0F;
-   m[10]=1.0F;
-   m[11]=0.0F;
-   
-   m[12]=0.0F;
-   m[13]=0.0F;
-   m[14]=0.0F;
-   m[15]=1.0F;
+  const register float m0 = m[0],  m4 = m[4],  m8 = m[8];
+  const register float m1 = m[1],  m5 = m[5],  m9 = m[9];
+  const register float m2 = m[2],  m6 = m[6],  m10 = m[10];
+  const register float m3 = m[3],  m7 = m[7],  m11 = m[11];
+  register float p0,p1,p2;
+  while(n--) {
+    p0=*(p++);
+    p1=*(p++);
+    p2=*(p++);
+    *(q++) = m0 * p0 + m1 * p1 + m2  * p2 + m3;
+    *(q++) = m4 * p0 + m5 * p1 + m6  * p2 + m7;
+    *(q++) = m8 * p0 + m9 * p1 + m10 * p2 + m11;
+  }
 }
 /*========================================================================*/
-void MatrixRotation44f( float *m44, const float angle, const float x,const float y,const float z)
+void MatrixGetRotationC44f( float *m44, const float angle, 
+                             const float x,const float y,const float z)
 {
   float m33[9];
   rotation_matrix3f(angle,x,y,z,m33);
@@ -420,7 +417,7 @@ void MatrixRotation44f( float *m44, const float angle, const float x,const float
   m44[15]=1.0F;
 }
 /*========================================================================*/
-void MatrixRotate44f3f( float *m, const float angle, const float x,const float y,const float z)
+void MatrixRotateC44f( float *m, const float angle, const float x,const float y,const float z)
 {
   float m33[9];
   float m44[16];
@@ -441,20 +438,20 @@ void MatrixRotate44f3f( float *m, const float angle, const float x,const float y
   m44[13]=0.0F;
   m44[14]=0.0F;
   m44[15]=1.0F;
-  MatrixMultiply44f(m44,m);
+  MatrixMultiplyC44f(m44,m);
 }
 
 /*========================================================================*/
-void MatrixTranslate44f3f( float *m, const float x,const float y,const float z)
+void MatrixTranslateC44f( float *m, const float x,const float y,const float z)
 {
    m[12] = m[0] * x + m[4] * y + m[8]  * z + m[12];
    m[13] = m[1] * x + m[5] * y + m[9]  * z + m[13];
    m[14] = m[2] * x + m[6] * y + m[10] * z + m[14];
-   m[15] = m[3] * x + m[7] * y + m[11] * z + m[15];                                                                    
+   m[15] = m[3] * x + m[7] * y + m[11] * z + m[15];
 }
 
 /*========================================================================*/
-void MatrixMultiply44f( const float *b, float *m )
+void MatrixMultiplyC44f( const float *b, float *m )
 {
   /* This routine included in PyMOL under the terms of the 
    * MIT consortium license for Brian Paul's Mesa, from which it was derived. */
@@ -481,7 +478,7 @@ void MatrixMultiply44f( const float *b, float *m )
 }
                                                                                                                              
 /*========================================================================*/
-void MatrixTransform44f3f(float *m, float *q,float *p)
+void MatrixTransformC44f3f(float *m, float *q,float *p)
 {
   register float q0  = *q    , q1  = *(q+1), q2  = *(q+2);
   *(p++) = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2+m[12];
@@ -489,7 +486,7 @@ void MatrixTransform44f3f(float *m, float *q,float *p)
   *(p++) = m[ 2]*q0+m[ 6]*q1+m[10]*q2+m[14];
 }
 /*========================================================================*/
-void MatrixTransform44f4f(float *m, float *q,float *p)
+void MatrixTransformC44f4f(float *m, float *q,float *p)
 {
   register float q0  = *q    , q1  = *(q+1), q2  = *(q+2);
   *(p++) = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2+m[12];
@@ -498,7 +495,7 @@ void MatrixTransform44f4f(float *m, float *q,float *p)
   *(p++) = m[ 3]*q0+m[ 7]*q1+m[11]*q2+m[15];
 }
 /*========================================================================*/
-void MatrixInvTransform44fAs33f3f(float *m, float *q,float *p)
+void MatrixInvTransformC44fAs33f3f(float *m, float *q,float *p)
 {
   /* multiplying a column major rotation matrix as row-major will
    * give the inverse rotation */
@@ -508,7 +505,7 @@ void MatrixInvTransform44fAs33f3f(float *m, float *q,float *p)
   *(p++) = m[ 8]*q0+m[ 9]*q1+m[10]*q2;
 }
 /*========================================================================*/
-void MatrixTransform44fAs33f3f(float *m, float *q, float *p) 
+void MatrixTransformC44fAs33f3f(float *m, float *q, float *p) 
 {
   register float q0  = *q    , q1  = *(q+1), q2  = *(q+2);
   *(p++) = m[ 0]*q0+m[ 4]*q1+m[ 8]*q2;
@@ -516,7 +513,7 @@ void MatrixTransform44fAs33f3f(float *m, float *q, float *p)
   *(p++) = m[ 2]*q0+m[ 6]*q1+m[10]*q2;
 }
 /*========================================================================*/
-float MatrixGetRMS(int n,float *v1,float *v2,float *wt)
+float MatrixGetRMS(PyMOLGlobals *G,int n,float *v1,float *v2,float *wt)
 {
   /* Just Compute RMS given current coordinates */
 
@@ -560,13 +557,13 @@ float MatrixGetRMS(int n,float *v1,float *v2,float *wt)
 }
 
 /*========================================================================*/
-float MatrixFitRMS(int n,float *v1,float *v2,float *wt,float *ttt)
+float MatrixFitRMSTTTf(PyMOLGlobals *G,int n,float *v1,float *v2,float *wt,float *ttt)
 {
   /*
 	Subroutine to do the actual RMS fitting of two sets of vector coordinates
 	This routine does not rotate the actual coordinates, but instead returns 
 	the RMS fitting value, along with the center-of-mass translation vectors 
-	T1 and T2 and the rotation vector M, which rotates the translated 
+	T1 and T2 and the rotation matrix M, which rotates the translated 
 	coordinates of molecule 2 onto the translated coordinates of molecule 1.
   */
 
@@ -590,8 +587,8 @@ float MatrixFitRMS(int n,float *v1,float *v2,float *wt,float *ttt)
   }
 
   sumwt = 0.0F;
-  tol = SettingGet(cSetting_fit_tolerance);
-  maxiter = (int)SettingGet(cSetting_fit_iterations);
+  tol = SettingGet(G,cSetting_fit_tolerance);
+  maxiter = (int)SettingGet(G,cSetting_fit_iterations);
 
   /* Calculate center-of-mass vectors */
 
@@ -680,9 +677,9 @@ float MatrixFitRMS(int n,float *v1,float *v2,float *wt,float *ttt)
 
       if(iters>=maxiter) 
         {
-          PRINTFB(FB_Matrix,FB_Details)
+          PRINTFB(G,FB_Matrix,FB_Details)
             " Matrix: Warning: no convergence (%1.8f<%1.8f after %d iterations).\n",(float)tol,(float)gam,iters
-            ENDFB;
+            ENDFB(G);
           break;
         }
 
@@ -738,7 +735,7 @@ float MatrixFitRMS(int n,float *v1,float *v2,float *wt,float *ttt)
 
   err=err/sumwt;
   err=sqrt1d(err);
-
+  /*
   ttt[0]=(float)m[0][0];
   ttt[1]=(float)m[0][1];
   ttt[2]=(float)m[0][2];
@@ -754,7 +751,28 @@ float MatrixFitRMS(int n,float *v1,float *v2,float *wt,float *ttt)
   ttt[12]=(float)t2[0];
   ttt[13]=(float)t2[1];
   ttt[14]=(float)t2[2];
-  ttt[15]=1.0F; /* for compatibility with normal 4x4 matrices */
+  ttt[15]=1.0F; 
+  */
+
+  /* NOTE: TTT's are now row-major (to be more like homogenous matrices) */
+
+  ttt[ 0]=(float)m[0][0];
+  ttt[ 1]=(float)m[1][0];
+  ttt[ 2]=(float)m[2][0];
+  ttt[ 3]=(float)t2[0];
+  ttt[ 4]=(float)m[0][1];
+  ttt[ 5]=(float)m[1][1];
+  ttt[ 6]=(float)m[2][1];
+  ttt[ 7]=(float)t2[1];
+  ttt[ 8]=(float)m[0][2];
+  ttt[ 9]=(float)m[1][2];
+  ttt[10]=(float)m[2][2];
+  ttt[11]=(float)t2[2];
+  ttt[12]=(float)-t1[0];
+  ttt[13]=(float)-t1[1];
+  ttt[14]=(float)-t1[2];
+
+/* for compatibility with normal 4x4 matrices */
 
   if(fabs(err)<R_SMALL4)
     err=0.0F;
@@ -774,27 +792,17 @@ float MatrixFitRMS(int n,float *v1,float *v2,float *wt,float *ttt)
 
 /* eispack.f -- translated by f2c (version 19970805).*/
 
-#ifndef F2C_INCLUDE
-#define F2C_INCLUDE
 
-typedef long int integer;
-typedef unsigned long uinteger;
+typedef unsigned int uinteger;
 typedef char *address;
 typedef short int shortint;
 typedef float real;
-typedef double doublereal;
 typedef struct { real r, i; } complex;
 typedef struct { doublereal r, i; } doublecomplex;
-typedef long int logical;
+typedef int logical;
 typedef short int shortlogical;
 typedef char logical1;
 typedef char integer1;
-#if 0	/* Adjust for integer*8. */
-typedef long long longint;		/* system-dependent */
-typedef unsigned long long ulongint;	/* system-dependent */
-#define qbit_clear(a,b)	((a) & ~((ulongint)1 << (b)))
-#define qbit_set(a,b)	((a) |  ((ulongint)1 << (b)))
-#endif
 
 #define TRUE_ (1)
 #define FALSE_ (0)
@@ -817,113 +825,8 @@ typedef long int ftnlen;
 typedef long int ftnint;
 #endif
 
-/*external read, write*/
-typedef struct
-{	flag cierr;
-	ftnint ciunit;
-	flag ciend;
-	char *cifmt;
-	ftnint cirec;
-} cilist;
-
-/*internal read, write*/
-typedef struct
-{	flag icierr;
-	char *iciunit;
-	flag iciend;
-	char *icifmt;
-	ftnint icirlen;
-	ftnint icirnum;
-} icilist;
-
-/*open*/
-typedef struct
-{	flag oerr;
-	ftnint ounit;
-	char *ofnm;
-	ftnlen ofnmlen;
-	char *osta;
-	char *oacc;
-	char *ofm;
-	ftnint orl;
-	char *oblnk;
-} olist;
-
-/*close*/
-typedef struct
-{	flag cerr;
-	ftnint cunit;
-	char *csta;
-} cllist;
-
-/*rewind, backspace, endfile*/
-typedef struct
-{	flag aerr;
-	ftnint aunit;
-} alist;
-
-/* inquire */
-typedef struct
-{	flag inerr;
-	ftnint inunit;
-	char *infile;
-	ftnlen infilen;
-	ftnint	*inex;	/*parameters in standard's order*/
-	ftnint	*inopen;
-	ftnint	*innum;
-	ftnint	*innamed;
-	char	*inname;
-	ftnlen	innamlen;
-	char	*inacc;
-	ftnlen	inacclen;
-	char	*inseq;
-	ftnlen	inseqlen;
-	char 	*indir;
-	ftnlen	indirlen;
-	char	*infmt;
-	ftnlen	infmtlen;
-	char	*inform;
-	ftnint	informlen;
-	char	*inunf;
-	ftnlen	inunflen;
-	ftnint	*inrecl;
-	ftnint	*innrec;
-	char	*inblank;
-	ftnlen	inblanklen;
-} inlist;
-
 #define VOID void
 
-union Multitype {	/* for multiple entry points */
-	integer1 g;
-	shortint h;
-	integer i;
-	/* longint j; */
-	real r;
-	doublereal d;
-	complex c;
-	doublecomplex z;
-	};
-
-typedef union Multitype Multitype;
-
-/*typedef long int Long;*/	/* No longer used; formerly in Namelist */
-
-struct Vardesc {	/* for Namelist */
-	char *name;
-	char *addr;
-	ftnlen *dims;
-	int  type;
-	};
-typedef struct Vardesc Vardesc;
-
-struct Namelist {
-	char *name;
-	Vardesc **vars;
-	int nvars;
-	};
-typedef struct Namelist Namelist;
-
 #define abs(x) ((x) >= 0 ? (x) : -(x))
 #define dabs(x) (doublereal)abs(x)
 #define mymin(a,b) ((a) <= (b) ? (a) : (b))
@@ -934,80 +837,9 @@ typedef struct Namelist Namelist;
 #define bit_clear(a,b)	((a) & ~((uinteger)1 << (b)))
 #define bit_set(a,b)	((a) |  ((uinteger)1 << (b)))
 
-/* procedure parameter types for -A and -C++ */
-
-#if 0
-#define F2C_proc_par_types 1
-#ifdef __cplusplus
-typedef int /* Unknown procedure type */ (*U_fp)(...);
-typedef shortint (*J_fp)(...);
-typedef integer (*I_fp)(...);
-typedef real (*R_fp)(...);
-typedef doublereal (*D_fp)(...), (*E_fp)(...);
-typedef /* Complex */ VOID (*C_fp)(...);
-typedef /* Double Complex */ VOID (*Z_fp)(...);
-typedef logical (*L_fp)(...);
-typedef shortlogical (*K_fp)(...);
-typedef /* Character */ VOID (*H_fp)(...);
-typedef /* Subroutine */ int (*S_fp)(...);
-#else
-typedef int /* Unknown procedure type */ (*U_fp)();
-typedef shortint (*J_fp)();
-typedef integer (*I_fp)();
-typedef real (*R_fp)();
-typedef doublereal (*D_fp)(), (*E_fp)();
-typedef /* Complex */ VOID (*C_fp)();
-typedef /* Double Complex */ VOID (*Z_fp)();
-typedef logical (*L_fp)();
-typedef shortlogical (*K_fp)();
-typedef /* Character */ VOID (*H_fp)();
-typedef /* Subroutine */ int (*S_fp)();
-#endif
-/* E_fp is for real functions when -R is not specified */
-typedef VOID C_f;	/* complex function */
-typedef VOID H_f;	/* character function */
-typedef VOID Z_f;	/* double complex function */
-typedef doublereal E_f;	/* real function with -R not specified */
-#endif
-
-/* undef any lower-case symbols that your C compiler predefines, e.g.: */
-
-#ifndef Skip_f2c_Undefs
-#undef cray
-#undef gcos
-#undef mc68010
-#undef mc68020
-#undef mips
-#undef pdp11
-#undef sgi
-#undef sparc
-#undef sun
-#undef sun2
-#undef sun3
-#undef sun4
-#undef u370
-#undef u3b
-#undef u3b2
-#undef u3b5
-#undef unix
-#undef vax
-#endif
-#endif
-
-
-
-int pymol_rg_(
-integer *nm, integer *n,
-doublereal *a, doublereal *wr, doublereal *wi,
-integer *matz,
-doublereal *z__,
-integer *iv1,
-doublereal *fv1,
-integer *ierr);
-
 /*========================================================================*/
 
-int MatrixEigensolve33d(double *a, double *wr, double *wi, double *v)
+int MatrixEigensolveC33d(PyMOLGlobals *G,double *a, double *wr, double *wi, double *v)
 {
   integer n,nm;
   integer iv1[3];
@@ -1021,7 +853,7 @@ int MatrixEigensolve33d(double *a, double *wr, double *wi, double *v)
   n = 3;
   matz = 1;
 
-  for(x=0;x<9;x++) /* make a copy cause eispack trashes the matrix */
+  for(x=0;x<9;x++) /* make a copy -- eispack trashes the matrix */
 	 at[x]=a[x];
 
   pymol_rg_(&nm,&n,at,wr,wi,&matz,v,iv1,fv1,&ierr);
@@ -1032,7 +864,7 @@ int MatrixEigensolve33d(double *a, double *wr, double *wi, double *v)
      IS that because we're actually solving the transpose?
   */
 
-  if(Feedback(FB_Matrix,FB_Blather)) {
+  if(Feedback(G,FB_Matrix,FB_Blather)) {
     printf(" Eigensolve: eigenvectors %8.3f %8.3f %8.3f\n",v[0],v[1],v[2]);
     printf(" Eigensolve:              %8.3f %8.3f %8.3f\n",v[3],v[4],v[5]);
     printf(" Eigensolve:              %8.3f %8.3f %8.3f\n",v[6],v[7],v[8]);
@@ -1043,17 +875,6 @@ int MatrixEigensolve33d(double *a, double *wr, double *wi, double *v)
   return(ierr);
 }
 
-
-
-/*========================================================================*/
-/* DON'T GO BELOW HERE - JUST DON'T DO IT - */
-/*========================================================================*/
-/*========================================================================*/
-/*========================================================================*/
-/*========================================================================*/
-/* A MIND IS A TERRIBLE THING TO WASTE (on linear algebra) */
-/*========================================================================*/
-
 static double d_sign(doublereal *a, doublereal *b);
 
 static int balanc_(integer *nm, integer *n, doublereal *a, integer *low, integer *igh, doublereal *scale);
@@ -1091,10 +912,6 @@ doublereal *h__, doublereal *wr, doublereal *wi,
 integer *ierr);
 
 
-
-
-
-
 double d_sign(doublereal *a, doublereal *b)
 {
 double x;
@@ -1117,12 +934,12 @@ integer a_dim1, a_offset, i__1, i__2;
 doublereal d__1;
 
 /* Local variables */
-static integer iexc;
-static doublereal c__, f, g;
-static integer i__, j, k, l, m;
-static doublereal r__, s, radix, b2;
-static integer jj;
-static logical noconv;
+ integer iexc;
+ doublereal c__, f, g;
+ integer i__, j, k, l, m;
+ doublereal r__, s, radix, b2;
+ integer jj;
+ logical noconv;
 
 
 
@@ -1376,9 +1193,9 @@ doublereal *z__;
 integer z_dim1, z_offset, i__1, i__2;
 
 /* Local variables */
-static integer i__, j, k;
-static doublereal s;
-static integer ii;
+ integer i__, j, k;
+ doublereal s;
+ integer ii;
 
 
 
@@ -1490,7 +1307,7 @@ doublereal *ar, *ai, *br, *bi, *cr, *ci;
 doublereal d__1, d__2;
 
 /* Local variables */
-static doublereal s, ais, bis, ars, brs;
+doublereal s, ais, bis, ars, brs;
 
 
 /*     complex division, (cr,ci) = (ar,ai)/(br,bi) */
@@ -1520,9 +1337,9 @@ integer a_dim1, a_offset, i__1, i__2, i__3;
 doublereal d__1;
 
 /* Local variables */
-static integer i__, j, m;
-static doublereal x, y;
-static integer la, mm1, kp1, mp1;
+integer i__, j, m;
+doublereal x, y;
+integer la, mm1, kp1, mp1;
 
 
 
@@ -1668,7 +1485,7 @@ doublereal *z__;
     integer a_dim1, a_offset, z_dim1, z_offset, i__1, i__2;
 
     /* Local variables */
-    static integer i__, j, kl, mm, mp, mp1;
+     integer i__, j, kl, mm, mp, mp1;
 
 
 
@@ -1786,14 +1603,14 @@ integer *ierr;
     doublereal d__1, d__2;
 
     /* Local variables */
-    static doublereal norm;
-    static integer i__, j, k, l, m;
-    static doublereal p, q, r__, s, t, w, x, y;
-    static integer na, en, ll, mm;
-    static doublereal zz;
-    static logical notlas;
-    static integer mp2, itn, its, enm2;
-    static doublereal tst1, tst2;
+     doublereal norm;
+     integer i__, j, k, l=0, m=0;
+     doublereal p, q=0.0, r__=0.0, s, t, w, x, y;
+     integer na, en, ll, mm;
+     doublereal zz;
+     logical notlas;
+     integer mp2, itn, its, enm2;
+     doublereal tst1, tst2;
 
 /*  RESTORED CORRECT INDICES OF LOOPS (200,210,230,240). (9/29/89 BSG) */
 
@@ -2140,16 +1957,16 @@ integer *ierr;
 
 
     /* Local variables */
-    static doublereal norm;
-    static integer i__, j, k, l, m;
-    static doublereal p, q, r__, s, t, w, x, y;
-    static integer na, ii, en, jj;
-    static doublereal ra, sa;
-    static integer ll, mm, nn;
-    static doublereal vi, vr, zz;
-    static logical notlas;
-    static integer mp2, itn, its, enm2;
-    static doublereal tst1, tst2;
+     doublereal norm;
+     integer i__, j, k, l=0, m=0;
+     doublereal p, q, r__=0.0, s=0.0, t, w, x, y;
+     integer na, ii, en, jj;
+     doublereal ra, sa;
+     integer ll, mm, nn;
+     doublereal vi, vr, zz;
+     logical notlas;
+     integer mp2, itn, its, enm2;
+     doublereal tst1, tst2;
 
 
 
@@ -2822,7 +2639,10 @@ L1001:
     return 0;
 } /* hqr2_ */
 
-int pymol_rg_(nm, n, a, wr, wi, matz, z__, iv1, fv1, ierr)
+int pymol_rg_(integer *nm, integer *n, doublereal *a, doublereal *wr, 
+              doublereal *wi, integer *matz,doublereal *z__,integer *iv1,
+              doublereal  *fv1,integer  *ierr)
+#if 0
 integer *nm, *n;
 doublereal *a, *wr, *wi;
 integer *matz;
@@ -2830,12 +2650,14 @@ doublereal *z__;
 integer *iv1;
 doublereal *fv1;
 integer *ierr;
+#endif
+
 {
     /* System generated locals */
     integer a_dim1, a_offset, z_dim1, z_offset;
 
     /* Local variables */
-    static integer is1, is2;
+     integer is1, is2;
 
 /*     this subroutine calls the recommended sequence of */
 /*     subroutines from the eigensystem subroutine package (eispack) */
