@@ -100,7 +100,6 @@ void MemoryZero(char *p,char *q)
 		*p++=0;
 		count--;
 	 }
-  /*  if(p!=q) ErrFatal("MemoryZero","is broken.");*/
 #endif
 
 }
@@ -133,7 +132,7 @@ void *VLAExpand(void *ptr,unsigned int rec)
 }
 
 
-void *VLACacheExpand(void *ptr,unsigned int rec,int thread_index,int block_id)
+void *VLACacheExpand(PyMOLGlobals *G,void *ptr,unsigned int rec,int thread_index,int block_id)
 {
   VLARec *vla;
   char *start,*stop;
@@ -144,7 +143,7 @@ void *VLACacheExpand(void *ptr,unsigned int rec,int thread_index,int block_id)
 		if(vla->autoZero)
 		  soffset = sizeof(VLARec)+(vla->recSize*vla->nAlloc);
 		vla->nAlloc = (rec*(vla->growFactor+10)/10)+1;
-		vla=(void*)MemoryCacheRealloc(vla,(vla->recSize*vla->nAlloc)+sizeof(VLARec),thread_index,block_id);
+		vla=(void*)MemoryCacheRealloc(G,vla,(vla->recSize*vla->nAlloc)+sizeof(VLARec),thread_index,block_id);
 		if(!vla)
 		  {
 			 printf("VLAExpand-ERR: realloc failed.\n");
@@ -195,15 +194,15 @@ void *_VLAMalloc(const char *file,int line,unsigned int initSize,unsigned int re
 
 
 #ifndef _MemoryDebug_ON
-void *VLACacheMalloc(unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int id)
+void *VLACacheMalloc(PyMOLGlobals *G,unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int id)
 #else
-void *_VLACacheMalloc(const char *file,int line,unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int id)
+void *_VLACacheMalloc(PyMOLGlobals *G,const char *file,int line,unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int id)
 #endif
 {
   VLARec *vla;
   char *start,*stop;
 
-  vla=(void*)MemoryCacheMalloc((initSize*recSize)+sizeof(VLARec),thread,id);
+  vla=(void*)MemoryCacheMalloc(G,(initSize*recSize)+sizeof(VLARec),thread,id);
 
   if(!vla)
 	 {
@@ -236,7 +235,7 @@ void VLAFree(void *ptr)
   mfree(vla);
 }
 
-void VLACacheFree(void *ptr,int thread,int id,int force)
+void VLACacheFree(PyMOLGlobals *G,void *ptr,int thread,int id,int force)
 {
   VLARec *vla;
   if(!ptr)
@@ -245,7 +244,7 @@ void VLACacheFree(void *ptr,int thread,int id,int force)
 		exit(EXIT_FAILURE);
 	 }
   vla = &(((VLARec*)ptr)[-1]);
-  MemoryCacheFree(vla,thread,id,force);
+  MemoryCacheFree(G,vla,thread,id,force);
 }
 
 unsigned int VLAGetSize(void *ptr)
@@ -274,7 +273,7 @@ void *VLANewCopy(void *ptr)
   return((void*)&(new_vla[1]));
 }
 
-void *VLACacheSetSize(void *ptr,unsigned int newSize,int group_id,int block_id)
+void *VLACacheSetSize(PyMOLGlobals *G,void *ptr,unsigned int newSize,int group_id,int block_id)
 {
   VLARec *vla;
   char *start=NULL;
@@ -285,7 +284,7 @@ void *VLACacheSetSize(void *ptr,unsigned int newSize,int group_id,int block_id)
 	 soffset = sizeof(VLARec)+(vla->recSize*vla->nAlloc);
   }
   vla->nAlloc = newSize;
-  vla=(void*)MemoryCacheRealloc(vla,(vla->recSize*vla->nAlloc)+sizeof(VLARec),group_id,block_id);
+  vla=(void*)MemoryCacheRealloc(G,vla,(vla->recSize*vla->nAlloc)+sizeof(VLARec),group_id,block_id);
   if(!vla)
 	 {
 		printf("VLASetSize-ERR: realloc failed.\n");
