@@ -17,6 +17,7 @@ Z* -------------------------------------------------------------------
 #define _H_MemoryDebug
 
 #include "os_std.h"
+#include "PyMOLGlobals.h"
 
 /* This file can be included by C and C++ programs for
    debugging of malloc, realloc, free in C and in addition,
@@ -65,10 +66,10 @@ typedef struct VLARec {
 /* NOTE: in VLACheck, rec is a zero based array index, not a record count */
 #define VLACheck(ptr,type,rec) (ptr=(type*)(((((unsigned)rec)>=((VLARec*)(ptr))[-1].nAlloc) ? VLAExpand(ptr,(rec)) : (ptr))))
 
-#define VLACacheCheck(ptr,type,rec,t,i) (ptr=(type*)(((((unsigned)rec)>=((VLARec*)(ptr))[-1].nAlloc) ? VLACacheExpand(ptr,(rec),t,i) : (ptr))))
-#define VLACacheAlloc(type,initSize,t,i) (type*)VLACacheMalloc(initSize,sizeof(type),5,0,t,i)
-#define VLACacheFreeP(ptr,t,i,f) {if(ptr) {VLACacheFree(ptr,t,i,f);ptr=NULL;}}
-#define VLACacheSize(ptr,type,size,t,i) {ptr=(type*)VLACacheSetSize(ptr,size,t,i);}
+#define VLACacheCheck(G,ptr,type,rec,t,i) (ptr=(type*)(((((unsigned)rec)>=((VLARec*)(ptr))[-1].nAlloc) ? VLACacheExpand(G,ptr,(rec),t,i) : (ptr))))
+#define VLACacheAlloc(G,type,initSize,t,i) (type*)VLACacheMalloc(G,initSize,sizeof(type),5,0,t,i)
+#define VLACacheFreeP(G,ptr,t,i,f) {if(ptr) {VLACacheFree(G,ptr,t,i,f);ptr=NULL;}}
+#define VLACacheSize(G,ptr,type,size,t,i) {ptr=(type*)VLACacheSetSize(G,ptr,size,t,i);}
 
 #define VLAlloc(type,initSize) (type*)VLAMalloc(initSize,sizeof(type),5,0)
 #define VLACalloc(type,initSize) (type*)VLAMalloc(initSize,sizeof(type),5,1)
@@ -83,26 +84,26 @@ typedef struct VLARec {
 #define FreeP(ptr) {if(ptr) {mfree(ptr);ptr=NULL;}}
 
 void *VLAExpand(void *ptr,unsigned int rec); /* NOTE: rec is index (total-1) */
-void *VLACacheExpand(void *ptr,unsigned int rec,int thread_index,int block_id);
+void *VLACacheExpand(PyMOLGlobals *G,void *ptr,unsigned int rec,int thread_index,int block_id);
 void *MemoryReallocForSure(void *ptr, unsigned int newSize);
 
 #ifndef _MemoryDebug_ON
 void *VLAMalloc(unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero); /*growfactor 1-10*/
 
-void *VLACacheMalloc(unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int index); /*growfactor 1-10*/
+void *VLACacheMalloc(PyMOLGlobals *G,unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int index); /*growfactor 1-10*/
 #else
 #define VLAMalloc(a,b,c,d) _VLAMalloc(__FILE__,__LINE__,a,b,c,d)
 
 void *_VLAMalloc(const char *file,int line,unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero); /*growfactor 1-10*/
-#define VLACacheMalloc(a,b,c,d,t,i) _VLACacheMalloc(__FILE__,__LINE__,a,b,c,d,t,i)
-void *_VLACacheMalloc(const char *file,int line,unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int index); /*growfactor 1-10*/
+#define VLACacheMalloc(G,a,b,c,d,t,i) _VLACacheMalloc(G,__FILE__,__LINE__,a,b,c,d,t,i)
+void *_VLACacheMalloc(PyMOLGlobals *G,const char *file,int line,unsigned int initSize,unsigned int recSize,unsigned int growFactor,int autoZero,int thread,int index); /*growfactor 1-10*/
 #endif
 
 void VLAFree(void *ptr);
-void VLACacheFree(void *ptr,int thread,int id,int force);
+void VLACacheFree(PyMOLGlobals *G,void *ptr,int thread,int id,int force);
 
 void *VLASetSize(void *ptr,unsigned int newSize);
-void *VLACacheSetSize(void *ptr,unsigned int newSize,int group_id,int block_id);
+void *VLACacheSetSize(PyMOLGlobals *G,void *ptr,unsigned int newSize,int group_id,int block_id);
 unsigned int VLAGetSize(void *ptr);
 void *VLANewCopy(void *ptr);
 void MemoryZero(char *p,char *q);
@@ -151,7 +152,8 @@ void *MemoryDebugMalloc(size_t size,const char *file,int line,int type);
 void *MemoryDebugCalloc(size_t nmemb,size_t size,const char *file,int line,int type);
 void *MemoryDebugRealloc(void *ptr,size_t size,
 			 const char *file,int line,int type);
-void *MemoryDebugReallocForSure(void *ptr, unsigned int newSize,const char *file,int line,int type);
+void *MemoryDebugReallocForSure(void *ptr,size_t size,const char *file,
+                                int line,int type);
 
 void MemoryDebugFree(void *ptr,const char *file,int line,int type);
 void MemoryDebugQuietFree(void *ptr,int type);
