@@ -20,6 +20,7 @@ Z* -------------------------------------------------------------------
 #include"MemoryDebug.h"
 #include"Crystal.h"
 #include"Field.h"
+#include"PyMOLGlobals.h"
 
 #define F3(field,P1,P2,P3) Ffloat3(field,P1,P2,P3)
 #define F3Ptr(field,P1,P2,P3) Ffloat3p(field,P1,P2,P3)
@@ -27,12 +28,13 @@ Z* -------------------------------------------------------------------
 #define F4(field,P1,P2,P3,P4) Ffloat4(field,P1,P2,P3,P4)
 #define F4Ptr(field,P1,P2,P3,P4) Ffloat4p(field,P1,P2,P3,P4)
 
-int	TetsurfVolume(Isofield *field,float level,int **num,
+int	TetsurfVolume(PyMOLGlobals *G,Isofield *field,float level,int **num,
                     float **vert,int *range,int mode,
-                    MapType *voxelmap,float *a_vert,
+                    MapType *voxelmap, float *a_vert,
                     float carvebuffer,int side);
 void TetsurfGetRange(Isofield *field,CCrystal *cryst,float *mn,float *mx,int *range);
 
-int	TetsurfInit(void);
+int	TetsurfInit(PyMOLGlobals *G);
+void  TetsurfFree(PyMOLGlobals *G);
 
 #endif
