@@ -3,10 +3,6 @@
 
 #include<stdio.h>
 
-#ifdef _PYMOL_PRETEND_GLUT_FONT
-void plutBitmapCharacter(int c);
-#endif
-
 void PyMOLReadPixels(GLint x,
                   GLint y,
                   GLsizei width,
@@ -36,10 +32,20 @@ void PyMOLReadPixels(GLint x,
   glPixelStorei(GL_PACK_SKIP_ROWS, 0);
   glPixelStorei(GL_PACK_SKIP_PIXELS, 0);
   glPixelStorei(GL_PACK_ALIGNMENT, 1);
-  
+
+  /* call glFlush & glFinish first to avoid full system crash on buggy
+   * ATI Radeon drivers, such as Radeon 7000 & 9000 series.  (calling
+   * both is probably redundant and ultra-paranoid, but so what?) */
+  glFlush(); glFinish();
+
+  /* now get the pixels */
   glReadPixels(x,y,width,height,format,type,pixels);
-  
-  /* Restore current pixel store state. */
+
+  /* now flush once again just to be extra sure that we don't encounter
+   * the dreaded ATI driver bug system freeze-up */
+  glFlush(); glFinish();  
+
+  /* and then estore current pixel store state. */
   glPixelStorei(GL_PACK_SWAP_BYTES, swapbytes);
   glPixelStorei(GL_PACK_LSB_FIRST, lsbfirst);
   glPixelStorei(GL_PACK_ROW_LENGTH, rowlength);
@@ -49,7 +55,6 @@ void PyMOLReadPixels(GLint x,
 
 }
 
-
 void PyMOLDrawPixels(GLsizei width,
                   GLsizei height,
                   GLenum format,
@@ -100,6 +105,12 @@ void PyMOLCheckOpenGLErr(char *pos)
     }
 }
 
+#ifndef _PYMOL_NO_GLUT
+
+#ifdef _PYMOL_PRETEND_GLUT_FONT
+void plutBitmapCharacter(int c);
+#endif
+
 #ifdef _PYMOL_MIN_GLUT
 
 /* NULL GLUT: Bare Minimum GLUT implementation for getting PyMOL up in 
@@ -111,12 +122,9 @@ void PyMOLCheckOpenGLErr(char *pos)
 #define NULL ((void*)0)
 #endif
 
-
 static void (*idleFunc)(void) = NULL;
 static int WinX = 640,WinY=480;
 
-
-
 int      p_glutCreateWindow(const char *title) {
 
   char **myArgv,*myArgvv[2],myArgvvv[1024] = "pymol";
@@ -148,6 +156,8 @@ void     p_glutSwapBuffers(void){}
 
 void     p_glutPopWindow(void){}
 void     p_glutShowWindow(void){}
+void     p_glutHideWindow(void){}
+
 void     p_glutDestroyWindow(int theWindow) {}
 
 void     p_glutReshapeWindow(int width, int height){ WinX=width;WinY=height;}
@@ -270,6 +280,7 @@ void     p_glutSwapBuffers(void){PBlock();PyRun_SimpleString("pymol._swap_buffer
 
 void     p_glutPopWindow(void){}
 void     p_glutShowWindow(void){}
+void     p_glutHideWindow(void){}
 void     p_glutDestroyWindow(int theWindow) {}
 
 void     p_glutReshapeWindow(int width, int height){ WinX=width;WinY=height;}
@@ -2444,3 +2455,4 @@ void plutBitmapCharacter(int c)
 }
 
 #endif
+#endif
