@@ -44,6 +44,7 @@ typedef struct {
 } CPrimitive;
 
 typedef struct {
+  PyMOLGlobals *G;
   MapType *Map;
   float *Vertex,*Normal,*Precomp;
   float *Radius,*Radius2,MaxRadius,MinVoxel;
@@ -56,7 +57,7 @@ typedef struct {
 } CBasis;
 
 typedef struct {
-  float base[3];
+  float base[3]; /* where is this light ray starting from */
   CPrimitive *prim;
   float impact[3];
   float tri1,tri2;
@@ -66,6 +67,8 @@ typedef struct {
   float dotgle,flat_dotgle;
   float reflect[3];
   float trans;
+  float dir[3]; /* what is the direction of this light ray? */
+  float skip[3];
 } RayInfo;
 
 typedef struct {
@@ -81,26 +84,32 @@ typedef struct {
   int check_interior;
   CPrimitive *prim;
   MapCache cache;
-
+  float fudge0,fudge1;
   /* returns */
   int interior_flag; 
+  int pass;
+  float back_dist;
 } BasisCallRec; 
 
 
-void BasisInit(CBasis *I,int group_id);
+void BasisInit(PyMOLGlobals *G,CBasis *I,int group_id);
 void BasisFinish(CBasis *I,int group_id);
 void BasisMakeMap(CBasis *I,int *vert2prim,CPrimitive *prim,float *volume,
-                  int group_id,int block_base);
+                  int group_id,int block_base,
+                  int perspective,float front);
 
 void BasisSetupMatrix(CBasis *I);
-void BasisGetTriangleNormal(CBasis *I,RayInfo *r,int i,float *fc);
+void BasisGetTriangleNormal(CBasis *I,RayInfo *r,int i,float *fc,int perspective);
 void BasisTrianglePrecompute(float *v1,float *v2,float *v3,float *pre);
+void BasisTrianglePrecomputePerspective(float *v1,float *v2,float *v3,float *pre);
 
+int BasisHitPerspective(BasisCallRec *BC);
 int BasisHitNoShadow(BasisCallRec *BC);
 int BasisHitShadow(BasisCallRec *BC);
 
 void BasisGetTriangleFlatDotgle(CBasis *I,RayInfo *r,int i);
-void BasisSetFudge(float fudge);
+void BasisGetTriangleFlatDotglePerspective(CBasis *I,RayInfo *r,int i);
+
 void BasisCylinderSausagePrecompute(float *dir,float *pre);
 
 #define PROFILE_BASIS_OFF
