@@ -28,14 +28,33 @@ Z* -------------------------------------------------------------------
 #include"GadgetSet.h"
 #include"VFont.h"
 #include"P.h"
+#include"PyMOLGlobals.h"
+#include"Ray.h"
 
 #define CGO_read_int(p) (*((int*)(p++)))
 #define CGO_get_int(p) (*((int*)(p)))
 #define CGO_write_int(p,i) ((*((int*)(p++)))=(i))
 
-static float global_alpha = 1.0F;
+struct _CCGORenderer {
+  float alpha;
+};
 
-CGO *DebugCGO = NULL; /* initialized in Scene.c */
+int CGORendererInit(PyMOLGlobals *G)
+{
+  register CCGORenderer *I= NULL;
+  
+  I = (G->CGORenderer = Calloc(CCGORenderer,1));
+  if(I) {
+    I->alpha = 1.0F;
+    return 1;
+  } else 
+    return 0;
+}
+
+void CGORendererFree(PyMOLGlobals *G)
+{
+  FreeP(G->CGORenderer);
+}
 
 int CGO_sz[] = {
   CGO_NULL_SZ,
@@ -81,7 +100,7 @@ int CGO_sz[] = {
 
 };
 
-typedef void CGO_op(float *);
+typedef void CGO_op(CCGORenderer *I,float *);
 typedef CGO_op *CGO_op_fn;
 
 static float *CGO_add(CGO *I,int c);
@@ -98,7 +117,7 @@ CGO *CGOProcessShape(CGO *I,struct GadgetSet *gs,CGO *result)
   int sz;
 
   if(!result)
-    result = CGONew();
+    result = CGONew(I->G);
   CGOReset(result);
   VLACheck(result->op,float,I->c+32);
   
@@ -171,8 +190,11 @@ CGO *CGOProcessShape(CGO *I,struct GadgetSet *gs,CGO *result)
   return(result);
 }
 
+#ifndef _PYMOL_NOPY
+
 static PyObject *CGOArrayAsPyList(CGO *I)
 {
+
   register float *pc = I->op;
   register int op;
   int i;
@@ -205,6 +227,9 @@ static PyObject *CGOArrayAsPyList(CGO *I)
   }
   return(result);
 }
+#endif
+
+#ifndef _PYMOL_NOPY
 
 static int CGOArrayFromPyListInPlace(PyObject *list,CGO *I)
 {
@@ -253,21 +278,30 @@ static int CGOArrayFromPyListInPlace(PyObject *list,CGO *I)
   }
   return(ok);
 }
+#endif
 
 PyObject *CGOAsPyList(CGO *I)
 {
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
   PyObject *result;
   result = PyList_New(2);
   PyList_SetItem(result,0,PyInt_FromLong(I->c));
   PyList_SetItem(result,1,CGOArrayAsPyList(I));
   return(result);
+#endif
 }
 
-CGO *CGONewFromPyList(PyObject *list,int version)
+CGO *CGONewFromPyList(PyMOLGlobals *G,PyObject *list,int version)
 {
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
   int ok=true;
   int ll;
-  OOAlloc(CGO);
+  OOAlloc(G,CGO);
+  I->G=G;
   I->op=NULL;
   if(ok) ok=(list!=NULL);
   if(ok) ok=PyList_Check(list);
@@ -287,19 +321,22 @@ CGO *CGONewFromPyList(PyObject *list,int version)
     I=NULL;
   }
   return(I);
+#endif
 }
 
-CGO *CGONew(void)
+CGO *CGONew(PyMOLGlobals *G)
 {
-  OOAlloc(CGO);
+  OOAlloc(G,CGO);
+  I->G=G;
   I->op=VLAlloc(float,33);
   I->c=0;
   return(I);
 }
 
-CGO *CGONewSized(int size)
+CGO *CGONewSized(PyMOLGlobals *G,int size)
 {
-  OOAlloc(CGO);
+  OOAlloc(G,CGO);
+  I->G=G;
   I->op=VLAlloc(float,size+32);
   I->c=0;
   return(I);
@@ -658,9 +695,9 @@ int CGOCheckComplex(CGO *I)
   int op;
   SphereRec *sp;
   
-  sp = Sphere1;
+  sp = I->G->Sphere->Sphere[1];
 
-  nEdge= (int) SettingGet(cSetting_stick_quality);
+  nEdge= (int) SettingGet(I->G,cSetting_stick_quality);
 
   while((op=(CGO_MASK&CGO_read_int(pc)))) {
     switch(op) {
@@ -691,12 +728,12 @@ int CGOPreloadFonts(CGO *I)
   while((op=(CGO_MASK&CGO_read_int(pc)))) {
     switch(op) {
     case CGO_FONT:
-      ok = ok && (VFontLoad(1.0,1,1,true));
+      ok = ok && (VFontLoad(I->G,1.0,1,1,true));
       font_seen = true;
       break;
     case CGO_CHAR:
       if(!font_seen) {
-        font_id = VFontLoad(1.0,1,1,true);
+        font_id = VFontLoad(I->G,1.0,1,1,true);
         ok = ok && font_id;
         font_seen=true;
       }
@@ -732,7 +769,7 @@ int CGOCheckForText(CGO *I)
     }
     pc+=CGO_sz[op];
   }
-  PRINTFD(FB_CGO)
+  PRINTFD(I->G,FB_CGO)
    " CGOCheckForText-Debug: %d\n",fc
     ENDFD;
 
@@ -757,7 +794,7 @@ CGO *CGODrawText(CGO *I,int est,float *camera)
                 0.0F,0.0F,1.0F};
   float scale[2] = { 1.0,1.0};
 
-  cgo=CGONewSized(I->c+est);
+  cgo=CGONewSized(I->G,I->c+est);
 
   while((op=(CGO_MASK&CGO_read_int(pc)))) {
     save_pc=pc;
@@ -775,14 +812,14 @@ CGO *CGODrawText(CGO *I,int est,float *camera)
       break;
     case CGO_INDENT:
       text[0]=(unsigned char)*pc;
-      VFontIndent(font_id,text,pos,scale,axes,pc[1]);
+      VFontIndent(I->G,font_id,text,pos,scale,axes,pc[1]);
       break;
     case CGO_CHAR:
       if(!font_id) {
-        font_id = VFontLoad(1.0,1,1,false);
+        font_id = VFontLoad(I->G,1.0,1,1,false);
       }
       text[0]=(unsigned char)*pc;
-      VFontWriteToCGO(font_id,cgo,text,pos,scale,axes);
+      VFontWriteToCGO(I->G,font_id,cgo,text,pos,scale,axes);
       break;
     default:
       sz=CGO_sz[op];
@@ -808,7 +845,7 @@ CGO *CGOSimplify(CGO *I,int est)
   float *save_pc;
   int sz;
   
-  cgo=CGONewSized(I->c+est);
+  cgo=CGONewSized(I->G,I->c+est);
 
   while((op=(CGO_MASK&CGO_read_int(pc)))) {
     save_pc=pc;
@@ -906,22 +943,22 @@ void CGORenderRay(CGO *I,CRay *ray,float *color,CSetting *set1,CSetting *set2)
   float *n0=NULL,*n1=NULL,*n2=NULL,*v0=NULL,*v1=NULL,*v2=NULL,*c0=NULL,*c1=NULL,*c2=NULL;
   int mode = -1;
 
-  global_alpha = 1.0F;
+  I->G->CGORenderer->alpha = 1.0F;
 
-  widthscale = SettingGet_f(set1,set2,cSetting_cgo_ray_width_scale);
+  widthscale = SettingGet_f(I->G,set1,set2,cSetting_cgo_ray_width_scale);
 
-  /*  printf("debug %8.9f\n",SceneGetScreenVertexScale(zee));*/
-  linewidth = SettingGet_f(set1,set2,cSetting_cgo_line_width);
+  /*  printf("debug %8.9f\n",SceneGetScreenVertexScale(I->G,zee));*/
+  linewidth = SettingGet_f(I->G,set1,set2,cSetting_cgo_line_width);
   if(linewidth<0.0F) linewidth = 1.0F;
-  lineradius = SettingGet_f(set1,set2,cSetting_cgo_line_radius);
-  dotwidth = SettingGet_f(set1,set2,cSetting_cgo_dot_width);
-  dotradius = SettingGet_f(set1,set2,cSetting_cgo_dot_radius);
+  lineradius = SettingGet_f(I->G,set1,set2,cSetting_cgo_line_radius);
+  dotwidth = SettingGet_f(I->G,set1,set2,cSetting_cgo_dot_width);
+  dotradius = SettingGet_f(I->G,set1,set2,cSetting_cgo_dot_radius);
   if(lineradius<0.0F) 
-    lineradius = linewidth * ray->PixelRadius/2.0;
+    lineradius = linewidth * ray->PixelRadius/2.0F;
   if(dotradius<0.0F) 
-    dotradius = dotwidth * ray->PixelRadius/2.0;
+    dotradius = dotwidth * ray->PixelRadius/2.0F;
   if(widthscale<0.0F)
-    widthscale = ray->PixelRadius/2.0;
+    widthscale = ray->PixelRadius/2.0F;
 
   if(color)
     c0=color;
@@ -964,8 +1001,8 @@ void CGORenderRay(CGO *I,CRay *ray,float *color,CSetting *set1,CSetting *set2)
       ray->fColor3fv(ray,c0);
       break;
     case CGO_ALPHA:
-      global_alpha = *pc;
-      ray->fTransparentf(ray,1.0F - global_alpha);
+      I->G->CGORenderer->alpha = *pc;
+      ray->fTransparentf(ray,1.0F - *pc);
       break;
     case CGO_VERTEX:
       v0=pc;
@@ -1055,68 +1092,56 @@ void CGORenderRay(CGO *I,CRay *ray,float *color,CSetting *set1,CSetting *set2)
 
 /* ======== GL Rendering ======== */
 
-static void CGO_gl_begin(float *pc)
+static void CGO_gl_begin(CCGORenderer *I,float *pc)
 {
   glBegin(CGO_read_int(pc));
 
 }
 
-static void CGO_gl_end(float *pc)
+static void CGO_gl_end(CCGORenderer *I,float *pc)
 {
   glEnd();
 }
 
-static void CGO_gl_linewidth(float *pc)
+static void CGO_gl_linewidth(CCGORenderer *I,float *pc)
 {
   glLineWidth(*pc);
 }
 
-static void CGO_gl_dotwidth(float *pc)
+static void CGO_gl_dotwidth(CCGORenderer *I,float *pc)
 {
   glPointSize(*pc);
 }
 
-static void CGO_gl_enable(float *pc)
+static void CGO_gl_enable(CCGORenderer *I,float *pc)
 {
   glEnable(CGO_read_int(pc));
 }
 
-static void CGO_gl_disable(float *pc)
+static void CGO_gl_disable(CCGORenderer *I,float *pc)
 {
   glDisable(CGO_read_int(pc));
 }
 
-static void CGO_gl_alpha(float *pc)
+static void CGO_gl_alpha(CCGORenderer *I,float *pc)
 {
-  global_alpha = *pc;
+  I->alpha = *pc;
 }
 
-static void CGO_gl_null(float *pc) {
+static void CGO_gl_null(CCGORenderer *I,float *pc) {
 }
 #ifdef CYGWIN
 #define WIN32
 #endif
 
-#ifdef WIN32
-
-/* Under windows, these functions needed to be wrapped....
- * That may also be the case for any operating system which
- * doesn't use canonical gl calls */
-
-static void CGO_gl_vertex(float *v) {
+static void CGO_gl_vertex(CCGORenderer *I,float *v) {
   glVertex3fv(v);
 }
-static void CGO_gl_normal(float *v) {
+static void CGO_gl_normal(CCGORenderer *I,float *v) {
   glNormal3fv(v);
 }
-#endif
-
-static void CGO_gl_color(float *v) {
-  if(global_alpha==1.0F) 
-    glColor3fv(v);
-  else {
-    glColor4f(v[0],v[1],v[2],global_alpha);
-  }
+static void CGO_gl_color(CCGORenderer *I,float *v) {
+  glColor4f(v[0],v[1],v[2],I->alpha);
 }
 
 /* dispatch table for OpenGL */
@@ -1126,13 +1151,8 @@ CGO_op_fn CGO_gl[] = {
   CGO_gl_null,             /* 0x01 */
   CGO_gl_begin,            /* 0x02 */
   CGO_gl_end,              /* 0x03 */
-#ifdef WIN32
   CGO_gl_vertex,           /* 0x04 */
   CGO_gl_normal,           /* 0x05 */
-#else
-  (CGO_op_fn)glVertex3fv,  /* 0x04 */
-  (CGO_op_fn)glNormal3fv,  /* 0x05 */
-#endif
   CGO_gl_color,            /* 0x06 */
   CGO_gl_null,             /* 0x07 */
   CGO_gl_null,             /* 0x08 */
@@ -1169,48 +1189,52 @@ CGO_op_fn CGO_gl[] = {
 
 void CGORenderGLPickable(CGO *I,Pickable **pick,void *ptr,CSetting *set1,CSetting *set2)
 {
-  register float *pc = I->op;
-  register int op;
-  int i,j;
-  Pickable *p;
-
-  if(I->c) {
-    i=(*pick)->index;
-
-    glLineWidth(SettingGet_f(set1,set2,cSetting_cgo_line_width));
-
-    while((op=(CGO_MASK&CGO_read_int(pc)))) {
-      if(op!=CGO_PICK_COLOR) {
-        if(op!=CGO_COLOR) {
-          CGO_gl[op](pc); /* ignore color changes */
+  register PyMOLGlobals *G = I->G;
+  if(G->ValidContext) {
+    register float *pc = I->op;
+    register int op;
+    register CCGORenderer *R =G->CGORenderer;
+    
+    int i,j;
+    Pickable *p;
+    
+    if(I->c) {
+      i=(*pick)->index;
+      
+      glLineWidth(SettingGet_f(G,set1,set2,cSetting_cgo_line_width));
+      
+      while((op=(CGO_MASK&CGO_read_int(pc)))) {
+        if(op!=CGO_PICK_COLOR) {
+          if(op!=CGO_COLOR) {
+            CGO_gl[op](R,pc); /* ignore color changes */
+          }
+        } else {
+          i++;
+          if(!(*pick)[0].ptr) {
+            /* pass 1 - low order bits */
+            
+            glColor3ub((uchar)((i&0xF)<<4),(uchar)((i&0xF0)|0x8),(uchar)((i&0xF00)>>4)); /* we're encoding the index into the color */
+            VLACheck((*pick),Pickable,i);
+            p=(*pick)+i;
+            p->ptr = ptr;
+            p->index = (int)*pc;
+            p->bond = (int)*(pc+1); /* actually holds state information */
+          } else { 
+            /* pass 2 - high order bits */
+            
+            j=i>>12;
+            
+            glColor3ub((uchar)((j&0xF)<<4),(uchar)((j&0xF0)|0x8),(uchar)((j&0xF00)>>4)); 
+            
+          }			 
+          
         }
-      } else {
-        i++;
-        if(!(*pick)[0].ptr) {
-          /* pass 1 - low order bits */
-          
-          glColor3ub((uchar)((i&0xF)<<4),(uchar)((i&0xF0)|0x8),(uchar)((i&0xF00)>>4)); /* we're encoding the index into the color */
-          VLACheck((*pick),Pickable,i);
-          p=(*pick)+i;
-          p->ptr = ptr;
-          p->index = (int)*pc;
-          p->bond = (int)*(pc+1); /* actually holds state information */
-        } else { 
-          /* pass 2 - high order bits */
-          
-          j=i>>12;
-          
-          glColor3ub((uchar)((j&0xF)<<4),(uchar)((j&0xF0)|0x8),(uchar)((j&0xF00)>>4)); 
-          
-        }			 
-        
+        pc+=CGO_sz[op];
       }
-      pc+=CGO_sz[op];
+      (*pick)[0].index = i; /* pass the count */
+      
     }
-    (*pick)[0].index = i; /* pass the count */
-
   }
-  
 }
 
 
@@ -1219,24 +1243,25 @@ void CGORenderGL(CGO *I,float *color,CSetting *set1,CSetting *set2)
       * the ASM loop is about 2X long as raw looped GL calls,
       * but hopefully superscaler processors won't care */
 {
-  register float *pc = I->op;
-  register int op;
-  global_alpha = 1.0F;
-
-  if(I->c) {
-    if(color) 
-      glColor3fv(color);
-    else
-      glColor3f(1.0,1.0,1.0);
-    glLineWidth(SettingGet_f(set1,set2,cSetting_cgo_line_width));
-    glPointSize(SettingGet_f(set1,set2,cSetting_cgo_dot_width));
-
-    while((op=(CGO_MASK&CGO_read_int(pc)))) {
-      /*      printf("%d\n",op);
-      if((op==CGO_VERTEX)||(op==CGO_COLOR))
-      dump3f(pc,"where");*/
-      CGO_gl[op](pc);
-      pc+=CGO_sz[op];
+  register PyMOLGlobals *G = I->G;
+  if(G->ValidContext) {
+    register float *pc = I->op;
+    register int op;
+    register CCGORenderer *R = G->CGORenderer;
+    
+    if(I->c) {
+      R->alpha = 1.0F;
+      if(color) 
+        glColor3fv(color);
+      else
+        glColor3f(1.0,1.0,1.0);
+      glLineWidth(SettingGet_f(I->G,set1,set2,cSetting_cgo_line_width));
+      glPointSize(SettingGet_f(I->G,set1,set2,cSetting_cgo_dot_width));
+      
+      while((op=(CGO_MASK&CGO_read_int(pc)))) {
+        CGO_gl[op](R,pc);
+        pc+=CGO_sz[op];
+      }
     }
   }
 }
@@ -1250,14 +1275,10 @@ void CGOSimpleSphere(CGO *I,float *v,float vdw)
   int b,c;
   int ds;
 
-  ds = SettingGet_i(NULL,NULL,cSetting_cgo_sphere_quality);
+  ds = SettingGet_i(I->G,NULL,NULL,cSetting_cgo_sphere_quality);
   if(ds<0) ds=0;
-  switch(ds) {
-  case 0: sp=Sphere0; break;
-  case 1: sp=Sphere1; break;
-  case 2: sp=Sphere2; break;
-  default: sp=Sphere3; break;
-  }
+  if(ds>3) ds=3;
+  sp = I->G->Sphere->Sphere[ds];
   
   q=sp->Sequence;
 
@@ -1304,9 +1325,9 @@ void CGOSimpleCylinder(CGO *I,float *v1,float *v2,float tube_size,float *c1,floa
   int c;
 
   v=v_buf;
-  nEdge= (int)SettingGet(cSetting_stick_quality);
-  overlap = tube_size*SettingGet(cSetting_stick_overlap);
-  nub = tube_size*SettingGet(cSetting_stick_nub);
+  nEdge= (int)SettingGet(I->G,cSetting_stick_quality);
+  overlap = tube_size*SettingGet(I->G,cSetting_stick_overlap);
+  nub = tube_size*SettingGet(I->G,cSetting_stick_nub);
 
   if(nEdge>MAX_EDGE)
     nEdge=MAX_EDGE;
