@@ -21,6 +21,8 @@ Z* -------------------------------------------------------------------
 #include "Text.h"
 #include "Ray.h"
 #include "Character.h"
+#include "Scene.h"
+#include "Util.h"
 
 static void FontGLUTSave(CFontGLUT *I)
 {
@@ -51,54 +53,86 @@ static void FontGLUTRestore(CFontGLUT *I)
 
 static char *FontGLUTRenderOpenGL(CFontGLUT *I,char *st)
 {
-  int c;
-  FontGLUTBitmapFontRec *font_info = I->glutFont;
-  int first,last;
-  FontGLUTBitmapCharRec const *ch;
-
-  if(st&&(*st)) {
+  register PyMOLGlobals *G = I->Font.G;
+  if(G->ValidContext) {
+    int c;
+    FontGLUTBitmapFontRec *font_info = I->glutFont;
+    int first,last;
+    FontGLUTBitmapCharRec const *ch;
+    int textured = SettingGetGlobal_b(G,cSetting_texture_fonts);
+    int pushed = OrthoGetPushed(G);
     
-    glColor3fv(TextGetColor());
-    glRasterPos4fv(TextGetPos());
-    FontGLUTSave(I);
     
-    first = font_info->first;
-    last = first + font_info->num_chars;
-
-    while((c=*(st++))) {
-      if ((c >= first) && (c < last))
-        {
-          ch = font_info->ch[c - first];
-          if (ch) {
-            glBitmap(ch->width, ch->height, 
-                     ch->xorig, ch->yorig,
-                     ch->advance, 0, ch->bitmap);
-#if 0
-            /* testing code for Character engine */
-            { 
-              CharFngrprnt fprnt;
-              unsigned char *rgba;
-              fprnt.u.i.text_id = I->Font.TextID;
-              rgba = fprnt.u.i.color;
-              TextGetColorUChar(rgba,rgba+1,rgba+2,rgba+3);
-              fprnt.u.i.ch = c;
-              {
-                int id = CharacterFind(&fprnt);
-                if(!id) {
-                  id = CharacterNewFromBitmap(ch->width, 
-                                              ch->height, 
-                                              (unsigned char*)ch->bitmap,
-                                            &fprnt);
+    if(st&&(*st)) {
+      
+      if(!textured) {
+        glColor3fv(TextGetColor(G));
+        glRasterPos4fv(TextGetPos(G));
+        FontGLUTSave(I);
+      }
+      
+      first = font_info->first;
+      last = first + font_info->num_chars;
+      
+      
+      
+      if(textured && !pushed) {
+        float *v = TextGetPos(G);
+        float zero[3]= {0.0F,0.0F,0.0F};
+        ScenePushRasterMatrix(G,v);
+        TextSetPos(G,zero);
+      }
+      
+      while((c=*(st++))) {
+        if ((c >= first) && (c < last))
+          {
+            ch = font_info->ch[c - first];
+            if (ch) {
+              if(!textured) {
+                
+                glBitmap(ch->width, ch->height, 
+                         ch->xorig, ch->yorig,
+                         ch->advance, 0, ch->bitmap);
+                TextAdvance(G,ch->advance);
+              } else {
+                CharFngrprnt fprnt;
+                unsigned char *rgba;
+                UtilZeroMem(&fprnt,sizeof(fprnt));
+                fprnt.u.i.text_id = I->Font.TextID;
+                rgba = fprnt.u.i.color;
+                TextGetColorUChar(G,rgba,rgba+1,rgba+2,rgba+3);
+                fprnt.u.i.ch = c;
+                {
+                  int id = CharacterFind(G,&fprnt);
+                  if(!id) {
+                    id = CharacterNewFromBitmap(G,ch->width, 
+                                                ch->height, 
+                                                (unsigned char*)ch->bitmap,
+                                                &fprnt);
+                  }
+                  if(id) {
+                    CharacterRenderOpenGL(G,id,
+                                          (float)ch->xorig,
+                                          (float)ch->yorig,
+                                          (float)ch->advance); /* handles advance */
+                  }
                 }
               }
             }
-            /* end testing code */
-#endif
-
           }
-        }
+      }
+                
+      if(textured && !pushed) {
+        ScenePopRasterMatrix(G);
+      }
+      if(!textured) {
+        FontGLUTRestore(I); 
+        #ifdef _PYMOL_OSX
+        /* workaround for screen flashes on late-model nVidia hardware */
+        glFlush();
+        #endif
+      }
     }
-    FontGLUTRestore(I); 
   }
   return st;
 }
@@ -113,11 +147,12 @@ static char *FontGLUTRenderRay(CRay *ray, CFontGLUT *I,char *st)
   unsigned char *rgba;
   
   if(st&&(*st)) {
+    UtilZeroMem(&fprnt,sizeof(fprnt));
     first = font_info->first;
     last = first + font_info->num_chars;
     fprnt.u.i.text_id = I->Font.TextID;
     rgba = fprnt.u.i.color;
-    TextGetColorUChar(rgba,rgba+1,rgba+2,rgba+3);
+    TextGetColorUChar(I->Font.G,rgba,rgba+1,rgba+2,rgba+3);
 
     while((c=*(st++))) {
       if ((c >= first) && (c < last))
@@ -126,17 +161,17 @@ static char *FontGLUTRenderRay(CRay *ray, CFontGLUT *I,char *st)
           if (ch) {
             fprnt.u.i.ch = c;
             {
-              int id = CharacterFind(&fprnt);
+              int id = CharacterFind(I->Font.G,&fprnt);
               if(!id) {
-                id = CharacterNewFromBitmap(ch->width, 
-                                          ch->height, 
-                                          (unsigned char*)ch->bitmap,
-                                          &fprnt);
+                id = CharacterNewFromBitmap(I->Font.G,ch->width, 
+                                            ch->height, 
+                                            (unsigned char*)ch->bitmap,
+                                            &fprnt);
               }
                ray->fCharacter(ray,id,
                             (float)ch->xorig,
                             (float)ch->yorig,
-                            (float)ch->advance);
+                               (float)ch->advance); /* handles advance */
             }
           }
         }
@@ -150,10 +185,10 @@ void FontGLUTFree(CFont *I)
   OOFreeP(I);
 }
 
-CFont* FontGLUTNew(int font_code)
+CFont* FontGLUTNew(PyMOLGlobals *G,int font_code)
 {  
-  OOAlloc(CFontGLUT);
-  FontInit(&I->Font);
+  OOAlloc(G,CFontGLUT);
+  FontInit(G,&I->Font);
   I->Font.fRenderOpenGL = (FontRenderOpenGLFn*)FontGLUTRenderOpenGL;
   I->Font.fRenderRay = (FontRenderRayFn*)FontGLUTRenderRay;
   I->Font.fFree = FontGLUTFree;
