@@ -25,6 +25,8 @@ Z* -------------------------------------------------------------------
 #include"P.h"
 #include"Util.h"
 
+#ifndef _PYMOL_NOPY
+
 PyObject *PConvAutoNone(PyObject *result) /* automatically own Py_None */
 {
   if(result==Py_None)
@@ -405,6 +407,36 @@ int PConvPyListToFloatArray(PyObject *obj,float **f)
   return(ok);
 }
 
+int PConvPyListToFloatVLANoneOkay(PyObject *obj,float **f)
+{
+  int a,l;
+  float *ff;
+  int ok=true;
+  if(!obj) {
+    *f=NULL;
+    ok=false;
+  } else if(obj==Py_None) {
+    *f=NULL;
+    ok=true;
+  } else if(!PyList_Check(obj)) {
+    *f=NULL;
+    ok=false;
+  } else {
+    l=PyList_Size(obj);
+    if(!l)
+      ok=-1;
+    else 
+      ok=l;
+    (*f) = VLAlloc(float,l);
+    ff = (*f);
+    for(a=0;a<l;a++)
+      *(ff++) = (float)PyFloat_AsDouble(PyList_GetItem(obj,a));
+    VLASize((*f),float,l);
+  }
+  return(ok);
+
+}
+
 int PConvPyListToFloatVLA(PyObject *obj,float **f)
 {
   int a,l;
@@ -493,6 +525,31 @@ int PConvPyListToIntArray(PyObject *obj,int **f)
   return(ok);
 }
 
+int PConvPyListToDoubleArray(PyObject *obj,double **f)
+{
+  int a,l;
+  double *ff;
+  int ok=true;
+  if(!obj) {
+    *f=NULL;
+    l=0;
+  } else if(!PyList_Check(obj)) {
+    *f=NULL;
+    ok=false;
+  } else {
+    l=PyList_Size(obj);
+    if(!l)
+      ok=-1;
+    else
+      ok=l;
+    (*f) = Alloc(double,l);
+    ff = (*f);
+    for(a=0;a<l;a++)
+      *(ff++) = PyFloat_AsDouble(PyList_GetItem(obj,a));
+  }
+  return(ok);
+}
+
 int PConvPyListToIntVLA(PyObject *obj,int **f)
 {
   int a,l;
@@ -606,7 +663,7 @@ int PConvPyListToIntArrayInPlaceAutoZero(PyObject *obj,int *ii,int ll)
       ok=-1;
     else
       ok=l;
-    for(a=0;a<l;a++) 
+    for(a=0;(a<l)&&(a<ll);a++) 
       *(ii++) = PyInt_AsLong(PyList_GetItem(obj,a)); 
     while(l<ll) {
       *(ii++)=0;
@@ -630,7 +687,7 @@ int PConvPyListToSIntArrayInPlaceAutoZero(PyObject *obj,short int *ii,int ll)
       ok=-1;
     else
       ok=l;
-    for(a=0;a<l;a++) 
+    for(a=0;(a<l)&&(a<ll);a++) 
       *(ii++) = (short int)PyInt_AsLong(PyList_GetItem(obj,a)); 
     while(l<ll) {
       *(ii++)=0;
@@ -654,7 +711,7 @@ int PConvPyListToFloatArrayInPlaceAutoZero(PyObject *obj,float *ii,int ll)
       ok=-1;
     else
       ok=l;
-    for(a=0;a<l;a++) 
+    for(a=0;(a<l)&&(a<ll);a++) 
       *(ii++) = (float)PyFloat_AsDouble(PyList_GetItem(obj,a)); 
     while(l<ll) {
       *(ii++)=0.0f;
@@ -675,6 +732,20 @@ PyObject *PConvFloatArrayToPyList(float *f,int l)
   return(result);
 }
 
+PyObject *PConvFloatArrayToPyListNullOkay(float *f,int l)
+{
+  int a;
+  PyObject *result = Py_None;
+  if(!f) {
+    result = PConvAutoNone(NULL);
+  } else {
+    result=PyList_New(l);
+    for(a=0;a<l;a++) 
+      PyList_SetItem(result,a,PyFloat_FromDouble((double)*(f++)));
+  }
+  return(result);
+}
+
 PyObject *PConvDoubleArrayToPyList(double *f,int l)
 {
   int a;
@@ -846,5 +917,8 @@ void PConv44PyListTo44f(PyObject *src,float *dest) /* note lost of precision */
   }
 }
 
+#else
+typedef int this_file_is_no_longer_empty;
+#endif
 
 
