@@ -18,6 +18,8 @@ Z* -------------------------------------------------------------------
 
 #include"os_python.h"
 
+#ifndef _PYMOL_NOPY
+
 /* Convenient conversion routines for C<->Python data interchange
    
    Note that all of these routines assume that we have the global
@@ -103,14 +105,17 @@ PyObject *PConvStringVLAToPyList(char *str);
 void PConv44PyListTo44f(PyObject *src,float *dest); /* note loss of precision */
 
 int PConvPyListToFloatVLA(PyObject *obj,float **f);
+int PConvPyListToFloatVLANoneOkay(PyObject *obj,float **f);
 int PConvPyList3ToFloatVLA(PyObject *obj,float **f);
 int PConvPyListToFloatArray(PyObject *obj,float **f);
+int PConvPyListToDoubleArray(PyObject *obj,double **f);
 int PConvPyListToFloatArrayInPlace(PyObject *obj,float *ff,int ll);
 int PConvPyListToFloatArrayInPlaceAutoZero(PyObject *obj,float *ii,int ll);
 
 int PConvPyListToDoubleArrayInPlace(PyObject *obj,double *ff,int ll);
 
 PyObject *PConvFloatArrayToPyList(float *f,int l);
+PyObject *PConvFloatArrayToPyListNullOkay(float *f,int l);
 PyObject *PConvDoubleArrayToPyList(double *f,int l);
 
 int PConvPyListToIntArray(PyObject *obj,int **f);
@@ -125,6 +130,7 @@ PyObject *PConv3DIntArrayTo3DPyList(int ***array,int *dim);
 PyObject *PConvAutoNone(PyObject *result); /* automatically own Py_None */
 
 #endif
+#endif
 
 
 
