@@ -36,7 +36,7 @@ struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int rep)
 {
   Rep *tmp = NULL;
 
-  PRINTFD(FB_Rep)
+  PRINTFD(I->G,FB_Rep)
     " RepRebuild-Debug: entered: rep %d I->fNew %p\n",rep,(void*)I->fNew
     ENDFD;
 
@@ -57,7 +57,7 @@ struct Rep *RepRebuild(struct Rep *I,struct CoordSet *cs,int rep)
 struct Rep *RepUpdate(struct Rep *I,struct CoordSet *cs,int rep)
 {
 
-  PRINTFD(FB_Rep)
+  PRINTFD(I->G,FB_Rep)
     " RepUpdate-Debug: entered: rep %d I->MaxInvalid %d\n",rep,I->MaxInvalid
     ENDFD;
 
@@ -108,8 +108,9 @@ void RepInvalidate(struct Rep *I,struct CoordSet *cs,int level)
   if(level>I->MaxInvalid) I->MaxInvalid=level;
 }
 /*========================================================================*/
-void RepInit(Rep *I)
+void RepInit(PyMOLGlobals *G,Rep *I)
 {
+  I->G = G;
   I->fInvalidate = RepInvalidate;
   I->fUpdate = RepUpdate;
   I->fRender = RepRenderBox;
@@ -124,13 +125,16 @@ void RepInit(Rep *I)
   I->displayList = 0;
 }
 /*========================================================================*/
-void RepFree(Rep *I)
+void RepPurge(Rep *I)
 {
-  if(PMGUI) {
+  register PyMOLGlobals *G = I->G;
+  if(G->HaveGUI) {
     if(I->displayList) {
       if(PIsGlutThread()) {
-        glDeleteLists(I->displayList,1);
-        I->displayList = 0;
+        if(G->ValidContext) {
+          glDeleteLists(I->displayList,1);
+          I->displayList = 0;
+        }
       } else {
         char buffer[255]; /* pass this off to the main thread */
         sprintf(buffer,"_cmd.gl_delete_lists(%d,%d)\n",I->displayList,1);
@@ -143,7 +147,8 @@ void RepFree(Rep *I)
 /*========================================================================*/
 void RepRenderBox(struct Rep *this,CRay *ray,Pickable **pick)
 {
-  if(PMGUI) {
+  register PyMOLGlobals *G = this->G;
+  if(G->HaveGUI && G->ValidContext) {
     glBegin(GL_LINE_LOOP);
     glVertex3f(-0.5F,-0.5F,-0.5F);
     glVertex3f(-0.5F,-0.5F, 0.5F);
