@@ -27,8 +27,9 @@ Z* -------------------------------------------------------------------
 #define cCartoon_tube 4
 #define cCartoon_arrow 5
 #define cCartoon_dumbbell 6
-#define cCartoon_skip_helix -2
+#define cCartoon_putty 7
 
+#define cCartoon_skip_helix -2
 #define cRepAll       -1
 
 /* WARNING: don't change these -- you'll break sessions!
@@ -50,8 +51,12 @@ Z* -------------------------------------------------------------------
 #define cRepCGO        13
 #define cRepCallback   14
 #define cRepExtent     15
+#define cRepSlice      16
 
-#define cRepCnt        16
+#define cRepAngle      17
+#define cRepDihedral    18
+
+#define cRepCnt        19
 
 /* Hierarchical invalidation scheme - 
  * each higher level event implies all of the lower levels 
@@ -68,6 +73,7 @@ Z* -------------------------------------------------------------------
 #define cRepInvPick   19
 /* invalidate visible atoms */
 #define cRepInvVisib  20
+#define cRepInvVisib2 21
 /* invalidate atomic properties */
 #define cRepInvProp   22
 /* invalidate coordinates */
@@ -85,6 +91,7 @@ struct CoordSet;
 struct Object;
 
 typedef struct Rep {
+  PyMOLGlobals *G;
   void            (*fRender)(struct Rep *I,CRay *ray,Pickable **pick);  
   struct Rep     *(*fUpdate)(struct Rep *I,struct CoordSet *cs,int rep);
   void        (*fInvalidate)(struct Rep *I,struct CoordSet *cs,int level);
@@ -101,7 +108,7 @@ typedef struct Rep {
   int displayList;
 } Rep;
 
-void RepInit(Rep *I);
-void RepFree(Rep *I);
+void RepInit(PyMOLGlobals *G,Rep *I);
+void RepPurge(Rep *I);
 
 #endif
