@@ -17,6 +17,7 @@ Z* -------------------------------------------------------------------
 #define _H_Setting
 
 #include"os_python.h"
+#include"PyMOLGlobals.h"
 
 typedef char SettingName[255];
 
@@ -28,11 +29,12 @@ typedef struct {
   unsigned int max_size;
 } SettingRec;
 
-typedef struct {
+struct _CSetting {
+  PyMOLGlobals *G;
   unsigned int size;
   char *data;
   SettingRec *info;
-} CSetting;
+};
 
 #define cSetting_blank       0
 #define cSetting_boolean     1
@@ -47,14 +49,14 @@ typedef struct {
  * in contrast, set commands expand the current list 
  */
 
-void SettingInitGlobal(int alloc,int reset_gui);
-void SettingFreeGlobal(void);
+void SettingInitGlobal(PyMOLGlobals *G,int alloc,int reset_gui);
+void SettingFreeGlobal(PyMOLGlobals *G);
 
-CSetting *SettingNew(void);
+CSetting *SettingNew(PyMOLGlobals *G);
 void SettingFreeP(CSetting *I);
-void SettingInit(CSetting *I);
+void SettingInit(PyMOLGlobals *G,CSetting *I);
 void SettingPurge(CSetting *I);
-void SettingCheckHandle(CSetting **handle);
+void SettingCheckHandle(PyMOLGlobals *G,CSetting **handle);
 
 int SettingSet_b(CSetting *I,int index, int value);
 int SettingSet_i(CSetting *I,int index, int value);
@@ -63,71 +65,75 @@ int SettingSet_s(CSetting *I,int index, char *value);
 int SettingSet_3f(CSetting *I,int index, float value1,float value2,float value3);
 int SettingSet_3fv(CSetting *I,int index, float *value);
 
-int SettingGetTextValue(CSetting *set1,CSetting *set2,int index,char *buffer);
+int SettingGetTextValue(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index,char *buffer);
 
-int SettingSetTuple(CSetting *I,int index,PyObject *tuple);
 int SettingUnset(CSetting *I,int index);
 
 void SettingClear(CSetting *I,int index); /* don't call this for the global list! */
 
-int SettingGetType(int index); /* based on global types, always succeeds */
+int SettingGetType(PyMOLGlobals *G,int index); /* based on global types, always succeeds */
 
-int   SettingGetGlobal_b(int index); /* always succeed */
-int   SettingGetGlobal_i(int index); /* always succeed */
-float SettingGetGlobal_f(int index); /* always succeed */
-char *SettingGetGlobal_s(int index); /* always succeeds */
-void  SettingGetGlobal_3f(int index,float *value); /* always succeeds */
-float *SettingGetGlobal_3fv(int index); /* always succeed */
+int   SettingGetGlobal_b(PyMOLGlobals *G,int index); /* always succeed */
+int   SettingGetGlobal_i(PyMOLGlobals *G,int index); /* always succeed */
+float SettingGetGlobal_f(PyMOLGlobals *G,int index); /* always succeed */
+char *SettingGetGlobal_s(PyMOLGlobals *G,int index); /* always succeeds */
+void  SettingGetGlobal_3f(PyMOLGlobals *G,int index,float *value); /* always succeeds */
+float *SettingGetGlobal_3fv(PyMOLGlobals *G,int index); /* always succeed */
 
-int   SettingSetGlobal_b(int index,int value);
-int   SettingSetGlobal_i(int index,int value);
-int   SettingSetGlobal_f(int index,float value);
+int   SettingSetGlobal_b(PyMOLGlobals *G,int index,int value);
+int   SettingSetGlobal_i(PyMOLGlobals *G,int index,int value);
+int   SettingSetGlobal_f(PyMOLGlobals *G,int index,float value);
 
-int   SettingSetSmart_i(CSetting *set1,CSetting *set2,int index, int value);
+int   SettingSetSmart_i(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index, int value);
 /* more to come */
 
-int SettingGetIfDefined_i(CSetting *set1,int index,int *value);
-int SettingGetIfDefined_b(CSetting *set1,int index,int *value);
-int SettingGetIfDefined_f(CSetting *set1,int index,float *value);
-int SettingGetIfDefined_s(CSetting *set1,int index,char **value);
-int SettingGetIfDefined_3fv(CSetting *set1,int index,float **value);
-int SettingGetIfDefined_color(CSetting *set1,int index,int *value);
+int SettingGetIfDefined_i(PyMOLGlobals *G,CSetting *set1,int index,int *value);
+int SettingGetIfDefined_b(PyMOLGlobals *G,CSetting *set1,int index,int *value);
+int SettingGetIfDefined_f(PyMOLGlobals *G,CSetting *set1,int index,float *value);
+int SettingGetIfDefined_s(PyMOLGlobals *G,CSetting *set1,int index,char **value);
+int SettingGetIfDefined_3fv(PyMOLGlobals *G,CSetting *set1,int index,float **value);
+int SettingGetIfDefined_color(PyMOLGlobals *G,CSetting *set1,int index,int *value);
 
 /* more to come */
 
 int SettingSet_color(CSetting *I,int index, char *value);
 
 
-int   SettingGet_b  (CSetting *set1,CSetting *set2,int index);
-int   SettingGet_i  (CSetting *set1,CSetting *set2,int index);
-float SettingGet_f  (CSetting *set1,CSetting *set2,int index);
-char  *SettingGet_s  (CSetting *set1,CSetting *set2,int index);
-void  SettingGet_3f (CSetting *set1,CSetting *set2,int index,float *value);
-float *SettingGet_3fv (CSetting *set1,CSetting *set2,int index);
-int   SettingGet_color(CSetting *set1,CSetting *set2,int index);
+int   SettingGet_b(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
+int   SettingGet_i(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
+float SettingGet_f(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
+char  *SettingGet_s(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
+void  SettingGet_3f(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index,float *value);
+float *SettingGet_3fv(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
+int   SettingGet_color(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index);
 
-PyObject *SettingGetTuple(CSetting *set1,CSetting *set2,int index); /* (type,(value,)) */
-PyObject *SettingGetDefinedTuple(CSetting *set1,int index);
+int SettingSetFromString(PyMOLGlobals *G,CSetting *I,int index,char *st);
 
-void SettingGenerateSideEffects(int index,char *sele,int state);
-PyObject *SettingGetUpdateList(CSetting *I);
+#ifndef _PYMOL_NOPY
+int SettingSetFromTuple(PyMOLGlobals *G,CSetting *I,int index,PyObject *tuple);
+PyObject *SettingGetTuple(PyMOLGlobals *G,CSetting *set1,CSetting *set2,int index); /* (type,(value,)) */
+PyObject *SettingGetDefinedTuple(PyMOLGlobals *G,CSetting *set1,int index);
+PyObject *SettingGetUpdateList(PyMOLGlobals *G,CSetting *I);
+#endif
+
+void SettingGenerateSideEffects(PyMOLGlobals *G,int index,char *sele,int state);
 
 /* Legacy API below */
 
-int SettingGetIndex(char *name);
-float SettingGet(int index);
-int SettingSet(int index,float v);
-int SettingSetfv(int index,float *value);
-float *SettingGetfv(int index);
-int SettingSetNamed(char *name,char *value);
-float SettingGetNamed(char *name);
-int SettingGetName(int index,SettingName name);
+int SettingGetIndex(PyMOLGlobals *G,char *name);
+float SettingGet(PyMOLGlobals *G,int index);
+int SettingSet(PyMOLGlobals *G,int index,float v);
+int SettingSetfv(PyMOLGlobals *G,int index,float *value);
+float *SettingGetfv(PyMOLGlobals *G,int index);
+int SettingSetNamed(PyMOLGlobals *G,char *name,char *value);
+float SettingGetNamed(PyMOLGlobals *G,char *name);
+int SettingGetName(PyMOLGlobals *G,int index,SettingName name);
 
 PyObject *SettingAsPyList(CSetting *I);
 int SettingFromPyList(CSetting *I,PyObject *list);
-int SettingSetGlobalsFromPyList(PyObject *list);
-PyObject *SettingGetGlobalsPyList(void);
-CSetting *SettingNewFromPyList(PyObject *list);
+int SettingSetGlobalsFromPyList(PyMOLGlobals *G,PyObject *list);
+PyObject *SettingGetGlobalsPyList(PyMOLGlobals *G);
+CSetting *SettingNewFromPyList(PyMOLGlobals *G,PyObject *list);
 
 /* WARNING: do not delete or change indices
    since they are used in session objects */
@@ -328,7 +334,7 @@ CSetting *SettingNewFromPyList(PyObject *list);
 #define cSetting_state                       193
 #define cSetting_frame                       194
 #define cSetting_ray_shadows                 195
-#define cSetting_ribbon_trace                196
+#define cSetting_ribbon_trace_atoms          196
 #define cSetting_security                    197
 #define cSetting_stick_transparency          198 
 #define cSetting_ray_transparency_shadows    199
@@ -407,7 +413,7 @@ CSetting *SettingNewFromPyList(PyObject *list);
 #define cSetting_retain_order                266
 #define cSetting_pdb_hetatm_sort             267
 #define cSetting_pdb_use_ter_records         268
-#define cSetting_cartoon_trace               269
+#define cSetting_cartoon_trace_atoms         269
 #define cSetting_ray_oversample_cutoff       270
 #define cSetting_gaussian_resolution         271
 #define cSetting_gaussian_b_floor            272
@@ -511,10 +517,67 @@ CSetting *SettingNewFromPyList(PyObject *list);
 #define cSetting_stereo                     365
 #define cSetting_wizard_prompt_mode         366
 #define cSetting_coulomb_cutoff             367
+#define cSetting_slice_track_camera         368
+#define cSetting_slice_height_scale         369 
+#define cSetting_slice_height_map           370
+#define cSetting_slice_grid                 371
+#define cSetting_slice_dynamic_grid         372
+#define cSetting_slice_dynamic_grid_resolution 373
+#define cSetting_pdb_insure_orthogonal        374
+#define cSetting_ray_direct_shade           375
+#define cSetting_stick_color                376
+#define cSetting_cartoon_putty_radius       377
+#define cSetting_cartoon_putty_quality      378
+#define cSetting_cartoon_putty_scale_min    379
+#define cSetting_cartoon_putty_scale_max    380
+#define cSetting_cartoon_putty_scale_power  381
+#define cSetting_cartoon_putty_range        382
+#define cSetting_cartoon_side_chain_helper  383
+#define cSetting_surface_optimize_subsets   384
+#define cSetting_multiplex                  385
+#define cSetting_texture_fonts              386
+#define cSetting_pqr_no_chain_id            387
+#define cSetting_animation                  388
+#define cSetting_animation_duration         389
+#define cSetting_scene_animation            390
+#define cSetting_line_stick_helper          391
+#define cSetting_ray_orthoscopic            392
+#define cSetting_ribbon_side_chain_helper   393
+#define cSetting_selection_width_max        394
+#define cSetting_selection_width_scale      395
+#define cSetting_scene_current_name         396
+#define cSetting_presentation               397
+#define cSetting_presentation_mode          398
+#define cSetting_pdb_truncate_residue_name  399
+#define cSetting_scene_loop                 400
+#define cSetting_sweep_mode                 401
+#define cSetting_sweep_phase                402
+#define cSetting_scene_restart_movie_delay  403
+#define cSetting_mouse_restart_movie_delay  404
+#define cSetting_angle_size                 405
+#define cSetting_angle_label_position       406
+#define cSetting_dihedral_size              407
+#define cSetting_dihedral_label_position    408
+#define cSetting_defer_builds_mode          409
+#define cSetting_seq_view_discrete_by_state 410
+#define cSetting_scene_animation_duration   411
+#define cSetting_wildcard                   412
+#define cSetting_atom_name_wildcard         413
+#define cSetting_ignore_case                414
+#define cSetting_presentation_auto_quit     415
+#define cSetting_editor_auto_dihedral       416
+#define cSetting_presentation_auto_start    417
+#define cSetting_validate_object_names      418
+
+/* when you add a new settingg also remember:
+   layer1/Setting.c
+   modules/pymol/setting.py
+  layer5/PyMOL.c 
+*/
 
 /* cSetting_ss_INIT must always be last setting_index +1 */
 
-#define cSetting_INIT                       368
+#define cSetting_INIT                       419
 
 #endif
 
