@@ -31,6 +31,7 @@ Z* -------------------------------------------------------------------
 
 #include"Executive.h"
 #include"Block.h"
+#include"Text.h"
 
 #define cWizBlank      0
 #define cWizTypeText   1
@@ -51,7 +52,7 @@ typedef struct {
   OrthoLineType code;
 } WizardLine;
 
-typedef struct {
+struct _CWizard {
   Block *Block;
   PyObject **Wiz;
   WizardLine *Line;
@@ -63,41 +64,39 @@ typedef struct {
   int LastUpdatedState;
   int LastUpdatedFrame;
   
-}  CWizard;
-
-CWizard Wizard;
+};
 
 #define cWizardLeftMargin 2
 #define cWizardTopMargin 0
 #define cWizardClickOffset 2
 
-void WizardDirty(void)
+void WizardDirty(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+  register CWizard *I=G->Wizard;
   I->Dirty=true;
-  OrthoDirty();
+  OrthoDirty(G);
 }
 
-int WizardUpdate(void)
+int WizardUpdate(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+  register CWizard *I=G->Wizard;
   int result = false;
 
   {
-    int frame = SettingGetGlobal_i(cSetting_frame);
-    int state = SettingGetGlobal_i(cSetting_state);
+    int frame = SettingGetGlobal_i(G,cSetting_frame);
+    int state = SettingGetGlobal_i(G,cSetting_state);
     if(frame!=I->LastUpdatedFrame) {
       I->LastUpdatedFrame = frame;
-      WizardDoFrame();
+      WizardDoFrame(G);
     }
     if(state!=I->LastUpdatedState) {
       I->LastUpdatedState = state;
-      WizardDoState();
+      WizardDoState(G);
     }
   }
 
   if(I->Dirty) {
-    WizardRefresh();
+    WizardRefresh(G);
     I->Dirty=false;
     result = true;
   }
@@ -105,22 +104,26 @@ int WizardUpdate(void)
   return result;
 }
 
-void WizardPurgeStack(void)
+void WizardPurgeStack(PyMOLGlobals *G)
 {
+#ifndef _PYMOL_NOPY
   int blocked;
   int a;
-  CWizard *I=&Wizard;
+  register CWizard *I=G->Wizard;
   blocked = PAutoBlock();
   for(a=I->Stack;a>=0;a--)
     Py_XDECREF(I->Wiz[a]);
   I->Stack = -1;
   PAutoUnblock(blocked);
-
+#endif
 }
-int WizardDoSelect(char *name)
+int WizardDoSelect(PyMOLGlobals *G,char *name)
 {
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
   OrthoLineType buf;
-  CWizard *I=&Wizard;
+  register CWizard *I=G->Wizard;
   int result = false;
 
   if(I->EventMask & cWizEventSelect) 
@@ -136,11 +139,14 @@ int WizardDoSelect(char *name)
         PUnblock();
       }
   return result;
+#endif
 }
 /*========================================================================*/
-void WizardRefresh(void)
+void WizardRefresh(PyMOLGlobals *G)
 {
-  CWizard *I = &Wizard;
+
+#ifndef _PYMOL_NOPY
+  register CWizard *I = G->Wizard;
   char *vla = NULL;
   PyObject *P_list;
   int ll;
@@ -162,7 +168,7 @@ void WizardRefresh(void)
       }
     }
 
-  OrthoSetWizardPrompt(vla);
+  OrthoSetWizardPrompt(G,vla);
 
   /* get the current panel list */
   
@@ -212,17 +218,19 @@ void WizardRefresh(void)
       }
     }
   if(I->NLine) {
-    int LineHeight = SettingGetGlobal_i(cSetting_internal_gui_control_size);
-    OrthoReshapeWizard(LineHeight*I->NLine+4);
+    int LineHeight = SettingGetGlobal_i(G,cSetting_internal_gui_control_size);
+    OrthoReshapeWizard(G,LineHeight*I->NLine+4);
   } else {
-    OrthoReshapeWizard(0);
+    OrthoReshapeWizard(G,0);
   }
   PAutoUnblock(blocked);
+#endif
 }
 /*========================================================================*/
-void WizardSet(PyObject *wiz,int replace)
+void WizardSet(PyMOLGlobals *G,PyObject *wiz,int replace)
 {
-  CWizard *I = &Wizard;
+#ifndef _PYMOL_NOPY
+  register CWizard *I = G->Wizard;
   int blocked;
   blocked = PAutoBlock();
   if(I->Wiz) {
@@ -249,13 +257,14 @@ void WizardSet(PyObject *wiz,int replace)
       }
     }
   }
-  WizardRefresh();
+  WizardRefresh(G);
   PAutoUnblock(blocked);
+#endif
 }
 /*========================================================================*/
-int WizardActive(void)
+int WizardActive(PyMOLGlobals *G)
 {
-  CWizard *I = &Wizard;
+  register CWizard *I = G->Wizard;
   if(!I->Wiz)
     return(false);
   if(I->Stack<0)
@@ -263,15 +272,18 @@ int WizardActive(void)
   return(I->Wiz[I->Stack]&&1);
 }
 /*========================================================================*/
-Block *WizardGetBlock(void)
+Block *WizardGetBlock(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+  register CWizard *I=G->Wizard;
   {return(I->Block);}
 }
 /*========================================================================*/
-int WizardDoPick(int bondFlag)
+int WizardDoPick(PyMOLGlobals *G,int bondFlag)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  register CWizard *I=G->Wizard;
   int result=false;
   if(I->EventMask & cWizEventPick) 
     if(I->Stack>=0) 
@@ -292,11 +304,15 @@ int WizardDoPick(int bondFlag)
         PUnblock();
       }
   return result;
+#endif
 }
 
-int WizardDoKey(unsigned char k, int x, int y, int mod)
+int WizardDoKey(PyMOLGlobals *G,unsigned char k, int x, int y, int mod)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  register CWizard *I=G->Wizard;
   int result=false;
   if(I->EventMask & cWizEventKey) 
     if(I->Stack>=0) 
@@ -315,11 +331,15 @@ int WizardDoKey(unsigned char k, int x, int y, int mod)
         PUnblock();
       }
   return result;
+#endif
 }
 
-int WizardDoScene(void)
+int WizardDoScene(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  register CWizard *I=G->Wizard;
   int result=false;
   if(I->EventMask & cWizEventScene) 
     if(I->Stack>=0) 
@@ -338,18 +358,22 @@ int WizardDoScene(void)
         PUnblock();
       }
   return result;
+#endif
 }
 
 
-int WizardDoState(void)
+int WizardDoState(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  register CWizard *I=G->Wizard;
   int result=false;
   if(I->EventMask & cWizEventState) 
     if(I->Stack>=0) 
       if(I->Wiz[I->Stack]) {
         OrthoLineType buffer;
-        int state = SettingGetGlobal_i(cSetting_state) + 1;
+        int state = SettingGetGlobal_i(G,cSetting_state) + 1;
         sprintf(buffer,"cmd.get_wizard().do_state(%d)",state);
         PLog(buffer,cPLog_pym);
         PBlock(); 
@@ -363,18 +387,22 @@ int WizardDoState(void)
         PUnblock();
       }
   return result;
+#endif
 }
 
 
-int WizardDoFrame(void)
+int WizardDoFrame(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  register CWizard *I=G->Wizard;
   int result=false;
   if(I->EventMask & cWizEventFrame) 
     if(I->Stack>=0) 
       if(I->Wiz[I->Stack]) {
         OrthoLineType buffer;
-        int frame = SettingGetGlobal_i(cSetting_frame) + 1;
+        int frame = SettingGetGlobal_i(G,cSetting_frame) + 1;
         sprintf(buffer,"cmd.get_wizard().do_frame(%d)",frame);
         PLog(buffer,cPLog_pym);
         PBlock(); 
@@ -388,11 +416,15 @@ int WizardDoFrame(void)
         PUnblock();
       }
   return result;
+#endif
 }
 
-int WizardDoSpecial(int k, int x, int y, int mod)
+int WizardDoSpecial(PyMOLGlobals *G,int k, int x, int y, int mod)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  register CWizard *I=G->Wizard;
   int result=false;
 
   if(I->EventMask & cWizEventSpecial) 
@@ -412,24 +444,29 @@ int WizardDoSpecial(int k, int x, int y, int mod)
         PUnblock();
       }
   return result;
+#endif
 }
 
 /*========================================================================*/
 static int WizardClick(Block *block,int button,int x,int y,int mod)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+  PyMOLGlobals *G=block->G;
+  register CWizard *I=G->Wizard;
 
   int a;
   PyObject *menuList=NULL;
-  int LineHeight = SettingGetGlobal_i(cSetting_internal_gui_control_size);
+  int LineHeight = SettingGetGlobal_i(G,cSetting_internal_gui_control_size);
 
   a=((I->Block->rect.top-(y+cWizardClickOffset))-cWizardTopMargin)/LineHeight;
   if((a>=0)&&(a<I->NLine)) {
     switch(I->Line[a].type) {
     case cWizTypeButton:
-      OrthoGrab(I->Block);
+      OrthoGrab(G,I->Block);
       I->Pressed=a;
-      OrthoDirty();
+      OrthoDirty(G);
       break;
     case cWizTypePopUp:
       PBlock(); 
@@ -445,7 +482,7 @@ static int WizardClick(Block *block,int button,int x,int y,int mod)
       if(menuList&&(menuList!=Py_None)) {
         int my = I->Block->rect.top-(cWizardTopMargin + a*LineHeight) - 2;
         
-        PopUpNew(x,my,x,y,menuList,NULL);
+        PopUpNew(G,x,my,x,y,false,menuList,NULL);
       }
       Py_XDECREF(menuList);
       PUnblock();
@@ -454,12 +491,19 @@ static int WizardClick(Block *block,int button,int x,int y,int mod)
     }
   }
   return(1);
+#endif
 }
 /*========================================================================*/
 static int WizardDrag(Block *block,int x,int y,int mod)
 {
-  CWizard *I=&Wizard;
-  int LineHeight = SettingGetGlobal_i(cSetting_internal_gui_control_size);
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+
+  PyMOLGlobals *G=block->G;
+
+  register CWizard *I=G->Wizard;
+  int LineHeight = SettingGetGlobal_i(G,cSetting_internal_gui_control_size);
 
   int a;
   a=((I->Block->rect.top-(y+cWizardClickOffset))-
@@ -470,7 +514,7 @@ static int WizardDrag(Block *block,int x,int y,int mod)
 
   if(I->Pressed!=a) {
     I->Pressed=-1;
-    OrthoDirty();
+    OrthoDirty(G);
   }
   if((a>=0)&&(a<I->NLine)) {
 
@@ -478,18 +522,21 @@ static int WizardDrag(Block *block,int x,int y,int mod)
     case cWizTypeButton:
       if(I->Pressed!=a) {
         I->Pressed=a;
-        OrthoDirty();
+        OrthoDirty(G);
       }
       break;
     }
   }
   return(1);
+#endif
 }
 /*========================================================================*/
 static int WizardRelease(Block *block,int button,int x,int y,int mod)
 {
-  CWizard *I=&Wizard;
-    int LineHeight = SettingGetGlobal_i(cSetting_internal_gui_control_size);
+  PyMOLGlobals *G=block->G;
+
+  register CWizard *I=G->Wizard;
+    int LineHeight = SettingGetGlobal_i(G,cSetting_internal_gui_control_size);
 
   int a;
   a=((I->Block->rect.top-(y+cWizardClickOffset))-
@@ -497,9 +544,9 @@ static int WizardRelease(Block *block,int button,int x,int y,int mod)
 
   if(I->Pressed)
     I->Pressed=-1;
-  OrthoDirty();
+  OrthoDirty(G);
 
-  OrthoUngrab();
+  OrthoUngrab(G);
 
   if((a>=0)&&(a<I->NLine)) {
     switch(I->Line[a].type) {
@@ -557,21 +604,21 @@ static void draw_button(int x2,int y2, int w, int h, float *light, float *dark,
 
 }
 
-static void draw_text(char *c,int xx,int yy,float *color)
+static void draw_text(PyMOLGlobals *G,char *c,int xx,int yy,float *color)
 {
-  glColor3fv(color);
+  TextSetColor(G,color);
   while(*c) {
     if(*c=='\\') if(*(c+1)) if(*(c+2)) if(*(c+3)) {
       if(*(c+1)=='-') {
-        glColor3fv(color);
+        TextSetColor(G,color);
         c+=4;
       } else {
-        glColor3f((*(c+1)-'0')/9.0F,(*(c+2)-'0')/9.0F,(*(c+3)-'0')/9.0F);
+        TextSetColor3f(G,(*(c+1)-'0')/9.0F,(*(c+2)-'0')/9.0F,(*(c+3)-'0')/9.0F);
         c+=4;
       }
     }
-    glRasterPos4d((double)(xx),(double)(yy),0.0,1.0);
-    p_glutBitmapCharacter(P_GLUT_BITMAP_8_BY_13,*(c++));
+    TextSetPos2i(G,xx,yy);
+    TextDrawChar(G,*(c++));
     xx = xx + 8;
   }
 }
@@ -579,7 +626,9 @@ static void draw_text(char *c,int xx,int yy,float *color)
 /*========================================================================*/
 static void WizardDraw(Block *block)
 {
-  CWizard *I=&Wizard;
+  PyMOLGlobals *G=block->G;
+
+  register CWizard *I=G->Wizard;
   int x,y;
   int a;
 
@@ -592,18 +641,19 @@ static void WizardDraw(Block *block)
   float dimDarkEdge[3] = {0.25F, 0.25F,0.25F};
 
   float menuBGColor[3] = {0.5F, 0.5F,1.0};
-  float menuLightEdge[3] = {0.7,0.7,0.9F};
-  float menuDarkEdge[3] = {0.3,0.3,0.5F};
+  float menuLightEdge[3] = {0.7F,0.7F,0.9F};
+  float menuDarkEdge[3] = {0.3F,0.3F,0.5F};
 
   float black_color[3] = {0.0F,0.0F,0.0F};
   float menuColor[3] = { 0.0,0.0,0.0};
-  int LineHeight = SettingGetGlobal_i(cSetting_internal_gui_control_size);
+  int LineHeight = SettingGetGlobal_i(G,cSetting_internal_gui_control_size);
   int text_lift = (LineHeight/2)-5;
   float *text_color ;
 
   text_color = menuColor;
 
-  if(PMGUI) {
+  if(G->HaveGUI && G->ValidContext) {
+
     glColor3fv(I->Block->BackColor);
     BlockFill(I->Block);
     
@@ -655,16 +705,16 @@ static void WizardDraw(Block *block)
           break;
         }
       }
-      draw_text(I->Line[a].text,x+1,y+text_lift,text_color);
+      draw_text(G,I->Line[a].text,x+1,y+text_lift,text_color);
       /*GrapDrawStr(I->Line[a].text,x+1,y+text_lift);*/
       y-=LineHeight;
     }
   }
 }
 /*========================================================================*/
-PyObject *WizardGet(void)
+PyObject *WizardGet(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+  register CWizard *I=G->Wizard;
   if(!I->Wiz)
     return(NULL);
   if(I->Stack<0)
@@ -672,9 +722,12 @@ PyObject *WizardGet(void)
   return(I->Wiz[I->Stack]);
 }
 /*========================================================================*/
-PyObject *WizardGetStack(void)
+PyObject *WizardGetStack(PyMOLGlobals *G)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
+  register CWizard *I=G->Wizard;
   int a;
   PyObject *result;
 
@@ -686,16 +739,21 @@ PyObject *WizardGetStack(void)
     }
   }
   return(result);
+#endif
 }
 /*========================================================================*/
-int WizardSetStack(PyObject *list)
+int WizardSetStack(PyMOLGlobals *G,PyObject *list)
 {
-  CWizard *I=&Wizard;
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
+
+  register CWizard *I=G->Wizard;
   int a;
   int ok= true;
   
   if(I->Wiz) {
-    WizardPurgeStack();
+    WizardPurgeStack(G);
     if(ok) ok = (list!=NULL);
     if(ok) ok = PyList_Check(list);
     if(ok) {
@@ -708,17 +766,19 @@ int WizardSetStack(PyObject *list)
         }
       }
     }
-    if(ok) WizardRefresh();
-    if(ok) OrthoDirty();
+    if(ok) WizardRefresh(G);
+    if(ok) OrthoDirty(G);
   }
   return(ok);
+#endif
 }
 /*========================================================================*/
-void WizardInit(void)
+int WizardInit(PyMOLGlobals *G)
 {
-  CWizard *I = &Wizard;
+  register CWizard *I=NULL;
+  if( (I=(G->Wizard=Calloc(CWizard,1)))) {
 
-  I->Block = OrthoNewBlock(NULL);
+  I->Block = OrthoNewBlock(G,NULL);
   I->Block->fClick = WizardClick;
   I->Block->fDrag  = WizardDrag;
   I->Block->fDraw    = WizardDraw;
@@ -733,23 +793,26 @@ void WizardInit(void)
   I->LastUpdatedState = -1;
   I->LastUpdatedFrame = -1;
 
-  OrthoAttach(I->Block,cOrthoTool);
+  OrthoAttach(G,I->Block,cOrthoTool);
 
-  I->Line = VLAlloc(WizardLine,10);
+  I->Line = VLAlloc(WizardLine,1);
   I->NLine = 0;
   I->Pressed = -1;
   I->EventMask = 0;
   I->Stack = -1;
   I->Wiz = VLAlloc(PyObject*,10);
+  return 1;
+  } else
+    return 0;
 }
 /*========================================================================*/
-void WizardFree(void)
+void WizardFree(PyMOLGlobals *G)
 {
-  CWizard *I = &Wizard;
-  WizardPurgeStack();
-  OrthoFreeBlock(I->Block);
+  register CWizard *I = G->Wizard;
+  WizardPurgeStack(G);
+  OrthoFreeBlock(G,I->Block);
   VLAFreeP(I->Line);
-  VLAFreeP(I->Wiz)
-    
+  VLAFreeP(I->Wiz);
+  FreeP(G->Wizard);
 }
 
