@@ -25,10 +25,32 @@ Z* -------------------------------------------------------------------
 #include"Color.h"
 #include"PConv.h"
 
+struct _CAtomInfo {
+  int NColor,CColor,DColor,HColor,OColor,SColor;
+  int BrColor, ClColor, FColor, IColor;
+  int PColor, MgColor, MnColor, NaColor, KColor, CaColor;
+  int CuColor, FeColor, ZnColor;
+  int SeColor;
+  int DefaultColor;
+  
+};
+
+int AtomInfoInit(PyMOLGlobals *G)
+{
+  register CAtomInfo *I=NULL;
+  if( (I=(G->AtomInfo=Calloc(CAtomInfo,1)))) {
+    AtomInfoPrimeColors(G);
+    return 1;
+  } else
+    return 0;
+}
+void AtomInfoFree(PyMOLGlobals *G)
+{
+  FreeP(G->AtomInfo);
+}
 
-static int NColor,CarbColor,HColor,OColor,SColor,MColor,IColor;
 /*========================================================================*/
-void AtomInfoGetPDB3LetHydroName(char *resn, char *iname, char *oname) 
+void AtomInfoGetPDB3LetHydroName(PyMOLGlobals *G,char *resn, char *iname, char *oname) 
 {
   oname[0]=' ';	
   strcpy(oname+1,iname);
@@ -296,9 +318,8 @@ void AtomInfoGetPDB3LetHydroName(char *resn, char *iname, char *oname)
   }
 }
 
-int AtomInfoKnownWaterResName(char *resn) 
+int AtomInfoKnownWaterResName(PyMOLGlobals *G,char *resn) 
 {
-
   switch(resn[0]) {
   case 'H':
     switch(resn[1]) {
@@ -317,6 +338,17 @@ int AtomInfoKnownWaterResName(char *resn)
       }
       break;
     }
+  case 'D':
+    switch(resn[1]) {
+    case 'O': 
+      switch(resn[2]) {
+      case 'D':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
   case 'T':
     switch(resn[1]) {
     case 'I': 
@@ -339,11 +371,22 @@ int AtomInfoKnownWaterResName(char *resn)
       break;
     }
     break;
+  case 'S':
+    switch(resn[1]) {
+    case 'O': 
+      switch(resn[2]) {
+      case 'L':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
   }
   return false;
 }
 
-int AtomInfoKnownPolymerResName(char *resn) 
+int AtomInfoKnownPolymerResName(PyMOLGlobals *G,char *resn) 
 {
   switch(resn[0]) {
   case 'A':
@@ -541,10 +584,9 @@ int AtomInfoKnownPolymerResName(char *resn)
 
 /*========================================================================*/
 
-int AtomInfoInOrder(AtomInfoType *atom,int atom1,int atom2);
-int AtomInfoInOrigOrder(AtomInfoType *atom,int atom1,int atom2);
-int AtomInfoInOrderIgnoreHet(AtomInfoType *atom,int atom1,int atom2);
-int AtomNameCompare(char *name1,char *name2);
+int AtomInfoInOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2);
+int AtomInfoInOrigOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2);
+int AtomInfoInOrderIgnoreHet(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2);
 
 int AtomResvFromResi(char *resi)
 {
@@ -561,11 +603,14 @@ int AtomResvFromResi(char *resi)
 }
 
 /*========================================================================*/
-PyObject *AtomInfoAsPyList(AtomInfoType *I)
+PyObject *AtomInfoAsPyList(PyMOLGlobals *G,AtomInfoType *I)
 {
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
   PyObject *result = NULL;
 
-  result = PyList_New(36);
+  result = PyList_New(40);
   PyList_SetItem(result, 0,PyInt_FromLong(I->resv));
   PyList_SetItem(result, 1,PyString_FromString(I->chain));
   PyList_SetItem(result, 2,PyString_FromString(I->alt));
@@ -602,11 +647,19 @@ PyObject *AtomInfoAsPyList(AtomInfoType *I)
   PyList_SetItem(result,33,PyInt_FromLong(I->stereo));
   PyList_SetItem(result,34,PyInt_FromLong(I->discrete_state));
   PyList_SetItem(result,35,PyFloat_FromDouble(I->bohr_radius));
+  PyList_SetItem(result,36,PyInt_FromLong(I->rank));
+  PyList_SetItem(result,37,PyInt_FromLong((int)I->hb_donor));
+  PyList_SetItem(result,38,PyInt_FromLong((int)I->hb_acceptor));
+  PyList_SetItem(result,39,PyInt_FromLong((int)I->atomic_color));
   return(PConvAutoNone(result));
+#endif
 }
 
-int AtomInfoFromPyList(AtomInfoType *I,PyObject *list)
+int AtomInfoFromPyList(PyMOLGlobals *G,AtomInfoType *I,PyObject *list)
 {
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
   int ok=true;
   int hetatm;
   int ll = 0;
@@ -649,11 +702,20 @@ int AtomInfoFromPyList(AtomInfoType *I,PyObject *list)
   if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,33),&I->stereo);
   if(ok&&(ll>34)) ok = PConvPyIntToInt(PyList_GetItem(list,34),&I->discrete_state);
   if(ok&&(ll>35)) ok = PConvPyFloatToFloat(PyList_GetItem(list,35),&I->bohr_radius);
+  if(ok&&(ll>36)) ok = PConvPyIntToInt(PyList_GetItem(list,36),&I->rank);
+  if(ok&&(ll>37)) ok = PConvPyIntToChar(PyList_GetItem(list,37),(char*)&I->hb_donor);
+  if(ok&&(ll>38)) ok = PConvPyIntToChar(PyList_GetItem(list,38),(char*)&I->hb_acceptor);
+  if(ok&&(ll>39)) 
+    ok = PConvPyIntToInt(PyList_GetItem(list,39),&I->atomic_color);
+  else {
+    I->atomic_color = AtomInfoGetColor(G,I);
+  }
   return(ok);
+#endif
 }
 
 /*========================================================================*/
-void AtomInfoCombine(AtomInfoType *dst,AtomInfoType *src,int mask)
+void AtomInfoCombine(PyMOLGlobals *G,AtomInfoType *dst,AtomInfoType *src,int mask)
 {
   if(mask&cAIC_tt) strcpy(dst->textType,src->textType); /* use the new types */
   if(mask&cAIC_ct) dst->customType = src->customType;
@@ -664,6 +726,7 @@ void AtomInfoCombine(AtomInfoType *dst,AtomInfoType *src,int mask)
   if(mask&cAIC_q) dst->q = src->q;
   if(mask&cAIC_id) dst->id = src->id;
   if(mask&cAIC_state) dst->discrete_state = src->discrete_state;
+  if(mask&cAIC_rank) dst->rank = src->rank;
   dst->temp1 = src->temp1;
   dst->sculpt_id = src->sculpt_id;
   /* keep all existing names, identifiers, etc. */
@@ -672,7 +735,7 @@ void AtomInfoCombine(AtomInfoType *dst,AtomInfoType *src,int mask)
 
 }
 /*========================================================================*/
-void AtomInfoUniquefyNames(AtomInfoType *atInfo0,int n0,AtomInfoType *atInfo1,int n1)
+void AtomInfoUniquefyNames(PyMOLGlobals *G,AtomInfoType *atInfo0,int n0,AtomInfoType *atInfo1,int n1)
 {
   /* makes sure all names in atInfo1 are unique WRT 0 and 1 */
 
@@ -702,20 +765,20 @@ void AtomInfoUniquefyNames(AtomInfoType *atInfo0,int n0,AtomInfoType *atInfo1,in
       /* check within object 1 */
       
       if(!lai1) bracketFlag=true;
-      else if(!AtomInfoSameResidue(lai1,ai1))
+      else if(!AtomInfoSameResidue(G,lai1,ai1))
         bracketFlag=true;
       else
         bracketFlag=false;
       if(bracketFlag) {
         c=1;
-        AtomInfoBracketResidue(atInfo1,n1,ai1,&st1,&nd1);
+        AtomInfoBracketResidue(G,atInfo1,n1,ai1,&st1,&nd1);
         lai1=ai1;
       }
       ai0 = atInfo1 + st1;
       for(a=st1;a<=nd1;a++) {
         if(strcmp(ai1->name,ai0->name))
           ai0++;
-        else if(!AtomInfoSameResidue(ai1,ai0))
+        else if(!AtomInfoSameResidue(G,ai1,ai0))
           ai0++;
         else if(ai1!=ai0) {
           matchFlag=true;
@@ -730,19 +793,19 @@ void AtomInfoUniquefyNames(AtomInfoType *atInfo0,int n0,AtomInfoType *atInfo1,in
         /* check within object 2 */
         
         if(!lai0) bracketFlag=true;
-        else if(!AtomInfoSameResidue(lai0,ai1))
+        else if(!AtomInfoSameResidue(G,lai0,ai1))
           bracketFlag=true;
         else
           bracketFlag=false;
         if(bracketFlag) {
-          AtomInfoBracketResidue(atInfo0,n0,ai1,&st0,&nd0);
+          AtomInfoBracketResidue(G,atInfo0,n0,ai1,&st0,&nd0);
           lai0=ai1;
         }
         ai0 = atInfo0 + st0;
         for(a=st0;a<=nd0;a++) {
           if(strcmp(ai1->name,ai0->name))
             ai0++;
-          else if(!AtomInfoSameResidue(ai1,ai0))
+          else if(!AtomInfoSameResidue(G,ai1,ai0))
             ai0++;
           else if(ai1!=ai0) {
             matchFlag=true;
@@ -773,7 +836,7 @@ void AtomInfoUniquefyNames(AtomInfoType *atInfo0,int n0,AtomInfoType *atInfo1,in
 }
 
 /*========================================================================*/
-void AtomInfoBracketResidue(AtomInfoType *ai0,int n0,AtomInfoType *ai,int *st,int *nd)
+void AtomInfoBracketResidue(PyMOLGlobals *G,AtomInfoType *ai0,int n0,AtomInfoType *ai,int *st,int *nd)
 {
   /* inefficient but reliable way to find where residue atoms are located in an object 
    * for purpose of residue-based operations */
@@ -784,21 +847,21 @@ void AtomInfoBracketResidue(AtomInfoType *ai0,int n0,AtomInfoType *ai,int *st,in
   *nd=n0-1;
   ai1=ai0;
   for(a=0;a<n0;a++) {
-    if(!AtomInfoSameResidue(ai,ai1++))
+    if(!AtomInfoSameResidue(G,ai,ai1++))
       *st=a;
     else 
       break;
   }
   ai1=ai0+n0-1;
   for(a=n0-1;a>=0;a--) {
-    if(!AtomInfoSameResidue(ai,ai1--)) {
+    if(!AtomInfoSameResidue(G,ai,ai1--)) {
       *nd=a;
     } else 
       break;
   }
 }
 /*========================================================================*/
-void AtomInfoBracketResidueFast(AtomInfoType *ai0,int n0,int cur,int *st,int *nd)
+void AtomInfoBracketResidueFast(PyMOLGlobals *G,AtomInfoType *ai0,int n0,int cur,int *st,int *nd)
 {
   /* efficient but unreliable way to find where residue atoms are located in an object 
    * for purpose of residue-based operations */
@@ -810,38 +873,63 @@ void AtomInfoBracketResidueFast(AtomInfoType *ai0,int n0,int cur,int *st,int *nd
   ai0=ai0+cur;
   ai1=ai0-1;
   for(a=cur-1;a>=0;a--) {
-    if(!AtomInfoSameResidue(ai0,ai1--))
+    if(!AtomInfoSameResidue(G,ai0,ai1--))
       break;
     *st=a;
   }
   ai1=ai0+1;
   for(a=cur+1;a<n0;a++) {
-    if(!AtomInfoSameResidue(ai0,ai1++))
+    if(!AtomInfoSameResidue(G,ai0,ai1++))
       break;
     *nd=a;
   }
 }
 /*========================================================================*/
-int AtomInfoGetCarbColor(void)
+int AtomInfoUpdateAutoColor(PyMOLGlobals *G)
 {
-  return CarbColor;
-}
-/*========================================================================*/
-void AtomInfoPrimeColors(void)
-{
-  NColor=ColorGetIndex("nitrogen");
-  if(SettingGet(cSetting_auto_color))
-    CarbColor=ColorGetNext();
+  CAtomInfo *I=G->AtomInfo;
+  if(SettingGet(G,cSetting_auto_color))
+    I->CColor = ColorGetNext(G);
   else
-    CarbColor=ColorGetIndex("carbon");
-  HColor=ColorGetIndex("hydrogen");
-  OColor=ColorGetIndex("oxygen");
-  SColor=ColorGetIndex("sulfur");
-  MColor=ColorGetIndex("magenta");
-  IColor=ColorGetIndex("yellow");
+    I->CColor = ColorGetIndex(G,"carbon");
+  return I->CColor;
+}
+    
+/*========================================================================*/
+void AtomInfoPrimeColors(PyMOLGlobals *G)
+{
+  CAtomInfo *I=G->AtomInfo;
+
+  I->NColor=ColorGetIndex(G,"nitrogen");
+  I->CColor=ColorGetIndex(G,"carbon");
+
+  I->HColor=ColorGetIndex(G,"hydrogen");
+  I->OColor=ColorGetIndex(G,"oxygen");
+  I->SColor=ColorGetIndex(G,"sulfur");
+
+  I->ClColor=ColorGetIndex(G,"chlorine");
+  I->BrColor=ColorGetIndex(G,"bromine");
+  I->FColor=ColorGetIndex(G,"fluorine");
+  I->IColor=ColorGetIndex(G,"iodine");
+
+  I->PColor=ColorGetIndex(G,"phosphorus");
+
+  I->MgColor=ColorGetIndex(G,"magnesium");
+  I->MnColor=ColorGetIndex(G,"manganese");
+
+  I->NaColor=ColorGetIndex(G,"sodium");
+  I->KColor=ColorGetIndex(G,"potassium");
+  I->CaColor=ColorGetIndex(G,"calcium");
+
+  I->CuColor=ColorGetIndex(G,"copper");
+  I->FeColor=ColorGetIndex(G,"iron");
+  I->ZnColor=ColorGetIndex(G,"zinc");
+
+  I->SeColor=ColorGetIndex(G,"selenium");
+  I->DColor=ColorGetIndex(G,"deuterium");
 }
 /*========================================================================*/
-float AtomInfoGetBondLength(AtomInfoType *ai1,AtomInfoType *ai2)
+float AtomInfoGetBondLength(PyMOLGlobals *G,AtomInfoType *ai1,AtomInfoType *ai2)
 {
   float result = 1.6F;
   AtomInfoType *a1,*a2;
@@ -1050,67 +1138,206 @@ float AtomInfoGetBondLength(AtomInfoType *ai1,AtomInfoType *ai2)
   return(result);
 }
 
-int AtomInfoGetColor(AtomInfoType *at1)
+void AtomInfoAssignColors(PyMOLGlobals *G,AtomInfoType *at1)
 {
+  at1->atomic_color = (at1->color = AtomInfoGetColor(G,at1));
+}
+
+int AtomInfoGetColor(PyMOLGlobals *G,AtomInfoType *at1)
+{
+  CAtomInfo *I=G->AtomInfo;
   char *n=at1->elem;
-  int color = 0;
+  int color = I->DefaultColor;
 
-  while(((*n)>='0')&&((*n)<='9')&&(*(n+1))) n++;
-  switch ( (*n) )
-    {
-    case 'N' : color = NColor; break;
-    case 'C' : 
+  if(!n[0]) {
+    n=at1->name;
+    while(((*n)>='0')&&((*n)<='9')&&(*(n+1))) 
       n++;
-      switch(*(n)) {
-      case 'A':
-      case 'a':
-        if(at1->hetatm) 
-          color = IColor;
-        else
-          color = CarbColor;
-        break;
-      case 0:
-      case 32:
-        color = CarbColor; 
-        break;
-      case 'l':
-      case 'L':
-      default:
-        color = MColor; break;
-      }
-      break;
-    case 'O' : color = OColor; break;
-    case 'I' : color = MColor; break;
-    case 'P' : color = MColor; break;
-    case 'B' : color = MColor; break;
-    case 'S' : color = SColor; break;
-    case 'F' : color = MColor; break;
-    case 'H' :
-    case 'D' :
-      color=HColor; 
-      break;
-    default  : color=MColor; break;
+  }
+  {
+    register char c1=n[0], c2=n[1];
+    c2 = tolower(c2);
+    if(c2<=32) c2=0;
+    switch(c1) {
+    case 'A': switch(c2) {
+      case 'c': color=ColorGetIndex(G,"actinium"); break;
+      case 'g': color=ColorGetIndex(G,"silver"); break;
+      case 'l': color=ColorGetIndex(G,"aluminum"); break;
+      case 'm': color=ColorGetIndex(G,"americium"); break;
+      case 'r': color=ColorGetIndex(G,"argon"); break;
+      case 's': color=ColorGetIndex(G,"arsenic"); break;
+      case 't': color=ColorGetIndex(G,"astatine"); break;
+      case 'u': color=ColorGetIndex(G,"gold"); break;
+      } break;
+    case 'B': switch(c2) {
+      case 0:    color=ColorGetIndex(G,"boron"); break;
+      case 'a': color=ColorGetIndex(G,"barium"); break;
+      case 'e': color=ColorGetIndex(G,"beryllium"); break;
+      case 'h': color=ColorGetIndex(G,"bohrium"); break;
+      case 'i': color=ColorGetIndex(G,"bismuth"); break;
+      case 'k': color=ColorGetIndex(G,"berkelium"); break;
+      case 'r': color=I->BrColor; break;
+      } break;
+    case 'C': switch(c2) {
+      case 0:  color=I->CColor; break;
+      case 'a': color=ColorGetIndex(G,"calcium"); break;
+      case 'd': color=ColorGetIndex(G,"cadmium"); break;
+      case 'e': color=ColorGetIndex(G,"cerium"); break;
+      case 'f': color=ColorGetIndex(G,"californium"); break;
+      case 'l': color=I->ClColor; break;
+      case 'm': color=ColorGetIndex(G,"curium"); break;
+      case 'o': color=ColorGetIndex(G,"cobalt"); break;
+      case 'r': color=ColorGetIndex(G,"chromium"); break;
+      case 's': color=ColorGetIndex(G,"cesium"); break;
+      case 'u': color=I->CuColor; break;
+      } break;
+    case 'D': switch(c2) {
+      case 0: color=I->DColor; break;
+      case 'b': color=ColorGetIndex(G,"dubnium"); break;
+      case 'y': color=ColorGetIndex(G,"dysprosium"); break;
+      } break;
+    case 'E': switch(c2) {
+      case 'r': color=ColorGetIndex(G,"erbium"); break;
+      case 's': color=ColorGetIndex(G,"einsteinium"); break;
+      case 'u': color=ColorGetIndex(G,"europium"); break;
+      } break;
+    case 'F': switch(c2) {
+      case 0:   color=I->FColor; break;
+      case 'e': color=I->FeColor; break;
+      case 'm': color=ColorGetIndex(G,"fermium"); break;
+      case 'r': color=ColorGetIndex(G,"francium"); break;
+      } break;
+    case 'G': switch(c2) {
+      case 'a': color=ColorGetIndex(G,"gallium"); break;
+      case 'd': color=ColorGetIndex(G,"gadolinium"); break;
+      case 'e': color=ColorGetIndex(G,"germanium"); break;
+      } break;
+    case 'H': switch(c2) {
+      case 0:   color=I->HColor; break;
+      case 'e': color=ColorGetIndex(G,"helium"); break;
+      case 'f': color=ColorGetIndex(G,"hafnium"); break;
+      case 'g': color=ColorGetIndex(G,"mercury"); break;
+      case 'o': color=ColorGetIndex(G,"holmium"); break;
+      case 's': color=ColorGetIndex(G,"hassium"); break;
+      } break;
+    case 'I': switch(c2) {
+      case 0:    color=I->IColor; break;
+      case 'n': color=ColorGetIndex(G,"indium"); break;
+      case 'r': color=ColorGetIndex(G,"iridium"); break;
+      } break;
+    case 'K': switch(c2) {
+      case 0:    color=ColorGetIndex(G,"potassium"); break;
+      case 'r': color=ColorGetIndex(G,"krypton"); break;
+      } break;
+    case 'L': switch(c2) {
+      case 'a': color=ColorGetIndex(G,"lanthanum"); break;
+      case 'i': color=ColorGetIndex(G,"lithium"); break;
+      case 'r': color=ColorGetIndex(G,"lawrencium"); break;
+      case 'u': color=ColorGetIndex(G,"lutetium"); break;
+      } break;
+    case 'M': switch(c2) {
+      case 'd': color=ColorGetIndex(G,"mendelevium"); break;
+      case 'g': color=I->MgColor; break;
+      case 'n': color=I->MnColor; break;
+      case 'o': color=ColorGetIndex(G,"molybdenum"); break;
+      case 't': color=ColorGetIndex(G,"meitnerium"); break;
+      } break;
+    case 'N': switch(c2) {
+      case 0:   color=I->NColor; break;
+      case 'a': color=I->NaColor; break;
+      case 'b': color=ColorGetIndex(G,"niobium"); break;
+      case 'd': color=ColorGetIndex(G,"neodymium"); break;
+      case 'e': color=ColorGetIndex(G,"neon"); break;
+      case 'i': color=ColorGetIndex(G,"nickel"); break;
+      case 'o': color=ColorGetIndex(G,"nobelium"); break;
+      case 'p': color=ColorGetIndex(G,"neptunium"); break;
+      } break;
+    case 'O': switch(c2) {
+      case 0:    color=I->OColor; break;
+      case 's': color=ColorGetIndex(G,"osmium"); break;
+      } break;
+    case 'P': switch(c2) {
+      case 0:    color=I->PColor; break;
+      case 'a': color=ColorGetIndex(G,"protactinium"); break;
+      case 'b': color=ColorGetIndex(G,"lead"); break;
+      case 'd': color=ColorGetIndex(G,"palladium"); break;
+      case 'm': color=ColorGetIndex(G,"promethium"); break;
+      case 'o': color=ColorGetIndex(G,"polonium"); break;
+      case 'r': color=ColorGetIndex(G,"praseodymium"); break;
+      case 't': color=ColorGetIndex(G,"platinum"); break;
+      case 'u': color=ColorGetIndex(G,"plutonium"); break;
+      } break;
+    case 'R': switch(c2) {
+      case 'a': color=ColorGetIndex(G,"radium"); break;
+      case 'b': color=ColorGetIndex(G,"rubidium"); break;
+      case 'e': color=ColorGetIndex(G,"rhenium"); break;
+      case 'f': color=ColorGetIndex(G,"rutherfordium"); break;
+      case 'h': color=ColorGetIndex(G,"rhodium"); break;
+      case 'n': color=ColorGetIndex(G,"radon"); break;
+      case 'u': color=ColorGetIndex(G,"ruthenium"); break;
+      } break;
+    case 'S': switch(c2) {
+      case 0:    color=I->SColor; break;
+      case 'b': color=ColorGetIndex(G,"antimony"); break;
+      case 'c': color=ColorGetIndex(G,"scandium"); break;
+      case 'e': color=I->SeColor; break;
+      case 'g': color=ColorGetIndex(G,"seaborgium"); break;
+      case 'i': color=ColorGetIndex(G,"silicon"); break;
+      case 'm': color=ColorGetIndex(G,"samarium"); break;
+      case 'n': color=ColorGetIndex(G,"tin"); break;
+      case 'r': color=ColorGetIndex(G,"strontium"); break;
+      } break;
+    case 'T': switch(c2) {
+      case 'a': color=ColorGetIndex(G,"tantalum"); break;
+      case 'b': color=ColorGetIndex(G,"terbium"); break;
+      case 'c': color=ColorGetIndex(G,"technetium"); break;
+      case 'e': color=ColorGetIndex(G,"tellurium"); break;
+      case 'h': color=ColorGetIndex(G,"thorium"); break;
+      case 'i': color=ColorGetIndex(G,"titanium"); break;
+      case 'l': color=ColorGetIndex(G,"thallium"); break;
+      case 'm': color=ColorGetIndex(G,"thulium"); break;
+      } break;
+    case 'U': switch(c2) {
+      case 0:    color=ColorGetIndex(G,"uranium"); break;
+      } break;
+    case 'V': switch(c2) {
+      case 0:    color=ColorGetIndex(G,"vanadium"); break;
+      } break;
+    case 'W': switch(c2) {
+      case 0:    color=ColorGetIndex(G,"tungsten"); break;
+      } break;
+    case 'X': switch(c2) {
+      case 'e': color=ColorGetIndex(G,"xenon"); break;
+      } break;
+    case 'Y': switch(c2) {
+      case 0:    color=ColorGetIndex(G,"yttrium"); break;
+      case 'b': color=ColorGetIndex(G,"ytterbium"); break;
+      } break;
+    case 'Z': switch(c2) {
+      case 'n': color=I->ZnColor; break;
+      case 'r': color=ColorGetIndex(G,"zirconium"); break;
+      } break;
     }
-
+  }
   return(color);
 }
 
 
-int *AtomInfoGetSortedIndex(AtomInfoType *rec,int n,int **outdex)
+int *AtomInfoGetSortedIndex(PyMOLGlobals *G,AtomInfoType *rec,int n,int **outdex)
 {
   int *index;
   int a;
   index = Alloc(int,n+1);
-  ErrChkPtr(index);
+  ErrChkPtr(G,index);
   (*outdex)=Alloc(int,n+1);
-  ErrChkPtr(*outdex);
+  ErrChkPtr(G,*outdex);
 
-  if((int)SettingGet(cSetting_retain_order)) {
-    UtilSortIndex(n,rec,index,(UtilOrderFn*)AtomInfoInOrigOrder);
-  } else if((int)SettingGet(cSetting_pdb_hetatm_sort)) {
-    UtilSortIndex(n,rec,index,(UtilOrderFn*)AtomInfoInOrder);    
+  if((int)SettingGet(G,cSetting_retain_order)) {
+    UtilSortIndexGlobals(G,n,rec,index,(UtilOrderFnGlobals*)AtomInfoInOrigOrder);
+  } else if((int)SettingGet(G,cSetting_pdb_hetatm_sort)) {
+    UtilSortIndexGlobals(G,n,rec,index,(UtilOrderFnGlobals*)AtomInfoInOrder);    
   } else {
-    UtilSortIndex(n,rec,index,(UtilOrderFn*)AtomInfoInOrderIgnoreHet);    
+    UtilSortIndexGlobals(G,n,rec,index,(UtilOrderFnGlobals*)AtomInfoInOrderIgnoreHet);    
   }
 
   for(a=0;a<n;a++)
@@ -1118,13 +1345,13 @@ int *AtomInfoGetSortedIndex(AtomInfoType *rec,int n,int **outdex)
   return(index);
 }
 
-void AtomInfoFreeSortedIndexes(int *index,int *outdex)
+void AtomInfoFreeSortedIndexes(PyMOLGlobals *G,int *index,int *outdex)
 {
   FreeP(index);
   FreeP(outdex);
 }
 
-int AtomNameCompare(char *name1,char *name2)
+static int AtomInfoNameCompare(PyMOLGlobals *G,char *name1,char *name2)
 {
   char *n1,*n2;
   int cmp;
@@ -1137,15 +1364,15 @@ int AtomNameCompare(char *name1,char *name2)
     n2=name2+1;
   else
     n2=name2;
-  cmp = WordCompare(n1,n2,true);
+  cmp = WordCompare(G,n1,n2,true);
 
   if(cmp) 
     return cmp;
-  return WordCompare(name1,name2,true);
+  return WordCompare(G,name1,name2,true);
  
 }
 
-int AtomInfoCompare(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoCompare(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   int result;
   int wc;
@@ -1158,19 +1385,24 @@ int AtomInfoCompare(AtomInfoType *at1,AtomInfoType *at2)
 	  at2->segi,at2->chain,at2->resv,at2->resi,at2->resn,at2->priority,at2->name);
   */
 
-  wc=WordCompare(at1->segi,at2->segi,true);
+  wc=WordCompare(G,at1->segi,at2->segi,true);
   if(!wc) {
 	 if(at1->chain[0]==at2->chain[0]) {
       if(at1->hetatm==at2->hetatm) {
         if(at1->resv==at2->resv) {
-          wc=WordCompare(at1->resi,at2->resi,true);
+          wc=WordCompare(G,at1->resi,at2->resi,true);
           if(!wc) {
-            wc=WordCompare(at1->resn,at2->resn,true);
+            wc=WordCompare(G,at1->resn,at2->resn,true);
             if(!wc) {
               if(at1->discrete_state==at2->discrete_state) {
                 if(at1->priority==at2->priority) {
                   if(at1->alt[0]==at2->alt[0]) {
-                    result=AtomNameCompare(at1->name,at2->name);
+                    if( (result=AtomInfoNameCompare(G,at1->name,at2->name))==0) {
+                      if(at1->rank<at2->rank)
+                        result = -1;
+                      else if(at1->rank>at2->rank)
+                        result = 1;
+                    }
                   } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
                     result=-1;
                   } else {
@@ -1191,7 +1423,7 @@ int AtomInfoCompare(AtomInfoType *at1,AtomInfoType *at2)
             }
           } else {
             /* NOTE: don't forget to synchronize with below */
-            if(SettingGet(cSetting_pdb_insertions_go_first)) {
+            if(SettingGet(G,cSetting_pdb_insertions_go_first)) {
               int sl1,sl2;
               sl1 = strlen(at1->resi);
               sl2 = strlen(at2->resi);
@@ -1227,7 +1459,7 @@ int AtomInfoCompare(AtomInfoType *at1,AtomInfoType *at2)
 }
 
 
-int AtomInfoCompareIgnoreHet(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoCompareIgnoreHet(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   int result;
   int wc;
@@ -1240,19 +1472,24 @@ int AtomInfoCompareIgnoreHet(AtomInfoType *at1,AtomInfoType *at2)
 	  at2->segi,at2->chain,at2->resv,at2->resi,at2->resn,at2->priority,at2->name);
   */
 
-  wc=WordCompare(at1->segi,at2->segi,true);
+  wc=WordCompare(G,at1->segi,at2->segi,true);
   if(!wc) {
 	 if(at1->chain[0]==at2->chain[0]) {
       /*      if(at1->hetatm==at2->hetatm) {*/
       if(at1->resv==at2->resv) {
-        wc=WordCompare(at1->resi,at2->resi,true);
+        wc=WordCompare(G,at1->resi,at2->resi,true);
         if(!wc) {
-          wc=WordCompare(at1->resn,at2->resn,true);
+          wc=WordCompare(G,at1->resn,at2->resn,true);
           if(!wc) {
             if(at1->discrete_state==at2->discrete_state) {
               if(at1->priority==at2->priority) {
                 if(at1->alt[0]==at2->alt[0]) {
-                  result=AtomNameCompare(at1->name,at2->name);
+                  if( (result=AtomInfoNameCompare(G,at1->name,at2->name))==0) {
+                    if(at1->rank<at2->rank)
+                      result = -1;
+                    else if(at1->rank>at2->rank)
+                      result = 1;
+                  }
                 } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
                   result=-1;
                 } else {
@@ -1274,7 +1511,7 @@ int AtomInfoCompareIgnoreHet(AtomInfoType *at1,AtomInfoType *at2)
         } else {
           /* NOTE: don't forget to synchronize with above */
 
-            if(SettingGet(cSetting_pdb_insertions_go_first)) {
+            if(SettingGet(G,cSetting_pdb_insertions_go_first)) {
               int sl1,sl2;
               sl1 = strlen(at1->resi);
               sl2 = strlen(at2->resi);
@@ -1310,12 +1547,12 @@ int AtomInfoCompareIgnoreHet(AtomInfoType *at1,AtomInfoType *at2)
   return(result);
 }
 
-int AtomInfoNameOrder(AtomInfoType *at1,AtomInfoType *at2) 
+int AtomInfoNameOrder(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2) 
 {
   int result;
   if(at1->alt[0]==at2->alt[0]) {
     if(at1->priority==at2->priority) {
-      result=AtomNameCompare(at1->name,at2->name);
+      result=AtomInfoNameCompare(G,at1->name,at2->name);
     } else if(at1->priority<at2->priority) {
       result=-1;
     } else {
@@ -1329,75 +1566,76 @@ int AtomInfoNameOrder(AtomInfoType *at1,AtomInfoType *at2)
   return(result);
 }
 
-int AtomInfoInOrder(AtomInfoType *atom,int atom1,int atom2)
+int AtomInfoInOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
 {
-  return(AtomInfoCompare(atom+atom1,atom+atom2)<=0);
+  return(AtomInfoCompare(G,atom+atom1,atom+atom2)<=0);
 }
 
-int AtomInfoInOrderIgnoreHet(AtomInfoType *atom,int atom1,int atom2)
+int AtomInfoInOrderIgnoreHet(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
 {
-  return(AtomInfoCompareIgnoreHet(atom+atom1,atom+atom2)<=0);
+  return(AtomInfoCompareIgnoreHet(G,atom+atom1,atom+atom2)<=0);
 }
 
-int AtomInfoInOrigOrder(AtomInfoType *atom,int atom1,int atom2)
+int AtomInfoInOrigOrder(PyMOLGlobals *G,AtomInfoType *atom,int atom1,int atom2)
 {
-  if(atom[atom1].id!=atom[atom2].id)
-    return (atom[atom1].id<atom[atom2].id);
+  if(atom[atom1].rank!=atom[atom2].rank)
+    return (atom[atom1].rank<atom[atom2].rank);
   else
-    return(AtomInfoCompare(atom+atom1,atom+atom2)<=0);
+    return(AtomInfoCompare(G,atom+atom1,atom+atom2)<=0);
 }
 
-int AtomInfoSameResidue(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoSameResidue(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   if(at1->hetatm==at2->hetatm)
     if(at1->chain[0]==at2->chain[0])
       if(at1->resv==at2->resv)
+
         if(at1->discrete_state==at2->discrete_state)
-          if(WordMatch(at1->resi,at2->resi,true)<0) 
-            if(WordMatch(at1->segi,at2->segi,true)<0) 
-              if(WordMatch(at1->resn,at2->resn,true)<0) 
+          if(WordMatch(G,at1->resi,at2->resi,true)<0) 
+            if(WordMatch(G,at1->segi,at2->segi,true)<0) 
+              if(WordMatch(G,at1->resn,at2->resn,true)<0) 
                 return 1;
   return 0;
 }
 
-int AtomInfoSameResidueP(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoSameResidueP(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   if(at1&&at2)
     if(at1->hetatm==at2->hetatm)
       if(at1->chain[0]==at2->chain[0])
         if(at1->resv==at2->resv)
           if(at1->discrete_state==at2->discrete_state)
-            if(WordMatch(at1->resi,at2->resi,true)<0) 
-              if(WordMatch(at1->segi,at2->segi,true)<0) 
-                if(WordMatch(at1->resn,at2->resn,true)<0) 
+            if(WordMatch(G,at1->resi,at2->resi,true)<0) 
+              if(WordMatch(G,at1->segi,at2->segi,true)<0) 
+                if(WordMatch(G,at1->resn,at2->resn,true)<0) 
                   return 1;
   return 0;
 }
 
-int AtomInfoSameChainP(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoSameChainP(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   if(at1&&at2)
     if(at1->chain[0]==at2->chain[0])
-        if(WordMatch(at1->segi,at2->segi,true)<0) 
+        if(WordMatch(G,at1->segi,at2->segi,true)<0) 
           return 1;
   return 0;
 }
 
-int AtomInfoSameSegmentP(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoSameSegmentP(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   if(at1&&at2)
-    if(WordMatch(at1->segi,at2->segi,true)<0) 
+    if(WordMatch(G,at1->segi,at2->segi,true)<0) 
       return 1;
   return 0;
 }
 
-int AtomInfoSequential(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoSequential(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   char last1=0,last2=0;
   char *p;
   if(at1->hetatm==at2->hetatm)
     if(at1->chain[0]==at2->chain[0]) {
-      if(WordMatch(at1->segi,at2->segi,true)<0) {
+      if(WordMatch(G,at1->segi,at2->segi,true)<0) {
         if(at1->resv==at2->resv) {
           p=at1->resi;
           while(*p) {
@@ -1420,19 +1658,19 @@ int AtomInfoSequential(AtomInfoType *at1,AtomInfoType *at2)
   return 0;
 }
 
-int AtomInfoMatch(AtomInfoType *at1,AtomInfoType *at2)
+int AtomInfoMatch(PyMOLGlobals *G,AtomInfoType *at1,AtomInfoType *at2)
 {
   if((tolower(at1->chain[0]))==(tolower(at2->chain[0])))
-	 if(WordMatch(at1->name,at2->name,true)<0)
-		if(WordMatch(at1->resi,at2->resi,true)<0)
-		  if(WordMatch(at1->resn,at2->resn,true)<0)
-			 if(WordMatch(at1->segi,at2->segi,true)<0)
+	 if(WordMatch(G,at1->name,at2->name,true)<0)
+		if(WordMatch(G,at1->resi,at2->resi,true)<0)
+		  if(WordMatch(G,at1->resn,at2->resn,true)<0)
+			 if(WordMatch(G,at1->segi,at2->segi,true)<0)
             if((tolower(at1->alt[0]))==(tolower(at2->alt[0])))
               return 1;
   return 0;
 }
 
-int AtomInfoGetExpectedValence(AtomInfoType *I) {
+int AtomInfoGetExpectedValence(PyMOLGlobals *G,AtomInfoType *I) {
   int result=-1; /* negative indicates minimum expected valence (abs)
                 but it could be higher  */
 
@@ -1485,9 +1723,99 @@ int AtomInfoGetExpectedValence(AtomInfoType *I) {
   }
   return(result);
 }
-void AtomInfoAssignParameters(AtomInfoType *I)
+
+static void set_protons(AtomInfoType *I,char *elem) 
 {
-  char *n,*e;
+  char *e = I->elem;
+
+  if(elem)
+    e=elem;
+  while((*e>='0')&&(*e<='9')&&(*(e+1))) e++;
+
+  if(!e[1]) { /* single letter */
+    switch(e[0]) {
+    case 'H': I->protons=cAN_H; break;
+    case 'D': I->protons=cAN_H; break;
+    case 'Q': I->protons=cAN_H; break; /* for NMR structures */
+    case 'C': I->protons=cAN_C; break;
+    case 'N': I->protons=cAN_N; break;
+    case 'O': I->protons=cAN_O; break;
+    case 'F': I->protons=cAN_F; break;
+    case 'S': I->protons=cAN_S; break;
+    case 'P': I->protons=cAN_P; break;
+    case 'K': I->protons=cAN_K; break;
+    case 'I': I->protons=cAN_I; break;
+    default: /* unrecognized element (possible garbage in this column?) */
+      if(!elem)
+        set_protons(I,I->name);
+      break;
+    }
+  } else {
+    switch(e[0]) {
+    case 'C':
+      switch(e[1]) {
+      case 'L':
+      case 'l': I->protons=cAN_Cl; break;
+      case 'A':
+      case 'a': I->protons=cAN_Ca; break;
+      case 'U':
+      case 'u': I->protons=cAN_Cu; break;
+      }
+      break;
+    case 'B':
+      switch(e[1]) {
+      case 'R':
+      case 'r': I->protons=cAN_Br; break;
+      }
+      break;
+    case 'F':
+      switch(e[1]) {
+      case 'E':
+      case 'e': I->protons=cAN_Fe; break;
+      }
+      break;
+    case 'H':
+      switch(e[1]) {
+      case 'G':
+      case 'g': I->protons=cAN_Hg; break;
+      case 'E':
+      case 'e': I->protons=cAN_He; break;
+      default:
+        I->protons=cAN_H; /* if unrecognized but begins with H, then assume H */
+        break;
+      }
+      break;
+    case 'M':
+      switch(e[1]) {
+      case 'G':
+      case 'g': I->protons=cAN_Mg; break;
+      }
+      break;
+    case 'Z':
+      switch(e[1]) {
+      case 'N':
+      case 'n': I->protons=cAN_Zn; break;
+      }
+      break;
+    case 'S':
+      switch(e[1]) {
+      case 'I':
+      case 'i': I->protons=cAN_Si; break;
+      case 'E':
+      case 'e': I->protons=cAN_Se; break;
+      }
+      break;
+    default: /* unrecognized element (possible garbage?) */
+      if(!elem)
+        set_protons(I,I->name);
+      break;
+    }
+  }
+}
+
+void AtomInfoAssignParameters(PyMOLGlobals *G,AtomInfoType *I)
+{
+  char *n=NULL,*e=NULL;
   int pri;
   float vdw;
 
@@ -1503,7 +1831,7 @@ void AtomInfoAssignParameters(AtomInfoType *I)
     switch ( *e ) {
     case 'C':
       if(*(e+1)=='A') {
-        if(!(WordMatch("CA",I->resn,true)<0)&&(!(WordMatch("CA+",I->resn,true)<0)))
+        if(!(WordMatch(G,"CA",I->resn,true)<0)&&(!(WordMatch(G,"CA+",I->resn,true)<0)))
           *(e+1)=0; 
       } else if(!(
            (*(e+1)=='a')||/* CA intpreted as carbon, not calcium */
@@ -1545,20 +1873,23 @@ void AtomInfoAssignParameters(AtomInfoType *I)
       if(!((*(e+1)=='s')))
         *(e+1)=0;
       break;
+    case 'Q':
+      *(e+1)=0;
+      break;
     default:
       break;
     }
     if(*(e+1)) *(e+1)=tolower(*(e+1));
   }
-  I->hydrogen=((
-                ((*I->elem)=='H')||((*I->elem)=='D')
-                )&&(!(*(I->elem+1))));
-  
+  I->hydrogen=((((*I->elem)=='H')||
+                ((*I->elem)=='D')||
+                ((*I->elem)=='Q'))
+               &&(!(*(I->elem+1))));
   n = I->name;
   while((*n>='0')&&(*n<='9')&&(*(n+1))) n++;
   if(toupper(*n)!=I->elem[0]) {
     pri=1000; /* unconventional atom name -- make no assignments */
-  } else if((int)SettingGet(cSetting_pdb_standard_order)) {
+  } else if((int)SettingGet(G,cSetting_pdb_standard_order)) {
     switch ( *n )
       {
         
@@ -1744,101 +2075,36 @@ void AtomInfoAssignParameters(AtomInfoType *I)
   }
   
   I->priority=pri;
-  e = I->elem;  
-  while((*e>='0')&&(*e<='9')&&(*(e+1))) e++;
-  switch ( *e )
-    {
-    case 'N':
-      switch (*(e+1)) {
-      case 0:
-        vdw=1.55F;  break;
-      case 'a':
-      case 'A':
-        vdw=2.27F; break;
-      default:
-        vdw = 1.80F; break;
-      }
-      break;
-    case 'C' :	
-      switch (*(e+1)) 
-        {
-        case 'U':
-        case 'u':
-          vdw=1.40F; break; /* Cu */
-        case 'L':
-        case 'l':
-          vdw=1.75F; break; /* Cl */
-        case 0:
-          vdw=1.7F;  break; /* Carbon */
-        default:
-          vdw=1.8F;  break; 
-        }
-      break;
-    case 'O' : vdw=1.52F;  break;
-    case 'I' :	vdw=1.98F; break;
-    case 'P' :	vdw=1.80F; break;
-    case 'B' :	vdw=1.85F; break; /* incl B, BR */
-    case 'S' :	
-      switch (*(e+1)) {
-      case 'e':
-      case 'E':
-        vdw=1.9F;
-        break;
-      default:
-        vdw=1.80F; break;
+  
+  e = I->elem;
+  set_protons(I,NULL);
 
-      }
-      break;
-    case 'F' : 
-      switch (*(e+1))
-        {
-        case 0:
-          vdw=1.47F; break;
-        case 'E': 
-        case 'e': 
-          /*          vdw=0.64F; break;*/
-          vdw=1.80F; break; /* default */
-        default:
-          vdw=1.35F; break;
-        }
-      break;
-    case 'K':
-      switch (*(e+1)) {
-      case 0:
-        vdw=2.75F; break;
-      default:
-        vdw = 1.80F; break;
-      }
-      break;
-    case 'M':
-      switch (*(e+1)) {
-      case 'n':
-      case 'N':
-      case 'g':
-      case 'G':
-        vdw=1.73F; break;
-      default:
-        vdw = 1.80F; break;
-      }
-      break;
-    case 'H' :
-    case 'D' :
-      vdw = 1.2F; /* WLD */
-      break;
-    case 'Z':
-      switch (*(e+1)) {
-      case 'n':
-      case 'N':
-        vdw=1.39F; break;
-      default:
-        vdw = 1.80F; break;
-      }
-      break;
-    default:
-      vdw=1.8F;
-      break;
-    }
-  if(SettingGet(cSetting_legacy_vdw_radii)) { /* ver<0.75, old, incorrect VDW */
+  /* vdw radii */
+
+  switch(I->protons) {
+  case cAN_H:  vdw=1.20F; break;
+  case cAN_N:  vdw=1.55F; break;
+  case cAN_Na: vdw=2.27F; break;
+  case cAN_Cu: vdw=1.40F; break; 
+  case cAN_Cl: vdw=1.75F; break;
+  case cAN_C:  vdw=1.70F; break; 
+  case cAN_O:  vdw=1.52F; break;
+  case cAN_I:	vdw=1.98F; break;
+  case cAN_P:	vdw=1.80F; break;
+  case cAN_B:	vdw=1.85F; break; 
+  case cAN_Br:	vdw=1.85F; break; 
+  case cAN_S:  vdw=1.80F; break;
+  case cAN_Se: vdw=1.90F; break;
+  case cAN_F:  vdw=1.47F; break;
+  case cAN_Fe: vdw=1.80F; break; 
+  case cAN_K:  vdw=2.75F; break;
+  case cAN_Mn: vdw=1.73F; break;
+  case cAN_Mg: vdw=1.73F; break;
+  case cAN_Zn: vdw=1.39F; break;
+  default: vdw=1.80F; break;
+  }
+  
+  if(SettingGet(G,cSetting_legacy_vdw_radii)) { /* ver<0.75, old, incorrect VDW */
     if(!strcmp(e,"N")) vdw=1.8F; /* slow but compact */
     if(!strcmp(e,"C")) vdw=1.8F;
     if(!strcmp(e,"Cl")) vdw=1.8F;
@@ -1850,86 +2116,13 @@ void AtomInfoAssignParameters(AtomInfoType *I)
     if(!strcmp(e,"F")) vdw=1.35F;
     if(!strcmp(e,"H")) vdw=1.1F;
   }
-  e = I->elem;
-  if(!e[1]) { /* single letter */
-    switch(e[0]) {
-    case 'H': I->protons=cAN_H; break;
-    case 'D': I->protons=cAN_H; break;
-    case 'C': I->protons=cAN_C; break;
-    case 'N': I->protons=cAN_N; break;
-    case 'O': I->protons=cAN_O; break;
-    case 'F': I->protons=cAN_F; break;
-    case 'S': I->protons=cAN_S; break;
-    case 'P': I->protons=cAN_P; break;
-    case 'K': I->protons=cAN_K; break;
-    case 'I': I->protons=cAN_I; break;
-    }
-  } else {
-    switch(e[0]) {
-    case 'C':
-      switch(e[1]) {
-      case 'L':
-      case 'l': I->protons=cAN_Cl; break;
-      case 'A':
-      case 'a': I->protons=cAN_Ca; break;
-      case 'U':
-      case 'u': I->protons=cAN_Cu; break;
-      }
-      break;
-    case 'B':
-      switch(e[1]) {
-      case 'R':
-      case 'r': I->protons=cAN_Br; break;
-      }
-      break;
-    case 'F':
-      switch(e[1]) {
-      case 'E':
-      case 'e': I->protons=cAN_Fe; break;
-      }
-      break;
-    case 'H':
-      switch(e[1]) {
-      case 'G':
-      case 'g': I->protons=cAN_Hg; break;
-      }
-      break;
-    case 'M':
-      switch(e[1]) {
-      case 'G':
-      case 'g': I->protons=cAN_Mg; break;
-      }
-      break;
-    case 'Z':
-      switch(e[1]) {
-      case 'N':
-      case 'n': I->protons=cAN_Zn; break;
-      }
-      break;
-    case 'S':
-      switch(e[1]) {
-      case 'I':
-      case 'i': I->protons=cAN_Si; break;
-      case 'E':
-      case 'e': I->protons=cAN_Se; break;
-      }
-      break;
-    }
-  }
-  if(I->vdw==0.0)
-    I->vdw = vdw;
 
+  if(I->vdw==0.0) /* only assigned if not yet assigned */
+    I->vdw = vdw;
+  if(!I->protons)
+    I->protons = cAN_C; /* default assumption */
+  else if(I->protons==cAN_H) 
+    I->hydrogen = true;
   /*  printf("I->name %s I->priority %d\n",I->name,I->priority);*/
 }
-
-
-
-
-
-
-
-
-
-
-
-
+  
