@@ -41,6 +41,8 @@ Z* -------------------------------------------------------------------
 #include"RepNonbonded.h"
 #include"RepNonbondedSphere.h"
 
+#include"PyMOLGlobals.h"
+
 void CoordSetUpdate(CoordSet *I);
 
 void CoordSetFree(CoordSet *I);
@@ -55,33 +57,39 @@ void CoordSetAppendIndices(CoordSet *I,int offset);
 static  char sATOM[]="ATOM  ";
 static  char sHETATM[]="HETATM";
 
+/*========================================================================*/
+int BondCompare(BondType *a,BondType *b)
+{
+  register int ai0=a->index[0];
+  register int bi0=b->index[0];
+  if(ai0==bi0) {
+    register int ai1=a->index[1];
+    register int bi1=b->index[1];
+    if(ai1==bi1) {
+      return 0;
+    } else if(ai1>bi1) {
+      return 1;
+    } else {
+      return -1;
+    }
+  } else if(ai0>bi0) {
+    return 1;
+  } else {
+    return -1;
+  }
+}
+
 /*========================================================================*/
 int BondInOrder(BondType *a,int b1,int b2)
 {
   return(BondCompare(a+b1,a+b2)<=0);
 }
-/*========================================================================*/
-int BondCompare(BondType *a,BondType *b)
-{
-  int result;
-  if(a->index[0]==b->index[0]) {
-	if(a->index[1]==b->index[1]) {
-	  result=0;
-	} else if(a->index[1]>b->index[1]) {
-	  result=1;
-	} else {
-	  result=-1;
-	}
-  } else if(a->index[0]>b->index[0]) {
-	result=1;
-  } else {
-	result=-1;
-  }
-  return(result);
-}
 
-int CoordSetFromPyList(PyObject *list,CoordSet **cs)
+int CoordSetFromPyList(PyMOLGlobals *G,PyObject *list,CoordSet **cs)
 {
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
   CoordSet *I = NULL;
   PyObject *tmp;
   int ok = true;
@@ -96,7 +104,7 @@ int CoordSetFromPyList(PyObject *list,CoordSet **cs)
     *cs = NULL;
   } else {
   
-    if(ok) I=CoordSetNew();
+    if(ok) I=CoordSetNew(G);
     if(ok) ok = (I!=NULL);
     if(ok) ok = (list!=NULL);
     if(ok) ok = PyList_Check(list);
@@ -113,7 +121,7 @@ int CoordSetFromPyList(PyObject *list,CoordSet **cs)
         ok = PConvPyListToIntArray(tmp,&I->AtmToIdx);
     }
     if(ok&&(ll>5)) ok = PConvPyStrToStr(PyList_GetItem(list,5),I->Name,sizeof(WordType));
-    
+    if(ok&&(ll>6))  ok = ObjectStateFromPyList(G,PyList_GetItem(list,6),&I->State);
     if(!ok) {
       if(I)
         CoordSetFree(I);
@@ -122,14 +130,19 @@ int CoordSetFromPyList(PyObject *list,CoordSet **cs)
     }
   }
   return(ok);
+#endif
 }
 
 PyObject *CoordSetAsPyList(CoordSet *I)
 {
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
+
   PyObject *result = NULL;
 
   if(I) {
-    result = PyList_New(6);
+    result = PyList_New(7);
     
     PyList_SetItem(result,0,PyInt_FromLong(I->NIndex));
     PyList_SetItem(result,1,PyInt_FromLong(I->NAtIndex));
@@ -140,10 +153,12 @@ PyObject *CoordSetAsPyList(CoordSet *I)
     else 
       PyList_SetItem(result,4,PConvAutoNone(NULL));
     PyList_SetItem(result,5,PyString_FromString(I->Name));
+    PyList_SetItem(result,6,ObjectStateAsPyList(&I->State));
     /* TODO symmetry, spheroid, setting, periodic box ... */
-
   }
   return(PConvAutoNone(result));
+#endif
+
 }
 
 void CoordSetAdjustAtmIdx(CoordSet *I,int *lookup,int nAtom)
@@ -153,7 +168,7 @@ void CoordSetAdjustAtmIdx(CoordSet *I,int *lookup,int nAtom)
   int a;
   int a0;
 
-  PRINTFD(FB_CoordSet)
+  PRINTFD(I->State.G,FB_CoordSet)
     " CoordSetAdjustAtmIdx-Debug: entered NAtIndex: %d NIndex %d\n I->AtmToIdx %p\n",
     I->NAtIndex,I->NIndex,(void*)I->AtmToIdx
     ENDFD;
@@ -169,7 +184,7 @@ void CoordSetAdjustAtmIdx(CoordSet *I,int *lookup,int nAtom)
   for(a=0;a<I->NIndex;a++) { 
     I->IdxToAtm[a] = lookup[I->IdxToAtm[a]];
   }
-  PRINTFD(FB_CoordSet)
+  PRINTFD(I->State.G,FB_CoordSet)
     " CoordSetAdjustAtmIdx-Debug: leaving... NAtIndex: %d NIndex %d\n",
     I->NAtIndex,I->NIndex
     ENDFD;
@@ -205,7 +220,7 @@ void CoordSetPurge(CoordSet *I)
   float *c0,*c1;
   obj=I->Obj;
 
-  PRINTFD(FB_CoordSet)
+  PRINTFD(I->State.G,FB_CoordSet)
     " CoordSetPurge-Debug: entering..."
     ENDFD;
 
@@ -234,20 +249,20 @@ void CoordSetPurge(CoordSet *I)
     I->NIndex+=offset;
     VLASize(I->Coord,float,I->NIndex*3);
     I->IdxToAtm=Realloc(I->IdxToAtm,int,I->NIndex);
-    PRINTFD(FB_CoordSet)
+    PRINTFD(I->State.G,FB_CoordSet)
       " CoordSetPurge-Debug: I->IdxToAtm shrunk to %d\n",I->NIndex
       ENDFD;
     if(I->fInvalidateRep)
       I->fInvalidateRep(I,cRepAll,cRepInvAtoms); /* this will free Color */
   }
-  PRINTFD(FB_CoordSet)
+  PRINTFD(I->State.G,FB_CoordSet)
     " CoordSetPurge-Debug: leaving NAtIndex %d NIndex %d...\n",
     I->NAtIndex,I->NIndex
     ENDFD;
 
 }
 /*========================================================================*/
-int CoordSetTransformAtom(CoordSet *I,int at,float *TTT)
+int CoordSetTransformAtomTTTf(CoordSet *I,int at,float *TTT)
 {
   ObjectMolecule *obj;
   int a1 = 01;
@@ -264,11 +279,58 @@ int CoordSetTransformAtom(CoordSet *I,int at,float *TTT)
   if(a1>=0) {
     result = 1;
     v1 = I->Coord+3*a1;
-    MatrixApplyTTTfn3f(1,v1,TTT,v1);
+    MatrixTransformTTTfN3f(1,v1,TTT,v1);
   }
 
   return(result);
 }
+
+/*========================================================================*/
+int CoordSetTransformAtomR44f(CoordSet *I,int at,float *matrix)
+{
+  ObjectMolecule *obj;
+  int a1 = 01;
+  int result = 0;
+  float *v1;
+
+  obj = I->Obj;
+  if(obj->DiscreteFlag) {
+    if(I==obj->DiscreteCSet[at])
+      a1=obj->DiscreteAtmToIdx[at];
+  } else 
+    a1=I->AtmToIdx[at];
+  
+  if(a1>=0) {
+    result = 1;
+    v1 = I->Coord+3*a1;
+    MatrixTransformR44fN3f(1,v1,matrix,v1);
+  }
+
+  return(result);
+}
+/*========================================================================*/
+void CoordSetRecordTxfApplied(CoordSet *I,float *matrix,int homogenous)
+{
+  if(I->State.Matrix) { 
+    double temp[16];
+    if(!homogenous) {
+      convertTTTfR44d(matrix,temp);
+    } else {
+      convert44f44d(matrix,temp);
+    }
+    left_multiply44d44d(temp,I->State.Matrix);
+  } else {
+    I->State.Matrix=Alloc(double,16);
+    if ( I->State.Matrix ) {
+      if(!homogenous)
+        convertTTTfR44d(matrix,I->State.Matrix);
+      else {
+        convert44f44d(matrix,I->State.Matrix);
+      }
+    }
+  }
+  /*  dump44d(I->State.Matrix,"history");*/
+}
 /*========================================================================*/
 int CoordSetMoveAtom(CoordSet *I,int at,float *v,int mode)
 {
@@ -362,6 +424,18 @@ void CoordSetTransform44f(CoordSet *I,float *mat)
   }  
 }
 /*========================================================================*/
+
+void CoordSetTransform33f(CoordSet *I,float *mat)
+{
+  int a;
+  float *v;
+  v=I->Coord;
+  for(a=0;a<I->NIndex;a++) {
+    transform33f3f(mat,v,v);
+    v+=3;
+  }  
+}
+/*========================================================================*/
 void CoordSetGetAverage(CoordSet *I,float *v0)
 {
   int a;
@@ -395,15 +469,22 @@ void CoordSetFracToReal(CoordSet *I,CCrystal *cryst)
 }
 
 /*========================================================================*/
-void CoordSetAtomToPDBStrVLA(char **charVLA,int *c,AtomInfoType *ai,
+void CoordSetAtomToPDBStrVLA(PyMOLGlobals *G,char **charVLA,int *c,AtomInfoType *ai,
                              float *v,int cnt,PDBInfoRec *pdb_info)
 {
   char *aType;
   AtomName name;
   ResIdent resi; 
+  ResName resn;
+  Chain chain;
   int rl;
-  int literal = (int)SettingGet(cSetting_pdb_literal_names);
-  int reformat = (int)SettingGet(cSetting_pdb_reformat_names_mode);
+  int literal = (int)SettingGet(G,cSetting_pdb_literal_names);
+  int reformat = (int)SettingGet(G,cSetting_pdb_reformat_names_mode);
+
+  strcpy(resn,ai->resn); 
+  if(SettingGetGlobal_b(G,cSetting_pdb_truncate_residue_name)) {
+    resn[3]=0; /* enforce 3-letter residue name in PDB files */
+  }
 
   if(ai->hetatm)
 	aType=sHETATM;
@@ -442,7 +523,7 @@ void CoordSetAtomToPDBStrVLA(char **charVLA,int *c,AtomInfoType *ai,
             case 1: /* pdb with internal pdb */
             case 3: /* pdb with internal iupac */
               if((ai->elem[0]=='H')&&(!ai->elem[1])&&(ai->name[2])) {
-                AtomInfoGetPDB3LetHydroName(ai->resn,ai->name,name);
+                AtomInfoGetPDB3LetHydroName(G,resn,ai->name,name);
               } else {
                 name[0]=' ';	
                 strcpy(name+1,ai->name);
@@ -538,32 +619,43 @@ void CoordSetAtomToPDBStrVLA(char **charVLA,int *c,AtomInfoType *ai,
       strcpy(name,ai->name); /* otherwise, stick in the first */
     }
   }
-  if((int)SettingGet(cSetting_pdb_retain_ids)) {
+  if((int)SettingGet(G,cSetting_pdb_retain_ids)) {
     cnt = ai->id - 1;
   }
   if(cnt>99998)
     cnt=99998;
+
   if((!pdb_info)||(!pdb_info->is_pqr_file)) { /* relying upon short-circuit */
 
-    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%3s %1s%5s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s\n",
-                  aType,cnt+1,name,ai->alt,ai->resn,
+    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%-4s%1s%5s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s\n",
+                  aType,cnt+1,name,ai->alt,resn,
                   ai->chain,resi,*v,*(v+1),*(v+2),ai->q,ai->b,ai->segi,ai->elem);
   } else {
-    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%3s %1s%5s   %8.3f%8.3f%8.3f %7.3f %7.3f\n",
-                  aType,cnt+1,name,ai->alt,ai->resn,
-                  ai->chain,resi,*v,*(v+1),*(v+2),ai->partialCharge,ai->bohr_radius);
+    if(pdb_info->pqr_no_chain_id) {
+      chain[0]=0;
+    } else {
+      chain[0] = ai->chain[0];
+      chain[1] = 0;
+    }
+      
+    (*c)+=sprintf((*charVLA)+(*c),"%6s%5i %-4s%1s%-4s%1s%5s   %8.3f%8.3f%8.3f %7.3f %7.3f\n",
+                  aType,cnt+1,name,ai->alt,resn,
+                  chain,resi,*v,*(v+1),*(v+2),ai->partialCharge,ai->bohr_radius);
   }
   
 }
 /*========================================================================*/
-PyObject *CoordSetAtomToChemPyAtom(AtomInfoType *ai,float *v,int index)
+PyObject *CoordSetAtomToChemPyAtom(PyMOLGlobals *G,AtomInfoType *ai,float *v,int index)
 {
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
   PyObject *atom;
   int ok = true;
 
   atom = PyObject_CallMethod(P_chempy,"Atom","");
   if (!atom) 
-    ok = ErrMessage("CoordSetAtomToChemPyAtom","can't create atom");
+    ok = ErrMessage(G,"CoordSetAtomToChemPyAtom","can't create atom");
   else {
     PConvFloat3ToPyObjAttr(atom,"coord",v);
     PConvStringToPyObjAttr(atom,"name",ai->name);
@@ -595,14 +687,20 @@ PyObject *CoordSetAtomToChemPyAtom(AtomInfoType *ai,float *v,int index)
   if(PyErr_Occurred())
     PyErr_Print();
   return(atom);
+#endif
 }
 /*========================================================================*/
-void CoordSetAtomToTERStrVLA(char **charVLA,int *c,AtomInfoType *ai,int cnt)
+void CoordSetAtomToTERStrVLA(PyMOLGlobals *G,char **charVLA,int *c,AtomInfoType *ai,int cnt)
 {
   ResIdent resi; 
+  ResName resn;
   int rl;
-  int retain_ids = (int)SettingGet(cSetting_pdb_retain_ids);
+  int retain_ids = (int)SettingGet(G,cSetting_pdb_retain_ids);
   int ter_id;
+
+  strcpy(resn,ai->resn); /* enforce 3-letter residue name in PDB files */
+  resn[3]=0;
+
   strcpy(resi,ai->resi);
   rl = strlen(resi)-1;
   if(rl>=0)
@@ -620,7 +718,7 @@ void CoordSetAtomToTERStrVLA(char **charVLA,int *c,AtomInfoType *ai,int cnt)
 
   (*c)+=sprintf((*charVLA)+(*c),
                 "%3s   %5i      %3s %1s%5s\n",
-                 "TER",ter_id,ai->resn,ai->chain,resi);
+                 "TER",ter_id,resn,ai->chain,resi);
   
 }
 
@@ -631,6 +729,37 @@ void CoordSetInvalidateRep(CoordSet *I,int type,int level)
 
   /*  printf("inv %d %d \n",type,level);fflush(stdout);*/
 
+  if(level==cRepInvVisib) {
+    if(SettingGet_b(I->State.G,I->Setting,I->Obj->Obj.Setting,
+                    cSetting_cartoon_side_chain_helper)) {
+      if((type==cRepCyl)||(type==cRepLine)||(type==cRepSphere))
+        CoordSetInvalidateRep(I,cRepCartoon,cRepInvVisib2);
+      else if(type==cRepCartoon) {
+        CoordSetInvalidateRep(I,cRepLine,cRepInvVisib2);
+        CoordSetInvalidateRep(I,cRepCyl,cRepInvVisib2);
+        CoordSetInvalidateRep(I,cRepSphere,cRepInvVisib2);
+      }
+    }
+    if(SettingGet_b(I->State.G,I->Setting,I->Obj->Obj.Setting,
+                    cSetting_ribbon_side_chain_helper)) {
+      if((type==cRepCyl)||(type==cRepLine)||(type==cRepSphere))
+        CoordSetInvalidateRep(I,cRepRibbon,cRepInvVisib2);
+      else if(type==cRepRibbon) {
+        CoordSetInvalidateRep(I,cRepLine,cRepInvVisib2);
+        CoordSetInvalidateRep(I,cRepCyl,cRepInvVisib2);
+        CoordSetInvalidateRep(I,cRepSphere,cRepInvVisib2);
+      }
+    }
+    if(SettingGet_b(I->State.G,I->Setting,I->Obj->Obj.Setting,
+                    cSetting_line_stick_helper)) {
+      if(type==cRepCyl) 
+        CoordSetInvalidateRep(I,cRepLine,cRepInvVisib2);
+      else if(type==cRepLine) {
+        CoordSetInvalidateRep(I,cRepCyl,cRepInvVisib2);
+      }
+    }
+  }
+
   if(I->Spheroid)
     if(I->NSpheroid!=I->NAtIndex*I->SpheroidSphereSize) {
       FreeP(I->Spheroid);
@@ -666,7 +795,7 @@ void CoordSetInvalidateRep(CoordSet *I,int type,int level)
 		}
 	 }
   }
-  SceneChanged();
+  SceneChanged(I->State.G);
 }
 /*========================================================================*/
 
@@ -681,9 +810,8 @@ void CoordSetInvalidateRep(CoordSet *I,int type,int level)
          I->Rep[rep] = I->Rep[rep]->fUpdate(I->Rep[rep],I,rep);\
     }\
   }\
-OrthoBusyFast(rep,I->NRep);\
+OrthoBusyFast(I->State.G,rep,I->NRep);\
 }
-
 /*========================================================================*/
 void CoordSetUpdate(CoordSet *I)
 {
@@ -714,8 +842,7 @@ void CoordSetUpdate(CoordSet *I)
             }
 		}
 	 }
-  OrthoBusyFast(0,I->NRep);
-
+  OrthoBusyFast(I->State.G,0,I->NRep);
   RepUpdateMacro(I, cRepLine,            RepWireBondNew        );
   RepUpdateMacro(I, cRepCyl,             RepCylBondNew         );
   RepUpdateMacro(I, cRepDot,             RepDotNew             );
@@ -732,8 +859,8 @@ void CoordSetUpdate(CoordSet *I)
     if(!I->Rep[a])
       I->Active[a]=false;
 
-  SceneDirty();
-  OrthoBusyFast(1,1);
+  SceneDirty(I->State.G);
+  OrthoBusyFast(I->State.G,1,1);
 }
 /*========================================================================*/
 void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
@@ -741,12 +868,12 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
   int a,aa;
   Rep *r;
 
-  PRINTFD(FB_CoordSet)
+  PRINTFD(I->State.G,FB_CoordSet)
     " CoordSetRender: entered (%p).\n",(void*)I
     ENDFD;
 
   if((!pass)&&I->Name[0])
-    ButModeCaption(I->Name);
+    ButModeCaption(I->State.G,I->Name);
   for(aa=0;aa<I->NRep;aa++) {
     if(aa==cRepSurface) { /* reorder */
       a=cRepCell;
@@ -765,13 +892,13 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
           } else {
             if(I->Obj) 
               ray->fWobble(ray,
-                           SettingGet_i(I->Setting,I->Obj->Obj.Setting,cSetting_ray_texture),
-                           SettingGet_3fv(I->Setting,I->Obj->Obj.Setting,cSetting_ray_texture_settings));
+                           SettingGet_i(I->State.G,I->Setting,I->Obj->Obj.Setting,cSetting_ray_texture),
+                           SettingGet_3fv(I->State.G,I->Setting,I->Obj->Obj.Setting,cSetting_ray_texture_settings));
             else
               ray->fWobble(ray,
-                           SettingGet_i(I->Setting,NULL,cSetting_ray_texture),
-                           SettingGet_3fv(I->Setting,NULL,cSetting_ray_texture_settings));
-            ray->fColor3fv(ray,ColorGet(I->Obj->Obj.Color));
+                           SettingGet_i(I->State.G,I->Setting,NULL,cSetting_ray_texture),
+                           SettingGet_3fv(I->State.G,I->Setting,NULL,cSetting_ray_texture_settings));
+            ray->fColor3fv(ray,ColorGet(I->State.G,I->Obj->Obj.Color));
           }
         
           if(r->fRender) { /* do OpenGL rendering in three passes */
@@ -780,6 +907,8 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
             } else 
               switch(a) {
               case cRepLabel:
+                if(pass==1) r->fRender(r,ray,pick);
+                break;
               case cRepNonbondedSphere:
               case cRepRibbon:
               case cRepDot:
@@ -796,7 +925,7 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
                 if(!pass) r->fRender(r,ray,pick);                
                 break;
               case cRepCyl: /* render sticks differently depending on transparency */
-                if(SettingGet_f(r->cs->Setting,
+                if(SettingGet_f(I->State.G,r->cs->Setting,
                                 r->obj->Setting,
                                 cSetting_stick_transparency)>0.0001) {
                   if(pass==-1)
@@ -807,7 +936,7 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
 
               case cRepSurface:
                 /*                if(pass==-1) r->fRender(r,ray,pick);              */
-                if(SettingGet_f(r->cs->Setting,
+                if(SettingGet_f(I->State.G,r->cs->Setting,
                                 r->obj->Setting,
                                 cSetting_transparency)>0.0001) {
                   if(pass==-1)
@@ -816,7 +945,7 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
                   r->fRender(r,ray,pick);
                 break;
               case cRepSphere: /* render spheres differently depending on transparency */
-                if(SettingGet_f(r->cs->Setting,
+                if(SettingGet_f(I->State.G,r->cs->Setting,
                                 r->obj->Setting,
                                 cSetting_sphere_transparency)>0.0001) {
                   if(pass==-1)
@@ -825,7 +954,7 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
                   r->fRender(r,ray,pick);
                 break;
               case cRepCartoon:
-                if(SettingGet_f(r->cs->Setting,
+                if(SettingGet_f(I->State.G,r->cs->Setting,
                                 r->obj->Setting,
                                 cSetting_cartoon_transparency)>0.0001) {
                   if(pass==-1)
@@ -841,17 +970,18 @@ void CoordSetRender(CoordSet *I,CRay *ray,Pickable **pick,int pass)
                       ray->fWobble(ray,0,NULL);*/
         }
   }
-  PRINTFD(FB_CoordSet)
+  PRINTFD(I->State.G,FB_CoordSet)
     " CoordSetRender: leaving...\n"
     ENDFD;
 
 }
 /*========================================================================*/
-CoordSet *CoordSetNew(void)
+CoordSet *CoordSetNew(PyMOLGlobals *G)
 {
   int a;
-  OOAlloc(CoordSet);
-
+  OOAlloc(G,CoordSet);
+  ObjectStateInit(G,&I->State);
+  I->State.G=G;
   I->fFree=CoordSetFree;
   I->fRender=CoordSetRender;
   I->fUpdate=CoordSetUpdate;
@@ -878,7 +1008,7 @@ CoordSet *CoordSetNew(void)
   I->Obj = NULL;
   I->Spheroid = NULL;
   I->SpheroidNormal = NULL;
-  I->SpheroidSphereSize = Sphere1->nDot;
+  I->SpheroidSphereSize = I->State.G->Sphere->Sphere[1]->nDot; /* does this make any sense? */
   for(a=0;a<I->NRep;a++)
 	 I->Rep[a] = NULL;
   I->Setting=NULL;
@@ -891,7 +1021,7 @@ CoordSet *CoordSetCopy(CoordSet *cs)
   int nAtom;
   float *v0,*v1;
   int *i0,*i1;
-  OOAlloc(CoordSet);
+  OOAlloc(cs->State.G,CoordSet);
 
   (*I)=(*cs);
   I->Symmetry=SymmetryCopy(cs->Symmetry);
@@ -964,7 +1094,7 @@ void CoordSetExtendIndices(CoordSet *I,int nAtom)
 		if(I->AtmToIdx) {
         I->AtmToIdx = Realloc(I->AtmToIdx,int,nAtom);
         if(nAtom){
-          ErrChkPtr(I->AtmToIdx);
+          ErrChkPtr(I->State.G,I->AtmToIdx);
           for(a=I->NAtIndex;a<nAtom;a++)
             I->AtmToIdx[a]=-1;
         }
@@ -985,7 +1115,7 @@ void CoordSetAppendIndices(CoordSet *I,int offset)
 
   I->IdxToAtm = Alloc(int,I->NIndex);
   if(I->NIndex){
-    ErrChkPtr(I->IdxToAtm);
+    ErrChkPtr(I->State.G,I->IdxToAtm);
     for(a=0;a<I->NIndex;a++)
       I->IdxToAtm[a]=a+offset;
   }
@@ -1000,7 +1130,7 @@ void CoordSetAppendIndices(CoordSet *I,int offset)
   } else {
     I->AtmToIdx = Alloc(int,I->NIndex+offset);
     if(I->NIndex+offset){
-      ErrChkPtr(I->AtmToIdx);
+      ErrChkPtr(I->State.G,I->AtmToIdx);
       for(a=0;a<offset;a++)
         I->AtmToIdx[a]=-1;
       for(a=0;a<I->NIndex;a++) 
@@ -1017,8 +1147,8 @@ void CoordSetEnumIndices(CoordSet *I)
   I->AtmToIdx = Alloc(int,I->NIndex);
   I->IdxToAtm = Alloc(int,I->NIndex);
   if(I->NIndex) {
-    ErrChkPtr(I->AtmToIdx);
-    ErrChkPtr(I->IdxToAtm);
+    ErrChkPtr(I->State.G,I->AtmToIdx);
+    ErrChkPtr(I->State.G,I->IdxToAtm);
     for(a=0;a<I->NIndex;a++)
       {
         I->AtmToIdx[a]=a;
@@ -1063,6 +1193,7 @@ void CoordSetFree(CoordSet *I)
     FreeP(I->Spheroid);
     FreeP(I->SpheroidNormal);
     SettingFreeP(I->Setting);
+    ObjectStatePurge(&I->State);
     OOFreeP(I);
   }
 }
