@@ -24,19 +24,27 @@ typedef struct DistSet {
   void (*fRender)(struct DistSet *I,CRay *ray,Pickable **pick,int pass);
   void (*fFree)(struct DistSet *I);
   void (*fInvalidateRep)(struct DistSet *I,int type,int level);
+  PyMOLGlobals *G;
   struct ObjectDist *Obj;
   float *Coord;
   int NIndex;
   Rep **Rep; /* an array of pointers to representations */
   int NRep;
   CSetting *Setting;
+  /* extended for mobile distance labels */
+  float *LabelCoord;
+  /* extended for angles and torsions, with labels embedded with coordinates */
+  float *AngleCoord;
+  int NAngleIndex;
+  float *DihedralCoord;
+  int NDihedralIndex;
 } DistSet;
 
 #include"ObjectDist.h"
 
-DistSet *DistSetNew(void);
+DistSet *DistSetNew(PyMOLGlobals *G);
 PyObject *DistSetAsPyList(DistSet *I);
-int DistSetFromPyList(PyObject *list,DistSet **cs);
+int DistSetFromPyList(PyMOLGlobals *G,PyObject *list,DistSet **cs);
 int DistSetGetExtent(DistSet *I,float *mn,float *mx);
 
 #endif
