@@ -34,14 +34,15 @@ Z* -------------------------------------------------------------------
 #include"Executive.h"
 #include"PConv.h"
 #include"P.h"
+#include"Matrix.h"
 
-ObjectMesh *ObjectMeshNew(void);
+ObjectMesh *ObjectMeshNew(PyMOLGlobals *G);
 
 static void ObjectMeshFree(ObjectMesh *I);
-void ObjectMeshStateInit(ObjectMeshState *ms);
+void ObjectMeshStateInit(PyMOLGlobals *G,ObjectMeshState *ms);
 void ObjectMeshRecomputeExtent(ObjectMesh *I);
 
-
+#ifndef _PYMOL_NOPY
 static PyObject *ObjectMeshStateAsPyList(ObjectMeshState *I)
 {
   PyObject *result = NULL;
@@ -89,11 +90,12 @@ static PyObject *ObjectMeshStateAsPyList(ObjectMeshState *I)
 
   return(PConvAutoNone(result));  
 }
+#endif
 
-
+#ifndef _PYMOL_NOPY
 static PyObject *ObjectMeshAllStatesAsPyList(ObjectMesh *I)
 {
-  
+
   PyObject *result=NULL;
   int a;
   result = PyList_New(I->NState);
@@ -105,10 +107,11 @@ static PyObject *ObjectMeshAllStatesAsPyList(ObjectMesh *I)
     }
   }
   return(PConvAutoNone(result));  
-
 }
+#endif
 
-static int ObjectMeshStateFromPyList(ObjectMeshState *I,PyObject *list)
+#ifndef _PYMOL_NOPY
+static int ObjectMeshStateFromPyList(PyMOLGlobals *G,ObjectMeshState *I,PyObject *list)
 {
   int ok=true;
   int ll;
@@ -118,7 +121,7 @@ static int ObjectMeshStateFromPyList(ObjectMeshState *I,PyObject *list)
     if(!PyList_Check(list))
       I->Active=false;
     else {
-      ObjectMeshStateInit(I);
+      ObjectMeshStateInit(G,I);
       if(ok) ok=(list!=NULL);
       if(ok) ok=PyList_Check(list);
       if(ok) ll = PyList_Size(list);
@@ -153,24 +156,31 @@ static int ObjectMeshStateFromPyList(ObjectMeshState *I,PyObject *list)
   }
   return(ok);
 }
+#endif
 
+#ifndef _PYMOL_NOPY
 static int ObjectMeshAllStatesFromPyList(ObjectMesh *I,PyObject *list)
 {
+
   int ok=true;
   int a;
   VLACheck(I->State,ObjectMeshState,I->NState);
   if(ok) ok=PyList_Check(list);
   if(ok) {
     for(a=0;a<I->NState;a++) {
-      ok = ObjectMeshStateFromPyList(I->State+a,PyList_GetItem(list,a));
+      ok = ObjectMeshStateFromPyList(I->Obj.G,I->State+a,PyList_GetItem(list,a));
       if(!ok) break;
     }
   }
   return(ok);
 }
+#endif
 
-int ObjectMeshNewFromPyList(PyObject *list,ObjectMesh **result)
+int ObjectMeshNewFromPyList(PyMOLGlobals *G,PyObject *list,ObjectMesh **result)
 {
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
   int ok = true;
   int ll;
   ObjectMesh *I=NULL;
@@ -182,10 +192,10 @@ int ObjectMeshNewFromPyList(PyObject *list,ObjectMesh **result)
   /* TO SUPPORT BACKWARDS COMPATIBILITY...
      Always check ll when adding new PyList_GetItem's */
 
-  I=ObjectMeshNew();
+  I=ObjectMeshNew(G);
   if(ok) ok = (I!=NULL);
   
-  if(ok) ok = ObjectFromPyList(PyList_GetItem(list,0),&I->Obj);
+  if(ok) ok = ObjectFromPyList(G,PyList_GetItem(list,0),&I->Obj);
   if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,1),&I->NState);
   if(ok) ok = ObjectMeshAllStatesFromPyList(I,PyList_GetItem(list,2));
   if(ok) {
@@ -195,12 +205,16 @@ int ObjectMeshNewFromPyList(PyObject *list,ObjectMesh **result)
     /* cleanup? */
   }
   return(ok);
+#endif
 }
 
 
 
 PyObject *ObjectMeshAsPyList(ObjectMesh *I)
 {
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
   
   PyObject *result=NULL;
 
@@ -210,15 +224,19 @@ PyObject *ObjectMeshAsPyList(ObjectMesh *I)
   PyList_SetItem(result,2,ObjectMeshAllStatesAsPyList(I));
 
   return(PConvAutoNone(result));  
+#endif
 }
 
 static void ObjectMeshStateFree(ObjectMeshState *ms)
 {
-  if(PMGUI) {
+  ObjectStatePurge(&ms->State);
+  if(ms->State.G->HaveGUI) {
     if(ms->displayList) {
       if(PIsGlutThread()) {
-        glDeleteLists(ms->displayList,1);
-        ms->displayList = 0;
+        if(ms->State.G->ValidContext) {
+          glDeleteLists(ms->displayList,1);
+          ms->displayList = 0;
+        }
       } else {
         char buffer[255]; /* pass this off to the main thread */
         sprintf(buffer,"_cmd.gl_delete_lists(%d,%d)\n",ms->displayList,1);
@@ -253,7 +271,7 @@ void ObjectMeshDump(ObjectMesh *I,char *fname,int state)
   FILE *f;
   f=fopen(fname,"wb");
   if(!f) 
-    ErrMessage("ObjectMeshDump","can't open file for writing");
+    ErrMessage(I->Obj.G,"ObjectMeshDump","can't open file for writing");
   else {
     if(state<I->NState) {
       n=I->State[state].N;
@@ -272,9 +290,9 @@ void ObjectMeshDump(ObjectMesh *I,char *fname,int state)
           }
     }
     fclose(f);
-    PRINTFB(FB_ObjectMesh,FB_Actions)
+    PRINTFB(I->Obj.G,FB_ObjectMesh,FB_Actions)
       " ObjectMeshDump: %s written to %s\n",I->Obj.Name,fname
-      ENDFB;
+      ENDFB(I->Obj.G);
   }
 }
 
@@ -288,9 +306,9 @@ static void ObjectMeshInvalidate(ObjectMesh *I,int rep,int level,int state)
     I->State[state].RefreshFlag=true;
     if(level>=cRepInvAll) {
       I->State[state].ResurfaceFlag=true;      
-      SceneChanged();
+      SceneChanged(I->Obj.G);
     } else {
-      SceneDirty();
+      SceneDirty(I->Obj.G);
     }
 
     if(once_flag) break;
@@ -329,6 +347,7 @@ int ObjectMeshSetLevel(ObjectMesh *I,float level,int state)
 
 static void ObjectMeshUpdate(ObjectMesh *I) 
 {
+  PyMOLGlobals *G = I->Obj.G;
   int a;
   int c;
   ObjectMeshState *ms;
@@ -355,12 +374,12 @@ static void ObjectMeshUpdate(ObjectMesh *I)
     ms = I->State+a;
     if(ms->Active) {
       
-      map = ExecutiveFindObjectMapByName(ms->MapName);
+      map = ExecutiveFindObjectMapByName(I->Obj.G,ms->MapName);
       if(!map) {
         ok=false;
-        PRINTFB(FB_ObjectMesh,FB_Errors)
+        PRINTFB(I->Obj.G,FB_ObjectMesh,FB_Errors)
           "ObjectMeshUpdate-Error: map '%s' has been deleted.\n",ms->MapName
-          ENDFB;
+          ENDFB(I->Obj.G);
         ms->ResurfaceFlag=false;
       }
       if(map) {
@@ -375,6 +394,11 @@ static void ObjectMeshUpdate(ObjectMesh *I)
               CGOFree(ms->UnitCellCGO);
             ms->UnitCellCGO = CrystalGetUnitCellCGO(&ms->Crystal);
           } 
+          if(oms->State.Matrix) {
+            ObjectStateSetMatrix(&ms->State,oms->State.Matrix);
+          } else if(ms->State.Matrix) {
+            ObjectStateResetMatrix(&ms->State);
+          }
           ms->RefreshFlag=false;
         }
       }
@@ -382,10 +406,27 @@ static void ObjectMeshUpdate(ObjectMesh *I)
       if(map&&oms&&ms->N&&ms->V&&I->Obj.RepVis[cRepMesh]) {
         if(ms->ResurfaceFlag) {
           ms->ResurfaceFlag=false;
-          PRINTF " ObjectMesh: updating \"%s\".\n" , I->Obj.Name ENDF;
+          PRINTFB(G,FB_ObjectMesh,FB_Actions)
+            " ObjectMesh: updating \"%s\".\n" , I->Obj.Name 
+          ENDFB(G);
           if(oms->Field) {
-            IsosurfGetRange(oms->Field,oms->Crystal,
-                            ms->ExtentMin,ms->ExtentMax,ms->Range);
+
+            {
+              float *min_ext,*max_ext;
+              float tmp_min[3],tmp_max[3];
+              if(MatrixInvTransformExtentsR44d3f(ms->State.Matrix,
+                                                 ms->ExtentMin,ms->ExtentMax,
+                                                 tmp_min,tmp_max)) {
+                min_ext = tmp_min;
+                max_ext = tmp_max;
+              } else {
+                min_ext = ms->ExtentMin;
+                max_ext = ms->ExtentMax;
+              }
+
+              IsosurfGetRange(I->Obj.G,oms->Field,oms->Crystal,
+                              min_ext,max_ext,ms->Range);
+            }
             /*          printf("%d %d %d %d %d %d\n",
                    ms->Range[0],
                    ms->Range[1],
@@ -394,11 +435,23 @@ static void ObjectMeshUpdate(ObjectMesh *I)
                    ms->Range[4],
                    ms->Range[5]);*/
                 
-            IsosurfVolume(oms->Field,
+            IsosurfVolume(I->Obj.G,oms->Field,
                           ms->Level,
                           &ms->N,&ms->V,
                           ms->Range,
                           ms->DotFlag); 
+
+            if(ms->State.Matrix && VLAGetSize(ms->N)&&VLAGetSize(ms->V)) { 
+              int count;
+              /* take map coordinates back to view coordinates if necessary */
+              v = ms->V;
+              count = VLAGetSize(ms->V)/3;
+              while(count--) {
+                transform44d3f(ms->State.Matrix,v,v);
+                v+=3;
+              }
+            }
+
           }
           if(ms->CarveFlag&&ms->AtomVertex&&
              VLAGetSize(ms->N)&&VLAGetSize(ms->V)) {
@@ -409,7 +462,9 @@ static void ObjectMeshUpdate(ObjectMesh *I)
             }
               
             /* cull my friend, cull */
-            voxelmap=MapNew(-carve_buffer,ms->AtomVertex,VLAGetSize(ms->AtomVertex)/3,NULL);
+            voxelmap=MapNew(I->Obj.G,
+                            -carve_buffer,ms->AtomVertex,
+                            VLAGetSize(ms->AtomVertex)/3,NULL);
             if(voxelmap) {
               
               MapSetupExpress(voxelmap);  
@@ -481,12 +536,13 @@ static void ObjectMeshUpdate(ObjectMesh *I)
         }
       }
     }
-    SceneDirty();
+    SceneDirty(I->Obj.G);
   }
 }
 
 static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,int pass)
 {
+  PyMOLGlobals *G = I->Obj.G;
   float *v = NULL;
   float *vc;
   float radius;
@@ -509,7 +565,7 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
     } else {
       if(!ms) {
         if(I->NState&&
-           ((SettingGet(cSetting_static_singletons)&&(I->NState==1))))
+           ((SettingGet(I->Obj.G,cSetting_static_singletons)&&(I->NState==1))))
           ms=I->State;
       }
     }
@@ -520,24 +576,24 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
         if(ray) {
           
           if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
-            CGORenderRay(ms->UnitCellCGO,ray,ColorGet(I->Obj.Color),
+            CGORenderRay(ms->UnitCellCGO,ray,ColorGet(I->Obj.G,I->Obj.Color),
                          I->Obj.Setting,NULL);
           if(!ms->DotFlag) {
-            radius=SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_radius);
+            radius=SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_radius);
             
             if(radius==0.0F) {
-              radius = ray->PixelRadius*SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_width)/2.0F;
+              radius = ray->PixelRadius*SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_width)/2.0F;
             } 
           } else {
-            radius=SettingGet_f(I->Obj.Setting,NULL,cSetting_dot_radius);            
+            radius=SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_dot_radius);            
             if(radius==0.0F) {
-              radius = ray->PixelRadius*SettingGet_f(I->Obj.Setting,NULL,cSetting_dot_width)/1.4142F;
+              radius = ray->PixelRadius*SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_dot_width)/1.4142F;
             } 
           }
                   
 
           if(n&&v&&I->Obj.RepVis[cRepMesh]) {
-            vc = ColorGet(I->Obj.Color);
+            vc = ColorGet(I->Obj.G,I->Obj.Color);
             if(ms->DotFlag) {
               ray->fColor3fv(ray,vc);
               while(*n)
@@ -569,18 +625,19 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
                 }
             }
           }
-        } else if(pick&&PMGUI) {
-        } else if(PMGUI) {
-          if(!pass) {
+        } else if(G->HaveGUI && G->ValidContext) {
+          if(pick) {
+          } else {
+            if(!pass) {
 
             int use_dlst;
             if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
-              CGORenderGL(ms->UnitCellCGO,ColorGet(I->Obj.Color),
+              CGORenderGL(ms->UnitCellCGO,ColorGet(I->Obj.G,I->Obj.Color),
                           I->Obj.Setting,NULL);
 
-            SceneResetNormal(false);
+            SceneResetNormal(I->Obj.G,false);
             ObjectUseColor(&I->Obj);
-            use_dlst = (int)SettingGet(cSetting_use_display_lists);
+            use_dlst = (int)SettingGet(I->Obj.G,cSetting_use_display_lists);
             if(use_dlst&&ms->displayList) {
               glCallList(ms->displayList);
             } else { 
@@ -596,9 +653,9 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
 
             if(n&&v&&I->Obj.RepVis[cRepMesh]) {
               if(ms->DotFlag) 
-                glPointSize(SettingGet_f(I->Obj.Setting,NULL,cSetting_dot_width));
+                glPointSize(SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_dot_width));
               else
-                glLineWidth(SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_width));
+                glLineWidth(SettingGet_f(I->Obj.G,I->Obj.Setting,NULL,cSetting_mesh_width));
               while(*n)
                 {
                   c=*(n++);
@@ -616,7 +673,7 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
             if(use_dlst&&ms->displayList) {
               glEndList();
             }
-                  
+            }
             }
           }
         }
@@ -636,11 +693,11 @@ static int ObjectMeshGetNStates(ObjectMesh *I)
 }
 
 /*========================================================================*/
-ObjectMesh *ObjectMeshNew(void)
+ObjectMesh *ObjectMeshNew(PyMOLGlobals *G)
 {
-  OOAlloc(ObjectMesh);
+  OOAlloc(G,ObjectMesh);
   
-  ObjectInit((CObject*)I);
+  ObjectInit(G,(CObject*)I);
   
   I->NState = 0;
   I->State=VLAMalloc(10,sizeof(ObjectMeshState),5,true); /* autozero important */
@@ -656,8 +713,11 @@ ObjectMesh *ObjectMeshNew(void)
 }
 
 /*========================================================================*/
-void ObjectMeshStateInit(ObjectMeshState *ms)
+void ObjectMeshStateInit(PyMOLGlobals *G,ObjectMeshState *ms)
 {
+  if(ms->Active) 
+    ObjectStatePurge(&ms->State);
+  ObjectStateInit(G,&ms->State);
   if(!ms->V) {
     ms->V = VLAlloc(float,10000);
   }
@@ -679,7 +739,7 @@ void ObjectMeshStateInit(ObjectMeshState *ms)
 }
 
 /*========================================================================*/
-ObjectMesh *ObjectMeshFromBox(ObjectMesh *obj,ObjectMap *map,
+ObjectMesh *ObjectMeshFromBox(PyMOLGlobals *G,ObjectMesh *obj,ObjectMap *map,
                               int map_state,
                               int state,float *mn,float *mx,
                               float level,int dotFlag,
@@ -690,7 +750,7 @@ ObjectMesh *ObjectMeshFromBox(ObjectMesh *obj,ObjectMap *map,
   ObjectMapState *oms;
 
   if(!obj) {
-    I=ObjectMeshNew();
+    I=ObjectMeshNew(G);
   } else {
     I=obj;
   }
@@ -702,7 +762,7 @@ ObjectMesh *ObjectMeshFromBox(ObjectMesh *obj,ObjectMap *map,
   }
 
   ms=I->State+state;
-  ObjectMeshStateInit(ms);
+  ObjectMeshStateInit(G,ms);
 
   strcpy(ms->MapName,map->Obj.Name);
   ms->MapState = map_state;
@@ -713,7 +773,28 @@ ObjectMesh *ObjectMeshFromBox(ObjectMesh *obj,ObjectMap *map,
   if(oms) {
     copy3f(mn,ms->ExtentMin); /* this is not exactly correct...should actually take vertex points from range */
     copy3f(mx,ms->ExtentMax);
-    IsosurfGetRange(oms->Field,oms->Crystal,mn,mx,ms->Range);
+
+    if(oms->State.Matrix) {
+      ObjectStateSetMatrix(&ms->State,oms->State.Matrix);
+    } else if(ms->State.Matrix) {
+      ObjectStateResetMatrix(&ms->State);
+    }
+    
+    {
+      float *min_ext,*max_ext;
+      float tmp_min[3],tmp_max[3];
+      if(MatrixInvTransformExtentsR44d3f(ms->State.Matrix,
+                                         ms->ExtentMin,ms->ExtentMax,
+                                         tmp_min,tmp_max)) {
+        min_ext = tmp_min;
+        max_ext = tmp_max;
+      } else {
+        min_ext = ms->ExtentMin;
+        max_ext = ms->ExtentMax;
+      }
+
+      IsosurfGetRange(G,oms->Field,oms->Crystal,min_ext,max_ext,ms->Range);
+    }
     ms->ExtentFlag = true;
   }
   if(carve!=0.0) {
@@ -726,8 +807,8 @@ ObjectMesh *ObjectMeshFromBox(ObjectMesh *obj,ObjectMap *map,
   }
   I->Obj.ExtentFlag=true;
   /*  printf("Brick %d %d %d %d %d %d\n",I->Range[0],I->Range[1],I->Range[2],I->Range[3],I->Range[4],I->Range[5]);*/
-  SceneChanged();
-  SceneCountFrames();
+  SceneChanged(G);
+  SceneCountFrames(G);
   return(I);
 }
 
