@@ -70,6 +70,7 @@ typedef struct ObjectMolecule {
   int BondCounter;
   int AtomCounter;
   struct CSculpt *Sculpt;
+  short int RepVisCache[cRepCnt]; /* for transient storage during updates */
 } ObjectMolecule;
 
 typedef struct ObjectMoleculeOpRec {
@@ -83,9 +84,11 @@ typedef struct ObjectMoleculeOpRec {
   char *charVLA;
   char *s1;
   ObjectMolecule **obj1VLA;
-  AtomInfoType *ai;
+  AtomInfoType *ai,**ai1VLA;
+  PyObject *py_ob1;
   float ttt[16],*mat1;
   int nvv1,nvv2;
+  int include_static_singletons;
 } ObjectMoleculeOpRec;
 
 typedef struct {
@@ -98,9 +101,18 @@ typedef struct {
 } HBondCriteria;
 
 typedef struct {
-  int is_pqr_file;
+  int flag[3];
+  float matrix[16];
+} PDBScale;
+
+typedef struct {
+  int is_pqr_file, pqr_no_chain_id;
+  PDBScale scale;
+  int ignore_header_names;
+  int multiplex;
 } PDBInfoRec;
 
+
 /* these four letter code are left over from an 
    earlier multicharacter constant implementation
    and should be replaced with something more verbose */
@@ -159,6 +171,7 @@ typedef struct {
 #define OMOP_CheckVis 52
 #define OMOP_OnOff 53
 #define OMOP_Pop 54
+#define OMOP_TransformR44f 55
 
 #include"CoordSet.h"
 
@@ -177,7 +190,7 @@ void ObjMolPairwisePurge(ObjMolPairwise *pairwise);
 typedef struct M4XBondType { /* now used for non-bonds as well as h-bonds */
   int atom1;
   int atom2;
-  int strength;
+  float strength;
 } M4XBondType;
 
 typedef struct {
@@ -206,12 +219,21 @@ typedef struct {
   M4XAlignType *align;
 } M4XAnnoType;
 
+typedef struct {
+  char name[ObjNameMax];
+} ObjMolMultiplexType;
+
 void M4XAnnoInit(M4XAnnoType *m4x);
 void M4XAnnoPurge(M4XAnnoType *m4x);
 
 void M4XAlignInit(M4XAlignType *align);
 void M4XAlignPurge(M4XAlignType *align);
 
+int ObjectMoleculeGetMatrix(ObjectMolecule *I,int state,double **history);
+int ObjectMoleculeGetTopNeighbor(PyMOLGlobals *G,
+                                 ObjectMolecule *I, 
+                                 int start, int excluded);
+
 int *ObjectMoleculeGetPrioritizedOtherIndexList(ObjectMolecule *I,struct CoordSet *cs);
 int ObjectMoleculeGetPrioritizedOther(int *other, int a1, int a2, int *double_sided);
 
@@ -219,9 +241,10 @@ void ObjectMoleculeM4XAnnotate(ObjectMolecule *I,M4XAnnoType *m4x,char *script_f
                                int nbr_sele);
 
 /* */
+int ObjectMoleculeAreAtomsBonded2(ObjectMolecule *obj0,int a0, ObjectMolecule *obj1,int a1);
 int ObjectMoleculeIsAtomBondedToName(ObjectMolecule *obj,int a0,char *name);
 void ObjectMoleculeOpRecInit(ObjectMoleculeOpRec *op);
-int ObjectMoleculeNewFromPyList(PyObject *list,ObjectMolecule **result);
+int ObjectMoleculeNewFromPyList(PyMOLGlobals *G,PyObject *list,ObjectMolecule **result);
 PyObject *ObjectMoleculeAsPyList(ObjectMolecule *I);
 int ObjectMoleculeGetSerial(ObjectMolecule *I);
 int ObjectMoleculeSetStateTitle(ObjectMolecule *I,int state,char *text);
@@ -229,45 +252,53 @@ char *ObjectMoleculeGetStateTitle(ObjectMolecule *I,int state);
 
 void ObjectMoleculeFree(ObjectMolecule *I); /* only for friends of ObjectMolecule */
 
-ObjectMolecule *ObjectMoleculeNew(int discreteFlag);
+ObjectMolecule *ObjectMoleculeNew(PyMOLGlobals *G,int discreteFlag);
 void ObjectMoleculeSort(ObjectMolecule *I);
 ObjectMolecule *ObjectMoleculeCopy(ObjectMolecule *obj);
 void ObjectMoleculeFixChemistry(ObjectMolecule *I, int sele1, int sele2, int invalidate);
 
-ObjectMolecule *ObjectMoleculeLoadXYZFile(ObjectMolecule *obj,char *fname,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeLoadPDBFile(ObjectMolecule *obj,char *fname,int frame,int discrete,M4XAnnoType *m4x,PDBInfoRec *pdb_info);
-ObjectMolecule *ObjectMoleculeLoadPMOFile(ObjectMolecule *obj,char *fname,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeLoadMOLFile(ObjectMolecule *obj,char *fname,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeLoadMOL2File(ObjectMolecule *obj,char *fname,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeLoadMMDFile(ObjectMolecule *obj,char *fname,
+ObjectMolecule *ObjectMoleculeLoadXYZFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeLoadPDBFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete,M4XAnnoType *m4x,PDBInfoRec *pdb_info);
+ObjectMolecule *ObjectMoleculeLoadPMOFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeLoadMOLFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeLoadMMDFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,
                                           int frame,char *sepPrefix,int discrete);
-ObjectMolecule *ObjectMoleculeLoadTOPFile(ObjectMolecule *obj,char *fname,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeLoadChemPyModel(ObjectMolecule *I,PyObject *model,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeLoadTOPFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeLoadChemPyModel(PyMOLGlobals *G,ObjectMolecule *I,PyObject *model,int frame,int discrete);
 
-ObjectMolecule *ObjectMoleculeLoadTRJFile(ObjectMolecule *obj,char *fname,int frame,
+ObjectMolecule *ObjectMoleculeLoadTRJFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame,
                                           int interval,int average,int start,
                                           int stop,int max,char *sele,int image,
                                           float *shift);
 
-ObjectMolecule *ObjectMoleculeLoadRSTFile(ObjectMolecule *obj,char *fname,int frame);
+ObjectMolecule *ObjectMoleculeLoadRSTFile(PyMOLGlobals *G,ObjectMolecule *obj,char *fname,int frame);
 
-ObjectMolecule *ObjectMoleculeLoadCoords(ObjectMolecule *I,PyObject *coords,int frame);
+ObjectMolecule *ObjectMoleculeLoadCoords(PyMOLGlobals *G,ObjectMolecule *I,PyObject *coords,int frame);
 
-ObjectMolecule *ObjectMoleculeReadPMO(ObjectMolecule *obj,CRaw *pmo,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeReadPMO(PyMOLGlobals *G,ObjectMolecule *obj,CRaw *pmo,int frame,int discrete);
 
-ObjectMolecule *ObjectMoleculeReadMOLStr(ObjectMolecule *obj,char *molstr,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeReadMOL2Str(ObjectMolecule *obj,char *molstr,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeReadPDBStr(ObjectMolecule *obj,char *molstr,
+ObjectMolecule *ObjectMoleculeReadMOLStr(PyMOLGlobals *G,ObjectMolecule *obj,char *molstr,
+                                         int frame,int discrete,int finish);
+ObjectMolecule *ObjectMoleculeReadMOL2Str(PyMOLGlobals *G,ObjectMolecule *I,
+                                          char *MOLStr,int frame,int discrete,
+                                          int quiet,int multiplex, char *new_name,
+                                          char **next_entry);
+ObjectMolecule *ObjectMoleculeReadStr(PyMOLGlobals *G,ObjectMolecule *I,
+                                   char *st,int content_format, int frame,int discrete,
+                                   int quiet,int multiplex, char *new_name,
+                                   char **next_entry);
+
+ObjectMolecule *ObjectMoleculeReadPDBStr(PyMOLGlobals *G,ObjectMolecule *obj,char *molstr,
                                          int frame,int discrete,
                                          M4XAnnoType *m4x,char *pdb_name,
-                                         char **next_pdb,PDBInfoRec *pdb_info);
+                                         char **next_pdb,PDBInfoRec *pdb_info,int quiet);
 
-ObjectMolecule *ObjectMoleculeReadMMDStr(ObjectMolecule *I,char *MMDStr,int frame,int discrete);
-ObjectMolecule *ObjectMoleculeReadXYZStr(ObjectMolecule *I,char *PDBStr,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeReadMMDStr(PyMOLGlobals *G,ObjectMolecule *I,char *MMDStr,int frame,int discrete);
+ObjectMolecule *ObjectMoleculeReadXYZStr(PyMOLGlobals *G,ObjectMolecule *I,char *PDBStr,int frame,int discrete);
 
 void ObjectMoleculeExtendIndices(ObjectMolecule *I);
 
-void ObjectMoleculeInvalidate(ObjectMolecule *I,int rep,int level);
+void ObjectMoleculeInvalidate(ObjectMolecule *I,int rep,int level,int state);
 
 void ObjectMoleculeRenderSele(ObjectMolecule *I,int curState,int sele);
 
@@ -277,14 +308,15 @@ struct CoordSet *ObjectMoleculeGetCoordSet(ObjectMolecule *I,int setIndex);
 void ObjectMoleculeBlindSymMovie(ObjectMolecule *I);
 void ObjectMoleculeMerge(ObjectMolecule *I,AtomInfoType *ai,
                          struct CoordSet *cs,int bondSearchFlag,
-                         int aic_mask);
+                         int aic_mask,int invalidate);
 void ObjectMoleculeUpdateNonbonded(ObjectMolecule *I);
 void ObjectMoleculeUpdateNeighbors(ObjectMolecule *I);
 int ObjectMoleculeMoveAtom(ObjectMolecule *I,int state,int index,float *v,int mode,int log);
 int ObjectMoleculeGetAtomVertex(ObjectMolecule *I,int state,int index,float *v);
 int ObjectMoleculeGetAtomIndex(ObjectMolecule *I,int sele);
 int ObjectMoleculeTransformSelection(ObjectMolecule *I,int state,
-                                      int sele,float *TTT,int log,char *sname);
+                                      int sele,float *TTT,int log,
+                                     char *sname,int homogenous);
 int ObjectMoleculeDoesAtomNeighborSele(ObjectMolecule *I, int index, int sele);
 void ObjectMoleculeInferChemFromNeighGeom(ObjectMolecule *I,int state);
 void ObjectMoleculeInferChemForProtein(ObjectMolecule *I,int state);
@@ -294,6 +326,8 @@ void ObjectMoleculePurge(ObjectMolecule *I);
 int ObjectMoleculeAddBond(ObjectMolecule *I,int sele0,int sele1,int order);
 int ObjectMoleculeRemoveBonds(ObjectMolecule *I,int sele1,int sele2);
 
+int ObjectMoleculeAutoDisableAtomNameWildcard(ObjectMolecule *I);
+
 void ObjectMoleculeSaveUndo(ObjectMolecule *I,int state,int log);
 void ObjectMoleculeUndo(ObjectMolecule *I,int dir);
 void ObjectMoleculePrepareAtom(ObjectMolecule *I,int index,AtomInfoType *ai);
@@ -306,7 +340,7 @@ int ObjectMoleculeFillOpenValences(ObjectMolecule *I,int index);
 int ObjectMoleculeGetTotalAtomValence(ObjectMolecule *I,int atom);
 int ObjectMoleculeAdjustBonds(ObjectMolecule *I,int sele0,int sele1,int mode,int order);
 void ObjectMoleculeAttach(ObjectMolecule *I,int index,AtomInfoType *nai);
-void ObjectMoleculeFuse(ObjectMolecule *I,int index0,ObjectMolecule *src,int index1,int mode);
+void ObjectMoleculeFuse(ObjectMolecule *I,int index0,ObjectMolecule *src,int index1,int mode,int move_flag);
 void ObjectMoleculeRenameAtoms(ObjectMolecule *I,int force);
 int ObjectMoleculeAreAtomsBonded(ObjectMolecule *I,int i0,int i1);
 void ObjectGotoState(ObjectMolecule *I,int state);
@@ -339,26 +373,29 @@ int ObjectMoleculeGetCheckHBond(ObjectMolecule *don_obj,
                                 int acc_atom,
                                 int acc_state,
                                 HBondCriteria *hbc);
-void ObjectMoleculeInitHBondCriteria(HBondCriteria *hbc);
+void ObjectMoleculeInitHBondCriteria(PyMOLGlobals *G,HBondCriteria *hbc);
 int ObjectMoleculeConvertIDsToIndices(ObjectMolecule *I,int *id,int n_id);
 
 #define cObjectMoleculeDummyOrigin 1
 #define cObjectMoleculeDummyCenter 2
 
-ObjectMolecule *ObjectMoleculeDummyNew(int mode);
+ObjectMolecule *ObjectMoleculeDummyNew(PyMOLGlobals *G,int mode);
 void ObjectMoleculeDummyUpdate(ObjectMolecule *I,int mode);
 
 /* internal to ObjectMolecule */
 
-struct CoordSet *ObjectMoleculePDBStr2CoordSet(char *buffer,
+struct CoordSet *ObjectMoleculePDBStr2CoordSet(PyMOLGlobals *G,
+                                               char *buffer,
                                                AtomInfoType **atInfoPtr,
                                                char **restart_model,
                                                char *segi_override,
                                                M4XAnnoType *m4x,
                                                char *pdb_name,
                                                char **next_pdb,
-                                               PDBInfoRec *pdb_info);
+                                               PDBInfoRec *pdb_info,
+                                               int quiet);
 
+#if 0
 /* legacy binary file suppoort */
 
 typedef struct {
@@ -371,6 +408,7 @@ typedef struct {
   int order;
   int stereo;
 } BondType083;
+#endif
 
 #endif
 
