@@ -35,14 +35,16 @@ Z* -------------------------------------------------------------------
 #include"PConv.h"
 #include"P.h"
 #include"Util.h"
+#include"PyMOLGlobals.h"
+#include"Matrix.h"
 
-ObjectSurface *ObjectSurfaceNew(void);
+ObjectSurface *ObjectSurfaceNew(PyMOLGlobals *G);
 
 static void ObjectSurfaceFree(ObjectSurface *I);
-void ObjectSurfaceStateInit(ObjectSurfaceState *ms);
+void ObjectSurfaceStateInit(PyMOLGlobals *G,ObjectSurfaceState *ms);
 void ObjectSurfaceRecomputeExtent(ObjectSurface *I);
 
-
+#ifndef _PYMOL_NOPY
 static PyObject *ObjectSurfaceStateAsPyList(ObjectSurfaceState *I)
 {
   PyObject *result = NULL;
@@ -111,7 +113,7 @@ static PyObject *ObjectSurfaceAllStatesAsPyList(ObjectSurface *I)
 
 }
 
-static int ObjectSurfaceStateFromPyList(ObjectSurfaceState *I,PyObject *list)
+static int ObjectSurfaceStateFromPyList(PyMOLGlobals *G,ObjectSurfaceState *I,PyObject *list)
 {
   int ok=true;
   int ll=0;
@@ -121,7 +123,7 @@ static int ObjectSurfaceStateFromPyList(ObjectSurfaceState *I,PyObject *list)
     if(!PyList_Check(list))
       I->Active=false;
     else {
-      ObjectSurfaceStateInit(I);
+      ObjectSurfaceStateInit(G,I);
       if(ok) ok=PyList_Check(list);
       if(ok) ll=PyList_Size(list);
       if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,0),&I->Active);
@@ -165,15 +167,19 @@ static int ObjectSurfaceAllStatesFromPyList(ObjectSurface *I,PyObject *list)
   if(ok) ok=PyList_Check(list);
   if(ok) {
     for(a=0;a<I->NState;a++) {
-      ok = ObjectSurfaceStateFromPyList(I->State+a,PyList_GetItem(list,a));
+      ok = ObjectSurfaceStateFromPyList(I->Obj.G,I->State+a,PyList_GetItem(list,a));
       if(!ok) break;
     }
   }
   return(ok);
 }
+#endif
 
-int ObjectSurfaceNewFromPyList(PyObject *list,ObjectSurface **result)
+int ObjectSurfaceNewFromPyList(PyMOLGlobals *G,PyObject *list,ObjectSurface **result)
 {
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
   int ok = true;
   ObjectSurface *I=NULL;
   (*result) = NULL;
@@ -181,10 +187,10 @@ int ObjectSurfaceNewFromPyList(PyObject *list,ObjectSurface **result)
   if(ok) ok=(list!=NULL);
   if(ok) ok=PyList_Check(list);
 
-  I=ObjectSurfaceNew();
+  I=ObjectSurfaceNew(G);
   if(ok) ok = (I!=NULL);
 
-  if(ok) ok = ObjectFromPyList(PyList_GetItem(list,0),&I->Obj);
+  if(ok) ok = ObjectFromPyList(G,PyList_GetItem(list,0),&I->Obj);
   if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,1),&I->NState);
   if(ok) ok = ObjectSurfaceAllStatesFromPyList(I,PyList_GetItem(list,2));
   if(ok) {
@@ -194,10 +200,15 @@ int ObjectSurfaceNewFromPyList(PyObject *list,ObjectSurface **result)
     /* cleanup? */
   }
   return(ok);
+#endif
 }
 
+
 PyObject *ObjectSurfaceAsPyList(ObjectSurface *I)
 {
+#ifdef _PYMOL_NOPY
+  return 0;
+#else
   
   PyObject *result=NULL;
 
@@ -207,15 +218,19 @@ PyObject *ObjectSurfaceAsPyList(ObjectSurface *I)
   PyList_SetItem(result,2,ObjectSurfaceAllStatesAsPyList(I));
 
   return(PConvAutoNone(result));  
+#endif
 }
 
 static void ObjectSurfaceStateFree(ObjectSurfaceState *ms)
 {
-  if(PMGUI) {
+  ObjectStatePurge(&ms->State);
+  if(ms->State.G->HaveGUI) {
     if(ms->displayList) {
       if(PIsGlutThread()) {
-        glDeleteLists(ms->displayList,1);
-        ms->displayList = 0;
+        if(ms->State.G->ValidContext) {
+          glDeleteLists(ms->displayList,1);
+          ms->displayList = 0;
+        }
       } else {
         char buffer[255]; /* pass this off to the main thread */
         sprintf(buffer,"_cmd.gl_delete_lists(%d,%d)\n",ms->displayList,1);
@@ -252,7 +267,7 @@ void ObjectSurfaceDump(ObjectSurface *I,char *fname,int state)
   FILE *f;
   f=fopen(fname,"wb");
   if(!f) 
-    ErrMessage("ObjectSurfaceDump","can't open file for writing");
+    ErrMessage(I->Obj.G,"ObjectSurfaceDump","can't open file for writing");
   else {
     if(state<I->NState) {
       n=I->State[state].N;
@@ -279,9 +294,9 @@ void ObjectSurfaceDump(ObjectSurface *I,char *fname,int state)
           }
     }
     fclose(f);
-    PRINTFB(FB_ObjectSurface,FB_Actions)
+    PRINTFB(I->Obj.G,FB_ObjectSurface,FB_Actions)
       " ObjectSurfaceDump: %s written to %s\n",I->Obj.Name,fname
-      ENDFB;
+      ENDFB(I->Obj.G);
   }
 }
 
@@ -295,9 +310,9 @@ static void ObjectSurfaceInvalidate(ObjectSurface *I,int rep,int level,int state
     I->State[state].RefreshFlag=true;
     if(level>=cRepInvAll) {
       I->State[state].ResurfaceFlag=true;     
-      SceneChanged();
+      SceneChanged(I->Obj.G);
     } else {
-      SceneDirty();
+      SceneDirty(I->Obj.G);
     }
     if(once_flag) break;
   }
@@ -315,12 +330,12 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
   for(a=0;a<I->NState;a++) {
     ms = I->State+a;
     if(ms->Active) {
-      map = ExecutiveFindObjectMapByName(ms->MapName);
+      map = ExecutiveFindObjectMapByName(I->Obj.G,ms->MapName);
       if(!map) {
         ok=false;
-        PRINTFB(FB_ObjectSurface,FB_Errors)
+        PRINTFB(I->Obj.G,FB_ObjectSurface,FB_Errors)
           "ObjectSurfaceUpdate-Error: map '%s' has been deleted.\n",ms->MapName
-          ENDFB;
+          ENDFB(I->Obj.G);
         ms->ResurfaceFlag=false;
       }
       if(map) {
@@ -335,6 +350,13 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
               CGOFree(ms->UnitCellCGO);
             ms->UnitCellCGO = CrystalGetUnitCellCGO(&ms->Crystal);
           } 
+          
+          if(oms->State.Matrix) {
+            ObjectStateSetMatrix(&ms->State,oms->State.Matrix);
+          } else if(ms->State.Matrix) {
+            ObjectStateResetMatrix(&ms->State);
+          }
+          
           ms->RefreshFlag=false;
         }
       }
@@ -343,10 +365,27 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
 
 
           ms->ResurfaceFlag=false;
-          PRINTF " ObjectSurface: updating \"%s\".\n" , I->Obj.Name ENDF;
+          PRINTFB(I->Obj.G,FB_ObjectSurface,FB_Actions)
+           " ObjectSurface: updating \"%s\".\n" , I->Obj.Name 
+            ENDFB(I->Obj.G);
           if(oms->Field) {
-            TetsurfGetRange(oms->Field,oms->Crystal,
-                            ms->ExtentMin,ms->ExtentMax,ms->Range);
+
+            {
+              float *min_ext,*max_ext;
+              float tmp_min[3],tmp_max[3];
+              if(MatrixInvTransformExtentsR44d3f(ms->State.Matrix,
+                                                 ms->ExtentMin,ms->ExtentMax,
+                                                 tmp_min,tmp_max)) {
+                min_ext = tmp_min;
+                max_ext = tmp_max;
+              } else {
+                min_ext = ms->ExtentMin;
+                max_ext = ms->ExtentMax;
+              }
+              
+              TetsurfGetRange(oms->Field,oms->Crystal,
+                              min_ext,max_ext,ms->Range);
+            }
 
             if(ms->CarveFlag&&ms->AtomVertex) {
               carve_buffer = ms->CarveBuffer;
@@ -354,29 +393,81 @@ static void ObjectSurfaceUpdate(ObjectSurface *I)
                 carve_buffer = -carve_buffer;
               }
 
-              voxelmap=MapNew(-carve_buffer,ms->AtomVertex,
+              voxelmap=MapNew(I->Obj.G,-carve_buffer,ms->AtomVertex,
                               VLAGetSize(ms->AtomVertex)/3,NULL);
               if(voxelmap)
                 MapSetupExpress(voxelmap);  
             }
 
-            ms->nT=TetsurfVolume(oms->Field,
-                          ms->Level,
-                          &ms->N,&ms->V,
-                          ms->Range,
-                          ms->Mode,
-                          voxelmap,
-                          ms->AtomVertex,
-                          ms->CarveBuffer,
-                          ms->Side);
+            ms->nT=TetsurfVolume(I->Obj.G,oms->Field,
+                                 ms->Level,
+                                 &ms->N,&ms->V,
+                                 ms->Range,
+                                 ms->Mode,
+                                 voxelmap,
+                                 ms->AtomVertex,
+                                 ms->CarveBuffer,
+                                 ms->Side);
+            
             if(voxelmap)
               MapFree(voxelmap);
+
+            if(ms->State.Matrix) { /* in we're in a different reference frame...*/
+              double *matrix = ms->State.Matrix;
+              float *v;
+              int *n,c;
+              
+              v=ms->V;
+              n=ms->N;
+
+              if(n&&v) {
+
+                while(*n) {
+                  c=*(n++);
+                  switch(ms->Mode) {
+                  case 3:
+                  case 2:
+                    transform44d3fas33d3f(matrix,v,v);
+                    transform44d3f(matrix,v+3,v+3);
+                    transform44d3fas33d3f(matrix,v+6,v+6);
+                    transform44d3f(matrix,v+9,v+9);
+                    v+=12;
+                    c-=4;
+                    while(c>0) {
+                      transform44d3fas33d3f(matrix,v,v);
+                      transform44d3f(matrix,v+3,v+3);
+                      v+=6;
+                      c-=2;
+                    }
+                    break;
+                  case 1:
+                    transform44d3f(matrix,v,v);
+                    c--;
+                    v+=3;
+                    while(c>0) {
+                      transform44d3f(matrix,v,v);
+                      v+=3;
+                      c--;
+                    }
+                    break;
+                  case 0:
+                  default:
+                    while(c>0) {
+                      transform44d3f(matrix,v,v);
+                      v+=3;
+                      c--;
+                    }
+                    break;
+                  }
+                }
+              }
+            }
           }
         }
       }
     }
   }
-  SceneDirty();
+  SceneDirty(I->Obj.G);
 }
 
 static int ZOrderFn(float *array,int l,int r)
@@ -391,6 +482,7 @@ static int ZRevOrderFn(float *array,int l,int r)
 
 static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **pick,int pass)
 {
+  PyMOLGlobals *G = I->Obj.G;
   float *v = NULL;
   float *vc;
   float *col;
@@ -402,7 +494,7 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
 
   ObjectPrepareContext(&I->Obj,ray);
 
-  alpha = SettingGet_f(NULL,I->Obj.Setting,cSetting_transparency);
+  alpha = SettingGet_f(G,NULL,I->Obj.Setting,cSetting_transparency);
   alpha=1.0F-alpha;
   if(fabs(alpha-1.0)<R_SMALL4)
     alpha=1.0F;
@@ -418,7 +510,7 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
     } else {
       if(!ms) {
         if(I->NState&&
-           ((SettingGet(cSetting_static_singletons)&&(I->NState==1))))
+           ((SettingGet(G,cSetting_static_singletons)&&(I->NState==1))))
           ms=I->State;
       }
     }
@@ -428,14 +520,14 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
         n=ms->N;
         if(ray) {
           if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
-            CGORenderRay(ms->UnitCellCGO,ray,ColorGet(I->Obj.Color),
+            CGORenderRay(ms->UnitCellCGO,ray,ColorGet(G,I->Obj.Color),
                          I->Obj.Setting,NULL);
 
 
           ray->fTransparentf(ray,1.0F-alpha);       
-          ms->Radius=SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_radius);
+          ms->Radius=SettingGet_f(G,I->Obj.Setting,NULL,cSetting_mesh_radius);
           if(n&&v&&I->Obj.RepVis[cRepSurface]) {
-            vc = ColorGet(I->Obj.Color);
+            vc = ColorGet(G,I->Obj.Color);
             while(*n)
               {
                 c=*(n++);
@@ -475,195 +567,197 @@ static void ObjectSurfaceRender(ObjectSurface *I,int state,CRay *ray,Pickable **
             
           }
           ray->fTransparentf(ray,0.0);
-        } else if(pick&&PMGUI) {
-        } else if(PMGUI) {
-
-          int render_now = false;
-          if(alpha>0.0001) {
-            render_now = (pass==-1);
-          } else 
-            render_now = (!pass);
-          if(render_now) {
-            int use_dlst;
-
-            if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
-              CGORenderGL(ms->UnitCellCGO,ColorGet(I->Obj.Color),
-                          I->Obj.Setting,NULL);
-
-            SceneResetNormal(false);
-            col = ColorGet(I->Obj.Color);
-            glColor4f(col[0],col[1],col[2],alpha);
-
-            use_dlst = (int)SettingGet(cSetting_use_display_lists);
-            if(use_dlst&&ms->displayList) {
-              glCallList(ms->displayList);
-            } else { 
+        } else if(G->HaveGUI && G->ValidContext) {
+          if(pick) {
+          } else {
+            
+            int render_now = false;
+            if(alpha>0.0001) {
+              render_now = (pass==-1);
+            } else 
+              render_now = (!pass);
+            if(render_now) {
+              int use_dlst;
               
-              if(use_dlst) {
-                if(!ms->displayList) {
-                  ms->displayList = glGenLists(1);
-                  if(ms->displayList) {
-                    glNewList(ms->displayList,GL_COMPILE_AND_EXECUTE);
+              if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
+                CGORenderGL(ms->UnitCellCGO,ColorGet(G,I->Obj.Color),
+                            I->Obj.Setting,NULL);
+              
+              SceneResetNormal(G,false);
+              col = ColorGet(G,I->Obj.Color);
+              glColor4f(col[0],col[1],col[2],alpha);
+              
+              use_dlst = (int)SettingGet(G,cSetting_use_display_lists);
+              if(use_dlst&&ms->displayList) {
+                glCallList(ms->displayList);
+              } else { 
+                
+                if(use_dlst) {
+                  if(!ms->displayList) {
+                    ms->displayList = glGenLists(1);
+                    if(ms->displayList) {
+                      glNewList(ms->displayList,GL_COMPILE_AND_EXECUTE);
+                    }
                   }
                 }
-              }
-
-            
-              if(n&&v&&I->Obj.RepVis[cRepSurface]) {
-
-                if((ms->Mode>1)&&(alpha!=1.0)) { /* transparent */
                 
-                  int t_mode;
-
-                  t_mode  = SettingGet_i(NULL,I->Obj.Setting,cSetting_transparency_mode);
                 
-                  if(t_mode) { /* high quality (sorted) transparency? */
+                if(n&&v&&I->Obj.RepVis[cRepSurface]) {
                   
-                    float **t_buf=NULL,**tb;
-                    float *z_value=NULL,*zv;
-                    int *ix=NULL;
-                    int n_tri = 0;
-                    float sum[3];
-                    float matrix[16];
-                    int parity;
-                    glGetFloatv(GL_MODELVIEW_MATRIX,matrix);
-                  
-                    t_buf = Alloc(float*,ms->nT*9);
-                  
-                    z_value = Alloc(float,ms->nT);
-                    ix = Alloc(int,ms->nT);
-                  
-                    zv = z_value;
-                    tb = t_buf;
-
-                    while(*n)
-                      {
-                        parity=true;
-                        c=*(n++);
-                        v+=12;
-                        c-=4;
-                        while(c>0) {
-
-                          if(parity) {
-                            *(tb++) = v-12;
-                            *(tb++) = v-9;
-                            *(tb++) = v-6;
-                            *(tb++) = v-3;
-                            *(tb++) = v;
-                            *(tb++) = v+3;
-                          } else {
-                            *(tb++) = v-12;
-                            *(tb++) = v-9;
-                            *(tb++) = v;
-                            *(tb++) = v+3;
-                            *(tb++) = v-6;
-                            *(tb++) = v-3;
+                  if((ms->Mode>1)&&(alpha!=1.0)) { /* transparent */
+                    
+                    int t_mode;
+                    
+                    t_mode  = SettingGet_i(G,NULL,I->Obj.Setting,cSetting_transparency_mode);
+                    
+                    if(t_mode) { /* high quality (sorted) transparency? */
+                      
+                      float **t_buf=NULL,**tb;
+                      float *z_value=NULL,*zv;
+                      int *ix=NULL;
+                      int n_tri = 0;
+                      float sum[3];
+                      float matrix[16];
+                      int parity;
+                      glGetFloatv(GL_MODELVIEW_MATRIX,matrix);
+                      
+                      t_buf = Alloc(float*,ms->nT*9);
+                      
+                      z_value = Alloc(float,ms->nT);
+                      ix = Alloc(int,ms->nT);
+                      
+                      zv = z_value;
+                      tb = t_buf;
+                      
+                      while(*n)
+                        {
+                          parity=true;
+                          c=*(n++);
+                          v+=12;
+                          c-=4;
+                          while(c>0) {
+                            
+                            if(parity) {
+                              *(tb++) = v-12;
+                              *(tb++) = v-9;
+                              *(tb++) = v-6;
+                              *(tb++) = v-3;
+                              *(tb++) = v;
+                              *(tb++) = v+3;
+                            } else {
+                              *(tb++) = v-12;
+                              *(tb++) = v-9;
+                              *(tb++) = v;
+                              *(tb++) = v+3;
+                              *(tb++) = v-6;
+                              *(tb++) = v-3;
+                            }
+                            
+                            parity=!parity;
+                            
+                            add3f(tb[-1],tb[-3],sum);
+                            add3f(sum,tb[-5],sum);
+                            
+                            *(zv++) = matrix[2]*sum[0]+matrix[6]*sum[1]+matrix[10]*sum[2];
+                            n_tri++;
+                            
+                            v+=6;
+                            c-=2;
                           }
-                        
-                          parity=!parity;
-
-                          add3f(tb[-1],tb[-3],sum);
-                          add3f(sum,tb[-5],sum);
-                        
-                          *(zv++) = matrix[2]*sum[0]+matrix[6]*sum[1]+matrix[10]*sum[2];
-                          n_tri++;
-                        
-                          v+=6;
-                          c-=2;
                         }
+                      switch(t_mode) {
+                      case 1:
+                        UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZOrderFn);
+                        break;
+                      default:
+                        UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZRevOrderFn);
+                        break;
                       }
-                    switch(t_mode) {
-                    case 1:
-                      UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZOrderFn);
-                      break;
-                    default:
-                      UtilSortIndex(n_tri,z_value,ix,(UtilOrderFn*)ZRevOrderFn);
-                      break;
-                    }
-                    
-                    c=n_tri;
-                    
-                    col=ColorGet(I->Obj.Color);
-                    
-                    glColor4f(col[0],col[1],col[2],alpha);
-                    glBegin(GL_TRIANGLES);
-                    for(c=0;c<n_tri;c++) {
                       
-                      tb = t_buf+6*ix[c];
+                      c=n_tri;
                       
-                      glNormal3fv(*(tb++));
-                      glVertex3fv(*(tb++));
-                      glNormal3fv(*(tb++));
-                      glVertex3fv(*(tb++));
-                      glNormal3fv(*(tb++));
-                      glVertex3fv(*(tb++));
+                      col=ColorGet(G,I->Obj.Color);
+                      
+                      glColor4f(col[0],col[1],col[2],alpha);
+                      glBegin(GL_TRIANGLES);
+                      for(c=0;c<n_tri;c++) {
+                        
+                        tb = t_buf+6*ix[c];
+                        
+                        glNormal3fv(*(tb++));
+                        glVertex3fv(*(tb++));
+                        glNormal3fv(*(tb++));
+                        glVertex3fv(*(tb++));
+                        glNormal3fv(*(tb++));
+                        glVertex3fv(*(tb++));
+                      }
+                      glEnd();
+                      
+                      FreeP(ix);
+                      FreeP(z_value);
+                      FreeP(t_buf);
+                    } else { 
+                      while(*n)
+                        {
+                          c=*(n++);
+                          
+                          glBegin(GL_TRIANGLE_STRIP);
+                          while(c>0) {
+                            glNormal3fv(v);
+                            v+=3;
+                            glVertex3fv(v);
+                            v+=3;
+                            c-=2;
+                          }
+                          glEnd();
+                        }
                     }
-                    glEnd();
-                    
-                    FreeP(ix);
-                    FreeP(z_value);
-                    FreeP(t_buf);
-                  } else { 
+                  } else {
+                    glLineWidth(SettingGet_f(G,I->Obj.Setting,NULL,cSetting_mesh_width));
                     while(*n)
                       {
                         c=*(n++);
-                    
-                        glBegin(GL_TRIANGLE_STRIP);
-                        while(c>0) {
-                          glNormal3fv(v);
-                          v+=3;
-                          glVertex3fv(v);
-                          v+=3;
-                          c-=2;
+                        switch(ms->Mode) {
+                        case 3:
+                        case 2:
+                          glBegin(GL_TRIANGLE_STRIP);
+                          while(c>0) {
+                            glNormal3fv(v);
+                            v+=3;
+                            glVertex3fv(v);
+                            v+=3;
+                            c-=2;
+                          }
+                          glEnd();
+                          break;
+                        case 1:
+                          glBegin(GL_LINES);
+                          while(c>0) {
+                            glVertex3fv(v);
+                            v+=3;
+                            c--;
+                          }
+                          glEnd();
+                          break;
+                        case 0:
+                        default:
+                          glBegin(GL_POINTS);
+                          while(c>0) {
+                            glVertex3fv(v);
+                            v+=3;
+                            c--;
+                          }
+                          glEnd();
+                          break;
                         }
-                        glEnd();
                       }
                   }
-                } else {
-                  glLineWidth(SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_width));
-                  while(*n)
-                    {
-                      c=*(n++);
-                      switch(ms->Mode) {
-                      case 3:
-                      case 2:
-                        glBegin(GL_TRIANGLE_STRIP);
-                        while(c>0) {
-                          glNormal3fv(v);
-                          v+=3;
-                          glVertex3fv(v);
-                          v+=3;
-                          c-=2;
-                        }
-                        glEnd();
-                        break;
-                      case 1:
-                        glBegin(GL_LINES);
-                        while(c>0) {
-                          glVertex3fv(v);
-                          v+=3;
-                          c--;
-                        }
-                        glEnd();
-                        break;
-                      case 0:
-                      default:
-                        glBegin(GL_POINTS);
-                        while(c>0) {
-                          glVertex3fv(v);
-                          v+=3;
-                          c--;
-                        }
-                        glEnd();
-                        break;
-                      }
-                    }
                 }
               }
-            }
             
-            if(use_dlst&&ms->displayList) {
-              glEndList();
+              if(use_dlst&&ms->displayList) {
+                glEndList();
+              }
             }
           }
         }
@@ -683,11 +777,11 @@ static int ObjectSurfaceGetNStates(ObjectSurface *I)
 }
 
 /*========================================================================*/
-ObjectSurface *ObjectSurfaceNew(void)
+ObjectSurface *ObjectSurfaceNew(PyMOLGlobals *G)
 {
-  OOAlloc(ObjectSurface);
+  OOAlloc(G,ObjectSurface);
   
-  ObjectInit((CObject*)I);
+  ObjectInit(G,(CObject*)I);
   
   I->NState = 0;
   I->State=VLAMalloc(10,sizeof(ObjectSurfaceState),5,true); /* autozero important */
@@ -703,8 +797,11 @@ ObjectSurface *ObjectSurfaceNew(void)
 }
 
 /*========================================================================*/
-void ObjectSurfaceStateInit(ObjectSurfaceState *ms)
+void ObjectSurfaceStateInit(PyMOLGlobals *G,ObjectSurfaceState *ms)
 {
+  if(ms->Active)
+    ObjectStatePurge(&ms->State);
+  ObjectStateInit(G,&ms->State);
   if(!ms->V) {
     ms->V = VLAlloc(float,10000);
   }
@@ -728,7 +825,7 @@ void ObjectSurfaceStateInit(ObjectSurfaceState *ms)
 }
 
 /*========================================================================*/
-ObjectSurface *ObjectSurfaceFromBox(ObjectSurface *obj,ObjectMap *map,
+ObjectSurface *ObjectSurfaceFromBox(PyMOLGlobals *G,ObjectSurface *obj,ObjectMap *map,
                                     int map_state,
 int state,float *mn,float *mx,float level,int mode,
 float carve,float *vert_vla,int side)
@@ -738,7 +835,7 @@ float carve,float *vert_vla,int side)
   ObjectMapState *oms;
 
   if(!obj) {
-    I=ObjectSurfaceNew();
+    I=ObjectSurfaceNew(G);
   } else {
     I=obj;
   }
@@ -750,7 +847,7 @@ float carve,float *vert_vla,int side)
   }
 
   ms=I->State+state;
-  ObjectSurfaceStateInit(ms);
+  ObjectSurfaceStateInit(G,ms);
 
   strcpy(ms->MapName,map->Obj.Name);
   ms->MapState = map_state;
@@ -760,23 +857,58 @@ float carve,float *vert_vla,int side)
   ms->Mode = mode;
   ms->Side = side;
   if(oms) {
-    TetsurfGetRange(oms->Field,oms->Crystal,mn,mx,ms->Range);
+
+    if(oms->State.Matrix) {
+      ObjectStateSetMatrix(&ms->State,oms->State.Matrix);
+    } else if(ms->State.Matrix) {
+      ObjectStateResetMatrix(&ms->State);
+    }
+
     copy3f(mn,ms->ExtentMin); /* this is not exactly correct...should actually take vertex points from range */
     copy3f(mx,ms->ExtentMax);
+    
+    {
+      float *min_ext,*max_ext;
+      float tmp_min[3],tmp_max[3];
+      if(MatrixInvTransformExtentsR44d3f(ms->State.Matrix,
+                                         ms->ExtentMin,ms->ExtentMax,
+                                         tmp_min,tmp_max)) {
+        min_ext = tmp_min;
+        max_ext = tmp_max;
+      } else {
+        min_ext = ms->ExtentMin;
+        max_ext = ms->ExtentMax;
+      }
+      
+      TetsurfGetRange(oms->Field,oms->Crystal,min_ext,max_ext,ms->Range);
+    }
     ms->ExtentFlag = true;
   }
   if(carve!=0.0) {
     ms->CarveFlag=true;
     ms->CarveBuffer = carve;
     ms->AtomVertex = vert_vla;
+
+    if(ms->State.Matrix) {
+      int n=VLAGetSize(ms->AtomVertex)/3;
+      float *v = ms->AtomVertex;
+      double *matrix = ms->State.Matrix;
+      while(n--) {
+        /* convert back into original map coordinates 
+           for surface carving operation */
+        inverse_transform44d3f(matrix, v,v);
+        v+=3;
+      }
+    }
+
   }
   if(I) {
     ObjectSurfaceRecomputeExtent(I);
   }
   I->Obj.ExtentFlag=true;
   /*  printf("Brick %d %d %d %d %d %d\n",I->Range[0],I->Range[1],I->Range[2],I->Range[3],I->Range[4],I->Range[5]);*/
-  SceneChanged();
-  SceneCountFrames();
+  SceneChanged(G);
+  SceneCountFrames(G);
   return(I);
 }
 
