@@ -8,7 +8,7 @@ F* -------------------------------------------------------------------
 G* Please see the accompanying LICENSE file for further information. 
 H* -------------------------------------------------------------------
 I* Additional authors of this source file include:
--* 
+-* Cameron Mura
 -* 
 -*
 Z* -------------------------------------------------------------------
@@ -47,14 +47,15 @@ void RepCartoonFree(RepCartoon *I)
     CGOFree(I->ray);
   if(I->std)
     CGOFree(I->std);
-  RepFree(&I->R);
+  RepPurge(&I->R);
   OOFreeP(I);
 }
 
 void RepCartoonRender(RepCartoon *I,CRay *ray,Pickable **pick)
 {
+  register PyMOLGlobals *G=I->R.G;
   if(ray) {
-    PRINTFD(FB_RepCartoon)
+    PRINTFD(G,FB_RepCartoon)
       " RepCartoonRender: rendering raytracable...\n"
       ENDFD;
     
@@ -65,38 +66,39 @@ void RepCartoonRender(RepCartoon *I,CRay *ray,Pickable **pick)
     else if(I->std)
       CGORenderRay(I->std,ray,NULL,I->R.cs->Setting,
                    I->R.obj->Setting);    
-  } else if(pick&&PMGUI) {
-    if(I->std) {
-      CGORenderGLPickable(I->std,pick,I->R.obj,
-                          I->R.cs->Setting,I->R.obj->Setting);
-    }
-  } else if(PMGUI) {
-    
-    int use_dlst;
-    use_dlst = (int)SettingGet(cSetting_use_display_lists);
-    if(use_dlst&&I->R.displayList) {
-      glCallList(I->R.displayList);
-    } else { 
-
-      if(use_dlst) {
-        if(!I->R.displayList) {
-          I->R.displayList = glGenLists(1);
-          if(I->R.displayList) {
-            glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
+  } else if(G->HaveGUI && G->ValidContext) {
+    if(pick) {
+      if(I->std) {
+        CGORenderGLPickable(I->std,pick,I->R.obj,
+                            I->R.cs->Setting,I->R.obj->Setting);
+      }
+    } else {
+      int use_dlst;
+      use_dlst = (int)SettingGet(G,cSetting_use_display_lists);
+      if(use_dlst&&I->R.displayList) {
+        glCallList(I->R.displayList);
+      } else { 
+        
+        if(use_dlst) {
+          if(!I->R.displayList) {
+            I->R.displayList = glGenLists(1);
+            if(I->R.displayList) {
+              glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
+            }
           }
         }
-      }
-
-    PRINTFD(FB_RepCartoon)
-      " RepCartoonRender: rendering GL...\n"
-      ENDFD;
-
-    if(I->std) 
-      CGORenderGL(I->std,NULL,I->R.cs->Setting,
-                   I->R.obj->Setting);
-
-      if(use_dlst&&I->R.displayList) {
-        glEndList();
+        
+        PRINTFD(G,FB_RepCartoon)
+          " RepCartoonRender: rendering GL...\n"
+          ENDFD;
+        
+        if(I->std) 
+          CGORenderGL(I->std,NULL,I->R.cs->Setting,
+                      I->R.obj->Setting);
+        
+        if(use_dlst&&I->R.displayList) {
+          glEndList();
+        }
       }
     }
   }
@@ -116,6 +118,7 @@ static float smooth(float x,float power)
 
 Rep *RepCartoonNew(CoordSet *cs)
 {
+  PyMOLGlobals *G=cs->State.G;
   ObjectMolecule *obj;
   int a,b,c,f,e,a1,a2,c1,c2,i0,*i,*s,*at,*seg,nAt,*atp,a3,a4=0,*car,*cc,*sstype;
   float *v,*v0,*v1,*v2,*v3,*v4,*v5,*vo,*vn,*va;
@@ -139,10 +142,12 @@ Rep *RepCartoonNew(CoordSet *cs)
   float power_b = 5;
   float loop_radius;
   float tube_radius;
+  float putty_radius;
+
   int visFlag;
   CExtrude *ex=NULL,*ex1;
   int n_p,n_pm1,n_pm2;
-  int loop_quality,oval_quality,tube_quality;
+  int loop_quality,oval_quality,tube_quality, putty_quality;
   float oval_width,oval_length;
   float dumbbell_radius,dumbbell_width,dumbbell_length;
   float throw;
@@ -162,6 +167,7 @@ Rep *RepCartoonNew(CoordSet *cs)
   int cylindrical_helices;
   int last_color,uniform_color;
   int cartoon_color,highlight_color;
+  int cartoon_side_chain_helper;
   int round_helices;
   int smooth_loops;
   int na_mode;
@@ -176,86 +182,94 @@ Rep *RepCartoonNew(CoordSet *cs)
   int skip_to;
   AtomInfoType *ai,*last_ai=NULL;
   float alpha;
-
+  int putty_flag = false;
+  float putty_mean=10.0F,putty_stdev=0.0F;
   /* THIS HAS GOT TO BE A CANDIDATE FOR THE WORST ROUTINE IN PYMOL!
    * DEVELOP ON IT ONLY AT EXTREME RISK TO YOUR MENTAL HEALTH */
-
-  OOAlloc(RepCartoon);
-
-PRINTFD(FB_RepCartoon)
-" RepCartoonNew-Debug: entered.\n"
-ENDFD;
+  
+  OOAlloc(G,RepCartoon);
+  
+  PRINTFD(G,FB_RepCartoon)
+    " RepCartoonNew-Debug: entered.\n"
+    ENDFD;
 	
 
   obj = cs->Obj;
   visFlag=false;
   for(a=0;a<cs->NIndex;a++) {
-	 if(obj->AtomInfo[cs->IdxToAtm[a]].visRep[cRepCartoon])
-		{
-		  visFlag=true;
-		  break;
-		}
+    if(obj->AtomInfo[cs->IdxToAtm[a]].visRep[cRepCartoon])
+      {
+	visFlag=true;
+	break;
+      }
   }
   if(!visFlag) {
     OOFreeP(I);
-    return(NULL); /* skip if no dots are visible */
+    return(NULL); /* skip if not visible */
   }
 
-  RepInit(&I->R);
-  power_a=SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_power);
-  power_b=SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_power_b);
+  RepInit(G,&I->R);
+  power_a=SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_power);
+  power_b=SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_power_b);
 
-  cartoon_debug=SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_debug);
-  length=SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_rect_length);
-  width=SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_rect_width);
-  trace=SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_trace);
+  cartoon_debug=SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_debug);
+  length=SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_rect_length);
+  width=SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_rect_width);
+  trace=SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_trace_atoms);
 
-  alpha=1.0F - SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_transparency);
-  throw=SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_throw);
+  alpha=1.0F - SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_transparency);
+  throw=SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_throw);
 
-  sampling = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_sampling);
+  sampling = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_sampling);
   if(sampling<1) sampling=1;
-  loop_radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_loop_radius);
+  loop_radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_loop_radius);
   if(loop_radius<0.01F) loop_radius=0.01F;
-  loop_quality = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_loop_quality);
+  loop_quality = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_loop_quality);
   if(loop_quality<3) loop_quality=3;
 
-  tube_radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_tube_radius);
+  tube_radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_tube_radius);
   if(tube_radius<0.01F) tube_radius=0.01F;
-  tube_quality = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_tube_quality);
+  tube_quality = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_tube_quality);
   if(tube_quality<3) tube_quality=3;
 
-  cartoon_color = SettingGet_color(cs->Setting,obj->Obj.Setting,cSetting_cartoon_color);
+  putty_radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_putty_radius);
+  if(putty_radius<0.01F) putty_radius=0.01F;
+  putty_quality = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_putty_quality);
+  if(putty_quality<3) putty_quality=3;
 
-  highlight_color = SettingGet_color(cs->Setting,obj->Obj.Setting,cSetting_cartoon_highlight_color);
+  cartoon_color = SettingGet_color(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_color);
+  cartoon_side_chain_helper = SettingGet_b(G,cs->Setting, obj->Obj.Setting,
+					   cSetting_cartoon_side_chain_helper);
 
-  oval_length = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_oval_length);
-  oval_width = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_oval_width);
-  oval_quality = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_oval_quality);
+  highlight_color = SettingGet_color(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_highlight_color);
+
+  oval_length = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_oval_length);
+  oval_width = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_oval_width);
+  oval_quality = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_oval_quality);
   if(oval_quality<3) tube_quality=3;
 
-  dumbbell_length = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_dumbbell_length);
-  dumbbell_width = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_dumbbell_width);
-  dumbbell_radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_dumbbell_radius);
+  dumbbell_length = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_dumbbell_length);
+  dumbbell_width = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_dumbbell_width);
+  dumbbell_radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_dumbbell_radius);
   if(dumbbell_radius<0.01F) dumbbell_radius=0.01F;
 
-  fancy_helices = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_fancy_helices);
-  fancy_sheets = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_fancy_sheets);
-  cylindrical_helices = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_cylindrical_helices);
-  refine = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_refine);
-  refine_tips = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_refine_tips);
+  fancy_helices = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_fancy_helices);
+  fancy_sheets = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_fancy_sheets);
+  cylindrical_helices = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_cylindrical_helices);
+  refine = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_refine);
+  refine_tips = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_refine_tips);
 
-  discrete_colors = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_discrete_colors);
-  round_helices = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_round_helices);
-  smooth_loops = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_loops);
-  helix_radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_helix_radius);
+  discrete_colors = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_discrete_colors);
+  round_helices = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_round_helices);
+  smooth_loops = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_loops);
+  helix_radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_helix_radius);
 
-  smooth_first = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_first);
-  smooth_last = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_last);
-  smooth_cycles = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_cycles);
-  flat_cycles = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_cycles);
+  smooth_first = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_first);
+  smooth_last = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_last);
+  smooth_cycles = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_smooth_cycles);
+  flat_cycles = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_cycles);
 
-  na_mode = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cartoon_nucleic_acid_mode);
+  na_mode = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_nucleic_acid_mode);
 
   I->R.fRender=(void (*)(struct Rep *, CRay *, Pickable **))RepCartoonRender;
   I->R.fFree=(void (*)(struct Rep *))RepCartoonFree;
@@ -291,7 +305,7 @@ ENDFD;
   parity = 1;
 
   for(a1=0;a1<cs->NAtIndex;a1++)
-	 {
+    {
       if(obj->DiscreteFlag) {
         if(cs==obj->DiscreteCSet[a1]) 
           a=obj->DiscreteAtmToIdx[a1];
@@ -299,32 +313,33 @@ ENDFD;
           a=-1;
       } else 
         a=cs->AtmToIdx[a1];
-		if(a>=0) {
+      if(a>=0) {
         ai = obj->AtomInfo+a1;
-		  if(ai->visRep[cRepCartoon]) {
+
+	if(ai->visRep[cRepCartoon]) {
           /*			 if(!obj->AtomInfo[a1].hetatm)*/
           if((!ai->alt[0])||
              (ai->alt[0]=='A')) {
             if(trace||(((ai->protons==cAN_C)&&
-               (WordMatch("CA",ai->name,1)<0))&&
-                       !AtomInfoSameResidueP(last_ai,ai)))
+			(WordMatch(G,"CA",ai->name,1)<0))&&
+                       !AtomInfoSameResidueP(G,last_ai,ai)))
               {
-                PRINTFD(FB_RepCartoon)
+                PRINTFD(G,FB_RepCartoon)
                   " RepCartoon: found CA in %s; a2 %d\n",ai->resi,a2
                   ENDFD;
                 if(!trace) 
                   if(a2>=0) {
-                  /*
-                    if((abs(obj->AtomInfo[a1].resv-obj->AtomInfo[a2].resv)>1)||
-                    (obj->AtomInfo[a1].chain[0]!=obj->AtomInfo[a2].chain[0])||
-                    (!WordMatch(obj->AtomInfo[a1].segi,obj->AtomInfo[a2].segi,1)))*/
-                  if(!ObjectMoleculeCheckBondSep(obj,a1,a2,3)) /* CA->N->C->CA = 3 bonds */
-                    a2=-1;
+		    /*
+		      if((abs(obj->AtomInfo[a1].resv-obj->AtomInfo[a2].resv)>1)||
+		      (obj->AtomInfo[a1].chain[0]!=obj->AtomInfo[a2].chain[0])||
+		      (!WordMatch(G,obj->AtomInfo[a1].segi,obj->AtomInfo[a2].segi,1)))*/
+		    if(!ObjectMoleculeCheckBondSep(obj,a1,a2,3)) /* CA->N->C->CA = 3 bonds */
+		      a2=-1;
                   
-                }
+		  }
                 last_ai = ai;
                 
-                PRINTFD(FB_RepCartoon)
+                PRINTFD(G,FB_RepCartoon)
                   " RepCartoon: found CA in %s; a2 %d\n",ai->resi,a2
                   ENDFD;
                 
@@ -338,6 +353,11 @@ ENDFD;
 
                 *fp = ai->flags; /* store atom flags */
 
+                if(cartoon_side_chain_helper) {
+                  if(ai->visRep[cRepLine]||ai->visRep[cRepCyl]||ai->visRep[cRepSphere])
+                    *fp |= cAtomFlag_no_smooth;
+                }
+                
                 switch (ai->ssType[0]) {
                 case 'H':
                 case 'h':
@@ -369,7 +389,12 @@ ENDFD;
                   *ss=0;
                   break;
                 }
+                
                 *(cc++)=cur_car;
+                
+                if(cur_car==cCartoon_putty)
+                  putty_flag=true;
+                
                 v1 = cs->Coord+3*a;
                 v_ca = v1;
                 *(v++)=*(v1++);
@@ -385,7 +410,7 @@ ENDFD;
                 v_n = NULL;
                 v_o = NULL;
                   
-                AtomInfoBracketResidueFast(obj->AtomInfo,obj->NAtom,a1,&st,&nd);
+                AtomInfoBracketResidueFast(G,obj->AtomInfo,obj->NAtom,a1,&st,&nd);
 
                 if(obj->DiscreteFlag) {
                   if(cs==obj->DiscreteCSet[nd]) 
@@ -394,7 +419,7 @@ ENDFD;
                   skip_to=cs->AtmToIdx[nd];
                   
                 for(a3=st;a3<=nd;a3++) {
-                    
+                  
                   if(obj->DiscreteFlag) {
                     if(cs==obj->DiscreteCSet[a3]) 
                       a4=obj->DiscreteAtmToIdx[a3];
@@ -403,11 +428,11 @@ ENDFD;
                   } else 
                     a4=cs->AtmToIdx[a3];
                   if(a4>=0) {
-                    if(WordMatch("C",obj->AtomInfo[a3].name,1)<0) {
+                    if(WordMatch(G,"C",obj->AtomInfo[a3].name,1)<0) {
                       v_c = cs->Coord+3*a4;		
-                    } else if(WordMatch("N",obj->AtomInfo[a3].name,1)<0) {
+                    } else if(WordMatch(G,"N",obj->AtomInfo[a3].name,1)<0) {
                       v_n = cs->Coord+3*a4;
-                    } else if(WordMatch("O",obj->AtomInfo[a3].name,1)<0) {
+                    } else if(WordMatch(G,"O",obj->AtomInfo[a3].name,1)<0) {
                       v_o = cs->Coord+3*a4;
                     }
                   }
@@ -434,12 +459,12 @@ ENDFD;
               } else if(trace|| 
                         ((
                           ((na_mode==0)&&(ai->protons==cAN_P) &&
-                           (WordMatch("P",ai->name,1)<0) ) ||
+                           (WordMatch(G,"P",ai->name,1)<0) ) ||
                           ((na_mode==1)&&(ai->protons==cAN_C) &&
-                           (WordMatchExact("C4*",ai->name,1) ||
-                            WordMatchExact("C4'",ai->name,1)))
+                           (WordMatchExact(G,"C4*",ai->name,1) ||
+                            WordMatchExact(G,"C4'",ai->name,1)))
                           )
-                         && !AtomInfoSameResidueP(last_ai,ai))) {
+                         && !AtomInfoSameResidueP(G,last_ai,ai))) {
                 if(!trace) 
                   if(a2>=0) {
                     if(!ObjectMoleculeCheckBondSep(obj,a1,a2,6)) /* six bonds between phosphates */
@@ -456,6 +481,9 @@ ENDFD;
                   cur_car = cCartoon_tube;
                 *ss=3; /* DNA/RNA */
 
+                if(cur_car==cCartoon_putty)
+                  putty_flag=true;
+
                 *(cc++)=cur_car;
                 v1 = cs->Coord+3*a;
                 v_ca = v1;
@@ -470,7 +498,7 @@ ENDFD;
                 v_n = NULL;
                 v_o = NULL;
                   
-                AtomInfoBracketResidueFast(obj->AtomInfo,obj->NAtom,a1,&st,&nd);
+                AtomInfoBracketResidueFast(G,obj->AtomInfo,obj->NAtom,a1,&st,&nd);
                   
                 for(a3=st;a3<=nd;a3++) {
                     
@@ -486,9 +514,9 @@ ENDFD;
                     /* need to figure out how to generate a how to do right for DNA...
                        then we can use oval or rectangle */
                       
-                    if(WordMatch("P",obj->AtomInfo[a3].name,1)<0) {
+                    if(WordMatch(G,"P",obj->AtomInfo[a3].name,1)<0) {
                       v_c = cs->Coord+3*a4;		
-                    } else if(WordMatch("C2",obj->AtomInfo[a3].name,1)<0) {
+                    } else if(WordMatch(G,"C2",obj->AtomInfo[a3].name,1)<0) {
                       v_o = cs->Coord+3*a4;
                     }
                   }
@@ -500,108 +528,132 @@ ENDFD;
                   vo+=3;
                 } else {
                   /* generate orientation vectors...*/
-                    
+                  
                   cross_product3f(v_c,v_o,vo);
                   normalize3f(vo);
-                    
+                  
                   vo+=3;
                 }
               }
           }
         }
       }
-	 }
+    }
 
-PRINTFD(FB_RepCartoon)
-" RepCartoon-Debug: path outlined, interpolating...\n"
-ENDFD;
+  if(nAt&&putty_flag) {
+    double sum=0.0,sumsq=0.0;
+    float value;
+    int cnt = 0;
+
+    for(a=0;a<obj->NAtom;a++) {
+      ai=obj->AtomInfo+a;
+      
+      if(ai->visRep[cRepCartoon]) {
+        value = ai->b;
+        sum+=value;
+        sumsq+=(value*value);
+        cnt++;
+      }
+    }
+
+    if(cnt) {
+      putty_mean = (float)(sum/cnt);
+      putty_stdev = (float)sqrt1d((sumsq - (sum*sum/cnt))/(cnt));
+    } else {
+      putty_mean = 10.0F;
+      putty_stdev = 10.0F;
+    }
+  }
+
+  PRINTFD(G,FB_RepCartoon)
+    " RepCartoon-Debug: path outlined, interpolating...\n"
+    ENDFD;
 
   if(nAt)
-	 {
-		/* compute differences and normals */
+    {
+      /* compute differences and normals */
 
-		s=seg;
-		v=pv;
+      s=seg;
+      v=pv;
 		
-		dv = Alloc(float,nAt*3);
-		nv = Alloc(float,nAt*3);
-		dl = Alloc(float,nAt);
-		v1=dv;
-		v2=nv;
-		d=dl;
-		for(a=0;a<(nAt-1);a++)
-		  {
-        PRINTFD(FB_RepCartoon)
-          " RepCartoon: seg %d *s %d , *(s+1) %d\n",a,*s,*(s+1)
-          ENDFD;
+      dv = Alloc(float,nAt*3);
+      nv = Alloc(float,nAt*3);
+      dl = Alloc(float,nAt);
+      v1=dv;
+      v2=nv;
+      d=dl;
+      for(a=0;a<(nAt-1);a++)
+	{
+	  PRINTFD(G,FB_RepCartoon)
+	    " RepCartoon: seg %d *s %d , *(s+1) %d\n",a,*s,*(s+1)
+	    ENDFD;
 
-			 if(*s==*(s+1))
-				{
-				  subtract3f(v+3,v,v1);
-				  *d = (float)length3f(v1);
+	  if(*s==*(s+1))
+	    {
+	      subtract3f(v+3,v,v1);
+	      *d = (float)length3f(v1);
               if(*d>R_SMALL4) {
-		float d_1;
-		d_1 = 1.0F/(*d);
+                float d_1;
+                d_1 = 1.0F/(*d);
                 scale3f(v1,d_1,v2);
               } else if(a)  {
                 copy3f(v2-3,v2); 
               } else {
                 zero3f(v2);
               }
-				}
+	    }
           else {
             zero3f(v2);	
           }
           
-			 d++;
-			 v+=3;
-			 v1+=3;
-			 v2+=3;
-			 s++;
-		  }
+	  d++;
+	  v+=3;
+	  v1+=3;
+	  v2+=3;
+	  s++;
+	}
 		
-		/* compute tangents */
+      /* compute tangents */
 		
-		s=seg;
-		v=nv;
+      s=seg;
+      v=nv;
 		
-		tv = Alloc(float,nAt*3+6);
-		v1=tv;
+      tv = Alloc(float,nAt*3+6);
+      v1=tv;
 		
-		*(v1++)=*(v++); /* first segment */
-		*(v1++)=*(v++);
-		*(v1++)=*(v++);
-		s++;
+      *(v1++)=*(v++); /* first segment */
+      *(v1++)=*(v++);
+      *(v1++)=*(v++);
+      s++;
 		
-		for(a=1;a<(nAt-1);a++)
-		  {
-			 if((*s==*(s-1))&&(*s==*(s+1)))
-				{
-				  add3f(v,(v-3),v1);
-				  normalize3f(v1);			 
-				}
-			 else if(*s==*(s-1))
-				{
-				  *(v1)=*(v-3);  /* end a segment */
-				  *(v1+1)=*(v-2); 
-				  *(v1+2)=*(v-1); 
-				}
-			 else if(*s==*(s+1))
-				{
-				  *(v1)=*(v);   /* new segment */
-				  *(v1+1)=*(v+1); 
-				  *(v1+2)=*(v+2); 
-				}
-			 v+=3;
-			 v1+=3;
-			 s++;
-		  }
-		
-		*(v1++)=*(v-3); /* last segment */
-		*(v1++)=*(v-2);
-		*(v1++)=*(v-1);
+      for(a=1;a<(nAt-1);a++)
+	{
+	  if((*s==*(s-1))&&(*s==*(s+1)))
+	    {
+	      add3f(v,(v-3),v1);
+	      normalize3f(v1);			 
+	    }
+	  else if(*s==*(s-1))
+	    {
+	      *(v1)=*(v-3);  /* end a segment */
+	      *(v1+1)=*(v-2); 
+	      *(v1+2)=*(v-1); 
+	    }
+	  else if(*s==*(s+1))
+	    {
+	      *(v1)=*(v);   /* new segment */
+	      *(v1+1)=*(v+1); 
+	      *(v1+2)=*(v+2); 
+	    }
+	  v+=3;
+	  v1+=3;
+	  s++;
+	}
+      *(v1++)=*(v-3); /* last segment */
+      *(v1++)=*(v-2);
+      *(v1++)=*(v-1);
 
-      PRINTFD(FB_RepCartoon)
+      PRINTFD(G,FB_RepCartoon)
         " RepCartoon-Debug: generating coordinate systems...\n"
         ENDFD;
       
@@ -668,11 +720,13 @@ ENDFD;
                     normalize3f(vo-12);
                   }
 
-                  /* now make sure there's no goofy flip on the end...
-                   of a short, tight helix */
+		  if(v4&&v5) {
+		    /* now make sure there's no goofy flip on the end...
+		       of a short, tight helix */
 
-                  if(dot_product3f(vo-9,vo-12)<-0.8F)
-                    invert3f(vo-12);
+		    if(dot_product3f(vo-9,vo-12)<-0.8F)
+		      invert3f(vo-12);
+		  }
                 }
               }
               v1 = NULL;
@@ -717,7 +771,7 @@ ENDFD;
         }
       }
 
-      if(SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_refine_normals)) {
+      if(SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_refine_normals)) {
         /* now generate alternative orientation vectors */
         
         v1 = tv;
@@ -776,7 +830,7 @@ ENDFD;
 
       }
 
-      if(SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_sheets)) {
+      if(SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_sheets)) {
         s = seg;
         v = pv;
         ss = sstype;
@@ -930,7 +984,7 @@ ENDFD;
       }
       
       if(smooth_loops ||
-         SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_sheets)) {
+         SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_sheets)) {
         
         /* recompute differences and normals */
         
@@ -983,7 +1037,8 @@ ENDFD;
             if((*s==*(s-1))&&(*s==*(s+1)))
               {
                 add3f(v,(v-3),v1);
-                normalize3f(v1);			 
+               
+		normalize3f(v1);			 
               }
             else if(*s==*(s-1))
               {
@@ -1007,7 +1062,7 @@ ENDFD;
         *(v1++)=*(v-1);
 
         
-        if(SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_sheets)) {
+        if(SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cartoon_flat_sheets)) {
           s=seg+1;
           ss = sstype+1;
           v2=tv+3; /* normal */
@@ -1025,17 +1080,19 @@ ENDFD;
                 }
               }
               
+
               v2+=3;
               s++;
               ss++;
             }
+
         }
 
       }
     }
 
 
-  I->ray = CGONew();
+  I->ray = CGONew(G);
   
   if(alpha!=1.0F)
     CGOAlpha(I->ray,alpha);
@@ -1078,18 +1135,18 @@ ENDFD;
     }
   }
 
-PRINTFD(FB_RepCartoon)
-" RepCartoon-Debug: creating 3D scaffold...\n"
-ENDFD;
+  PRINTFD(G,FB_RepCartoon)
+    " RepCartoon-Debug: creating 3D scaffold...\n"
+    ENDFD;
 
-/* okay, we now have enough info to generate smooth interpolations */
+  /* okay, we now have enough info to generate smooth interpolations */
 
   if(nAt>1) {
-    ex = ExtrudeNew();
+    ex = ExtrudeNew(G);
     ExtrudeAllocPointsNormalsColors(ex,cs->NIndex*(3*sampling+3));
   }
 
-/* process cylindrical helices first */
+  /* process cylindrical helices first */
 
   if((nAt>1)&&cylindrical_helices) {
 
@@ -1105,13 +1162,13 @@ ENDFD;
     last_color=-1;
     uniform_color=true;
 
-	 v1=pv; /* points */
-	 v2=tv; /* tangents */
+    v1=pv; /* points */
+    v2=tv; /* tangents */
     vo=pvo;
     d = dl;
-	 s=seg;
+    s=seg;
     cc=car;
-	 atp=at; /* cs index pointer */
+    atp=at; /* cs index pointer */
     a=0;
     contFlag=true;
     cur_car = cCartoon_skip;
@@ -1150,12 +1207,12 @@ ENDFD;
         }
       }
       if(!extrudeFlag) {
-		  if((a<(nAt-1))&&(*s==*(s+1))) /* working in the same segment... */
-			 {
+	if((a<(nAt-1))&&(*s==*(s+1))) /* working in the same segment... */
+	  {
             atom_index1 = cs->IdxToAtm[*atp];
             atom_index2 = cs->IdxToAtm[*(atp+1)];
-				c1=*(cs->Color+*atp);
-				c2=*(cs->Color+*(atp+1));
+	    c1=*(cs->Color+*atp);
+	    c2=*(cs->Color+*(atp+1));
             if(cartoon_color>=0) {
               c1 = (c2 = cartoon_color);
             }
@@ -1192,13 +1249,13 @@ ENDFD;
             }
             last_color=c1;
 
-            v0 = ColorGet(c1);
+            v0 = ColorGet(G,c1);
             *(vc++)=*(v0++);
             *(vc++)=*(v0++);
             *(vc++)=*(v0++);
             *(vi++)=atom_index1;
 
-            v0 = ColorGet(c2); /* kludge */
+            v0 = ColorGet(G,c2); /* kludge */
             *(vc  )=*(v0++);
             *(vc+1)=*(v0++);
             *(vc+2)=*(v0++);
@@ -1287,14 +1344,14 @@ ENDFD;
                   normalize3f(t0);
                   scale3f(t0,f0,t1);
                   add3f(t1,t3,h_start)
-                }
+		    }
 
                 subtract3f(h_end,t4,t0);
                 if(length3f(t0)>f0) {
                   normalize3f(t0);
                   scale3f(t0,f0,t1);
                   add3f(t1,t4,h_end)
-                }
+		    }
               }
             }
           }
@@ -1463,10 +1520,10 @@ ENDFD;
 
                     f0 = ((float)b)/sampling; /* fraction of completion */
                     if(f0<=0.5) {
-                      v0 = ColorGet(c1);
+                      v0 = ColorGet(G,c1);
                       i0 = atom_index1;
                     } else {
-                      v0 = ColorGet(c2);
+                      v0 = ColorGet(G,c2);
                       i0 = atom_index2;
                     }
                     f0 = smooth(f0,power_a); /* bias sampling towards the center of the curve */
@@ -1503,10 +1560,10 @@ ENDFD;
 
                   f0=((float)b+1)/sampling;
                   if(f0<=0.5) {
-                    v0 = ColorGet(c1);
+                    v0 = ColorGet(G,c1);
                     i0 = atom_index1;
                   } else {
-                    v0 = ColorGet(c2);
+                    v0 = ColorGet(G,c2);
                     i0 = atom_index2;
                   }
                   f0=smooth(f0,power_a); /* bias sampling towards the center of the curve */                
@@ -1518,6 +1575,7 @@ ENDFD;
                   *(vc++)=*(v0++);
                   *(vc++)=*(v0++);
                   *(vi++)=i0;
+
                   /* end of line/cylinder */
                 
                   f1=1.0F-f0;
@@ -1644,6 +1702,23 @@ ENDFD;
               ExtrudeBuildNormals1f(ex);
               ExtrudeCGOSurfaceTube(ex,I->ray,1,NULL);
               break;
+            case cCartoon_putty:
+              ExtrudeCircle(ex,putty_quality,putty_radius);
+              ExtrudeBuildNormals1f(ex);
+              ExtrudeComputeScaleFactors(ex,obj,0,
+                                         putty_mean,putty_stdev,
+					 SettingGet_f(G,cs->Setting,obj->Obj.Setting,
+						      cSetting_cartoon_putty_scale_power),
+					 SettingGet_f(G,cs->Setting,obj->Obj.Setting,
+						      cSetting_cartoon_putty_range),
+					 SettingGet_f(G,cs->Setting,obj->Obj.Setting,
+						      cSetting_cartoon_putty_scale_min),
+					 SettingGet_f(G,cs->Setting,obj->Obj.Setting,
+						      cSetting_cartoon_putty_scale_max),
+                                         sampling/2);
+
+              ExtrudeCGOSurfaceVariableTube(ex,I->ray,1);
+              break;
             case cCartoon_loop:
               ExtrudeCircle(ex,loop_quality,loop_radius);
               ExtrudeBuildNormals1f(ex);
@@ -1660,7 +1735,7 @@ ENDFD;
                 ExtrudeCGOSurfacePolygon(ex,I->ray,0,NULL);
                 ExtrudeRectangle(ex,width,length,2);
                 ExtrudeBuildNormals2f(ex);
-                ExtrudeCGOSurfacePolygon(ex,I->ray,1,ColorGet(highlight_color));
+                ExtrudeCGOSurfacePolygon(ex,I->ray,1,ColorGet(G,highlight_color));
               }
               break;
             case cCartoon_oval:
@@ -1669,7 +1744,7 @@ ENDFD;
               if(highlight_color<0) 
                 ExtrudeCGOSurfaceTube(ex,I->ray,1,NULL);
               else 
-                ExtrudeCGOSurfaceTube(ex,I->ray,1,ColorGet(highlight_color));
+                ExtrudeCGOSurfaceTube(ex,I->ray,1,ColorGet(G,highlight_color));
               break;
             case cCartoon_arrow:
               ExtrudeRectangle(ex,width,length,0);
@@ -1677,7 +1752,7 @@ ENDFD;
               if(highlight_color<0) 
                 ExtrudeCGOSurfaceStrand(ex,I->ray,sampling,NULL);
               else
-                ExtrudeCGOSurfaceStrand(ex,I->ray,sampling,ColorGet(highlight_color));                
+                ExtrudeCGOSurfaceStrand(ex,I->ray,sampling,ColorGet(G,highlight_color));                
               break;
             case cCartoon_dumbbell:
               if(highlight_color<0) {
@@ -1692,7 +1767,7 @@ ENDFD;
 
                 ExtrudeDumbbell1(ex,dumbbell_width,dumbbell_length,2);
                 ExtrudeBuildNormals2f(ex);
-                ExtrudeCGOSurfacePolygonTaper(ex,I->ray,sampling,ColorGet(highlight_color));
+                ExtrudeCGOSurfacePolygonTaper(ex,I->ray,sampling,ColorGet(G,highlight_color));
               }
               /*
                 ExtrudeCGOSurfacePolygonX(ex,I->ray,1);*/
@@ -1702,6 +1777,7 @@ ENDFD;
               ExtrudeComputeTangents(ex1);
               ExtrudeCircle(ex1,loop_quality,dumbbell_radius);
               ExtrudeBuildNormals1f(ex1);
+
               ExtrudeCGOSurfaceTube(ex1,I->ray,1,NULL);
               ExtrudeFree(ex1);
 
@@ -1751,6 +1827,7 @@ ENDFD;
           v3+=3;
         }
       CGOEnd(I->ray);
+
       CGOEnable(I->ray,GL_LIGHTING);
     }
   }
