@@ -29,6 +29,7 @@ Z* -------------------------------------------------------------------
 #include"Setting.h"
 #include"Color.h"
 #include"main.h"
+#include"PyMOLGlobals.h"
 
 typedef struct RepMesh {
   Rep R;
@@ -71,6 +72,7 @@ void RepMeshGetSolventDots(RepMesh *I,CoordSet *cs,float *min,float *max,float p
 
 void RepMeshRender(RepMesh *I,CRay *ray,Pickable **pick)
 {
+  PyMOLGlobals *G=I->R.G;
   float *v=I->V;
   float *vc=I->VC;
   int *n=I->N;
@@ -87,8 +89,8 @@ void RepMeshRender(RepMesh *I,CRay *ray,Pickable **pick)
         radius = I->Radius;
       }
       if(I->oneColorFlag) 
-        col=ColorGet(I->oneColor);
-		ray->fColor3fv(ray,ColorGet(I->R.obj->Color));
+        col=ColorGet(G,I->oneColor);
+		ray->fColor3fv(ray,ColorGet(G,I->R.obj->Color));
 		while(*n)
 		  {
 			 c=*(n++);
@@ -114,99 +116,102 @@ void RepMeshRender(RepMesh *I,CRay *ray,Pickable **pick)
 				}
 		  }
 	 }
-  } else if(pick&&PMGUI) {
-  } else if(PMGUI) {
-    int use_dlst;
-    int lighting = SettingGet_f(I->R.cs->Setting,I->R.obj->Setting,cSetting_mesh_lighting);
-    SceneResetNormal(true);
-    if(!lighting)
-      glDisable(GL_LIGHTING);
-    
-    use_dlst = (int)SettingGet(cSetting_use_display_lists);
-    if(use_dlst&&I->R.displayList) {
-      glCallList(I->R.displayList);
-    } else { 
-      SceneResetNormal(false);
+  } else if(G->HaveGUI && G->ValidContext) {
+    if(pick) {
+    } else {
+      int use_dlst;
+      int lighting = SettingGet_i(G,I->R.cs->Setting,I->R.obj->Setting,cSetting_mesh_lighting);
 
-      if(use_dlst) {
-        if(!I->R.displayList) {
-          I->R.displayList = glGenLists(1);
-          if(I->R.displayList) {
-            glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
-          }
-        }
-      }
-      
-      switch(I->mesh_type) {
-      case 0:
-        glLineWidth(I->Width);
-        if(n) {
-          if(I->oneColorFlag) {
-            while(*n)
-              {
-                glColor3fv(ColorGet(I->oneColor));
-                c=*(n++);
-                glBegin(GL_LINE_STRIP);
-                while(c--) {
-                  glVertex3fv(v);
-                  v+=3;
-                }
-                glEnd();
-              }
-          } else {
-            while(*n)
-              {
-                c=*(n++);
-                glBegin(GL_LINE_STRIP);
-                while(c--) {
-                  glColor3fv(vc);
-                  vc+=3;
-                  glVertex3fv(v);
-                  v+=3;
-                }
-                glEnd();
-              }
-          }
-        }
-      break;
-      case 1:
-        glPointSize(SettingGet_f(I->R.cs->Setting,I->R.obj->Setting,cSetting_dot_width));
-        if(n) {
-          if(I->oneColorFlag) {
-            while(*n)
-              {
-                glColor3fv(ColorGet(I->oneColor));
-                c=*(n++);
-                glBegin(GL_POINTS);
-                while(c--) {
-                  glVertex3fv(v);
-                  v+=3;
-                }
-                glEnd();
-              }
-          } else {
-            while(*n)
-              {
-                c=*(n++);
-                glBegin(GL_POINTS);
-                while(c--) {
-                  glColor3fv(vc);
-                  vc+=3;
-                  glVertex3fv(v);
-                  v+=3;
-                }
-                glEnd();
-              }
-          }
-        }
-        break;
-      }
+      SceneResetNormal(G,true);
+      if(!lighting)
+        glDisable(GL_LIGHTING);
+    
+      use_dlst = (int)SettingGet(G,cSetting_use_display_lists);
       if(use_dlst&&I->R.displayList) {
-        glEndList();
+        glCallList(I->R.displayList);
+      } else { 
+        SceneResetNormal(G,false);
+
+        if(use_dlst) {
+          if(!I->R.displayList) {
+            I->R.displayList = glGenLists(1);
+            if(I->R.displayList) {
+              glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
+            }
+          }
+        }
+      
+        switch(I->mesh_type) {
+        case 0:
+          glLineWidth(I->Width);
+          if(n) {
+            if(I->oneColorFlag) {
+              while(*n)
+                {
+                  glColor3fv(ColorGet(G,I->oneColor));
+                  c=*(n++);
+                  glBegin(GL_LINE_STRIP);
+                  while(c--) {
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glEnd();
+                }
+            } else {
+              while(*n)
+                {
+                  c=*(n++);
+                  glBegin(GL_LINE_STRIP);
+                  while(c--) {
+                    glColor3fv(vc);
+                    vc+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glEnd();
+                }
+            }
+          }
+          break;
+        case 1:
+          glPointSize(SettingGet_f(G,I->R.cs->Setting,I->R.obj->Setting,cSetting_dot_width));
+          if(n) {
+            if(I->oneColorFlag) {
+              while(*n)
+                {
+                  glColor3fv(ColorGet(G,I->oneColor));
+                  c=*(n++);
+                  glBegin(GL_POINTS);
+                  while(c--) {
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glEnd();
+                }
+            } else {
+              while(*n)
+                {
+                  c=*(n++);
+                  glBegin(GL_POINTS);
+                  while(c--) {
+                    glColor3fv(vc);
+                    vc+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glEnd();
+                }
+            }
+          }
+          break;
+        }
+        if(use_dlst&&I->R.displayList) {
+          glEndList();
+        }
       }
+      if(!lighting)
+        glEnable(GL_LIGHTING);
     }
-    if(!lighting)
-      glEnable(GL_LIGHTING);
   }
 }
 
@@ -239,6 +244,7 @@ int RepMeshSameVis(RepMesh *I,CoordSet *cs)
 
 void RepMeshColor(RepMesh *I,CoordSet *cs)
 {
+  PyMOLGlobals *G=cs->State.G;
   MapType *map;
   int a,i0,i,j,h,k,l,c1;
   float *v0,*vc,*c0;
@@ -255,9 +261,9 @@ void RepMeshColor(RepMesh *I,CoordSet *cs)
 
   obj=cs->Obj;
 
-  probe_radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_solvent_radius);
-  mesh_color = SettingGet_color(cs->Setting,obj->Obj.Setting,cSetting_mesh_color);
-  mesh_mode = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_mesh_mode);
+  probe_radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_solvent_radius);
+  mesh_color = SettingGet_color(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_color);
+  mesh_mode = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_mode);
   cullByFlag = (mesh_mode==cRepMesh_by_flags);
   inclH = !(mesh_mode==cRepMesh_heavy_atoms);
   
@@ -273,12 +279,12 @@ void RepMeshColor(RepMesh *I,CoordSet *cs)
       *(lc++) = *(cc++);
     }
 
-  I->Width = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_mesh_width);
-  I->Radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_mesh_radius);
+  I->Width = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_width);
+  I->Radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_radius);
 
   if(I->NTot) {
 	 obj=cs->Obj;
-    if(ColorCheckRamped(mesh_color)) {
+    if(ColorCheckRamped(G,mesh_color)) {
       I->oneColorFlag=false;
     } else {
       I->oneColorFlag=true;
@@ -287,7 +293,7 @@ void RepMeshColor(RepMesh *I,CoordSet *cs)
 	 if(!I->VC) I->VC = Alloc(float,3*I->NTot);
 	 vc=I->VC;
 	 /* now, assign colors to each point */
-	 map=MapNew(I->max_vdw+probe_radius,cs->Coord,cs->NIndex,NULL);
+	 map=MapNew(G,I->max_vdw+probe_radius,cs->Coord,cs->NIndex,NULL);
 	 if(map)
 		{
 		  MapSetupExpress(map);
@@ -328,15 +334,15 @@ void RepMeshColor(RepMesh *I,CoordSet *cs)
 				  }
 				}
 
-            if(ColorCheckRamped(mesh_color)) {
+            if(ColorCheckRamped(G,mesh_color)) {
               c1 = mesh_color;
             }
-            if(ColorCheckRamped(c1)) {
+            if(ColorCheckRamped(G,c1)) {
               I->oneColorFlag=false;
-              ColorGetRamped(c1,v0,vc);
+              ColorGetRamped(G,c1,v0,vc);
               vc+=3;
             } else {
-              c0 = ColorGet(c1);
+              c0 = ColorGet(G,c1);
               *(vc++) = *(c0++);
               *(vc++) = *(c0++);
               *(vc++) = *(c0++);
@@ -357,6 +363,7 @@ void RepMeshColor(RepMesh *I,CoordSet *cs)
 
 Rep *RepMeshNew(CoordSet *cs)
 {
+  PyMOLGlobals *G=cs->State.G;
   ObjectMolecule *obj;
   CoordSet *ccs;
   int a,b,c,d,h,k,l,*n;
@@ -385,43 +392,46 @@ Rep *RepMeshNew(CoordSet *cs)
 
   AtomInfoType *ai1;
 
-  OOAlloc(RepMesh);
+  OOAlloc(G,RepMesh);
 
-  PRINTFD(FB_RepMesh)
+  PRINTFD(G,FB_RepMesh)
 	 " RepMeshNew-DEBUG: entered with coord-set %p\n",(void*)cs
 	 ENDFD;
   obj = cs->Obj;
 
-  probe_radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_solvent_radius);
+  probe_radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_solvent_radius);
   probe_radius2 = probe_radius*probe_radius;
-  solv_acc = (SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_mesh_solvent));
-  mesh_type = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_mesh_type);
+  solv_acc = (SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_solvent));
+  mesh_type = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_type);
 
-  I->max_vdw = ObjectMoleculeGetMaxVDW(obj) + solv_acc*probe_radius;
 
-  mesh_mode = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_mesh_mode);
+  
+  mesh_mode = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_mode);
   cullByFlag = (mesh_mode==cRepMesh_by_flags);
   inclH = !(mesh_mode==cRepMesh_heavy_atoms);
   visFlag=false;
-  for(a=0;a<cs->NIndex;a++) {
-    ai1 = obj->AtomInfo+cs->IdxToAtm[a];
-	 if(ai1->visRep[cRepMesh]&&
-       (inclH||(!ai1->hydrogen))&&
-       ((!cullByFlag)||
-        (!(ai1->flags&(cAtomFlag_exclude|cAtomFlag_ignore)))))
-		{
-		  visFlag=true;
-		  break;
-		}
-  }
+  if(obj->RepVisCache[cRepMesh])
+    for(a=0;a<cs->NIndex;a++) {
+      ai1 = obj->AtomInfo+cs->IdxToAtm[a];
+      if(ai1->visRep[cRepMesh]&&
+         (inclH||(!ai1->hydrogen))&&
+         ((!cullByFlag)||
+          (!(ai1->flags&(cAtomFlag_exclude|cAtomFlag_ignore)))))
+        {
+          visFlag=true;
+          break;
+        }
+    }
   if(!visFlag) {
     OOFreeP(I);
     return(NULL); /* skip if no dots are visible */
   }
 
-  RepInit(&I->R);
+  I->max_vdw = ObjectMoleculeGetMaxVDW(obj) + solv_acc*probe_radius;
 
-  min_spacing = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_min_mesh_spacing);
+  RepInit(G,&I->R);
+
+  min_spacing = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_min_mesh_spacing);
 
   I->N=NULL;
   I->NTot=0;
@@ -438,7 +448,7 @@ Rep *RepMeshNew(CoordSet *cs)
   I->LastVisib=NULL;
   I->LastColor=NULL;
   I->mesh_type = mesh_type;
-  I->Radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_mesh_radius);
+  I->Radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_radius);
 
   meshFlag=true;
 
@@ -498,25 +508,25 @@ Rep *RepMeshNew(CoordSet *cs)
 	 for(c=0;c<3;c++)
 		dims[c] = (int)((sizeE[c]/gridSize)+1.5F);
 	 
-	 field = IsosurfFieldAlloc(dims);
+	 field = IsosurfFieldAlloc(G,dims);
 	 
 	 for(a=0;a<dims[0];a++)
 		for(b=0;b<dims[1];b++)
 		  for(c=0;c<dims[2];c++)
 			 F3(field->data,a,b,c) = 2.0;
 
-	 OrthoBusyFast(0,1);
+	 OrthoBusyFast(G,0,1);
 	 if(!solv_acc)
       RepMeshGetSolventDots(I,cs,minE,maxE,probe_radius);
-	 smap=MapNew(probe_radius,I->Dot,I->NDot,NULL);
-	 map=MapNew(I->max_vdw+probe_radius,cs->Coord,cs->NIndex,NULL);
+	 smap=MapNew(G,probe_radius,I->Dot,I->NDot,NULL);
+	 map=MapNew(G,I->max_vdw+probe_radius,cs->Coord,cs->NIndex,NULL);
 	 if(map&&smap)
 		{
 		  MapSetupExpress(smap);
 		  MapSetupExpress(map);
 		  for(a=0;a<dims[0];a++)
 			 {
-				OrthoBusyFast(dims[0]+a,dims[0]*3);
+				OrthoBusyFast(G,dims[0]+a,dims[0]*3);
 				point[0]=minE[0]+a*gridSize;
 				for(b=0;b<dims[1];b++)
 				  {
@@ -603,9 +613,9 @@ Rep *RepMeshNew(CoordSet *cs)
 	 MapFree(smap);
 	 MapFree(map);
 	 FreeP(I->Dot);	 
-	 OrthoBusyFast(2,3);
-    IsosurfVolume(field,1.0,&I->N,&I->V,NULL,mesh_type);
-    IsosurfFieldFree(field);
+	 OrthoBusyFast(G,2,3);
+    IsosurfVolume(G,field,1.0,&I->N,&I->V,NULL,mesh_type);
+    IsosurfFieldFree(G,field);
     
     /* someday add support for solid mesh representation....
        if(mesh_type==0||mesh_type==1) {
@@ -627,55 +637,46 @@ Rep *RepMeshNew(CoordSet *cs)
 	 I->NTot=0;
 	 while(*n) I->NTot+=*(n++);
 	 RepMeshColor(I,cs);
-	 OrthoBusyFast(3,4);
+	 OrthoBusyFast(G,3,4);
   }
-  OrthoBusyFast(4,4);
+  OrthoBusyFast(G,4,4);
   return((void*)(struct Rep*)I);
 }
 
 void RepMeshGetSolventDots(RepMesh *I,CoordSet *cs,float *min,float *max,float probe_radius)
 {
-  ObjectMolecule *obj;
+  PyMOLGlobals *G=cs->State.G;
+  ObjectMolecule *obj=cs->Obj;
   int a,b,c,a1,a2,flag,i,h,k,l,j;
   float *v,*v0,vdw;
   MapType *map;
   int inFlag,*p,*dot_flag;
-  SphereRec *sp = Sphere0;
+  SphereRec *sp = G->Sphere->Sphere[0];
   int cavity_cull;
   float probe_radius_plus;
   int dotCnt,maxCnt,maxDot=0;
   int cnt;
   int inclH,mesh_mode,cullByFlag;
   AtomInfoType *ai1,*ai2;
-  obj = cs->Obj;
-  
-  switch(SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_mesh_quality)) {
-  case 0: sp = Sphere0; break;
-  case 1: sp = Sphere1; break;
-  case 2: sp = Sphere2; break;
-  case 3: sp = Sphere3; break;
-  case 4: sp = Sphere4; break;
-  default:
-    if(SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_mesh_quality)>4)
-      sp = Sphere4; 
-    else
-      sp = Sphere0;      
-    break;
-  }
+  int ds = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_quality);
 
-  cavity_cull = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_cavity_cull);
+  if(ds<0) ds = 0;
+  if(ds>4) ds = 4;
+  sp = G->Sphere->Sphere[ds];
 
-  mesh_mode = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_mesh_mode);
+  cavity_cull = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_cavity_cull);
+
+  mesh_mode = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_mesh_mode);
   cullByFlag = (mesh_mode==cRepMesh_by_flags);
   inclH = !(mesh_mode==cRepMesh_heavy_atoms);
 
   I->Dot=(float*)mmalloc(sizeof(float)*cs->NIndex*3*sp->nDot);
-  ErrChkPtr(I->Dot);
+  ErrChkPtr(G,I->Dot);
 
   probe_radius_plus = probe_radius * 1.5F;
 
   I->NDot=0;
-  map=MapNew(I->max_vdw+probe_radius,cs->Coord,cs->NIndex,NULL);
+  map=MapNew(G,I->max_vdw+probe_radius,cs->Coord,cs->NIndex,NULL);
   if(map)
 	 {
 		MapSetupExpress(map);
@@ -688,7 +689,7 @@ void RepMeshGetSolventDots(RepMesh *I,CoordSet *cs,float *min,float *max,float p
           if((inclH||(!ai1->hydrogen))&&
              ((!cullByFlag)||
               (!(ai1->flags&(cAtomFlag_ignore))))) {
-            OrthoBusyFast(a,cs->NIndex*3);
+            OrthoBusyFast(G,a,cs->NIndex*3);
             dotCnt=0;
             a1 = cs->IdxToAtm[a];
             v0 = cs->Coord+3*a;
@@ -746,13 +747,13 @@ void RepMeshGetSolventDots(RepMesh *I,CoordSet *cs,float *min,float *max,float p
 
   if(cavity_cull>0) {
 	 dot_flag=Alloc(int,I->NDot);
-	 ErrChkPtr(dot_flag);
+	 ErrChkPtr(G,dot_flag);
 	 for(a=0;a<I->NDot;a++) {
 		dot_flag[a]=0;
 	 }
 	 dot_flag[maxDot]=1; /* this guarantees that we have a valid dot */
 
-	 map=MapNew(probe_radius_plus,I->Dot,I->NDot,NULL);
+	 map=MapNew(G,probe_radius_plus,I->Dot,I->NDot,NULL);
 	 if(map)
 		{
 		  MapSetupExpress(map);
