@@ -57,12 +57,13 @@ void RepSphereFree(RepSphere *I)
   FreeP(I->NT);
   FreeP(I->LastColor);
   FreeP(I->LastVisib);
-  RepFree(&I->R);
+  RepPurge(&I->R);
   OOFreeP(I);
 }
 
 void RepSphereRender(RepSphere *I,CRay *ray,Pickable **pick)
 {
+  PyMOLGlobals *G=I->R.G;
   float *v=I->V,*vc;
   int c=I->N;
   int cc=0,*nt=NULL;
@@ -71,7 +72,7 @@ void RepSphereRender(RepSphere *I,CRay *ray,Pickable **pick)
   SphereRec *sp;
   float restart;
   float alpha;
-  alpha = SettingGet_f(I->R.cs->Setting,I->R.obj->Setting,cSetting_sphere_transparency);
+  alpha = SettingGet_f(G,I->R.cs->Setting,I->R.obj->Setting,cSetting_sphere_transparency);
 
   alpha=1.0F-alpha;
   if(fabs(alpha-1.0)<R_SMALL4)
@@ -104,22 +105,53 @@ void RepSphereRender(RepSphere *I,CRay *ray,Pickable **pick)
       }
     }
     ray->fTransparentf(ray,0.0);
-  } else if(pick&&PMGUI) {
-    int trans_pick_mode = SettingGet_i(I->R.cs->Setting,
-                                       I->R.obj->Setting,
-                                       cSetting_transparency_picking_mode);
-    if(I->R.P&&((trans_pick_mode==1)||((trans_pick_mode==2)&&(alpha>0.9F)))) {
-      int i,j;
-      Pickable *p;
-		sp=I->SP;      
-      i=(*pick)->index;
+  } else if(G->HaveGUI && G->ValidContext) {
+    if(pick) {
+      int trans_pick_mode = SettingGet_i(G,I->R.cs->Setting,
+                                         I->R.obj->Setting,
+                                         cSetting_transparency_picking_mode);
+
+      if(I->R.P&&((trans_pick_mode==1)||((trans_pick_mode==2)&&(alpha>0.9F)))) {
+        int i,j;
+        Pickable *p;
+        sp=I->SP;      
+        i=(*pick)->index;
       
-      p=I->R.P;
+        p=I->R.P;
       
-      if(I->spheroidFlag) {
-        while(c--)
-          {
+        if(I->spheroidFlag) {
+          while(c--)
+            {
             
+              i++;          
+              if(!(*pick)[0].ptr) {
+                /* pass 1 - low order bits *            */
+                glColor3ub((uchar)((i&0xF)<<4),(uchar)((i&0xF0)|0x8),(uchar)((i&0xF00)>>4)); 
+                VLACheck((*pick),Pickable,i);
+                p++;
+                (*pick)[i] = *p; /* copy object and atom info */
+              } else { 
+                /* pass 2 - high order bits */           
+                j=i>>12;            
+                glColor3ub((uchar)((j&0xF)<<4),(uchar)((j&0xF0)|0x8),(uchar)((j&0xF00)>>4));             
+              }			 
+            
+              v+=3;
+              for(a=0;a<sp->NStrip;a++) {
+                cc=sp->StripLen[a];
+                glBegin(GL_TRIANGLE_STRIP);
+                while((cc--)>0) {
+                  glNormal3fv(v);
+                  glVertex3fv(v+3);
+                  v+=6;
+                }
+                glEnd();
+              }
+            }
+        } else {
+          v=I->VC;
+          c=I->NC;
+          while(c--) {
             i++;          
             if(!(*pick)[0].ptr) {
               /* pass 1 - low order bits *            */
@@ -132,206 +164,179 @@ void RepSphereRender(RepSphere *I,CRay *ray,Pickable **pick)
               j=i>>12;            
               glColor3ub((uchar)((j&0xF)<<4),(uchar)((j&0xF0)|0x8),(uchar)((j&0xF00)>>4));             
             }			 
-            
-            v+=3;
-            for(a=0;a<sp->NStrip;a++) {
-              cc=sp->StripLen[a];
-              glBegin(GL_TRIANGLE_STRIP);
-              while((cc--)>0) {
-                glNormal3fv(v);
-                glVertex3fv(v+3);
-                v+=6;
-              }
-              glEnd();
-            }
-          }
-      } else {
-        v=I->VC;
-        c=I->NC;
-        while(c--) {
-          i++;          
-          if(!(*pick)[0].ptr) {
-            /* pass 1 - low order bits *            */
-            glColor3ub((uchar)((i&0xF)<<4),(uchar)((i&0xF0)|0x8),(uchar)((i&0xF00)>>4)); 
-            VLACheck((*pick),Pickable,i);
-            p++;
-            (*pick)[i] = *p; /* copy object and atom info */
-          } else { 
-            /* pass 2 - high order bits */           
-            j=i>>12;            
-            glColor3ub((uchar)((j&0xF)<<4),(uchar)((j&0xF0)|0x8),(uchar)((j&0xF00)>>4));             
-          }			 
           
-          {
-            int *s,*q,b;
-            float *v0,vdw;
+            {
+              int *s,*q,b;
+              float *v0,vdw;
 
-            v0 = v+3;
-            vdw = v[6];
-            q=sp->Sequence;
-            s=sp->StripLen;
-            for(b=0;b<sp->NStrip;b++)
-              {
-                glBegin(GL_TRIANGLE_STRIP);
-                for(cc=0;cc<(*s);cc++)
-                  {
-                    glNormal3f(sp->dot[*q][0],
-                               sp->dot[*q][1],
-                               sp->dot[*q][2]);
-                    glVertex3f(v0[0]+vdw*sp->dot[*q][0],
-                               v0[1]+vdw*sp->dot[*q][1],
-                               v0[2]+vdw*sp->dot[*q][2]);
-                    q++;
-                  }
-                glEnd();
-                s++;
-              }
-            v+=7;
+              v0 = v+3;
+              vdw = v[6];
+              q=sp->Sequence;
+              s=sp->StripLen;
+              for(b=0;b<sp->NStrip;b++)
+                {
+                  glBegin(GL_TRIANGLE_STRIP);
+                  for(cc=0;cc<(*s);cc++)
+                    {
+                      glNormal3f(sp->dot[*q][0],
+                                 sp->dot[*q][1],
+                                 sp->dot[*q][2]);
+                      glVertex3f(v0[0]+vdw*sp->dot[*q][0],
+                                 v0[1]+vdw*sp->dot[*q][1],
+                                 v0[2]+vdw*sp->dot[*q][2]);
+                      q++;
+                    }
+                  glEnd();
+                  s++;
+                }
+              v+=7;
+            }
           }
         }
+        (*pick)[0].index = i;
       }
-	 (*pick)[0].index = i;
-    }
-  } else if(PMGUI) {
-    int use_dlst;
-    use_dlst = (int)SettingGet(cSetting_use_display_lists);
+    } else {
+      int use_dlst;
 
-    if(use_dlst&&I->R.displayList) {
-      glCallList(I->R.displayList);
-    } else { /* display list */
+      use_dlst = (int)SettingGet(G,cSetting_use_display_lists);
 
-      if(use_dlst) {
-        if(!I->R.displayList) {
-          I->R.displayList = glGenLists(1);
-          if(I->R.displayList) {
-            glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
-          }
-        }
-      }
-
-      if(I->cullFlag) {
-      
-      if(alpha==1.0) {
-        
-        nt=I->NT; /* number of passes for each sphere */
-        while(c--) /* iterate through all atoms */
-          {
-            glColor3fv(v);
-            v+=3;
-            cc=*(nt++);
-            flag=0;
-            glBegin(GL_TRIANGLE_STRIP);
-            while(cc--) { /* execute loop this many times */
-              restart=*(v++);
-              if(restart) {
-                if(flag) {
-                  glEnd();
-                  glBegin(GL_TRIANGLE_STRIP);
-                }
-                if(restart==2.0) { /* swap triangle polarity */
-                  glNormal3fv(v);
-                  glVertex3fv(v+3);
-                }
-                glNormal3fv(v);
-                v+=3;
-                glVertex3fv(v);
-                v+=3;
-                glNormal3fv(v);
-                v+=3;
-                glVertex3fv(v);
-                v+=3;
-              }
-              glNormal3fv(v);
-              v+=3;
-              glVertex3fv(v);
-              v+=3;
-              flag=1;
-            }
-            glEnd();
-          }
-      } else {
-    
-        nt=I->NT; /* number of passes for each sphere */
-        while(c--) /* iterate through all atoms */
-          {
-            glColor4f(v[0],v[1],v[2],alpha);
-            v+=3;
-            cc=*(nt++);
-            flag=0;
-            glBegin(GL_TRIANGLE_STRIP);
-            while(cc--) { /* execute loop this many times */
-              restart=*(v++);
-              if(restart) {
-                if(flag) {
-                  glEnd();
-                  glBegin(GL_TRIANGLE_STRIP);
-                }
-                if(restart==2.0) { /* swap triangle polarity */
-                  glNormal3fv(v);
-                  glVertex3fv(v+3);
-                }
-                glNormal3fv(v);
-                v+=3;
-                glVertex3fv(v);
-                v+=3;
-                glNormal3fv(v);
-                v+=3;
-                glVertex3fv(v);
-                v+=3;
-              }
-              glNormal3fv(v);
-              v+=3;
-              glVertex3fv(v);
-              v+=3;
-              flag=1;
-            }
-            glEnd();
-          }
-        
-      }
-	 } else {
-
-      if(alpha==1.0) {
-        
-        sp=I->SP;
-        while(c--)
-          {
-            glColor3fv(v);
-            v+=3;
-            for(a=0;a<sp->NStrip;a++) {
-              glBegin(GL_TRIANGLE_STRIP);
-              cc=sp->StripLen[a];
-              while(cc--) {
-                glNormal3fv(v);
-                v+=3;
-                glVertex3fv(v);
-                v+=3;
-              }
-              glEnd();
-            }
-          }
-      } else {
-        sp=I->SP;
-        while(c--)
-          {
-            glColor4f(v[0],v[1],v[2],alpha);
-            v+=3;
-            for(a=0;a<sp->NStrip;a++) {
-              glBegin(GL_TRIANGLE_STRIP);
-              cc=sp->StripLen[a];
-              while(cc--) {
-                glNormal3fv(v);
-                v+=3;
-                glVertex3fv(v);
-                v+=3;
-              }
-              glEnd();
-            }
-          }
-      }
-    }
       if(use_dlst&&I->R.displayList) {
-        glEndList();
+        glCallList(I->R.displayList);
+      } else { /* display list */
+
+        if(use_dlst) {
+          if(!I->R.displayList) {
+            I->R.displayList = glGenLists(1);
+            if(I->R.displayList) {
+              glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
+            }
+          }
+        }
+
+        if(I->cullFlag) {
+      
+          if(alpha==1.0) {
+        
+            nt=I->NT; /* number of passes for each sphere */
+            while(c--) /* iterate through all atoms */
+              {
+                glColor3fv(v);
+                v+=3;
+                cc=*(nt++);
+                flag=0;
+                glBegin(GL_TRIANGLE_STRIP);
+                while(cc--) { /* execute loop this many times */
+                  restart=*(v++);
+                  if(restart) {
+                    if(flag) {
+                      glEnd();
+                      glBegin(GL_TRIANGLE_STRIP);
+                    }
+                    if(restart==2.0) { /* swap triangle polarity */
+                      glNormal3fv(v);
+                      glVertex3fv(v+3);
+                    }
+                    glNormal3fv(v);
+                    v+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                    glNormal3fv(v);
+                    v+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glNormal3fv(v);
+                  v+=3;
+                  glVertex3fv(v);
+                  v+=3;
+                  flag=1;
+                }
+                glEnd();
+              }
+          } else {
+    
+            nt=I->NT; /* number of passes for each sphere */
+            while(c--) /* iterate through all atoms */
+              {
+                glColor4f(v[0],v[1],v[2],alpha);
+                v+=3;
+                cc=*(nt++);
+                flag=0;
+                glBegin(GL_TRIANGLE_STRIP);
+                while(cc--) { /* execute loop this many times */
+                  restart=*(v++);
+                  if(restart) {
+                    if(flag) {
+                      glEnd();
+                      glBegin(GL_TRIANGLE_STRIP);
+                    }
+                    if(restart==2.0) { /* swap triangle polarity */
+                      glNormal3fv(v);
+                      glVertex3fv(v+3);
+                    }
+                    glNormal3fv(v);
+                    v+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                    glNormal3fv(v);
+                    v+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glNormal3fv(v);
+                  v+=3;
+                  glVertex3fv(v);
+                  v+=3;
+                  flag=1;
+                }
+                glEnd();
+              }
+        
+          }
+        } else {
+
+          if(alpha==1.0) {
+        
+            sp=I->SP;
+            while(c--)
+              {
+                glColor3fv(v);
+                v+=3;
+                for(a=0;a<sp->NStrip;a++) {
+                  glBegin(GL_TRIANGLE_STRIP);
+                  cc=sp->StripLen[a];
+                  while(cc--) {
+                    glNormal3fv(v);
+                    v+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glEnd();
+                }
+              }
+          } else {
+            sp=I->SP;
+            while(c--)
+              {
+                glColor4f(v[0],v[1],v[2],alpha);
+                v+=3;
+                for(a=0;a<sp->NStrip;a++) {
+                  glBegin(GL_TRIANGLE_STRIP);
+                  cc=sp->StripLen[a];
+                  while(cc--) {
+                    glNormal3fv(v);
+                    v+=3;
+                    glVertex3fv(v);
+                    v+=3;
+                  }
+                  glEnd();
+                }
+              }
+          }
+        }
+        if(use_dlst&&I->R.displayList) {
+          glEndList();
+        }
       }
     }
   }
@@ -367,13 +372,14 @@ int RepSphereSameVis(RepSphere *I,CoordSet *cs)
 
 Rep *RepSphereNew(CoordSet *cs)
 {
+  PyMOLGlobals *G=cs->State.G;
   ObjectMolecule *obj;
   int a,b,c,a1,c1,a2,i,j,k,h,l;
   float *v,*v0,*vc,vdw,v1[3];
   float restart;
   int *q, *s,q0,q1,q2;
   int *lv,*lc,*cc;
-  SphereRec *sp = Sphere0; 
+  SphereRec *sp = G->Sphere->Sphere[0];
   int ds,*nt,flag;
   int *visFlag = NULL;
   MapType *map = NULL;
@@ -385,49 +391,53 @@ Rep *RepSphereNew(CoordSet *cs)
   float sphere_scale,sphere_add=0.0;
   int one_color;
   int *map_flag=NULL,*mf;
+  int cartoon_side_chain_helper = 0;
+  int ribbon_side_chain_helper = 0;
+  AtomInfoType *ati1;
+  int vis_flag;
 
 #ifdef _this_code_is_not_used
   float vv0[3],vv1[3],vv2[3];
   float tn[3],vt1[3],vt2[3],xtn[3],*tn0,*tn1,*tn2;
 #endif
 
-  OOAlloc(RepSphere);
+  OOAlloc(G,RepSphere);
 
   
   obj = cs->Obj;
   vFlag=false;
-  for(a=0;a<cs->NIndex;a++) {
-	 if(obj->AtomInfo[cs->IdxToAtm[a]].visRep[cRepSphere])
-		{
-		  vFlag=true;
-		  break;
-		}
-  }
+  if(obj->RepVisCache[cRepSphere])
+    for(a=0;a<cs->NIndex;a++) {
+      if(obj->AtomInfo[cs->IdxToAtm[a]].visRep[cRepSphere])
+        {
+          vFlag=true;
+          break;
+        }
+    }
   if(!vFlag) {
     OOFreeP(I);
     return(NULL); /* skip if no dots are visible */
   }
 
-  RepInit(&I->R);
-  ds = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_sphere_quality);
+  RepInit(G,&I->R);
+  ds = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_sphere_quality);
   if(ds<0) ds=0;
-  switch(ds) {
-  case 0: sp=Sphere0; break;
-  case 1: sp=Sphere1; break;
-  case 2: sp=Sphere2; break;
-  case 3: sp=Sphere3; break;
-  default: sp=Sphere4; break;
-  }
+  if(ds>4) ds=4;
+  sp = G->Sphere->Sphere[ds];
 
-  one_color=SettingGet_color(cs->Setting,obj->Obj.Setting,cSetting_sphere_color);
+  one_color=SettingGet_color(G,cs->Setting,obj->Obj.Setting,cSetting_sphere_color);
+  cartoon_side_chain_helper = SettingGet_b(G,cs->Setting, obj->Obj.Setting,
+                                         cSetting_cartoon_side_chain_helper);
+  ribbon_side_chain_helper = SettingGet_b(G,cs->Setting, obj->Obj.Setting,
+                                         cSetting_ribbon_side_chain_helper);
 
-  spheroid_scale=SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_spheroid_scale);
+  spheroid_scale=SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_spheroid_scale);
   if(spheroid_scale&&cs->Spheroid) 
     spheroidFlag=1;
   else
     spheroidFlag=0;
 
-  sphere_scale=SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sphere_scale);
+  sphere_scale=SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sphere_scale);
 
   I->R.fRender=(void (*)(struct Rep *, CRay *, Pickable **))RepSphereRender;
   I->R.fFree=(void (*)(struct Rep *))RepSphereFree;
@@ -443,7 +453,7 @@ Rep *RepSphereNew(CoordSet *cs)
   
 
   I->VC=(float*)mmalloc(sizeof(float)*cs->NIndex*7);
-  ErrChkPtr(I->VC);
+  ErrChkPtr(G,I->VC);
   I->NC=0;
   map_flag = Calloc(int,cs->NIndex);
 
@@ -452,22 +462,48 @@ Rep *RepSphereNew(CoordSet *cs)
   v=I->VC; 
   mf=map_flag;
 
-  if(SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_sphere_solvent)) { /* are we generating a solvent surface? */
-    sphere_add = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_solvent_radius); /* if so, get solvent radius */
+  if(SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_sphere_solvent)) { /* are we generating a solvent surface? */
+    sphere_add = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_solvent_radius); /* if so, get solvent radius */
   }
   
-  if(SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_pickable)) {
+  if(SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_pickable)) {
     I->R.P=Alloc(Pickable,cs->NIndex+1);
-    ErrChkPtr(I->R.P);
+    ErrChkPtr(G,I->R.P);
   }
 
   I->spheroidFlag=spheroidFlag;
   for(a=0;a<cs->NIndex;a++)
     {
       a1 = cs->IdxToAtm[a];
-      if(obj->AtomInfo[a1].visRep[cRepSphere])
-        {
+      ati1 = obj->AtomInfo+a1;
+      vis_flag = ati1->visRep[cRepSphere];
 
+      if(vis_flag &&
+         (!ati1->hetatm) &&
+         ((cartoon_side_chain_helper && ati1->visRep[cRepCartoon]) ||
+          (ribbon_side_chain_helper && ati1->visRep[cRepRibbon]))) {
+        
+        register char *name1=ati1->name;
+        register int prot1=ati1->protons;
+        if(prot1 == cAN_N) { 
+          if((!name1[1])&&(name1[0]=='N')) { /* N */
+            register char *resn1 = ati1->resn;
+            if(!((resn1[0]=='P')&&(resn1[1]=='R')&&(resn1[2]=='O')))
+              vis_flag=false;
+          }
+        } else if(prot1 == cAN_O) { 
+          if((!name1[1])&&(name1[0]=='O'))
+            vis_flag=false;
+        } else if(prot1 == cAN_C) {
+          if((!name1[1])&&(name1[0]=='C'))
+            vis_flag=false;
+        }
+      }
+      ati1->temp1 = vis_flag; /* store temporary visibility information */
+
+      if(vis_flag)
+        {
+          
           if(I->R.P) {
             I->NP++;
             
@@ -475,7 +511,7 @@ Rep *RepSphereNew(CoordSet *cs)
             I->R.P[I->NP].index = a1;
             I->R.P[I->NP].bond = -1;
           }
-
+          
           *mf=true;
           I->NC++;
           if(one_color==-1)
@@ -483,11 +519,11 @@ Rep *RepSphereNew(CoordSet *cs)
           else
             c1=one_color;
           v0 = cs->Coord+3*a;			 
-          if(ColorCheckRamped(c1)) {
-            ColorGetRamped(c1,v0,v);
+          if(ColorCheckRamped(G,c1)) {
+            ColorGetRamped(G,c1,v0,v);
             v+=3;
           } else {
-            vc = ColorGet(c1); /* save new color */
+            vc = ColorGet(G,c1); /* save new color */
             *(v++)=*(vc++);
             *(v++)=*(vc++);
             *(v++)=*(vc++);
@@ -499,7 +535,7 @@ Rep *RepSphereNew(CoordSet *cs)
         }
       mf++;
     }
-
+  
   if(I->NC) 
 	 I->VC=ReallocForSure(I->VC,float,(v-I->VC));
   else
@@ -510,30 +546,30 @@ Rep *RepSphereNew(CoordSet *cs)
   }
 
 
-  I->cullFlag = (int)SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_cull_spheres);
+  I->cullFlag = (int)SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_cull_spheres);
   if(spheroidFlag) I->cullFlag=false;
   if((I->cullFlag<2)&&
-     (SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpting))) 
+     (SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpting))) 
     /* optimize build-time performance when sculpting */
     I->cullFlag=false;
   if((I->cullFlag<2)&&
-     (SettingGet(cSetting_roving_spheres)!=0.0F))
+     (SettingGet(G,cSetting_roving_spheres)!=0.0F))
     I->cullFlag=false;
   if(I->cullFlag) {
 	 I->V=(float*)mmalloc(sizeof(float)*I->NC*(sp->NVertTot*30)); /* double check 30 */
-	 ErrChkPtr(I->V);
+	 ErrChkPtr(G,I->V);
 
 	 I->NT=Alloc(int,cs->NIndex);
-	 ErrChkPtr(I->NT);
+	 ErrChkPtr(G,I->NT);
 
 	 visFlag = Alloc(int,sp->nDot);
-	 ErrChkPtr(visFlag);
+	 ErrChkPtr(G,visFlag);
 
-    map=MapNewFlagged(MAX_VDW*sphere_scale+sphere_add,cs->Coord,cs->NIndex,NULL,map_flag);
+    map=MapNewFlagged(G,MAX_VDW*sphere_scale+sphere_add,cs->Coord,cs->NIndex,NULL,map_flag);
     if(map) MapSetupExpress(map);
   } else {
 	 I->V=(float*)mmalloc(sizeof(float)*I->NC*(3+sp->NVertTot*6));
-	 ErrChkPtr(I->V);
+	 ErrChkPtr(G,I->V);
   }
 
   /* rendering primitives */
@@ -546,7 +582,12 @@ Rep *RepSphereNew(CoordSet *cs)
   for(a=0;a<cs->NIndex;a++)
 	 {
 		a1 = cs->IdxToAtm[a];
-		if(obj->AtomInfo[a1].visRep[cRepSphere])
+      ati1 = obj->AtomInfo+a1;
+      vis_flag = ati1->temp1;
+      
+      /* don't show backbone atoms if side_chain_helper is on */
+
+		if(vis_flag)
 		  {
           if(one_color==-1)
             c1=*(cs->Color+a);
@@ -554,11 +595,11 @@ Rep *RepSphereNew(CoordSet *cs)
             c1=one_color;
 			 v0 = cs->Coord+3*a;
 			 vdw = cs->Obj->AtomInfo[a1].vdw*sphere_scale+sphere_add;
-          if(ColorCheckRamped(c1)) {
-            ColorGetRamped(c1,v0,v);
+          if(ColorCheckRamped(G,c1)) {
+            ColorGetRamped(G,c1,v0,v);
             v+=3;
           } else {
-            vc = ColorGet(c1);
+            vc = ColorGet(G,c1);
             *(v++)=*(vc++);
             *(v++)=*(vc++);
             *(v++)=*(vc++);
@@ -579,7 +620,7 @@ Rep *RepSphereNew(CoordSet *cs)
 						j=map->EList[i++];
 						while(j>=0) {
 						  a2 = cs->IdxToAtm[j];
-						  if(obj->AtomInfo[a2].visRep[cRepSphere]) {
+						  if(obj->AtomInfo[a2].temp1) {
 							 if(j!=a)
 								if(within3f(cs->Coord+3*j,v1,
                                     cs->Obj->AtomInfo[a2].vdw*
@@ -631,7 +672,7 @@ Rep *RepSphereNew(CoordSet *cs)
 							 j=map->EList[i++];
 							 while(j>=0) {
 								a2 = cs->IdxToAtm[j];
-								if(obj->AtomInfo[a2].visRep[cRepSphere]) {
+								if(obj->AtomInfo[a2].temp1) {
 								  if(j!=a)
 									 if(within3f(cs->Coord+3*j,v1,cs->Obj->AtomInfo[a2].vdw*sphere_scale+sphere_add))
 										{
@@ -752,7 +793,8 @@ Rep *RepSphereNew(CoordSet *cs)
 			 I->N++;
 			 if(nt) nt++;
 		  }
-	 }
+      }
+
 
   if(!I->LastVisib) I->LastVisib = Alloc(int,cs->NIndex);
   if(!I->LastColor) I->LastColor = Alloc(int,cs->NIndex);
@@ -764,13 +806,13 @@ Rep *RepSphereNew(CoordSet *cs)
   if(one_color==-1) 
     for(a=0;a<cs->NIndex;a++)
       {
-        *(lv++) = (ai2 + cs->IdxToAtm[a])->visRep[cRepSphere];
+        *(lv++) = (ai2 + cs->IdxToAtm[a])->temp1;
         *(lc++) = *(cc++);
       }
   else 
     for(a=0;a<cs->NIndex;a++)
       {
-        *(lv++) = (ai2 + cs->IdxToAtm[a])->visRep[cRepSphere];
+        *(lv++) = (ai2 + cs->IdxToAtm[a])->temp1;
         *(lc++) = one_color;
       }
 
