@@ -344,12 +344,13 @@ void RepWireBondFree(RepWireBond *I)
 {
   FreeP(I->VP);
   FreeP(I->V);
-  RepFree(&I->R);
+  RepPurge(&I->R);
   OOFreeP(I);
 }
 
 void RepWireBondRender(RepWireBond *I,CRay *ray,Pickable **pick)
 {
+  PyMOLGlobals *G=I->R.G;
   float *v=I->V;
   int c=I->N;
   unsigned int i,j;
@@ -374,90 +375,94 @@ void RepWireBondRender(RepWireBond *I,CRay *ray,Pickable **pick)
 		v+=9;
 	 }
 
-  } else if(pick&&PMGUI) {
+  } else if(G->HaveGUI && G->ValidContext) {
+    if(pick) {
 	 
-	 i=(*pick)->index;
+      i=(*pick)->index;
 
-	 v=I->VP;
-	 c=I->NP;
-	 p=I->R.P;
+      v=I->VP;
+      c=I->NP;
+      p=I->R.P;
 
-	 glBegin(GL_LINES);
+      glBegin(GL_LINES);
 	 
-	 while(c--) {
+      while(c--) {
 
-		i++;
+        i++;
 
-		if(!(*pick)[0].ptr) {
-		  /* pass 1 - low order bits */
+        if(!(*pick)[0].ptr) {
+          /* pass 1 - low order bits */
         
-        glColor3ub((uchar)((i&0xF)<<4),(uchar)((i&0xF0)|0x8),(uchar)((i&0xF00)>>4)); /* we're encoding the index into the color */
-		  VLACheck((*pick),Pickable,i);
-		  p++;
-		  (*pick)[i] = *p; /* copy object and atom info */
-		} else { 
-		  /* pass 2 - high order bits */
+          glColor3ub((uchar)((i&0xF)<<4),(uchar)((i&0xF0)|0x8),(uchar)((i&0xF00)>>4)); /* we're encoding the index into the color */
+          VLACheck((*pick),Pickable,i);
+          p++;
+          (*pick)[i] = *p; /* copy object and atom info */
+        } else { 
+          /* pass 2 - high order bits */
 
-		  j=i>>12;
+          j=i>>12;
 
           glColor3ub((uchar)((j&0xF)<<4),(uchar)((j&0xF0)|0x8),(uchar)((j&0xF00)>>4)); 
 
-		}			 
+        }			 
 
-		glVertex3fv(v);
-		v+=3;
-		glVertex3fv(v);
-		v+=3;
-
-	 }
-	 glEnd();
-	 (*pick)[0].index = i; /* pass the count */
-  } else if(PMGUI) {
-
-    int use_dlst;
-    use_dlst = (int)SettingGet(cSetting_use_display_lists);
-    if(use_dlst&&I->R.displayList) {
-      glCallList(I->R.displayList);
-    } else { 
-
-      if(use_dlst) {
-        if(!I->R.displayList) {
-          I->R.displayList = glGenLists(1);
-          if(I->R.displayList) {
-            glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
-          }
-        }
-      }
-      
-      glLineWidth(I->Width);
-      
-      v=I->V;
-      c=I->N;
-      
-      glDisable(GL_LIGHTING); 
-      glBegin(GL_LINES);	 
-      SceneResetNormal(true);
-      while(c--) {
-        glColor3fv(v);
-        v+=3;
         glVertex3fv(v);
         v+=3;
         glVertex3fv(v);
         v+=3;
+
       }
       glEnd();
-      glEnable(GL_LIGHTING);
+      (*pick)[0].index = i; /* pass the count */
+    } else {
+      int use_dlst;
+      use_dlst = (int)SettingGet(G,cSetting_use_display_lists);
+            
       if(use_dlst&&I->R.displayList) {
-        glEndList();
-      }
-    } 
+        glCallList(I->R.displayList);
+      } else { 
+
+        if(use_dlst) {
+          if(!I->R.displayList) {
+            I->R.displayList = glGenLists(1);
+            if(I->R.displayList) {
+              glNewList(I->R.displayList,GL_COMPILE_AND_EXECUTE);
+            }
+          }
+        }
+      
+        glLineWidth(I->Width);
+      
+        v=I->V;
+        c=I->N;
+      
+        glDisable(GL_LIGHTING); 
+        glBegin(GL_LINES);	 
+        SceneResetNormal(G,true);
+        while(c--) {
+          glColor3fv(v);
+          v+=3;
+          glVertex3fv(v);
+          v+=3;
+          glVertex3fv(v);
+          v+=3;
+        }
+        glEnd();
+        glEnable(GL_LIGHTING);
+        if(use_dlst&&I->R.displayList) {
+          glEndList();
+        }
+      } 
+    }
   }
 }
 
 Rep *RepWireBondNew(CoordSet *cs)
 {
+  PyMOLGlobals *G=cs->State.G;
   ObjectMolecule *obj;
-  int a,a1,a2,c1,c2,s1,s2,b1,b2,ord;
+  register int a1, a2, b1, b2;
+  int a,c1,c2,s1,s2,ord;
   BondType *b;
   int half_bonds,*other=NULL;
   float valence;
@@ -467,40 +472,54 @@ Rep *RepWireBondNew(CoordSet *cs)
   int maxBond = 0;
   float tmpColor[3];
   int valence_flag = false;
-
   Pickable *rp;
   AtomInfoType *ai1,*ai2;
-  OOAlloc(RepWireBond);
+  int cartoon_side_chain_helper = 0;
+  int ribbon_side_chain_helper = 0;
+  int line_stick_helper = 0;
+
+  OOAlloc(G,RepWireBond);
   obj = cs->Obj;
 
-  PRINTFD(FB_RepWireBond)
+  PRINTFD(G,FB_RepWireBond)
     " RepWireBondNew-Debug: entered.\n"
     ENDFD;
 
   visFlag=false;
   b=obj->Bond;
-  for(a=0;a<obj->NBond;a++)
-    {
-      b1 = b->index[0];
-      b2 = b->index[1];
-      b++;
-      if(obj->AtomInfo[b1].visRep[cRepLine]||
-         obj->AtomInfo[b2].visRep[cRepLine]) {
-        visFlag=true;
-        break;
+  if(obj->RepVisCache[cRepLine])
+    for(a=0;a<obj->NBond;a++)
+      {
+        b1 = b->index[0];
+        b2 = b->index[1];
+        b++;
+        if(obj->AtomInfo[b1].visRep[cRepLine]||
+           obj->AtomInfo[b2].visRep[cRepLine]) {
+          visFlag=true;
+          break;
+        }
       }
-    }
   if(!visFlag) {
     OOFreeP(I);
     return(NULL); /* skip if no dots are visible */
   }
 
-  valence = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_valence);
+  valence = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_valence);
   if(valence==1.0F) /* backwards compatibility... */
-    valence = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_valence_size);
+    valence = SettingGet_f(G,cs->Setting,
+                           obj->Obj.Setting,cSetting_valence_size);  
   valence_flag = (valence!=0.0F);
+  cartoon_side_chain_helper = SettingGet_b(G,cs->Setting, obj->Obj.Setting,
+                                         cSetting_cartoon_side_chain_helper);
+  ribbon_side_chain_helper = SettingGet_b(G,cs->Setting, obj->Obj.Setting,
+                                         cSetting_ribbon_side_chain_helper);
+  line_stick_helper = SettingGet_b(G,cs->Setting, obj->Obj.Setting,
+                                   cSetting_line_stick_helper);
 
-  half_bonds = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_half_bonds);
+  if(line_stick_helper && (SettingGet_f(G,cs->Setting, obj->Obj.Setting,
+                                        cSetting_stick_transparency)>R_SMALL4))
+    line_stick_helper = false;
+  half_bonds = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_half_bonds);
 
   b=obj->Bond;
   for(a=0;a<obj->NBond;a++)
@@ -538,12 +557,12 @@ Rep *RepWireBondNew(CoordSet *cs)
       b++;
     }
   
-  RepInit(&I->R);
+  RepInit(G,&I->R);
 
   I->R.fRender=(void (*)(struct Rep *, CRay *, Pickable **))RepWireBondRender;
   I->R.fFree=(void (*)(struct Rep *))RepWireBondFree;
-  I->Width = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_line_width);
-  I->Radius = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_line_radius);
+  I->Width = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_line_width);
+  I->Radius = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_line_radius);
 
 
   I->N=0;
@@ -559,7 +578,79 @@ Rep *RepWireBondNew(CoordSet *cs)
       other=ObjectMoleculeGetPrioritizedOtherIndexList(obj,cs);
     
 	 I->V=(float*)mmalloc(sizeof(float)*maxSegment*9);
-	 ErrChkPtr(I->V);
+	 ErrChkPtr(G,I->V);
+
+    if(cartoon_side_chain_helper || ribbon_side_chain_helper) {
+      /* several extra passes required for these fancy features */
+
+      /* pass 1, clear the temp1 field for all bonded atoms */
+
+      b=obj->Bond;
+      for(a=0;a<obj->NBond;a++)
+        {
+          b1 = b->index[0];
+          b2 = b->index[1];
+          ord = b->order;
+          b++;
+          
+          if(obj->DiscreteFlag) {
+            if((cs==obj->DiscreteCSet[b1])&&(cs==obj->DiscreteCSet[b2])) {
+              a1=obj->DiscreteAtmToIdx[b1];
+              a2=obj->DiscreteAtmToIdx[b2];
+            } else {
+              a1=-1;
+              a2=-1;
+            }
+          } else {
+            a1=cs->AtmToIdx[b1];
+            a2=cs->AtmToIdx[b2];
+          }
+          if((a1>=0)&&(a2>=0)) {
+            obj->AtomInfo[b1].temp1 = 0;
+            obj->AtomInfo[b2].temp1 = 0;
+          }
+        }
+
+      /* pass 2, set the temp1 field for atoms that are bonded to atoms without a
+         visible cartoon or ribbon */
+
+      b=obj->Bond;
+      for(a=0;a<obj->NBond;a++)
+        {
+          b1 = b->index[0];
+          b2 = b->index[1];
+          ord = b->order;
+          b++;
+          
+          if(obj->DiscreteFlag) {
+            if((cs==obj->DiscreteCSet[b1])&&(cs==obj->DiscreteCSet[b2])) {
+              a1=obj->DiscreteAtmToIdx[b1];
+              a2=obj->DiscreteAtmToIdx[b2];
+            } else {
+              a1=-1;
+              a2=-1;
+            }
+          } else {
+            a1=cs->AtmToIdx[b1];
+            a2=cs->AtmToIdx[b2];
+          }
+          if((a1>=0)&&(a2>=0)) {
+            register AtomInfoType *ati1=obj->AtomInfo+b1;
+            register AtomInfoType *ati2=obj->AtomInfo+b2;
+
+            if((!ati1->hetatm) && (!ati2->hetatm)) {
+              if(((cartoon_side_chain_helper && ati1->visRep[cRepCartoon] && !ati2->visRep[cRepCartoon]) ||
+                  (ribbon_side_chain_helper && ati1->visRep[cRepRibbon] && !ati2->visRep[cRepRibbon]))) {
+                ati1->temp1 = 1;
+              }
+              if(((cartoon_side_chain_helper && ati2->visRep[cRepCartoon] && !ati1->visRep[cRepCartoon]) ||
+                  (ribbon_side_chain_helper && ati2->visRep[cRepRibbon] && !ati1->visRep[cRepRibbon]))) {
+                ati2->temp1 = 1;
+              }
+            }
+          }
+        }
+    }
 	 	 
 	 v=I->V;
 	 b=obj->Bond;
@@ -588,27 +679,115 @@ Rep *RepWireBondNew(CoordSet *cs)
         }
 		  if((a1>=0)&&(a2>=0))
 			 {
-				s1=obj->AtomInfo[b1].visRep[cRepLine];
-				s2=obj->AtomInfo[b2].visRep[cRepLine];
+            register AtomInfoType *ati1=obj->AtomInfo+b1;
+            register AtomInfoType *ati2=obj->AtomInfo+b2;
 
-				if(!(s1&&s2))
+				s1=ati1->visRep[cRepLine];
+				s2=ati2->visRep[cRepLine];
+
+				if((s1||s2)&&!(s1&&s2))
               if(!half_bonds) {
-                s1 = 0;
-                s2 = 0;
+                if(line_stick_helper && 
+                    ( ( (!s1) && ati1->visRep[cRepCyl] && (!ati2->visRep[cRepCyl])) ||
+                      ( (!s2) && ati2->visRep[cRepCyl] && (!ati1->visRep[cRepCyl]))))
+                  s1 = s2 = 1; /* turn on line when both stick and line are alternately shown */
+                else {
+                  s1 = 0;
+                  s2 = 0;
+                }
               }
 
 				if(s1||s2)
 				  {	
+
+
 					 c1=*(cs->Color+a1);
 					 c2=*(cs->Color+a2);
 					 
 					 v1 = cs->Coord+3*a1;
 					 v2 = cs->Coord+3*a2;
 					 
-					 if((c1==c2)&&s1&&s2&&(!ColorCheckRamped(c1))) {
+
+                if( (!ati1->hetatm) && (!ati2->hetatm) &&
+                    ((cartoon_side_chain_helper && ati1->visRep[cRepCartoon] && ati2->visRep[cRepCartoon]) ||
+                     (ribbon_side_chain_helper && ati1->visRep[cRepRibbon] && ati2->visRep[cRepRibbon]))) {
+
+                    register char *name1=ati1->name;
+                    register int prot1=ati1->protons;
+                    register char *name2=ati2->name;
+                    register int prot2=ati2->protons;
+
+                    if(prot1 == cAN_C) { 
+                      if((name1[1]=='A')&&(name1[0]=='C')&&(!name1[2])) { /* CA */
+                        if(prot2 == cAN_C) { 
+                          if((name2[1]=='B')&&(name2[0]=='C')&&(!name2[2]))
+                            c1 = c2;  /* CA-CB */
+                          else if((!name2[1])&&(name2[0]=='C')&&(!ati2->temp1))
+                            s1 = s2 = 0; /* suppress CA-C */
+                        } else if(prot2 == cAN_H) 
+                          s1 = s2 = 0; /* suppress all CA-hydrogens */
+                      }
+                    } else if(prot1 == cAN_N) { 
+                      if((!name1[1])&&(name1[0]=='N')) { /* N */
+                        if(prot2 == cAN_C) {
+                          if((name2[1]=='D')&&(name2[0]=='C')&&(!name2[2])) 
+                            c1 = c2; /* N->CD in PRO */
+                          else if((name2[1]=='A')&&(name2[0]=='C')&&(!name2[2])&&(!ati1->temp1))
+                            s1 = s2 = 0; /* suppress N-CA */
+                          else if((!name2[1])&&(name2[0]=='C')&&(!ati1->temp1))
+                            s1 = s2 = 0; /* suppress N-C */
+                        } else if(prot2 == cAN_H)
+                          s1 = s2 = 0; /* suppress all N-hydrogens */
+                      }
+                    } else if((prot1 == cAN_O)&&(prot2 == cAN_C)) { 
+                      if((!name2[1])&&(name2[0]=='C')&&
+                         (((!name1[1])&&(name1[0]=='O'))||
+                          ((name1[3]==0)&&(name1[2]=='T')&&(name1[1]=='X')&&(name1[0]=='O')))
+                         &&(!ati2->temp1))
+                        s1 = s2 = 0; /* suppress C-O,OXT */
+                    }
+                    
+                    if(prot2 == cAN_C) {
+                      if((name2[1]=='A')&&(name2[0]=='C')&&(!name2[2])) { /* CA */
+                        if(prot1 == cAN_C) { 
+                          if((name1[1]=='B')&&(name1[0]=='C')&&(!name1[2]))
+                            c2 = c1; /* CA-CB */
+                          else if((!name1[1])&&(name1[0]=='C')&&(!ati1->temp1))
+                            s1 = s2 = 0; /* suppress CA-C */
+                        } else if(prot1 == cAN_H) 
+                          s1 = s2 = 0; /* suppress all CA-hydrogens */
+                      }
+                    } else if(prot2 == cAN_N) {
+                      if((!name2[1])&&(name2[0]=='N')) { /* N */
+                        if(prot1 == cAN_C) { 
+                          if((name1[1]=='D')&&(name1[0]=='C')&&(!name1[2]))
+                            c2 = c1; /* N->CD in PRO */
+                          else if((name1[1]=='A')&&(name1[0]=='C')&&(ati2->temp1))
+                            s1 = s2 = 0; /* suppress N-CA */
+                          else if((!name1[1])&&(name1[0]=='C')&&(!ati2->temp1))
+                            s1 = s2 = 0; /* suppress N-C */
+                        } else if(prot1 == cAN_H)
+                          s1 = s2 = 0; /* suppress all N-hydrogens */
+                      }
+                    } else if((prot2 == cAN_O)&&(prot1 == cAN_C)) {
+                      if((!name1[1])&&(name1[0]=='C')&&
+                         (((!name2[1])&&(name2[0]=='O'))||
+                          ((name2[3]==0)&&(name2[2]=='T')&&(name2[1]=='X')&&(name2[0]=='O')))
+                         &&(!ati1->temp1))
+                        s1 = s2 = 0; /* suppress C-O,OXT */
+                    }
+                }
+                
+                if(line_stick_helper) {
+                  if(ati1->visRep[cRepCyl] && 
+                     ati2->visRep[cRepCyl])
+                    s1 = s2 = 0;
+                }
+
+					 if((c1==c2)&&s1&&s2&&(!ColorCheckRamped(G,c1))) {
 						
 
-						v0 = ColorGet(c1);
+						v0 = ColorGet(G,c1);
 
                   if((valence_flag)&&(ord>1)&&(ord<4)) {
                     RepValence(v,v1,v2,other,a1,a2,cs->Coord,v0,ord,valence);
@@ -639,11 +818,11 @@ Rep *RepWireBondNew(CoordSet *cs)
 						if(s1)
 						  {
                       
-                      if(ColorCheckRamped(c1)) {
-                        ColorGetRamped(c1,v1,tmpColor);
+                      if(ColorCheckRamped(G,c1)) {
+                        ColorGetRamped(G,c1,v1,tmpColor);
                         v0=tmpColor;
                       } else {
-                        v0 = ColorGet(c1);
+                        v0 = ColorGet(G,c1);
                       }
 
                       if((valence_flag)&&(ord>1)&&(ord<4)) {
@@ -670,11 +849,11 @@ Rep *RepWireBondNew(CoordSet *cs)
                     }
 						if(s2)
 						  {
-                      if(ColorCheckRamped(c2)) {
-                        ColorGetRamped(c2,v2,tmpColor);
+                      if(ColorCheckRamped(G,c2)) {
+                        ColorGetRamped(G,c2,v2,tmpColor);
                         v0 = tmpColor;
                       } else {
-                        v0 = ColorGet(c2);
+                        v0 = ColorGet(G,c2);
                       }
                       if((valence_flag)&&(ord>1)&&(ord<4)) {
                         RepValence(v,h,v2,other,a1,a2,cs->Coord,v0,ord,valence);
@@ -699,20 +878,20 @@ Rep *RepWireBondNew(CoordSet *cs)
                       
 						  }
 					 }
-				  }
-			 }
-		}
-
-	 I->V = ReallocForSure(I->V,float,(v-I->V));
+              }
+          }
+      }
+    
+    I->V = ReallocForSure(I->V,float,(v-I->V));
 
 	 /* now create pickable verson */
 
-	 if(SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_pickable)) {
+	 if(SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_pickable)) {
 		I->VP=(float*)mmalloc(sizeof(float)*maxBond*6*2);
-		ErrChkPtr(I->VP);
+		ErrChkPtr(G,I->VP);
 		
 		I->R.P=Alloc(Pickable,2*maxBond+1);
-		ErrChkPtr(I->R.P);
+		ErrChkPtr(G,I->R.P);
 		rp = I->R.P + 1; /* skip first record! */
 
 		v=I->VP;
