@@ -74,7 +74,7 @@ static float ShakerDoDist(float target,float *v0,float *v1,float *d0to1,float *d
   subtract3f(v0,v1,d);
   len = (float)length3f(d);
   dev = target-len;
-  if((result=fabs(dev))>R_SMALL8) {
+  if((result=(float)fabs(dev))>R_SMALL8) {
     dev_2 = wt*dev/2.0F;
     if(len>R_SMALL8) { /* nonoverlapping */
       sc = dev_2/len;
@@ -82,8 +82,14 @@ static float ShakerDoDist(float target,float *v0,float *v1,float *d0to1,float *d
       add3f(push,d0to1,d0to1);
       subtract3f(d1to0,push,d1to0);
     } else { /* overlapping, so just push along X */
-      d0to1[0]-=dev_2;
-      d1to0[0]+=dev_2;
+      float rd[3];
+      get_random3f(rd);
+      d0to1[0]-=rd[0]*dev_2;
+      d1to0[0]+=rd[0]*dev_2;
+      d0to1[1]-=rd[1]*dev_2;
+      d1to0[1]+=rd[1]*dev_2;
+      d0to1[2]-=rd[2]*dev_2;
+      d1to0[2]+=rd[2]*dev_2;
     }
   } else
     result = 0.0;
@@ -140,14 +146,19 @@ static float ShakerDoTors(int type,float *v0,float *v1,float *v2,float *v3,
 
   switch(type) {
   case cShakerTorsSP3SP3:
-    
     if(dp>-0.5F)
       return 0.0F;
-    result = fabs(dp)-0.5F;
-    
+    result = ((float)fabs(dp))-0.5F;
     if(result<tole) /* discontinuous low bottom well */
       result = result/25.F;       
     break;
+  case cShakerTorsAmide: /* strongly favor trans over cis */
+    if((dp>-0.9F)&&(dp<1.0f)) {
+      result = 1.0F-dp;
+    } else if((dp<-0.9F)&&(dp>-1.0F))
+      result = -1.0F-dp;
+    result *= 20.0F; /* emphasize */
+    break;
   case cShakerTorsDisulfide:
     if(fabs(dp)<tole)
       return 0.0F;
@@ -158,9 +169,9 @@ static float ShakerDoTors(int type,float *v0,float *v1,float *v2,float *v3,
   }
 
 
-    cross_product3f(perp0,perp1,dir);
-    sign = dot_product3f(axis,dir);
-
+  cross_product3f(perp0,perp1,dir);
+  sign = dot_product3f(axis,dir);
+  
   if(sign<0.0F)
     sc=wt*result;
   else
@@ -202,10 +213,11 @@ static float ShakerDoTors(int type,float *v0,float *v1,float *v2,float *v3,
   
 }
 
-CSculpt *SculptNew(void)
+CSculpt *SculptNew(PyMOLGlobals *G)
 {
-  OOAlloc(CSculpt);
-  I->Shaker = ShakerNew();
+  OOAlloc(G,CSculpt);
+  I->G=G;
+  I->Shaker = ShakerNew(G);
   I->NBList = VLAlloc(int,150000);
   I->NBHash = Alloc(int,NB_HASH_SIZE);
   I->EXList = VLAlloc(int,100000);
@@ -222,6 +234,7 @@ CSculpt *SculptNew(void)
 
 void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
 {
+  PyMOLGlobals *G=I->G;
   int a,a0,a1,a2,a3,b0,b1,b2,b3;
   BondType *b;
   float *v0,*v1,*v2,*v3,d,dummy;
@@ -229,6 +242,7 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
   int n0,n1,n2;
   int *planer = NULL;
   int *linear = NULL;
+  int *single = NULL;
   int nex = 1;
   int *j,*k,xhash;
   int ex_type;
@@ -236,7 +250,7 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
   int xoffset;
   int use_cache = 1;
 
-  PRINTFD(FB_Sculpt)
+  PRINTFD(G,FB_Sculpt)
     " SculptMeasureObject-Debug: entered.\n"
     ENDFD;
 
@@ -257,7 +271,7 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
           I->Don[a]=false;
           I->Acc[a]=false;
           if(!ai->sculpt_id) /* insure all atoms have unique sculpt IDs */
-            ai->sculpt_id=SculptCacheNewID();
+            ai->sculpt_id=SculptCacheNewID(G);
           ai++;
         }
         
@@ -266,15 +280,17 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
 
         cs = obj->CSet[state];
 
-        use_cache = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_sculpt_memory);
+        use_cache = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_memory);
         if(obj->NBond) {
 
           planer=Alloc(int,obj->NAtom);
           linear=Alloc(int,obj->NAtom);
+          single=Alloc(int,obj->NAtom);
           ai = obj->AtomInfo;
           for(a=0;a<obj->NAtom;a++) {
             planer[a]=(ai->geom==cAtomInfoPlaner);
             linear[a]=(ai->geom==cAtomInfoLinear);
+            single[a]=(ai->geom==cAtomInfoSingle);
             ai++;
           }
 
@@ -462,10 +478,10 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
                   v2 = cs->Coord+3*a2;
                   d = (float)diff3f(v1,v2);
                   if(use_cache) {
-                    if(!SculptCacheQuery(cSculptBond,
+                    if(!SculptCacheQuery(G,cSculptBond,
                                          oai[b1].sculpt_id,
                                          oai[b2].sculpt_id,0,0,&d))
-                      SculptCacheStore(cSculptBond,
+                      SculptCacheStore(G,cSculptBond,
                                        oai[b1].sculpt_id,
                                        oai[b2].sculpt_id,0,0,d);
                   }
@@ -524,11 +540,11 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
                   v2 = cs->Coord+3*a2;
                   d = (float)diff3f(v1,v2);
                   if(use_cache) {
-                    if(!SculptCacheQuery(cSculptAngl,
+                    if(!SculptCacheQuery(G,cSculptAngl,
                                          oai[b0].sculpt_id,
                                          oai[b1].sculpt_id,
                                          oai[b2].sculpt_id,0,&d))
-                      SculptCacheStore(cSculptAngl,
+                      SculptCacheStore(G,cSculptAngl,
                                        oai[b0].sculpt_id,
                                        oai[b1].sculpt_id,
                                        oai[b2].sculpt_id,0,d);
@@ -539,11 +555,11 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
                   if(linear[b0]&&(linear[b1]||linear[b2])) {
                     
                     if(use_cache) {
-                      if(!SculptCacheQuery(cSculptLine,
+                      if(!SculptCacheQuery(G,cSculptLine,
                                            oai[b1].sculpt_id,
                                            oai[b0].sculpt_id,
                                            oai[b2].sculpt_id,0,&dummy))
-                        SculptCacheStore(cSculptLine,
+                        SculptCacheStore(G,cSculptLine,
                                          oai[b1].sculpt_id,
                                          oai[b0].sculpt_id,
                                          oai[b2].sculpt_id,0,0.0);
@@ -603,13 +619,13 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
                     if(planer[b0])
                       d=0.0;
                     if(use_cache) {
-                      if(!SculptCacheQuery(cSculptPyra,
+                      if(!SculptCacheQuery(G,cSculptPyra,
                                            oai[b1].sculpt_id,
                                            oai[b0].sculpt_id,
                                            oai[b2].sculpt_id,
                                            oai[b3].sculpt_id,
                                            &d))
-                        SculptCacheStore(cSculptPyra,
+                        SculptCacheStore(G,cSculptPyra,
                                          oai[b1].sculpt_id,
                                          oai[b0].sculpt_id,
                                          oai[b2].sculpt_id,
@@ -630,27 +646,33 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
         
           for(b0=0;b0<obj->NAtom;b0++) {
             n0 = obj->Neighbor[b0]+1;
-            while(obj->Neighbor[n0]>=0) {
-              b1 = obj->Neighbor[n0];
-
+            while((b1 = obj->Neighbor[n0])>=0) {
               n1 = obj->Neighbor[b0]+1;
-              while(obj->Neighbor[n1]>=0) {
-                b2 = obj->Neighbor[n1];
-
+              while((b2 = obj->Neighbor[n1])>=0) {
                 if(b1!=b2) {
                   n2 =  obj->Neighbor[b2]+1;
-                  while(obj->Neighbor[n2]>=0) {
-                    b3 = obj->Neighbor[n2];
+                  while((b3 = obj->Neighbor[n2])>=0) {
                     if((b3!=b0)&&(b3>b1)) {
-                      
                       if(!(planer[b0]||planer[b2]||linear[b0]||linear[b2])) {
                         int type;
                         if((oai[b0].protons == cAN_S)&&
                            (oai[b2].protons == cAN_S))
                           type = cShakerTorsDisulfide;
-                        else
+                        else 
                           type = cShakerTorsSP3SP3;
                         ShakerAddTorsCon(I->Shaker,b1,b0,b2,b3,type);
+                      } 
+                      if(planer[b0]&&planer[b2]) {
+                        if(((oai[b1].protons == cAN_H)&&single[b1]&&
+                            (oai[b0].protons == cAN_N)&&
+                            (oai[b2].protons == cAN_C)&&
+                            (oai[b3].protons == cAN_O)&&planer[b3])||
+                           ((oai[b1].protons == cAN_O)&&planer[b1]&&
+                            (oai[b0].protons == cAN_C)&&
+                            (oai[b2].protons == cAN_N)&&
+                            (oai[b3].protons == cAN_H)&&single[b3])) {
+                          ShakerAddTorsCon(I->Shaker,b1,b0,b2,b3,cShakerTorsAmide);
+                        }
                       }
                       /* check 1-4 exclusion */
                       xhash = ex_hash(b1,b3);
@@ -723,26 +745,98 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
                         v3 = cs->Coord+3*a3;
                         
                         d = 0.0;
-                        if(fabs(get_dihedral3f(v1,v0,v2,v3))<deg_to_rad(10.0))
-                          if(planer[b0]&&planer[b2]) {
+                        if(planer[b0]&&planer[b2]) {
+                          float deg = get_dihedral3f(v1,v0,v2,v3);
+                          if(fabs(deg)<deg_to_rad(10.0))
                             d = 1.0; 
+                          else if(fabs(deg)>deg_to_rad(170))
+                            d = -1.0;
+                          
+                          /*
+                            if(!(((oai[b1].protons == cAN_H)&&single[b1]&&
+                            (oai[b0].protons == cAN_N)&&planer[b0]&&
+                            (oai[b2].protons == cAN_C)&&planer[b2]&&
+                            (oai[b3].protons == cAN_O)&&planer[b3])||
+                            ((oai[b1].protons == cAN_O)&&planer[b1]&&
+                            (oai[b0].protons == cAN_C)&&planer[b0]&&
+                            (oai[b2].protons == cAN_N)&&planer[b2]&&
+                            (oai[b3].protons == cAN_H)&&single[b3]))) 
+                          */
+                          {
+                            int fixed = false;
+                            /* look for 4, 5, 6, 7, or 8 cycle that connects back to b1 
+                               if found, then this planer system is fixed (it can't flip it over) */
+                            {
+                              int b4,b5,b6,b7,b8,b9,b10;
+                              int n3,n4,n5,n6,n7,n8,n9;
+                              n3 = obj->Neighbor[b2]+1;
+                              while((!fixed)&&(b4 = obj->Neighbor[n3])>=0) {
+                                if(b4!=b0) {
+                                  n4 = obj->Neighbor[b4]+1;
+                                  while((!fixed)&&(b5 = obj->Neighbor[n4])>=0) {
+                                    if(b5!=b2) {
+                                      n5 = obj->Neighbor[b5]+1;
+                                      while((!fixed)&&(b6 = obj->Neighbor[n5])>=0) {
+                                        if(b6==b0) { /* 4-cycle */
+                                          fixed=true;
+                                        } else if(b6!=b4) {
+                                          n6 = obj->Neighbor[b6]+1;
+                                          while((!fixed)&&(b7 = obj->Neighbor[n6])>=0) {
+                                            if(b7==b0) {  /* 5-cycle */
+                                              fixed=true;
+                                            } else if(b7!=b5) {
+                                              n7 = obj->Neighbor[b7]+1;
+                                              while((!fixed)&&(b8 = obj->Neighbor[n7])>=0) {
+                                                if(b8==b0) {  /* 6-cycle */
+                                                  fixed=true;
+                                                } else if(b8!=b6) {
+                                                  n8 = obj->Neighbor[b8]+1;
+                                                  while((!fixed)&&(b9 = obj->Neighbor[n8])>=0) {
+                                                    if(b9==b0) {  /* 7-cycle */
+                                                      fixed=true;
+                                                    } else if(b9!=b7) {
+                                                      n9 = obj->Neighbor[b9]+1;
+                                                      while((!fixed)&&(b10 = obj->Neighbor[n9])>=0) {
+                                                        if(b10==b0) {  /* 8-cycle */
+                                                          fixed=true;
+                                                        } 
+                                                        n9+=2;
+                                                      }
+                                                    }
+                                                    n8+=2;
+                                                  }
+                                                }
+                                                n7+=2;
+                                              }
+                                            }
+                                            n6+=2;
+                                          }
+                                        }
+                                        n5+=2;
+                                      }
+                                    }
+                                    n4+=2;
+                                  }
+                                }
+                                n3+=2;
+                              }
+                            }
+                            if(use_cache) {
+                              if(!SculptCacheQuery(G,cSculptPlan,
+                                                   oai[b1].sculpt_id,
+                                                   oai[b0].sculpt_id,
+                                                   oai[b2].sculpt_id,
+                                                   oai[b3].sculpt_id,
+                                                   &d))
+                                SculptCacheStore(G,cSculptPlan,
+                                                 oai[b1].sculpt_id,
+                                                 oai[b0].sculpt_id,
+                                                 oai[b2].sculpt_id,
+                                                 oai[b3].sculpt_id,
+                                                 d);
+                            }
+                            ShakerAddPlanCon(I->Shaker,b1,b0,b2,b3,d,fixed); 
                           }
-                        if(use_cache) {
-                          if(!SculptCacheQuery(cSculptPlan,
-                                               oai[b1].sculpt_id,
-                                               oai[b0].sculpt_id,
-                                               oai[b2].sculpt_id,
-                                               oai[b3].sculpt_id,
-                                               &d))
-                            SculptCacheStore(cSculptPlan,
-                                             oai[b1].sculpt_id,
-                                             oai[b0].sculpt_id,
-                                             oai[b2].sculpt_id,
-                                             oai[b3].sculpt_id,
-                                             d);
-                        }
-                        if(d>0.0) {
-                          ShakerAddPlanCon(I->Shaker,b1,b0,b2,b3); 
                         }
                       }
                     }
@@ -756,25 +850,25 @@ void SculptMeasureObject(CSculpt *I,ObjectMolecule *obj,int state)
           }
           FreeP(planer);
           FreeP(linear);
+          FreeP(single);
         }
       }
-
-  PRINTFB(FB_Sculpt,FB_Blather)
+  
+  PRINTFB(G,FB_Sculpt,FB_Blather)
     " Sculpt: I->Shaker->NDistCon %d\n",I->Shaker->NDistCon
-    ENDFB;
-  PRINTFB(FB_Sculpt,FB_Blather)
+    ENDFB(G);
+  PRINTFB(G,FB_Sculpt,FB_Blather)
     " Sculpt: I->Shaker->NPyraCon %d\n",I->Shaker->NPyraCon
-    ENDFB;
-  PRINTFB(FB_Sculpt,FB_Blather)
+    ENDFB(G);
+  PRINTFB(G,FB_Sculpt,FB_Blather)
     " Sculpt: I->Shaker->NPlanCon %d\n",I->Shaker->NPlanCon
-    ENDFB;
-
-
- PRINTFD(FB_Sculpt)
+    ENDFB(G);
+  
+  
+  PRINTFD(G,FB_Sculpt)
     " SculptMeasureObject-Debug: leaving...\n"
     ENDFD;
-
-
+  
 }
 #ifdef _PYMOL_INLINE
 __inline__
@@ -806,7 +900,7 @@ static int SculptDoBump(float target,float actual,float *d,
   float dev,dev_2,sc,abs_dev;
 
   dev = target-actual;
-  if((abs_dev=fabs(dev))>R_SMALL8) {
+  if((abs_dev=(float)fabs(dev))>R_SMALL8) {
     dev_2 = wt*dev/2.0F;
     (*strain) += abs_dev;
     if(actual>R_SMALL8) { /* nonoverlapping */
@@ -827,6 +921,7 @@ static int SculptDoBump(float target,float actual,float *d,
        
 float SculptIterateObject(CSculpt *I,ObjectMolecule *obj,int state,int n_cycle)
 {
+  PyMOLGlobals *G=I->G;
   CShaker *shk;
   int a,aa,a0,a1,a2,a3,b0,b1,b2,b3;
   CoordSet *cs;
@@ -871,7 +966,7 @@ float SculptIterateObject(CSculpt *I,ObjectMolecule *obj,int state,int n_cycle)
   float total_strain=0.0F;
   int total_count=1;
 
-  PRINTFD(FB_Sculpt)
+  PRINTFD(G,FB_Sculpt)
     " SculptIterateObject-Debug: entered state=%d n_cycle=%d\n",state,n_cycle
     ENDFD;
 
@@ -885,28 +980,28 @@ float SculptIterateObject(CSculpt *I,ObjectMolecule *obj,int state,int n_cycle)
         active = Alloc(int,obj->NAtom);
         shk=I->Shaker;
 
-        PRINTFD(FB_Sculpt)
+        PRINTFD(G,FB_Sculpt)
           " SIO-Debug: NDistCon %d\n",shk->NDistCon
           ENDFD;
 
         cs = obj->CSet[state];
 
-        nb_skip = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_sculpt_nb_interval);
+        nb_skip = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_nb_interval);
         if(nb_skip<1) nb_skip=1;
-        vdw = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_scale);
-        vdw14 = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_scale14);
-        vdw_wt = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_weight);
-        vdw_wt14 = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_weight14);
-        bond_wt =  SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_bond_weight);
-        angl_wt =  SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_angl_weight);
-        pyra_wt =  SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_pyra_weight);
-        plan_wt =  SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_plan_weight);
-        line_wt =  SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_line_weight);
-        mask = SettingGet_i(cs->Setting,obj->Obj.Setting,cSetting_sculpt_field_mask);
-        hb_overlap = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_hb_overlap);
-        hb_overlap_base = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_hb_overlap_base);
-        tors_tole = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_tors_tolerance);
-        tors_wt = SettingGet_f(cs->Setting,obj->Obj.Setting,cSetting_sculpt_tors_weight);
+        vdw = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_scale);
+        vdw14 = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_scale14);
+        vdw_wt = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_weight);
+        vdw_wt14 = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_vdw_weight14);
+        bond_wt =  SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_bond_weight);
+        angl_wt =  SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_angl_weight);
+        pyra_wt =  SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_pyra_weight);
+        plan_wt =  SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_plan_weight);
+        line_wt =  SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_line_weight);
+        mask = SettingGet_i(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_field_mask);
+        hb_overlap = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_hb_overlap);
+        hb_overlap_base = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_hb_overlap_base);
+        tors_tole = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_tors_tolerance);
+        tors_wt = SettingGet_f(G,cs->Setting,obj->Obj.Setting,cSetting_sculpt_tors_weight);
         n_active = 0;
         ai0=obj->AtomInfo;
         for(a=0;a<obj->NAtom;a++) {
@@ -937,13 +1032,13 @@ float SculptIterateObject(CSculpt *I,ObjectMolecule *obj,int state,int n_cycle)
           /* first, create coordinate -> vertex mapping */
           /* and count number of constraints */
 
-          task_time = UtilGetSeconds();
+          task_time = UtilGetSeconds(G);
           vdw_magnify = 1.0F;
           nb_skip_count = n_cycle - nb_skip * (n_cycle/nb_skip);
           if(!nb_skip_count) nb_skip_count = nb_skip;
           while(n_cycle--) {
             
-            CGOReset(DebugCGO);
+            CGOReset(G->DebugCGO);
 
             total_strain = 0.0F;
             total_count = 0;
@@ -1101,6 +1196,8 @@ float SculptIterateObject(CSculpt *I,ObjectMolecule *obj,int state,int n_cycle)
                                              disp+b1*3,
                                              disp+b2*3,
                                              disp+b3*3,
+                                             snc->target,
+                                             snc->fixed,
                                              plan_wt);
                   total_count++;
                   cnt[b0]++;
@@ -1320,16 +1417,16 @@ float SculptIterateObject(CSculpt *I,ObjectMolecule *obj,int state,int n_cycle)
             if(cs->fInvalidateRep) {
               cs->fInvalidateRep(cs,cRepAll,cRepInvCoord);
             } else {
-              ObjectMoleculeInvalidate(obj,cRepAll,cRepInvCoord);
+              ObjectMoleculeInvalidate(obj,cRepAll,cRepInvCoord,-1);
             }
           
           }
           
-          task_time = UtilGetSeconds() - task_time;
-          PRINTFB(FB_Sculpt,FB_Blather)
+          task_time = UtilGetSeconds(G) - task_time;
+          PRINTFB(G,FB_Sculpt,FB_Blather)
             " Sculpt: %2.5f seconds %8.3f %d %8.3f\n",task_time,total_strain,total_count,
             100*total_strain/total_count
-            ENDFB;
+            ENDFB(G);
 
           if(total_count) 
             total_strain = (1000*total_strain)/total_count;
@@ -1340,11 +1437,11 @@ float SculptIterateObject(CSculpt *I,ObjectMolecule *obj,int state,int n_cycle)
         FreeP(atm2idx);
       }
 
-  PRINTFD(FB_Sculpt)
+  PRINTFD(G,FB_Sculpt)
     " SculptIterateObject-Debug: leaving...\n"
     ENDFD;
 
-  CGOStop(DebugCGO);
+  CGOStop(G->DebugCGO);
   
 
   return total_strain;
