@@ -28,6 +28,9 @@ Z* -------------------------------------------------------------------
 #include "Util.h"
 #include "P.h"
 #include "Util.h"
+#include "Color.h"
+#include "Text.h"
+#include "PyMOL.h"
 
 #define cPopUpLineHeight 17
 #define cPopUpTitleHeight 19
@@ -59,6 +62,7 @@ typedef struct CPopUp {
   double PassiveDelay;
   int DirtyDelayFlag;
   int NeverDragged;
+  int PlacementAffinity;
 }  CPopUp;
 
 int PopUpRelease(Block *block,int button,int x,int y,int mod);
@@ -69,9 +73,10 @@ int PopUpConvertY(CPopUp *I,int value,int mode);
 /*========================================================================*/
 static Block *PopUpRecursiveFind(Block *block,int x, int y)
 {
+  PyMOLGlobals *G=block->G;
   CPopUp *I = (CPopUp*)block->reference;
   if(BlockRecursiveFind(block,x,y) == block) {
-    OrthoGrab(block);
+    OrthoGrab(G,block);
     return block;
   } else if(I->Child) {
     if(PopUpRecursiveFind(I->Child,x,y)==I->Child)
@@ -80,8 +85,12 @@ static Block *PopUpRecursiveFind(Block *block,int x, int y)
   return NULL;
 }
 /*========================================================================*/
-Block *PopUpNew(int x,int y,int last_x,int last_y,PyObject *list,Block *parent)
+Block *PopUpNew(PyMOLGlobals *G,int x,int y,int last_x,int last_y,
+                int passive, PyObject *list,Block *parent)
 {
+#ifdef _PYMOL_NOPY
+  return NULL;
+#else
   /* assumes blocked threads (calls the Python C API) */
 
   int mx,a,l,cl,cmx;
@@ -89,9 +98,9 @@ Block *PopUpNew(int x,int y,int last_x,int last_y,PyObject *list,Block *parent)
   PyObject *elem;
   char *str,*c;
 
-  OOAlloc(CPopUp);
+  OOAlloc(G,CPopUp);
 
-  I->Block = OrthoNewBlock(NULL);
+  I->Block = OrthoNewBlock(G,NULL);
   I->Block->reference = (void*)I;
   I->Block->fDraw    = PopUpDraw;
   I->Block->fDrag    = PopUpDrag;
@@ -114,11 +123,12 @@ Block *PopUpNew(int x,int y,int last_x,int last_y,PyObject *list,Block *parent)
   I->Selected = -1;
   I->StartX = (I->LastX = last_x);
   I->StartY = (I->LastY = last_y);
-  I->ChildDelay = UtilGetSeconds() + cChildDelay*2.5;
-  I->PassiveDelay = UtilGetSeconds() + cPassiveDelay;
+  I->ChildDelay = UtilGetSeconds(G) + cChildDelay*2.5;
+  I->PassiveDelay = UtilGetSeconds(G) + cPassiveDelay;
   I->DirtyDelay = false;
   I->DirtyDelayFlag = false;
   I->NeverDragged = true;
+  I->PlacementAffinity = 0;
   mx=1;
   cmx=1;
   for(a=0;a<I->NLine;a++) {
@@ -129,7 +139,8 @@ Block *PopUpNew(int x,int y,int last_x,int last_y,PyObject *list,Block *parent)
     c = str;
     while(*c) {
       if(*c=='\\') { /* discount the markup */
-        if((c!=str)&&(*(c-1)!='\\'))
+        if((((c!=str)&&(c[-1])!='\\'))||
+           ((c==str)&&(c[1])&&(c[1]!='\\')))
           cl-=4;
       }
       c++;
@@ -184,11 +195,16 @@ Block *PopUpNew(int x,int y,int last_x,int last_y,PyObject *list,Block *parent)
 
   PopFitBlock(I->Block);
 
-  OrthoAttach(I->Block,cOrthoTool);
+  OrthoAttach(G,I->Block,cOrthoTool);
   I->Block->active=true;
-  OrthoGrab(I->Block);
-  OrthoDirty();
+  OrthoGrab(G,I->Block);
+  OrthoDirty(G);
+
+  if(passive) 
+    PyMOL_SetPassive(G->PyMOL,true);
   return I->Block;
+#endif
+
 }
 /*========================================================================*/
 int PopUpConvertY(CPopUp *I,int value,int mode)
@@ -259,9 +275,10 @@ int PopUpConvertY(CPopUp *I,int value,int mode)
 
 static void PopUpDetachRecursiveChild(Block *block)
 {
+  PyMOLGlobals *G=block->G;
   CPopUp *I = (CPopUp*)block->reference;
 
-  OrthoDetach(block);
+  OrthoDetach(G,block);
   if(I->Child)
     PopUpDetachRecursiveChild(I->Child);
 }
@@ -269,14 +286,16 @@ static void PopUpDetachRecursiveChild(Block *block)
 
 static void PopUpForgetChild(Block *block)
 {
+
   CPopUp *I = (CPopUp*)block->reference;
   I->Child = NULL;
 }
 
 static void PopUpRecursiveDetach(Block *block)
 {
+  PyMOLGlobals *G=block->G;
   CPopUp *I = (CPopUp*)block->reference;
-  OrthoDetach(block);
+  OrthoDetach(G,block);
   if(I->Child)
     PopUpDetachRecursiveChild(I->Child);
   if(I->Parent) {
@@ -288,20 +307,21 @@ static void PopUpRecursiveDetach(Block *block)
 
 static void PopUpFree(Block *block)
 {
+  PyMOLGlobals *G=block->G;
   CPopUp *I = (CPopUp*)block->reference;
 
-
-
+#ifndef _PYMOL_NOPY
   { /* purge code */
     int a;
     for(a=0;a<I->NLine;a++)
       if(I->Sub[a]) {
         Py_DECREF(I->Sub[a]);
       }
-
   }
-  OrthoDetach(I->Block);
-  OrthoFreeBlock(I->Block);
+#endif
+
+  OrthoDetach(G,I->Block);
+  OrthoFreeBlock(G,I->Block);
   FreeP(I->Sub);
   FreeP(I->Code);
   FreeP(I->Command);
@@ -312,6 +332,7 @@ static void PopUpFree(Block *block)
 
 static void PopUpRecursiveFree(Block *block)
 {
+
   CPopUp *I = (CPopUp*)block->reference;
 
   if(I->Child)
@@ -333,16 +354,18 @@ static void PopUpFreeRecursiveChild(Block *block)
   PopUpFree(block);
 }
 
+
 /*========================================================================*/
 int PopUpRelease(Block *block,int button,int x,int y,int mod)
 {
+  PyMOLGlobals *G=block->G;
   CPopUp *I = (CPopUp*)block->reference;
   int gone_passive = false;
 
   if(I->NeverDragged) {
-    if(I->PassiveDelay>UtilGetSeconds()) {    
+    if(I->PassiveDelay>UtilGetSeconds(G)) {    
       gone_passive = true;
-      I->PassiveDelay = UtilGetSeconds(); /* kill any further delay */
+      I->PassiveDelay = UtilGetSeconds(G); /* kill any further delay */
     }
   } 
   if(!gone_passive) {
@@ -358,9 +381,9 @@ int PopUpRelease(Block *block,int button,int x,int y,int mod)
     }
   }  
   if(gone_passive) {
-    MainSetPassiveDrag(true);
+    PyMOL_SetPassive(G->PyMOL,true);
   } else {
-    OrthoUngrab();
+    OrthoUngrab(G);
     PopUpRecursiveDetach(block);
     if(!I->NeverDragged) 
       if((I->Selected>=0)&&(!I->Sub[I->Selected])) {
@@ -370,7 +393,7 @@ int PopUpRelease(Block *block,int button,int x,int y,int mod)
       }
     PopUpRecursiveFree(block);
   }
-  OrthoDirty();
+  OrthoDirty(G);
   return(1);
 }
 
@@ -378,7 +401,7 @@ int PopUpRelease(Block *block,int button,int x,int y,int mod)
 /*========================================================================*/
 int PopUpDrag(Block *block,int x,int y,int mod)
 {
-
+  PyMOLGlobals *G=block->G;
   CPopUp *I = (CPopUp*)block->reference;
   
   int a;
@@ -412,7 +435,7 @@ int PopUpDrag(Block *block,int x,int y,int mod)
       }
     } 
   } else {
-    OrthoGrab(block);
+    OrthoGrab(G,block);
     a = PopUpConvertY(I,y,false);
     if(I->NLine&&(a==I->NLine))
       if((y-a*cPopUpLineHeight)<4)
@@ -422,16 +445,16 @@ int PopUpDrag(Block *block,int x,int y,int mod)
     else {
       if(I->Code[a]==1) {
         if((I->Child)&&(I->ChildLine!=a)) {
-          if(I->ChildDelay<UtilGetSeconds()) {
+          if(I->ChildDelay<UtilGetSeconds(G)) {
             PopUpDetachRecursiveChild(I->Child);
             PopUpFreeRecursiveChild(I->Child);
             I->Child=NULL;
             I->ChildLine=-1;
-            OrthoDirty();
+            OrthoDirty(G);
           } else {
             I->Selected = a;
           }
-          MainDragDirty();
+          PyMOL_NeedFakeDrag(G->PyMOL);
         }
       }
       
@@ -441,25 +464,27 @@ int PopUpDrag(Block *block,int x,int y,int mod)
         /* activate submenu */
         if(!I->Child) {
           I->ChildLine = a;
-          if(I->ChildDelay>UtilGetSeconds()) {
-            MainDragDirty(); /* keep coming back here... */
+          if(I->ChildDelay>UtilGetSeconds(G)) {
+            PyMOL_NeedFakeDrag(G->PyMOL); /* keep coming back here... */
           } else {
-            I->Child = PopUpNew(I->LastX-300,I->LastY,I->LastX,I->LastY,I->Sub[a],I->Block);
+            I->Child = PopUpNew(G,I->LastX-300,I->LastY,I->LastX,I->LastY,
+                                false,I->Sub[a],I->Block);
             {
               int target_y = block->rect.top - (PopUpConvertY(I,a,true)+ cPopUpCharMargin);
               CPopUp *child = (CPopUp*)(I->Child->reference);
               if(child->NLine)
                 if(child->Code[0]!=1)
                   target_y+=cPopUpTitleHeight+2;
-              PopPlaceChild(I->Child,block->rect.left-5,block->rect.right+5,target_y);
+              child->PlacementAffinity = PopPlaceChild(I->Child,block->rect.left-5,
+                                                       block->rect.right+5,target_y,I->PlacementAffinity);
             }
                           
-            OrthoGrab(I->Block);
-            I->ChildDelay = UtilGetSeconds() + cChildDelay; /* leave child up for a while */
+            OrthoGrab(G,I->Block);
+            I->ChildDelay = UtilGetSeconds(G) + cChildDelay; /* leave child up for a while */
           }
-          MainDragDirty(); /* keep coming back here... */
+          PyMOL_NeedFakeDrag(G->PyMOL); /* keep coming back here... */
         } else if(I->ChildLine==a) { /* on correct line */
-          I->ChildDelay = UtilGetSeconds() + cChildDelay; /* keep child here for a while */
+          I->ChildDelay = UtilGetSeconds(G) + cChildDelay; /* keep child here for a while */
         }
         I->Selected=a;
       } else 
@@ -470,27 +495,27 @@ int PopUpDrag(Block *block,int x,int y,int mod)
      can be comfortably accessed with sloppy mousing */
 
   if((I->Child)&&(I->Selected!=I->ChildLine))
-    MainDragDirty();
+    PyMOL_NeedFakeDrag(G->PyMOL);
 
   if(was!=I->Selected) {
 
     I->NeverDragged = false;
     if(!I->Child) {
       /* we moved, so renew the child delay */
-      I->ChildDelay = UtilGetSeconds() + cChildDelay;
-      MainDragDirty();
+      I->ChildDelay = UtilGetSeconds(G) + cChildDelay;
+      PyMOL_NeedFakeDrag(G->PyMOL);
     }
 
     if((I->Child)&&(I->Selected!=I->ChildLine)) {
       I->DirtyDelayFlag = true;
-      I->DirtyDelay = UtilGetSeconds() + cDirtyDelay;
+      I->DirtyDelay = UtilGetSeconds(G) + cDirtyDelay;
     }
     if(!I->DirtyDelayFlag)
-      OrthoDirty();
+      OrthoDirty(G);
   }
-  if(I->DirtyDelayFlag && (I->DirtyDelay<UtilGetSeconds())) {
+  if(I->DirtyDelayFlag && (I->DirtyDelay<UtilGetSeconds(G))) {
     I->DirtyDelayFlag = false;
-    OrthoDirty();
+    OrthoDirty(G);
   }
   return(1);
 }
@@ -498,19 +523,20 @@ int PopUpDrag(Block *block,int x,int y,int mod)
 void PopUpDraw(Block *block)
 {
   CPopUp *I = (CPopUp*)block->reference;
+  PyMOLGlobals *G=block->G;
   int x,y,a,xx;
   char *c;
 
   
-  if(PMGUI) {
+  if(G->HaveGUI && G->ValidContext ) {
 
-  if((I->Child)&&(I->Selected!=I->ChildLine))
-    MainDragDirty();
+    if((I->Child)&&(I->Selected!=I->ChildLine))
+      PyMOL_NeedFakeDrag(G->PyMOL);
     
     /* put raised border around pop-up menu */
-
+    
     /* bottom */
-
+    
     glColor3f(0.2F,0.2F,0.4F);
     glBegin(GL_POLYGON);
     glVertex2i(block->rect.left-2,block->rect.bottom-2);
@@ -630,9 +656,9 @@ void PopUpDraw(Block *block)
     for(a=0;a<I->NLine;a++)
       {
         if(a==I->Selected)
-          glColor3fv(I->Block->BackColor);
+          TextSetColor(G,I->Block->BackColor);
         else
-          glColor3fv(I->Block->TextColor);          
+          TextSetColor(G,I->Block->TextColor);          
         if(I->Code[a]) {
           c=I->Text[a];
           xx=x;
@@ -640,17 +666,21 @@ void PopUpDraw(Block *block)
             if(*c=='\\') if(*(c+1)) if(*(c+2)) if(*(c+3)) {
               if(*(c+1)=='-') {
                 if(a==I->Selected)
-                  glColor3fv(I->Block->BackColor);
+                  TextSetColor(G,I->Block->BackColor);
                 else
-                  glColor3fv(I->Block->TextColor);          
+                  TextSetColor(G,I->Block->TextColor);          
                 c+=4;
-              } else {
-                glColor3f((*(c+1)-'0')/9.0F,(*(c+2)-'0')/9.0F,(*(c+3)-'0')/9.0F);
+              } else if(*(c+1)=='+') {
+                c+=4;                
+                TextSetColor(G,ColorGetNamed(G,c));
+              }else {
+                TextSetColor3f(G,(*(c+1)-'0')/9.0F,(*(c+2)-'0')/9.0F,(*(c+3)-'0')/9.0F);
                 c+=4;
               }
             }
-            glRasterPos4d((double)(xx),(double)(y+cPopUpCharLift),0.0,1.0);
-            p_glutBitmapCharacter(P_GLUT_BITMAP_8_BY_13,*(c++));
+
+            TextSetPos2i(G,xx,y+cPopUpCharLift);
+            TextDrawChar(G,*(c++));
             xx = xx + 8;
           }
 
