@@ -94,6 +94,9 @@ class Setting:
       self.cartoon_cylindrical_helices = IntVar()
       self.cartoon_cylindrical_helices.set(int(cmd.get_setting_legacy('cartoon_cylindrical_helices')))
 
+      self.cartoon_side_chain_helper = IntVar()
+      self.cartoon_side_chain_helper.set(int(cmd.get_setting_legacy('cartoon_side_chain_helper')))
+
       self.ignore_pdb_segi = IntVar()
       self.ignore_pdb_segi.set(int(cmd.get_setting_legacy('ignore_pdb_segi')))
 
@@ -138,6 +141,12 @@ class Setting:
       self.seq_view = IntVar()
       self.seq_view.set(int(cmd.get_setting_legacy('seq_view')))
       
+      self.texture_fonts = IntVar()
+      self.texture_fonts.set(int(cmd.get_setting_legacy('texture_fonts')))
+
+      self.animation = IntVar()
+      self.animation.set(int(cmd.get_setting_legacy('animation')))
+
       self.F=[ None,
                IntVar(),
                IntVar(),
@@ -146,6 +155,21 @@ class Setting:
                IntVar(),
                IntVar(),
 
+               IntVar(),
+               IntVar(),
+               IntVar(),
+               IntVar(),
+               IntVar(),
+               IntVar(),
+               ]
+      self.SHFTF=[ None,
+               IntVar(),
+               IntVar(),
+               IntVar(),
+               IntVar(),
+               IntVar(),
+               IntVar(),
+
                IntVar(),
                IntVar(),
                IntVar(),
@@ -202,6 +226,9 @@ class Setting:
          (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_smooth_loops.get())),log=1))),
          'cartoon_cylindrical_helices'        :
          (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_cylindrical_helices.get())),log=1))),
+         'cartoon_side_chain_helper'        :
+         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_side_chain_helper.get())),log=1))),
+
 
          'ignore_pdb_segi'        :
          (lambda s,a: (cmd.set(a,("%1.0f" % (s.ignore_pdb_segi.get())),log=1))),
@@ -237,7 +264,12 @@ class Setting:
          
          'seq_view'         :
          (lambda s,a: (cmd.set(a,("%d" % s.seq_view.get()),log=1))),
-         
+         'texture_fonts'         :
+         (lambda s,a: (cmd.set(a,("%d" % s.texture_fonts.get()),log=1))),
+
+         'animation'        :
+         (lambda s,a: (cmd.set(a,("%1.0f" % (s.animation.get())),log=1))),
+
          }
 
       self.update_code = {
@@ -283,6 +315,8 @@ class Setting:
          (lambda s,t: (s.cartoon_smooth_loops.set(t[1][0]!=0))),
          'cartoon_cylindrical_helices':
          (lambda s,t: (s.cartoon_cylindrical_helices.set(t[1][0]!=0))),
+         'cartoon_side_chain_helper':
+         (lambda s,t: (s.cartoon_side_chain_helper.set(t[1][0]!=0))),
          'ignore_pdb_segi':
          (lambda s,t: (s.ignore_pdb_segi.set(t[1][0]!=0))),
          'log_box_selections':
@@ -313,8 +347,12 @@ class Setting:
          (lambda s,t: (s.virtual_trackball.set(t[1][0]!=0))),
          'seq_view':
          (lambda s,t: (s.seq_view.set(t[1][0]!=0))),
+         'texture_fonts':
+         (lambda s,t: (s.texture_fonts.set(t[1][0]!=0))),
          'stereo':
-         (lambda s,t: (s.stereo.set(t[1][0]!=0))),         
+         (lambda s,t: (s.stereo.set(t[1][0]!=0))),
+         'animation':
+         (lambda s,t: (s.animation.set(t[1][0]!=0))),         
         }
       self.active_list = [
          pymol.setting._get_index("ray_trace_frames"),
@@ -338,7 +376,8 @@ class Setting:
          pymol.setting._get_index("cartoon_fancy_sheets"),
          pymol.setting._get_index("cartoon_discrete_colors"),
          pymol.setting._get_index("cartoon_smooth_loops"),
-         pymol.setting._get_index("cartoon_cylindrical_helices"),                  
+         pymol.setting._get_index("cartoon_cylindrical_helices"),
+         pymol.setting._get_index("cartoon_side_chain_helper"),                  
          pymol.setting._get_index("ignore_pdb_segi"),         
          pymol.setting._get_index("log_box_selections"),
          pymol.setting._get_index("log_conformations"),
@@ -354,7 +393,9 @@ class Setting:
          pymol.setting._get_index("use_display_lists"),
          pymol.setting._get_index("virtual_trackball"),
          pymol.setting._get_index("seq_view"),
+         pymol.setting._get_index("texture_fonts"),
          pymol.setting._get_index("stereo"),
+         pymol.setting._get_index("animation"),
          ]
 
       self.active_dict = {}
@@ -389,6 +430,10 @@ class Setting:
             self.F[x].set(1)
          else:
             self.F[x].set(0)
+         if dict.has_key('SHFT-F%d'%x):
+            self.SHFTF[x].set(1)
+         else:
+            self.SHFTF[x].set(0)
             
    def refresh(self): # get any settings changes from PyMOL and update menus
       lst = cmd.get_setting_updates()
