@@ -1,11 +1,32 @@
-#
-# This simple plugin will automatically load a pdb from the
-# web given the pdb code.  It does error handling as well.
-#
-# Charles Moad (cmoad@indiana.edu)
-# Scientific Data Analysis Lab
-# Indiana University
-#
+# Copyright Notice
+# ================
+# 
+# The PyMOL Plugin source code in this file is copyrighted, but you can
+# freely use and copy it as long as you don't change or remove any of
+# the copyright notices.
+# 
+# ----------------------------------------------------------------------
+# This PyMOL Plugin is Copyright (C) 2004 by Charles Moad <cmoad@indiana.edu>
+# 
+#                        All Rights Reserved
+# 
+# Permission to use, copy, modify, distribute, and distribute modified
+# versions of this software and its documentation for any purpose and
+# without fee is hereby granted, provided that the above copyright
+# notice appear in all copies and that both the copyright notice and
+# this permission notice appear in supporting documentation, and that
+# the name(s) of the author(s) not be used in advertising or publicity
+# pertaining to distribution of the software without specific, written
+# prior permission.
+# 
+# THE AUTHOR(S) DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN
+# NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
+# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+# PERFORMANCE OF THIS SOFTWARE.
+# ----------------------------------------------------------------------
 
 from Tkinter import *
 from pymol import cmd
@@ -32,7 +53,7 @@ class FetchPDB:
                                          parent=app.root)
       
       if pdbCode: # None is returned for user cancel
-         pdcCode = string.upper(pdbCode)
+         pdbCode = string.upper(pdbCode)
          filename = urllib.urlretrieve('http://www.rcsb.org/pdb/cgi/export.cgi/' +
                                        pdbCode + '.pdb.gz?format=PDB&pdbId=' +
                                        pdbCode + '&compression=gz')[0]
@@ -56,3 +77,4 @@ class FetchPDB:
                                    parent=app.root)
 
          os.remove(filename) # Remove tmp file (leave the pdb)
+
