@@ -17,7 +17,15 @@ if __name__=='pymol.controlling':
    import string
    import selector
    import cmd
-   from cmd import _cmd,lock,unlock,Shortcut,QuietException
+   from cmd import _cmd,lock,unlock,Shortcut,QuietException,is_string
+   from cmd import boolean_dict, boolean_sc
+
+   location_code = {
+      'top' : -1,
+      'current' : 0,
+      'bottom' : 1
+   }
+   location_sc = Shortcut(location_code.keys())
 
    button_code = {
       'left' : 0,
@@ -27,6 +35,9 @@ if __name__=='pymol.controlling':
       'double_left' : 4,
       'double_middle' : 5,
       'double_right' : 6,
+      'single_left' : 7,
+      'single_middle' : 8,
+      'single_right' : 9
       }
    button_sc = Shortcut(button_code.keys())
 
@@ -75,6 +86,8 @@ if __name__=='pymol.controlling':
       '+/-'  : 31 ,
       '+box' : 32 ,
       '-box' : 33 ,
+
+      'mvsz' : 34 ,
       
       }
    but_act_sc = Shortcut(but_act_code.keys())
@@ -123,15 +136,18 @@ if __name__=='pymol.controlling':
                                  ('m','ctrl','pkat'),
                                  ('r','ctrl','pk1'),                 
                                  ('l','ctsh','Sele'),
-                                 ('m','ctsh','cent'),
+                                 ('m','ctsh','orig'),
                                  ('r','ctsh','menu'),
                                  ('w','none','slab'),
                                  ('w','shft','movs'),
-                                 ('w','ctrl','none'),
-                                 ('w','ctsh','none'),
+                                 ('w','ctrl','mvsz'),
+                                 ('w','ctsh','movz'),
                                  ('double_left','none','menu'),
-                                 ('double_middle','none','cent'),
+                                 ('double_middle','none','none'),
                                  ('double_right','none', 'pkat'),
+                                 ('single_left','none','+/-'),
+                                 ('single_middle','none','cent'),
+                                 ('single_right','none', 'menu'),
                                  ],
 
       'three_button_editing': [ ('l','none','rota'),
@@ -144,15 +160,18 @@ if __name__=='pymol.controlling':
                                 ('m','ctrl','pkat'),
                                 ('r','ctrl','pktb'),                  
                                 ('l','ctsh','mova'),
-                                ('m','ctsh','cent'),
+                                ('m','ctsh','orig'),
                                 ('r','ctsh','menu'),
                                 ('w','none','slab'),
                                 ('w','shft','movs'),
-                                ('w','ctrl','none'),
-                                ('w','ctsh','none'),
-                                ('double_left','none','menu'),
-                                ('double_middle','none','cent'),
-                                ('double_right','none', 'pkat'),
+                                ('w','ctrl','mvsz'),
+                                ('w','ctsh','movz'),
+                                ('double_left','none','torf'),
+                                ('double_middle','none','none'),
+                                ('double_right','none', 'pktb'),
+                                ('single_left','none','pkat'),
+                                ('single_middle','none','cent'),
+                                ('single_right','none', 'menu'),
                                 ],
 
       'two_button_viewing' : [ ('l','none','rota'),
@@ -174,6 +193,9 @@ if __name__=='pymol.controlling':
                                ('double_left','none','menu'),
                                ('double_middle','none','none'),                               
                                ('double_right','none','cent'),
+                               ('single_left','none','pkat'),
+                               ('single_middle','none','none'),
+                               ('single_right','none', 'menu'),
                                
                                ],
       'two_button_selecting' : [ ('l','none','rota'),
@@ -195,6 +217,8 @@ if __name__=='pymol.controlling':
                                ('double_left','none','menu'),
                                ('double_middle','none','none'),                               
                                ('double_right','none','cent'),
+                               ('single_left','none','+/-'),
+                               ('single_right','none', 'menu'),
                                  
                                ],
       'two_button_editing' : [ ('l','none','rota'),
@@ -216,10 +240,55 @@ if __name__=='pymol.controlling':
                                ('double_left','none','menu'),
                                ('double_middle','none','none'),                               
                                ('double_right','none','cent'),
-                               
+                               ('single_left','none','pkat'),
+                               ('single_middle','none','none'),                               
+                               ('single_right','none','menu'),                               
                                ],
       }
 
+   def order(names,sort=0,location='current'):
+      '''
+DESCRIPTION
+
+   "order" allows you to change ordering of names in the control panel
+
+USAGE
+
+   order names-list, sort, location
+
+EXAMPLES
+
+   order 1dn2 1fgh 1rnd  # sets the order of these three objects
+   order *,yes           # sorts all names
+   order 1dn2_*, yes     # sorts all names beginning with 1dn2_
+   order 1frg, location=top   # puts 1frg at the top of the list
+
+PYMOL API
+
+   cmd.order(string names-list, string sort, string location)
+
+NOTES
+
+   names-list: a space separated list of names
+   sort: yes or no
+   location: top, current, or bottom
+
+SEE ALSO
+
+   set_name
+      '''
+
+      r = 1
+      location=location_code[location_sc.auto_err(location,'location')]
+      if is_string(sort):
+         sort=boolean_dict[boolean_sc.auto_err(sort,'sort option')]
+      try:
+         lock()
+         _cmd.order(str(names),int(sort),int(location))
+      finally:
+         unlock()
+      return r
+
    def mouse(action=None,quiet=1):# INTERNAL
       # NOTE: PyMOL automatically runs this routine upon start-up
       try:
@@ -235,7 +304,17 @@ if __name__=='pymol.controlling':
             bm = (int(bm) - 1) % len(mouse_ring)
             cmd.set("button_mode",str(bm),quiet=1)
             action=None
-
+         elif action=='select_forward':
+            sm = _cmd.get_setting("mouse_selection_mode")
+            sm = sm + 1
+            if sm>6: sm = 0
+            cmd.set("mouse_selection_mode",sm,quiet=1)
+         elif action=='select_backward':
+            sm = _cmd.get_setting("mouse_selection_mode")
+            sm = sm - 1
+            if sm<0: sm = 6
+            cmd.set("mouse_selection_mode",sm,quiet=1)
+         
          mode_list = None
          if action==None:
             bm = _cmd.get_setting("button_mode")
@@ -246,23 +325,39 @@ if __name__=='pymol.controlling':
          elif action in mode_dict.keys():
             mode = action
             cmd.set("button_mode_name",mode_name_dict.get(mode,mode))
+            if mode in mouse_ring:
+               bm = mouse_ring.index(mode)
+               cmd.set("button_mode",bm)
             mode_list = mode_dict[mode]
          if mode_list!=None:
             for a in mode_list:
                apply(button,a)
             if not quiet:
                print " mouse: %s"%mode
+            if mode[-7:]!='editing': cmd.unpick()
+            if mode[-7:]=='editing': cmd.deselect()
       finally:
          unlock()
 
-   def edit_mode(mode=None):
+   def edit_mode(active=1,quiet=1):
       # legacy function
       if len(mouse_ring):
-         mouse_mode = mouse_ring[0]
-         if mouse_mode[0:10]=='two_button':
-            mouse(action='two_button_editing',quiet=0)
-         elif mouse_mode[0:12] == 'three_button':
-            mouse(action='three_button_editing',quiet=0)
+         bm = int(_cmd.get_setting("button_mode"))
+         mouse_mode = mouse_ring[bm]
+         if active:
+            if mouse_mode[0:10]=='two_button':
+               if mouse_mode!='two_button_editing':
+                  mouse(action='two_button_editing',quiet=quiet)
+            elif mouse_mode[0:12] == 'three_button':
+               if mouse_mode!='three_button_editing':
+                  mouse(action='three_button_editing',quiet=quiet)
+         else:
+            if mouse_mode[0:10]=='two_button':
+               if mouse_mode!='two_button_viewing':               
+                  mouse(action='two_button_viewing',quiet=quiet)
+            elif mouse_mode[0:12] == 'three_button':
+               if mouse_mode!='three_button_viewing':
+                  mouse(action='three_button_viewing',quiet=quiet)
 
    #   try:
    #      lock()
