@@ -7,7 +7,7 @@
 #G* Please see the accompanying LICENSE file for further information. 
 #H* -------------------------------------------------------------------
 #I* Additional authors of this source file include:
-#-* 
+#-* Filipe Maia (slicing code)
 #-* 
 #-*
 #Z* -------------------------------------------------------------------
@@ -18,8 +18,9 @@ if __name__=='pymol.creating':
    import traceback
    import operator
    import cmd
-   from cmd import _cmd,lock,unlock,Shortcut,QuietException,is_list
-   from cmd import file_ext_re
+   import string
+   from cmd import _cmd,lock,unlock,Shortcut,QuietException,is_list,is_string
+   from cmd import file_ext_re, safe_list_eval, safe_alpha_list_eval
    from chempy import fragments
 
    map_type_dict = {
@@ -27,11 +28,25 @@ if __name__=='pymol.creating':
       'coulomb' : 1,
       'gaussian' : 2,
       'coulomb_neutral' : 3,
+      'coulomb_local' : 4,
       }
 
    map_type_sc = Shortcut(map_type_dict.keys())
 
-   def map_new(name,type='gaussian',grid=None,selection="(all)",buffer=None,box=None,state=0):
+   ramp_spectrum_dict = {
+      "traditional" : 1,
+      "sludge" : 2,
+      "ocean" : 3,
+      "hot" : 4,
+      "grayable" : 5,
+      "rainbow" : 6,
+      "afmhot" : 7,
+      "grayscale" : 8,
+      }
+   
+   ramp_spectrum_sc = Shortcut(ramp_spectrum_dict.keys())
+   
+   def map_new(name,type='gaussian',grid=None,selection="(all)",buffer=None,box=None,state=0,quiet=1):
       '''
       state > 0: do indicated state
       state = 0: independent states in independent extents
@@ -44,15 +59,17 @@ if __name__=='pymol.creating':
       selection = selector.process(selection)
       if box!=None: # box should be [[x1,y1,z1],[x2,y2,z2]]
          if cmd.is_string(box):
-            box = eval(box)
+            box = safe_list_eval(box)
          box = (float(box[0][0]),
                 float(box[0][1]),
                 float(box[0][2]),
                 float(box[1][0]),
                 float(box[1][1]),
                 float(box[1][2]))
+         box_flag = 1
       else:
          box = (0.0,0.0,0.0,1.0,1.0,1.0)
+         box_flag = 0
       if grid==None:
          grid = cmd.get_setting_legacy('gaussian_resolution')/3.0
       if buffer==None:
@@ -62,7 +79,7 @@ if __name__=='pymol.creating':
       try:
          lock()
          r = _cmd.map_new(str(name),int(type),grid,str(selection),
-                          float(buffer),box,int(state)-1)
+                          float(buffer),box,int(state)-1,box_flag,quiet)
       finally:
          unlock()
 
@@ -71,20 +88,28 @@ if __name__=='pymol.creating':
                 map_state=1,selection='',
                 beyond=2.0,within=6.0,
                 sigma=2.0,zero=1):
-      # preprocess selection
-      color = eval(str(color))
+      safe_color = string.strip(str(color))
+      if(safe_color[0:1]=="["): # looks like a list
+         color = safe_alpha_list_eval(str(safe_color))
+      else: # looks like a literal
+         color = str(color)
       new_color = []
+      # preprocess selection
       if selection!='':
-         selection = selector.process(selection)      
-      for a in color:
-         if not is_list(a):
-            new_color.append(list(cmd.get_color_tuple(a)))
-         else:
-            new_color.append(a)
-         
+         selection = selector.process(selection)
+      if is_list(color):
+         for a in color:
+            if not is_list(a):
+               new_color.append(list(cmd.get_color_tuple(a)))
+            else:
+               new_color.append(a)
+      elif is_string(color):
+         new_color = ramp_spectrum_dict[ramp_spectrum_sc.auto_err(str(color),'ramp color spectrum')]
+      else:
+         new_color=int(color)
       try:
          lock()
-         r = _cmd.ramp_new(str(name),str(map_name),list(eval(str(range))),list(new_color),
+         r = _cmd.ramp_new(str(name),str(map_name),list(safe_list_eval(str(range))),new_color,
                            int(map_state)-1,str(selection),float(beyond),float(within),
                            float(sigma),int(zero))
       finally:
@@ -156,6 +181,42 @@ SEE ALSO
          unlock()
       return r
 
+   def slice_new(name,map,state=1,source_state=0):
+      '''
+DESCRIPTION
+
+   "slice_map" creates a slice object from a map object.
+
+USAGE
+
+   slice_map name, map, [opacity, [resolution, [state, [source_state]]]]
+
+   name = the name for the new slice object.
+
+   map = the name of the map object to use for computing the slice.
+
+   opacity = opacity of the new slice (default=1)
+
+   resolution = the number of pixels per sampling (default=5)
+
+   state = the state into which the object should be loaded (default=1)
+      (set state=0 to append new mesh as a new state)
+
+   source_state = the state of the map from which the object should be loaded (default=0)
+   
+SEE ALSO
+
+   isomesh, isodot, load
+'''
+
+      try:
+         lock()
+         r = _cmd.slice_new(str(name),str(map),int(state)-1,int(source_state)-1)
+      finally:
+         unlock()
+      return r
+
+
    def isosurface(name,map,level=1.0,selection='',buffer=0.0,state=1,carve=None,
                   source_state=0,side=1,mode=3):
       '''
@@ -384,7 +445,7 @@ USAGE
          if cmd._raising(): raise QuietException
       return r
 
-   def create(name,selection,source_state=0,target_state=0,discrete=0):
+   def create(name,selection,source_state=0,target_state=0,discrete=0,zoom=-1):
       '''
 DESCRIPTION
 
@@ -428,7 +489,7 @@ SEE ALSO
             _cmd.legacy_set("sel_counter","%1.0f" % sel_cnt)
             name = "obj%02.0f" % sel_cnt
          _cmd.create(str(name),"("+str(selection)+")",
-                     int(source_state)-1,int(target_state)-1,int(discrete))
+                     int(source_state)-1,int(target_state)-1,int(discrete),int(zoom))
       finally:
          unlock()
       return None
