@@ -19,6 +19,7 @@ if __name__=='pymol.editing':
 
    import cmd
    from cmd import _cmd,lock,unlock,Shortcut,QuietException
+   from cmd import safe_list_eval
    from chempy import cpv
 
    def sculpt_purge():
@@ -57,11 +58,13 @@ if __name__=='pymol.editing':
          unlock()
       return r
 
-   def split_states(object,first=1,last=0):
+   def split_states(object,first=1,last=0,prefix=None):
       '''
       '''
       r = 0
       object = str(object)
+      if prefix==None:
+         prefix = object+"_"
       first=int(first)
       last=int(last)
       n_state = cmd.count_states(object)
@@ -71,9 +74,9 @@ if __name__=='pymol.editing':
          try:
             name = cmd.get_title(object,a)
             if len(name)==0:
-               name = object+"%04d"%a
+               name = prefix+"%04d"%a
          except:
-            name = object+"%04d"%a
+            name = prefix+"%04d"%a
          cmd.frame(a)
          cmd.create(name,"%s and present"%object,a,1)
       return r
@@ -90,7 +93,7 @@ if __name__=='pymol.editing':
          unlock()
       return r
 
-   def smooth(selection="all",passes=1,window=5,first=1,last=0,ends=0):
+   def smooth(selection="all",passes=1,window=5,first=1,last=0,ends=0,quiet=1):
       '''
 DESCRIPTION
 
@@ -126,7 +129,7 @@ NOTES
       try:
          lock()
          r = _cmd.smooth(str(selection),int(passes),int(window),
-                         int(first)-1,int(last)-1,int(ends))
+                         int(first)-1,int(last)-1,int(ends),int(quiet))
 
       finally:
          unlock()
@@ -348,7 +351,7 @@ PYMOL API
 
 NOTE
 
-   The invert function is usually bound to CTRL-X in editing mode.
+   The invert function is usually bound to CTRL-E in Editing Mode.
 
 
    '''
@@ -390,7 +393,7 @@ SEE ALSO
          unlock()
       return r
 
-   def remove(selection):
+   def remove(selection,quiet=1):
       '''
 DESCRIPTION
 
@@ -418,13 +421,13 @@ SEE ALSO
       r = 1
       try:
          lock()   
-         r = _cmd.remove("("+selection+")")
+         r = _cmd.remove("("+selection+")",int(quiet))
       finally:
          unlock()
       return r
 
 
-   def remove_picked(hydrogens=1):
+   def remove_picked(hydrogens=1,quiet=1):
       '''
 DESCRIPTION
 
@@ -454,13 +457,13 @@ SEE ALSO
       r = 1
       try:
          lock()   
-         r = _cmd.remove_picked(int(hydrogens))
+         r = _cmd.remove_picked(int(hydrogens),int(quiet))
       finally:
          unlock()
       return r
 
 
-   def cycle_valence(h_fill=1):
+   def cycle_valence(h_fill=1,quiet=1):
       '''
 DESCRIPTION
 
@@ -493,15 +496,15 @@ SEE ALSO
       r = 1
       try:
          lock()   
-         r = _cmd.cycle_valence()
+         r = _cmd.cycle_valence(quiet)
       finally:
          unlock()
       if h_fill:
-         globals()['h_fill']()
+         globals()['h_fill'](quiet)
       return r
 
 
-   def attach(element,geometry,valence,name=''):
+   def attach(element,geometry,valence,name='',quiet=1):
       '''
 DESCRIPTION
 
@@ -529,7 +532,7 @@ NOTES
       return r
 
 
-   def fuse(selection1="(pk1)",selection2="(pk2)",mode=0):
+   def fuse(selection1="(pk1)",selection2="(pk2)",mode=0,recolor=1,move=1):
       '''
 DESCRIPTION
 
@@ -563,7 +566,8 @@ SEE ALSO
       #   
       try:
          lock()
-         r = _cmd.fuse(str(selection1),str(selection2),int(mode))
+         r = _cmd.fuse(str(selection1),str(selection2),
+                       int(mode),int(recolor),int(move))
       finally:
          unlock()
       return r
@@ -666,7 +670,7 @@ SEE ALSO
          unlock()
       return r
 
-   def h_fill():
+   def h_fill(quiet=1):
       '''
 DESCRIPTION
 
@@ -693,12 +697,12 @@ SEE ALSO
       r = 1
       try:
          lock()   
-         r = _cmd.h_fill()
+         r = _cmd.h_fill(int(quiet))
       finally:
          unlock()
       return r
 
-   def h_add(selection="(all)"):
+   def h_add(selection="(all)",quiet=1):
       '''
 DESCRIPTION
 
@@ -723,7 +727,7 @@ SEE ALSO
       r = 1
       try:
          lock()   
-         r = _cmd.h_add(selection)
+         r = _cmd.h_add(selection,int(quiet))
       finally:
          unlock()
       return r
@@ -759,7 +763,7 @@ SEE ALSO
       return r
 
 
-   def replace(element,geometry,valence,h_fill=1,name=""):
+   def replace(element,geometry,valence,h_fill=1,name="",quiet=1):
       '''
 DESCRIPTION
 
@@ -788,9 +792,9 @@ SEE ALSO
          raise QuietException
       try:
          if h_fill: # strip off existing hydrogens
-            remove("((neighbor pk1) and elem h)")
+            remove("((neighbor pk1) and elem h)",quiet=quiet)
          lock()
-         r = _cmd.replace(str(element),int(geometry),int(valence),str(name))
+         r = _cmd.replace(str(element),int(geometry),int(valence),str(name),quiet)
       finally:
          unlock()
       return r
@@ -894,7 +898,7 @@ NOTES
          unlock()   
       return r
 
-   def alter(selection,expression,quiet=1):
+   def alter(selection,expression,quiet=1,space=cmd.pymol.__dict__):
       '''
 DESCRIPTION
 
@@ -933,18 +937,32 @@ SEE ALSO
       #
       try:
          lock()
-         r = _cmd.alter("("+str(selection)+")",str(expression),0,int(quiet))
+         r = _cmd.alter("("+str(selection)+")",str(expression),0,int(quiet),dict(space))
+      finally:
+         unlock()   
+      return r
+
+   def alter_list(object,expr_list,quiet=1,space=cmd.pymol.__dict__):
+      '''
+DESCRIPTION
+   "alter_list" is currently in development
+   
+      '''
+      #
+      try:
+         lock()
+         r = _cmd.alter_list(str(object),list(expr_list),int(quiet),dict(space))
       finally:
          unlock()   
       return r
 
 
-   def iterate(selection,expression,quiet=1):
+   def iterate(selection,expression,quiet=1,space=cmd.pymol.__dict__):
       '''
 DESCRIPTION
 
    "iterate" iterates over an expression with a separate name space
-   for each atom.  However, unlike the "alter" command, atomic
+   for each atom.  However, unlike the "alter" comman3d, atomic
    properties can not be altered.  Thus, "iterate" is more efficient
    than "alter".
 
@@ -983,12 +1001,12 @@ SEE ALSO
       #
       try:
          lock()
-         r = _cmd.alter("("+str(selection)+")",str(expression),1,int(quiet))
+         r = _cmd.alter("("+str(selection)+")",str(expression),1,int(quiet),dict(space))
       finally:
          unlock()   
       return r
 
-   def alter_state(state,selection,expression,quiet=1):
+   def alter_state(state,selection,expression,quiet=1,space=cmd.pymol.__dict__):
       '''
 DESCRIPTION
 
@@ -1013,21 +1031,17 @@ SEE ALSO
       # preprocess selection
       selection = selector.process(selection)
       #
+      atomic_props = 1
       state = int(state)
-      if state<0: # hack -- need to replace with "alter_state_atom" command
-         state = -state
-         atomic_props = 1
-      else:
-         atomic_props = 0
       try:
          lock()
          r = _cmd.alter_state(int(state)-1,"("+str(selection)+")",str(expression),
-                              0,int(atomic_props),int(quiet))
+                              0,int(atomic_props),int(quiet),dict(space))
       finally:
          unlock()   
       return r
 
-   def iterate_state(state,selection,expression,quiet=1):
+   def iterate_state(state,selection,expression,quiet=1,space=cmd.pymol.__dict__):
       '''
 DESCRIPTION
 
@@ -1050,20 +1064,18 @@ SEE ALSO
       selection = selector.process(selection)
       state = int(state)
       #
-      if state<0: # hack -- need to replace with "alter_state_atom" command
-         state = -state
-         atomic_props = 1
-      else:
-         atomic_props = 0
+      atomic_props = 1
       try:
          lock()
          r = _cmd.alter_state(int(state)-1,"("+str(selection)+")",
-                              str(expression),1,int(atomic_props),int(quiet))
+                              str(expression),1,int(atomic_props),
+                              int(quiet),dict(space))
       finally:
          unlock()   
       return r
 
-   def translate(vector=[0.0,0.0,0.0],selection="all",state=0,camera=1,object=None):
+   def translate(vector=[0.0,0.0,0.0],selection="all",
+                 state=0,camera=1,object=None,object_mode=0):
       '''
 DESCRIPTION
 
@@ -1104,8 +1116,9 @@ NOTES
 
       '''
       r = 1
+      object_mode = int(object_mode)
       if cmd.is_string(vector):
-         vector = eval(vector)
+         vector = safe_list_eval(vector)
       if not cmd.is_list(vector):
          print "Error: bad vector."
          raise QuietException
@@ -1120,24 +1133,38 @@ NOTES
          else:
             shift = vector
          if object==None:
-            ttt = [1.0,0.0,0.0,0.0,
-                   0.0,1.0,0.0,0.0,
-                   0.0,0.0,1.0,0.0,
-                   shift[0],shift[1],shift[2],1.0]
+            ttt = [1.0,0.0,0.0,shift[0],
+                   0.0,1.0,0.0,shift[1],
+                   0.0,0.0,1.0,shift[2],
+                   0.0,0.0,0.0,1.0]
             r=cmd.transform_selection(selection,ttt,state=state)
+         elif object_mode==0: # update the TTT display matrix
+            try:
+               lock()
+               ttt = [1.0, 0.0, 0.0, shift[0],
+                      0.0, 1.0, 0.0, shift[1],
+                      0.0, 0.0, 1.0, shift[2],
+                      0.0, 0.0, 0.0, 1.0]
+               r=_cmd.combine_object_ttt(str(object),ttt)
+            finally:
+               unlock()
+         elif object_mode==1: # transform object coordinates & history matrix
+            matrix = [1.0, 0.0, 0.0, shift[0],
+                      0.0, 1.0, 0.0, shift[1],
+                      0.0, 0.0, 1.0, shift[2],
+                      0.0, 0.0, 0.0, 1.0]
+            try:
+               lock()
+               r = _cmd.transform_object(str(object),int(state)-1,
+                                         list(matrix),0,'',1)
+            finally:
+               unlock()
          else:
-            lock()
-            ttt = [1.0, 0.0, 0.0, shift[0],
-                   0.0, 1.0, 0.0, shift[1],
-                   0.0, 0.0, 1.0, shift[2],
-                   0.0, 0.0, 0.0, 1.0]
-            r=_cmd.combine_object_ttt(str(object),ttt)
-            unlock()
-
+            print " Error: translate: unrecognized object_mode"
       return r
 
    def rotate(axis='x',angle=0.0,selection="all",
-              state=0,camera=1,object=None,origin=None):
+              state=0,camera=1,object=None,origin=None,object_mode=0):
       '''
 DESCRIPTION
 
@@ -1175,6 +1202,7 @@ NOTES
 
       '''
       r = 1
+      object_mode = int(object_mode)
       if axis in ['x','X']:
          axis = [1.0,0.0,0.0]
       elif axis in ['y','Y']:
@@ -1182,7 +1210,7 @@ NOTES
       elif axis in ['z','Z']:
          axis = [0.0,0.0,1.0]
       else:
-         axis = eval(axis)
+         axis = safe_list_eval(str(axis))
       if not cmd.is_list(axis):
          print "Error: bad axis."
          raise QuietException
@@ -1192,7 +1220,7 @@ NOTES
          view = cmd.get_view(0)
          if origin!=None:
             if cmd.is_string(origin):
-               origin = eval(origin) # should be a sequence of floats
+               origin = safe_list_eval(origin) # should be a sequence of floats
             origin = [float(origin[0]),float(origin[1]),float(origin[2])]
          else:
             origin = [view[12],view[13],view[14]]
@@ -1202,21 +1230,36 @@ NOTES
             axis = cpv.transform(vmat,axis)
          mat = cpv.rotation_matrix(angle,axis)
          if object==None:
-            ttt = [mat[0][0],mat[1][0],mat[2][0],-origin[0],                   
-                   mat[0][1],mat[1][1],mat[2][1],-origin[1],
-                   mat[0][2],mat[1][2],mat[2][2],-origin[2],
-                   origin[0],origin[1],origin[2], view[15]]
+            ttt = [mat[0][0],mat[0][1],mat[0][2],origin[0],                   
+                   mat[1][0],mat[1][1],mat[1][2],origin[1],
+                   mat[2][0],mat[2][1],mat[2][2],origin[2],
+                   -origin[0],-origin[1],-origin[2], 1.0]
             r=cmd.transform_selection(selection,ttt,state=state)
-         else:
+         elif object_mode==0:
             lock()
-            ttt = [mat[0][0],mat[1][0],mat[2][0], 0.0,
-                   mat[0][1],mat[1][1],mat[2][1], 0.0,
-                   mat[0][2],mat[1][2],mat[2][2], 0.0,
+            ttt = [mat[0][0],mat[0][1],mat[0][2], 0.0,
+                   mat[1][0],mat[1][1],mat[1][2], 0.0,
+                   mat[2][0],mat[2][1],mat[2][2], 0.0,
                    0.0      ,0.0      ,0.0      , 1.0]
             r=_cmd.combine_object_ttt(str(object),ttt)
             unlock()
+         elif object_mode==1:
+            matrix = [mat[0][0],mat[0][1],mat[0][2], origin[0],     
+                      mat[1][0],mat[1][1],mat[1][2], origin[1],
+                      mat[2][0],mat[2][1],mat[2][2], origin[2],
+                      -origin[0],-origin[1],-origin[2], 1.0]
+            try:
+               lock()
+               r = _cmd.transform_object(str(object),int(state)-1,
+                                         list(matrix),0,'',0)
+            finally:
+               unlock()
+            
+         else:
+            print " Error: rotate: unrecognized object_mode"
       return r
 
+
    def set_title(object,state,text):
       '''
 DESCRIPTION
@@ -1241,14 +1284,45 @@ PYMOL API
       finally:
          unlock()
 
+   def set_object_ttt(object,ttt,state=0,quiet=1,homogenous=0):
+      r = None
+      if cmd.is_string(ttt):
+         ttt = safe_list_eval(str(ttt))
+      if homogenous: # passed a homogenous matrix, so do the best we can
+         ttt = [
+            ttt[ 0], ttt[ 4], ttt[ 8], 0.0,
+            ttt[ 1], ttt[ 5], ttt[ 9], 0.0,
+            ttt[ 2], ttt[ 6], ttt[10], 0.0,
+            ttt[ 3], ttt[ 7], ttt[11], 1.0]
+      try:
+         lock()
+         r = _cmd.set_object_ttt(str(object),
+                                 (
+            float(ttt[ 0]),
+            float(ttt[ 1]),
+            float(ttt[ 2]),
+            float(ttt[ 3]),            
+            float(ttt[ 4]),
+            float(ttt[ 5]),
+            float(ttt[ 6]),
+            float(ttt[ 7]),            
+            float(ttt[ 8]),
+            float(ttt[ 9]),
+            float(ttt[10]),
+            float(ttt[11]),            
+            float(ttt[12]),
+            float(ttt[13]),
+            float(ttt[14]),
+            float(ttt[15])),
+                                 int(state)-1,int(quiet))
+      finally:
+         unlock()
+      return r
 
-   def transform_selection(selection,matrix,state=0,log=0):
+   def transform_selection(selection,matrix,state=0,log=0,homogenous=0,transpose=0):
       '''
 
-"transform_selection" is currently UNSUPPORTED, but some information has
-been provided via the mailing list:
-
-RE: transform_selection
+"transform_selection" is UNSUPPORTED.
 
 cmd.transform_selection(string selection, list-of-16-floats matrix, int state-number):
 
@@ -1258,12 +1332,12 @@ following:
 
 1) a 3x3 matrix containing the rotation in the upper-left quadrant
 
-2) a 3x1 translation to be applied before rotation in the right-hand
-   column (matrix[3],matrix[7],matrix[11])
-
-3) a 1x3 translation to be applied after rotation in the bottom row
+2) a 1x3 translation to be applied *before* rotation in the bottom row
    (matrix[12],matrix[13],matrix[14]).
 
+3) a 3x1 translation to be applied *after* rotation in the right-hand
+   column (matrix[3],matrix[7],matrix[11])
+
 In other words, if the matrix is:
 
 [  m0  m1  m2  m3
@@ -1275,37 +1349,86 @@ Atoms will be transformed as follows
 
 Y = M X
 
-y0 = m0*(x0+m3) + m4*(x1+m7) +  m8*(x2+m11) + m12
-y1 = m1*(x0+m3) + m5*(x1+m7) +  m9*(x2+m11) + m13
-y2 = m2*(x0+m3) + m6*(x1+m7) + m10*(x2+m11) + m14
+y0 = m0*(x0+m12) + m1*(x1+m13) +  m8*(x2+m14) + m3
+y1 = m4*(x0+m12) + m5*(x1+m13) +  m9*(x2+m14) + m7
+y2 = m8*(x0+m12) + m9*(x1+m13) + m10*(x2+m14) + m11
 
       '''
+      if int(transpose):
+         matrix = [ matrix[0], matrix[4], matrix[8 ], matrix[12],
+                    matrix[1], matrix[5], matrix[9 ], matrix[13],
+                    matrix[2], matrix[6], matrix[10], matrix[14],
+                    matrix[3], matrix[7], matrix[11], matrix[15]]
       try:
          lock()
-         r = _cmd.transform_selection(str(selection),int(state)-1,list(matrix),int(log))
+         r = _cmd.transform_selection(str(selection),int(state)-1,
+                                      list(matrix),int(log),int(homogenous))
       finally:
          unlock()
 
-   def transform_object(name,matrix,state=0,log=0,selection=''):
+   def transform_object(name,matrix,state=0,log=0,selection='',homogenous=0,transpose=0):
       r = None
+      if int(transpose):
+         matrix = [ matrix[0], matrix[4], matrix[8 ], matrix[12],
+                    matrix[1], matrix[5], matrix[9 ], matrix[13],
+                    matrix[2], matrix[6], matrix[10], matrix[14],
+                    matrix[3], matrix[7], matrix[11], matrix[15]]
       try:
          lock()
-         r = _cmd.transform_object(str(name),int(state)-1,list(matrix),int(log),str(selection))
+         r = _cmd.transform_object(str(name),int(state)-1,
+                                   list(matrix),int(log),
+                                   str(selection),
+                                   int(homogenous))
       finally:
          unlock()
       return r
 
+   def matrix_transfer(source_name,    target_name,
+                       source_mode=0,  target_mode=0,
+                       source_state=1, target_state=1,
+                       target_undo=1, log=0, quiet=1):
+      r = None
+      try:
+         lock()
+         r = _cmd.matrix_transfer(str(source_name),
+                                  str(target_name),
+                                  int(source_mode),
+                                  int(target_mode),
+                                  int(source_state)-1,
+                                  int(target_state)-1,
+                                  int(target_undo),
+                                  int(log),
+                                  int(quiet))
+      finally:
+         unlock()
+      return r
+
+   def matrix_reset(name, state=1, mode=0, log=0, quiet=1):
+      r = None
+      try:
+         lock()
+         r = _cmd.reset_matrix(str(name),
+                               int(mode),
+                               int(state)-1,
+                               int(log),
+                               int(quiet))
+      finally:
+         unlock()
+      return r
+
+      
    def translate_atom(sele1,v0,v1,v2,state=0,mode=0,log=0):
       r = None
       sele1 = selector.process(sele1)
       try:
          lock()
-         r = _cmd.translate_atom(str(sele1),float(v0),float(v1),float(v2),int(state)-1,int(mode),int(log))
+         r = _cmd.translate_atom(str(sele1),float(v0),float(v1),
+                                 float(v2),int(state)-1,int(mode),int(log))
       finally:
          unlock()
       return r
 
-   def update(target,source,target_state=0,source_state=0):
+   def update(target,source,target_state=0,source_state=0,matchmaker=1,quiet=1):
       '''
 DESCRIPTION
 
@@ -1327,7 +1450,6 @@ SEE ALSO
 
    load
    '''
-
       a=target
       b=source
       # preprocess selections
@@ -1338,7 +1460,8 @@ SEE ALSO
       if b[0]!='(': b="("+str(b)+")"   
       try:
          lock()   
-         r = _cmd.update(str(a),str(b),int(target_state)-1,int(source_state)-1)
+         r = _cmd.update(str(a),str(b),int(target_state)-1,
+                         int(source_state)-1,int(matchmaker),int(quiet))
       finally:
          unlock()
       return r
@@ -1353,7 +1476,8 @@ SEE ALSO
       #   
       try:
          lock()
-         r = _cmd.set_dihe(str(atom1),str(atom2),str(atom3),str(atom4),float(angle),int(state)-1,int(quiet))
+         r = _cmd.set_dihe(str(atom1),str(atom2),str(atom3),str(atom4),
+                           float(angle),int(state)-1,int(quiet))
       finally:
          unlock()
       return r
@@ -1486,12 +1610,12 @@ SEE ALSO
 
    flag_action_sc = Shortcut(flag_action_dict.keys())
 
-   def fix_chemistry(selection1 = "all",selection2 = "all", quiet=1):
+   def fix_chemistry(selection1 = "all",selection2 = "all", invalidate=0, quiet=1):
       selection1 = selector.process(selection1)
       selection2 = selector.process(selection2)
       try:
          lock()   
-         r = _cmd.fix_chemistry("("+str(selection1)+")","("+str(selection2)+")",int(quiet))
+         r = _cmd.fix_chemistry("("+str(selection1)+")","("+str(selection2)+")",int(invalidate),int(quiet))
       finally:
          unlock()
       return r
@@ -1544,10 +1668,11 @@ RESERVED FLAGS
       exclude    4 = Atoms which should not be part of any simulation
    Flags 8-15 are free for end users to manipulate
    Flags 16-23 are reserved for external GUIs and linked applications
-   Flags 24-31 are reserved for PyMOL internal usage
+   Flags 24-30 are reserved for PyMOL internal usage
       exfoliate 24 = Remove surface from atoms when surfacing
       ignore    25 = Ignore atoms altogether when surfacing
       no_smooth 26 = Don\'t smooth atom position
+   Flag 31 is reserved for coverage tracking when assigning parameters, etc.
       '''
       # preprocess selection
       new_flag = flag_sc.interpret(str(flag))
@@ -1568,3 +1693,4 @@ RESERVED FLAGS
          unlock()
       return r
 
+   import pymol
