@@ -71,7 +71,7 @@ SEE ALSO
          unlock()
       return r
 
-   def intra_fit(selection,state=1,quiet=1):
+   def intra_fit(selection,state=1,quiet=1,mix=0):
       '''
 DESCRIPTION
 
@@ -105,9 +105,10 @@ SEE ALSO
       #   
       r = None
       state = int(state)
+      mix = int(mix)
       try:
          lock()
-         r = _cmd.intrafit("("+str(selection)+")",int(state)-1,2,int(quiet))
+         r = _cmd.intrafit("("+str(selection)+")",int(state)-1,2,int(quiet),int(mix))
       finally:
          unlock()
       if not r:
@@ -116,7 +117,10 @@ SEE ALSO
          st = 1
          for a in r:
             if a>=0.0:
-               print " cmd.intra_fit: %5.3f in state %d vs state %d"%(a,st,state)
+               if mix:
+                  print " cmd.intra_fit: %5.3f in state %d vs mixed target"%(a,st)
+               else:
+                  print " cmd.intra_fit: %5.3f in state %d vs state %d"%(a,st,state)
             st = st + 1
       return r
 
@@ -149,7 +153,7 @@ SEE ALSO
       state = int(state)
       try:
          lock()
-         r = _cmd.intrafit("("+str(selection)+")",int(state)-1,1,int(quiet))
+         r = _cmd.intrafit("("+str(selection)+")",int(state)-1,1,int(quiet),int(0))
       finally:
          unlock()
       if not r:
@@ -191,7 +195,7 @@ SEE ALSO
       state = int(state)
       try:
          lock()
-         r = _cmd.intrafit("("+str(selection)+")",int(state)-1,0,int(quiet))
+         r = _cmd.intrafit("("+str(selection)+")",int(state)-1,0,int(quiet),int(0))
       finally:
          unlock()
       if not r:
@@ -204,7 +208,8 @@ SEE ALSO
             st = st + 1
       return r
 
-   def fit(selection,target,source_state=0,target_state=0,quiet=1):
+   def fit(selection,target,source_state=0,target_state=0,
+           quiet=1,matchmaker=0,cutoff=2.0,cycles=0,object=None):
       '''
 DESCRIPTION
 
@@ -230,17 +235,25 @@ SEE ALSO
       a = selector.process(a)
       b = selector.process(b)
       #
+      if object==None: object=''
+      if matchmaker==0:
+         sele1 = "((%s) in (%s))" % (str(a),str(b))
+         sele2 = "((%s) in (%s))" % (str(b),str(a))
+      else:
+         sele1 = str(a)
+         sele2 = str(b)
       try:
-         lock()   
-         r = _cmd.fit("((%s) in (%s))" % (str(a),str(b)),
-                     "((%s) in (%s))" % (str(b),str(a)),2,
+         lock()
+         r = _cmd.fit(sele1,sele2,2,
                       int(source_state)-1,int(target_state)-1,
-                      int(quiet))
+                      int(quiet),int(matchmaker),float(cutoff),
+                      int(cycles),str(object))
       finally:
          unlock()
       return r
 
-   def rms(selection,target,source_state=0,target_state=0,quiet=1):
+   def rms(selection,target,source_state=0,target_state=0,quiet=1,
+           matchmaker=0,cutoff=2.0,cycles=0,object=None):
       '''
 DESCRIPTION
 
@@ -265,17 +278,25 @@ SEE ALSO
       a = selector.process(a)
       b = selector.process(b)
       #
+      if object==None: object=''      
+      if matchmaker==0:
+         sele1 = "((%s) in (%s))" % (str(a),str(b))
+         sele2 = "((%s) in (%s))" % (str(b),str(a))
+      else:
+         sele1 = str(a)
+         sele2 = str(b)
       try:
          lock()   
-         r = _cmd.fit("((%s) in (%s))" % (str(a),str(b)),
-                     "((%s) in (%s))" % (str(b),str(a)),1,
+         r = _cmd.fit(sele1,sele2,1,
                       int(source_state)-1,int(target_state)-1,
-                      int(quiet))
+                      int(quiet),int(matchmaker),float(cutoff),
+                      int(cycles),str(object))
       finally:
          unlock()
       return r
 
-   def rms_cur(selection,target,source_state=0,target_state=0,quiet=1):
+   def rms_cur(selection,target,source_state=0,target_state=0,
+               quiet=1,matchmaker=0,cutoff=2.0,cycles=0,object=None):
       '''
 DESCRIPTION
 
@@ -295,13 +316,20 @@ SEE ALSO
       # preprocess selections
       a = selector.process(a)
       b = selector.process(b)
-      #   
+      #
+      if object==None: object=''            
+      if matchmaker==0:
+         sele1 = "((%s) in (%s))" % (str(a),str(b))
+         sele2 = "((%s) in (%s))" % (str(b),str(a))
+      else:
+         sele1 = str(a)
+         sele2 = str(b)
       try:
-         lock()   
-         r = _cmd.fit("((%s) in (%s))" % (str(a),str(b)),
-                     "((%s) in (%s))" % (str(b),str(a)),0,
+         lock()
+         r = _cmd.fit(sele1,sele2,0,
                       int(source_state)-1,int(target_state)-1,
-                      int(quiet))
+                      int(quiet),int(matchmaker),float(cutoff),
+                      int(cycles),str(object))
       finally:
          unlock()
       return r
