@@ -21,15 +21,17 @@ if __name__=='pymol.importing':
    from cmd import _cmd,lock,unlock,Shortcut,QuietException, \
         _feedback,fb_module,fb_mask, \
         file_ext_re,safe_oname_re, \
-        _load, is_list, space_sc
+        _load, is_list, space_sc, safe_list_eval
+   
    import selector
    try:
       from pymol import m4x
    except ImportError:
       m4x = None
 
+   from pymol import parser
    from chempy.sdf import SDF,SDFRec
-   from chempy import io
+   from chempy import io,PseudoFile
    import pymol
    import copy
    import traceback
@@ -39,6 +41,7 @@ if __name__=='pymol.importing':
       mol = 1
       molstr = 3
       mmod = 4
+      mmodstr = 6
       xplor = 7
       model = 8
       pdbstr = 9    
@@ -48,7 +51,7 @@ if __name__=='pymol.importing':
       cgo = 13      # compiled graphic object
       r3d = 14      # r3d, only used within cmd.py
       xyz = 15      # xyz, tinker format
-      sdf = 16      # sdf, only used within cmd.py
+      sdf1 = 16     # sdf, only used within cmd.py
       cc1 = 17      # cc1 and cc2, only used within cmd.py
       ccp4 = 18     # CCP4 map, under development
       pmo = 19      # pmo, experimental molecular object format
@@ -66,7 +69,13 @@ if __name__=='pymol.importing':
       pqr = 31      # PQR file (modified PDB file for APBS)
       dx = 32       # DX file (APBS)
       mol2 = 33     # MOL2 file (TRIPOS)
-      
+      mol2str = 34  # MOL2 file string (TRIPOS)
+      p1m = 35      # P1M file (combined data & secure commands)
+      ccp4str = 36  # CCP4 map string
+      sdf = 37      # new default...
+      sdf2 = 37     # SDF using C-based SDF parser (instead of Python)
+      sdf2str = 38  # SDF ditto
+
    loadable_sc = Shortcut(loadable.__dict__.keys()) 
 
    def set_session(session):
@@ -93,7 +102,8 @@ if __name__=='pymol.importing':
          cmd.wizard("security")
       return r
 
-   def load_object(type,object,name,state=0,finish=1,discrete=0,quiet=1):
+   def load_object(type,object,name,state=0,finish=1,discrete=0,
+                   quiet=1,zoom=-1):
       '''
 DESCRIPTION
 
@@ -102,10 +112,10 @@ DESCRIPTION
 
 PYMOL API
 
-   cmd.load_object(type,object,name,state=0,finish=1,discrete=0)
+   cmd.load_object(type,object,name,state=0,finish=1,discrete=0,quiet=1)
 
-   type = one one of the numberic cmd.loadable types
-   object =
+   type = one one of the numeric cmd.loadable types
+   object = 
    name = object name (string)
    finish = perform (1) or defer (0) post-processing of structure after load
    discrete = treat each state as an independent, unrelated set of atoms
@@ -115,7 +125,8 @@ PYMOL API
       try:
          lock()   
          r = _cmd.load_object(str(name),object,int(state)-1,
-                                 int(type),int(finish),int(discrete),int(quiet))
+                              int(type),int(finish),int(discrete),
+                              int(quiet),int(zoom))
       finally:
          unlock()
       return r
@@ -284,7 +295,7 @@ SEE ALSO
          stop = int(stop)
          max = int(max)
          image = int(image)
-         shift = eval(shift) # dangerous
+         shift = safe_list_eval(str(shift)) # dangerous
          if is_list(shift):
             shift = [float(shift[0]),float(shift[1]),float(shift[2])]
          else:
@@ -338,20 +349,29 @@ SEE ALSO
          unlock()
       return r
 
-
-
-   def load(filename,object='',state=0,format='',finish=1,discrete=None,quiet=1):
+   def _processSDF(sdf,oname,state,quiet):
+      ftype = loadable.molstr
+      while 1:
+         rec = sdf.read()
+         if not rec: break
+         r = _load(oname,string.join(rec.get('MOL'),''),state,loadable.molstr,0,1,quiet)
+      del sdf
+      _cmd.finish_object(str(oname))
+      if _cmd.get_setting("auto_zoom")==1.0:
+         cmd._do("zoom (%s)"%oname)
+   
+   def load(filename,object='',state=0,format='',finish=1,
+            discrete=-1,quiet=1,multiplex=None,zoom=-1):
       '''
 DESCRIPTION
 
    "load" reads several file formats.  The file extension is used to
    determine the format.  PDB files must end in ".pdb", MOL files must
-   end in ".mol", Macromodel files must end in ".mmod", XPLOR
-   maps must end in ".xplor", CCP4 maps must end in ".ccp4",
-   Raster3D input (Molscript output) must end in ".r3d", PyMOL session
-   files must end in ".pse"
-
-   Pickled ChemPy models with a ".pkl" can also be directly read.
+   end in ".mol", Macromodel files must end in ".mmod", XPLOR maps
+   must end in ".xplor", CCP4 maps must end in ".ccp4", Raster3D input
+   (Molscript output) must end in ".r3d", PyMOL session files must end
+   in ".pse", and pickled ChemPy models with a ".pkl" can also be directly
+   read.
 
    If an object is specified, then the file is loaded into that object.
    Otherwise, an object is created with the same name as the file
@@ -377,7 +397,7 @@ NOTES
    'trj' : AMBER trajectory (use load_traj command for more control)
    'top' : AMBER topology file 'rst' : AMBER restart file
    'cex' : Metaphorics CEX format
-   'pse' : PyMOL Session file
+   'pse' : PyMOL Session file ('psw' PyMOL Show)
    'pqr' : PQR (a modified PDB file with charges and radii)
    'mol2' : MOL2
    
@@ -392,16 +412,20 @@ SEE ALSO
          ftype = 0
          state = int(state)
          finish = int(finish)
-         if discrete==None:
+         zoom = int(zoom)
+         if discrete==-1:
             discrete_default = 1
             discrete=0
          else:
             discrete_default = 0
             discrete = int(discrete)
+         if multiplex==None:
+            multiplex=-2
          fname = filename
          fname = os.path.expanduser(fname)
          fname = os.path.expandvars(fname)
-
+         go_to_first_scene = 0
+         
          if not len(str(type)):
             # determine file type if possible
             if re.search("\.pdb$|\.pdb1$|\.ent$|\.p5m",filename,re.I):
@@ -424,8 +448,8 @@ SEE ALSO
                ftype = loadable.cc1
             elif re.search("\.xyz_[0-9]*$",filename,re.I):
                ftype = loadable.xyz
-            elif re.search("\.sdf$",filename,re.I):
-               ftype = loadable.sdf
+            elif re.search("\.sdf$|\.sd$",filename,re.I): 
+               ftype = loadable.sdf2 # now using the C-based SDF reader by default...
             elif re.search("\.cex$",filename,re.I):
                ftype = loadable.cex
             elif re.search("\.pmo$",filename,re.I):
@@ -438,7 +462,9 @@ SEE ALSO
                ftype = loadable.crd
             elif re.search("\.rst$",filename,re.I):
                ftype = loadable.crd
-            elif re.search("\.pse$",filename,re.I):
+            elif re.search("\.pse$|\.psw$",filename,re.I):
+               if re.search("\.psw$",filename,re.I):
+                  go_to_first_scene = 1
                ftype = loadable.pse
             elif re.search("\.phi$",filename,re.I):
                ftype = loadable.phi
@@ -454,6 +480,8 @@ SEE ALSO
                ftype = loadable.brix
             elif re.search("\.grd$",filename,re.I):
                ftype = loadable.grd
+            elif re.search("\.p1m$",filename,re.I):
+               ftype = loadable.p1m
             else:
                ftype = loadable.pdb # default is PDB
          elif cmd.is_string(type):
@@ -479,20 +507,18 @@ SEE ALSO
          else:
             oname = string.strip(object)
 
-   # special handling of sdf files
-         if ftype == loadable.sdf:
-            ftype = loadable.molstr
+   # loadable.sdf1 is for the old Python-based SDF file reader
+         if ftype == loadable.sdf1:
             sdf = SDF(fname)
-            while 1:
-               rec = sdf.read()
-               if not rec: break
-               r = _load(oname,string.join(rec.get('MOL'),''),state,ftype,0,1,quiet)
-            del sdf
-            _cmd.finish_object(str(oname))
-            _cmd.do("zoom (%s)"%oname,0)
-            _cmd.do("set seq_view_format,4,"+oname,0)
+            _processSDF(sdf,oname,state,quiet)
             ftype = -1
 
+   # p1m embedded data script files
+   
+         if ftype == loadable.p1m:
+            cmd._do("_ @"+fname)
+            ftype = -1
+            
    # special handling for AMBER trj failes
    #      if ftype == loadable.trj:
    #         ftype = -1
@@ -511,22 +537,97 @@ SEE ALSO
          if ftype == loadable.pse:
             ftype = -1
             cmd.set_session(io.pkl.fromFile(fname))
+   # special handling for multi-model files (mol2, sdf)
 
-   # special handling for mol2 files
+         if ftype in ( loadable.mol2, loadable.sdf1, loadable.sdf2):
+            if discrete_default==1: # make such files discrete by default
+               discrete = -1
 
-         if ftype == loadable.mol2:
-            if discrete_default==1: # make mol2 files discrete by default
-               discrete = 1
-               
    # standard file handling
-
          if ftype>=0:
-            r = _load(oname,fname,state,ftype,finish,discrete,quiet)
+            r = _load(oname,fname,state,ftype,finish,
+                      discrete,quiet,multiplex,zoom)
+      finally:
+         unlock()
+      if go_to_first_scene:
+         if int(cmd.get_setting_legacy("presentation_auto_start"))!=0:
+            cmd.scene("auto","start",animate=0)
+      return r
+
+   def load_embedded(key=None,name=None,state=0,finish=1,discrete=1,quiet=1):
+      r = 1
+      list = parser.get_embedded(key)
+      if list == None:
+         print "Error: embedded data '%s' not found."%key
+         r = None
+      else:
+         if name == None:
+            if key != None:
+               name = key
+            else:
+               name = parser.get_default_key()
+         type = list[0]
+         data = list[1]
+         try:
+            ftype = int(type)
+         except:
+            type = loadable_sc.auto_err(type,'file type')
+            if hasattr(loadable,type):
+               ftype = getattr(loadable,type)
+            else:
+               print "Error: unknown type '%s'",type
+               raise QuietException
+         if ftype==loadable.pdb:
+            read_pdbstr(string.join(data,''),name,state,finish,discrete,quiet)
+         elif ftype==loadable.mol:
+            read_molstr(string.join(data,''),name,state,finish,discrete,quiet)
+         elif ftype==loadable.mol2:
+            read_mol2str(string.join(data,''),name,state,finish,discrete,quiet)
+         elif ftype==loadable.xplor:
+            read_xplorstr(string.join(data,''),name,state,finish,discrete,quiet)
+         elif ftype==loadable.sdf1: # Python-based SDF reader
+            sdf = SDF(PseudoFile(data),'pf')
+            _processSDF(sdf,name,state,quiet)
+         elif ftype==loadable.sdf2: # C-based SDF reader (much faster)
+            read_sdfstr(string.join(data,''),name,state,finish,discrete,quiet)
+      return r
+   
+   def read_sdfstr(sdfstr,name,state=0,finish=1,discrete=1,quiet=1,
+                   zoom=-1):
+      '''
+DESCRIPTION
+
+   "read_sdfstr" reads an MDL MOL format file as a string
+
+PYMOL API ONLY
+
+   cmd.read_sdfstr( string molstr, string name, int state=0,
+      int finish=1, int discrete=1 )
+
+NOTES
+
+   "state" is a 1-based state index for the object, or 0 to append.
+
+   "finish" is a flag (0 or 1) which can be set to zero to improve
+   performance when loading large numbers of objects, but you must
+   call "finish_object" when you are done.
+
+   "discrete" is a flag (0 or 1) which tells PyMOL that there will be
+   no overlapping atoms in the file being loaded.  "discrete"
+   objects save memory but can not be edited.
+      '''
+      r = 1
+      try:
+         lock()
+         r = _cmd.load(str(name),str(sdfstr),int(state)-1,
+                       loadable.sdf2str,int(finish),int(discrete),
+                       int(quiet),0,int(zoom))
       finally:
          unlock()
       return r
 
-   def read_molstr(molstr,name,state=0,finish=1,discrete=1,quiet=1):
+   def read_molstr(molstr,name,state=0,finish=1,discrete=1,quiet=1,
+                   zoom=-1):
       '''
 DESCRIPTION
 
@@ -553,7 +654,8 @@ NOTES
       try:
          lock()
          r = _cmd.load(str(name),str(molstr),int(state)-1,
-                       loadable.molstr,int(finish),int(discrete),int(quiet))
+                       loadable.molstr,int(finish),int(discrete),
+                       int(quiet),0,int(zoom))
       finally:
          unlock()
       return r
@@ -569,24 +671,30 @@ DESCRIPTION
       r = 1
       try:
          lock()   
-         ftype = 6
+         ftype = loadable.mmodstr
          if kw.has_key('quiet'):
             quiet = int(kw['quiet'])
          else:
             quiet = 1
+         if kw.has_key('zoom'):
+            zoom = int(kw['zoom'])
+         else:
+            zoom = -1
          if len(arg)==2:
             oname = string.strip(arg[1])
-            r = _cmd.load(str(oname),arg[0],-1,int(ftype),1,1,quiet)
+            r = _cmd.load(str(oname),arg[0],-1,int(ftype),1,1,quiet,0,zoom)
          elif len(arg)==3:
             oname = string.strip(arg[1])
-            r = _cmd.load(str(oname),arg[0],int(arg[2])-1,int(ftype),1,1,quiet)
+            r = _cmd.load(str(oname),arg[0],int(arg[2])-1,
+                          int(ftype),1,1,quiet,0,zoom)
          else:
             print "argument error."
       finally:
          unlock()
       return r
 
-   def read_pdbstr(pdb,name,state=0,finish=1,discrete=0,quiet=1):
+   def read_pdbstr(pdb,name,state=0,finish=1,discrete=0,quiet=1,
+                   multiplex=-2,zoom=-1):
       '''
 DESCRIPTION
 
@@ -617,12 +725,52 @@ NOTES
          ftype = loadable.pdbstr
          oname = string.strip(str(name))
          r = _cmd.load(str(oname),pdb,int(state)-1,int(ftype),
-                          int(finish),int(discrete),int(quiet))
+                       int(finish),int(discrete),int(quiet),
+                       0,int(zoom))
       finally:
          unlock()
       return r
 
-   def read_xplorstr(xplor,name,state=0,finish=1,discrete=0,quiet=1):
+   def read_mol2str(mol2,name,state=0,finish=1,discrete=0,
+                    quiet=1,zoom=-1):
+      '''
+DESCRIPTION
+
+   "read_mol2str" in an API-only function which reads a mol2 file from a
+   Python string.  This feature can be used to load or update
+   structures into PyMOL without involving any temporary files.
+
+PYMOL API ONLY
+
+   cmd.read_mol2str( string mol2-content, string object name 
+      [ ,int state [ ,int finish [ ,int discrete ] ] ] )
+
+NOTES
+
+   "state" is a 1-based state index for the object.
+
+   "finish" is a flag (0 or 1) which can be set to zero to improve
+   performance when loading large numbers of objects, but you must
+   call "finish_object" when you are done.
+
+   "discrete" is a flag (0 or 1) which tells PyMOL that there will be
+   no overlapping atoms in the MOL2 files being loaded.  "discrete"
+   objects save memory but can not be edited.
+   '''
+      r = 1
+      try:
+         lock()   
+         ftype = loadable.mol2str
+         oname = string.strip(str(name))
+         r = _cmd.load(str(oname),mol2,int(state)-1,int(ftype),
+                       int(finish),int(discrete),int(quiet),
+                       0,int(zoom))
+      finally:
+         unlock()
+      return r
+
+   def read_xplorstr(xplor,name,state=0,finish=1,discrete=0,
+                     quiet=1,zoom=-1):
       '''
 DESCRIPTION
 
@@ -646,7 +794,8 @@ NOTES
          ftype = loadable.xplorstr
          oname = string.strip(str(name))
          r = _cmd.load(str(oname),xplor,int(state)-1,int(ftype),
-                          int(finish),int(discrete),int(quiet))
+                       int(finish),int(discrete),int(quiet),
+                       0,int(zoom))
       finally:
          unlock()
       return r
