@@ -20,6 +20,31 @@ if __name__=='pymol.querying':
    from cmd import _cmd,lock,unlock,Shortcut,QuietException
    from cmd import _feedback,fb_module,fb_mask,is_list
 
+   def get_object_matrix(object,state=1):
+      r = None
+      object = str(object)
+      try:
+         lock()   
+         r = _cmd.get_object_matrix(str(object), int(state)-1)
+      finally:
+         unlock()
+      return r
+
+   def get_object_list(selection="(all)",quiet=1):
+      '''
+      '''
+      r = []
+      selection = selector.process(selection)
+      try:
+         lock()
+         r = _cmd.get_object_list(str(selection))
+         if not quiet:
+            if(is_list(r)):
+               print " get_object_list: ",str(r)
+      finally:
+         unlock()
+      return r
+
    def get_symmetry(selection="(all)",quiet=1):
       '''
 DESCRIPTION
@@ -101,8 +126,178 @@ PYMOL API
          unlock()
       return r
 
+   def angle(name=None,selection1="(pk1)",selection2="(pk2)",selection3="(pk3)", 
+                mode=None,labels=1,reset=0,zoom=0,quiet=1):
+      '''
+DESCRIPTION
+
+   "angle" shows the angle formed between any three atoms.
+
+USAGE
+
+   angle 
+   angle name, selection1, selection2, selection3
+
+PYMOL API
+
+   cmd.angle(string name=None,
+             string selection1="(pk1)",
+             string selection2="(pk2)",
+             string selection3="(pk3)")
+
+NOTES
+
+   "angle" alone will show the angle angle formed by selections (pk1),
+   (pk2), (pk3) which can be set using the "PkAt" mouse action
+   (typically, Ctrl-middle-click)
+
+   '''
+      
+      r = 1
+      if selection1=="(pk1)":
+         if "pk1" not in cmd.get_names('selections'):
+            if _feedback(fb_module.cmd,fb_mask.errors):
+               print "cmd-Error: The 'pk1' selection is undefined."
+            r = 0
+      if selection2=="(pk2)":
+         if "pk2" not in cmd.get_names('selections'):
+            if _feedback(fb_module.cmd,fb_mask.errors):         
+               print "cmd-Error: The 'pk2' selection is undefined."
+            r = 0
+      if selection3=="(pk3)":
+         if "pk3" not in cmd.get_names('selections'):
+            if _feedback(fb_module.cmd,fb_mask.errors):         
+               print "cmd-Error: The 'pk3' selection is undefined."
+            r = 0
+      if r:         
+         # if unlabeled, then get next name in series
+
+         if name!=None:
+            nam=name
+         else:
+            try:
+               lock()
+               cnt = _cmd.get("dist_counter") + 1.0
+               _cmd.legacy_set("dist_counter","%1.0f" % cnt)
+               nam = "angle%02.0f" % cnt
+            finally:
+               unlock()
+
+         # defaults
+         if mode == None:
+            mode = 0
+         # preprocess selections
+         selection1 = selector.process(selection1)
+         selection2 = selector.process(selection2)
+         selection3 = selector.process(selection3)
+         # now do the deed
+         try:
+            lock()
+            if selection2!="same":
+               selection2 = "("+selection2+")"
+            if selection3!="same":
+               selection3 = "("+selection3+")"
+            r = _cmd.angle(str(nam),"("+str(selection1)+")",
+                          str(selection2),
+                          str(selection3),
+                          int(mode),int(labels),int(reset),int(zoom),int(quiet))
+         finally:
+            unlock()
+      return r
+
+   def dihedral(name=None,selection1="(pk1)",selection2="(pk2)",
+                selection3="(pk3)",selection4="(pk4)",
+                mode=None,labels=1,reset=0,zoom=0,quiet=1):
+      '''
+DESCRIPTION
+
+   "dihedral" shows dihedral angles formed between any four atoms.
+
+USAGE
+
+   dihedral 
+   dihedral name, selection1, selection2, selection3, selection4
+
+PYMOL API
+
+   cmd.dihedral(string name=None,
+                string selection1="(pk1)",
+                string selection2="(pk2)",
+                string selection3="(pk3)",
+                string selection4="(pk4)")
+
+NOTES
+
+   "dihedral" alone will show the dihedral angle formed by selections
+   (pk1), (pk2), (pk3), and (pk4), which can be set using the "PkAt"
+   mouse action (typically, Ctrl-middle-click)
+
+   '''
+      
+      r = 1
+      if selection1=="(pk1)":
+         if "pk1" not in cmd.get_names('selections'):
+            if _feedback(fb_module.cmd,fb_mask.errors):
+               print "cmd-Error: The 'pk1' selection is undefined."
+            r = 0
+      if selection2=="(pk2)":
+         if "pk2" not in cmd.get_names('selections'):
+            if _feedback(fb_module.cmd,fb_mask.errors):         
+               print "cmd-Error: The 'pk2' selection is undefined."
+            r = 0
+      if selection3=="(pk3)":
+         if "pk3" not in cmd.get_names('selections'):
+            if _feedback(fb_module.cmd,fb_mask.errors):         
+               print "cmd-Error: The 'pk3' selection is undefined."
+            r = 0
+      if selection3=="(pk4)":
+         if "pk4" not in cmd.get_names('selections'):
+            if _feedback(fb_module.cmd,fb_mask.errors):         
+               print "cmd-Error: The 'pk4' selection is undefined."
+            r = 0
+      if r:         
+         # if unlabeled, then get next name in series
+
+         if name!=None:
+            nam=name
+         else:
+            try:
+               lock()
+               cnt = _cmd.get("dist_counter") + 1.0
+               _cmd.legacy_set("dist_counter","%1.0f" % cnt)
+               nam = "dihedral%02.0f" % cnt
+            finally:
+               unlock()
+
+         # defaults
+         if mode == None:
+            mode = 0
+         # preprocess selections
+         selection1 = selector.process(selection1)
+         selection2 = selector.process(selection2)
+         selection3 = selector.process(selection3)
+         selection4 = selector.process(selection4)
+         # now do the deed
+         try:
+            lock()
+            if selection2!="same":
+               selection2 = "("+selection2+")"
+            if selection3!="same":
+               selection3 = "("+selection3+")"
+            if selection4!="same":
+               selection4 = "("+selection4+")"
+            r = _cmd.dihedral(str(nam),"("+str(selection1)+")",
+                           str(selection2),
+                           str(selection3),
+                           str(selection4),
+                           int(mode),int(labels),int(reset),int(zoom),int(quiet))
+         finally:
+            unlock()
+      return r
+      
    def distance(name=None,selection1="(pk1)",selection2="(pk2)",cutoff=None,
-                mode=None,zoom=0,width=None,length=None,gap=None,labels=1,quiet=1):
+                mode=None,zoom=0,width=None,length=None,gap=None,labels=1,quiet=1,
+                reset=0):
       '''
 DESCRIPTION
 
@@ -191,7 +386,7 @@ NOTES
                selection2 = "("+selection2+")"
             r = _cmd.dist(str(nam),"("+str(selection1)+")",
                           str(selection2),int(mode),float(cutoff),
-                          int(labels),int(quiet))
+                          int(labels),int(quiet),int(reset))
             if width!=None:
                cmd.set("dash_width",width,nam)
             if length!=None:
@@ -349,17 +544,31 @@ SEE ALSO
          if cmd._raising(): raise QuietException
       return r
 
-   def get_color_indices():
+   def get_color_indices(all=0):
       r = None
       try:
          lock()
-         r = _cmd.get_color('',1)
+         if all:
+            r = _cmd.get_color('',2)
+         else:
+            r = _cmd.get_color('',1)            
       finally:
          unlock()
       if not r:
          if cmd._raising(): raise QuietException
       return r
 
+   def get_color_index(color):
+      r = None
+      try:
+         lock()
+         r = _cmd.get_color(str(color),3)
+      finally:
+         unlock()
+      if r == None:
+         if cmd._raising(): raise QuietException
+      return r
+         
    def get_renderer():  # 
       try:
          lock()
@@ -594,7 +803,7 @@ PYMOL API
       return r
 
 
-   def get_names(type='objects',enabled_only=0):
+   def get_names(type='public_objects',enabled_only=0):
       '''
 DESCRIPTION
 
@@ -602,7 +811,7 @@ DESCRIPTION
 
 PYMOL API
 
-   cmd.get_names( [string: "objects"|"selections"|"all"] )
+   cmd.get_names( [string: "objects"|"selections"|"all"|"public_objects"|"public_selections"] )
 
 NOTES
 
@@ -621,6 +830,10 @@ SEE ALSO
          mode = 0
       elif type=='public':
          mode = 3
+      elif type=='public_objects':
+         mode = 4
+      elif type=='public_selections':
+         mode = 5
       try:
          lock()
          r = _cmd.get_names(int(mode),int(enabled_only))
