@@ -24,7 +24,6 @@ from pymol import movie
 mload = movie.load
 mrock = movie.rock
 mroll = movie.roll
-
    
 def sum_formal_charges(selection="(all)",quiet=1):
    pymol.stored._util_sum_fc = 0.0
@@ -42,7 +41,7 @@ def sum_partial_charges(selection="(all)",quiet=1):
       print " util.sum_partial_charges: sum = %0.3f"%result
    return result
 
-def protein_vacuum_esp(selection, absolute=1,border=10.0,quiet = 1):
+def protein_vacuum_esp(selection, mode=2,border=10.0,quiet = 1):
 
    if ((string.split(selection)!=[selection]) or
        selection not in cmd.get_names('objects')):
@@ -112,15 +111,18 @@ def protein_vacuum_esp(selection, absolute=1,border=10.0,quiet = 1):
    sep = max_length/50.0
    if sep<1.0: sep = 1.0
    print " Util: Calculating electrostatic potential..."
-   if absolute:
+   if mode==0: # absolute, no cutoff
       cmd.map_new(map_name,"coulomb",sep,obj_name,border)
-   else:
+   elif mode==1: # neutral, no cutoff
       cmd.map_new(map_name,"coulomb_neutral",sep,obj_name,border)
+   else: # local, with cutoff
+      cmd.map_new(map_name,"coulomb_local",sep,obj_name,border)      
       
    cmd.ramp_new(pot_name, map_name, selection=obj_name,zero=1)
    cmd.hide("everything",obj_name)
    cmd.show("surface",obj_name)
    cmd.set("surface_color",pot_name,obj_name)
+   cmd.set("surface_ramp_above_mode",1,obj_name)
    
 def color_carbon(color,selection="(all)"):
    selection = str(selection)
@@ -137,101 +139,63 @@ def cbss(selection="(all)",helix_color="red",sheet_color="yellow",loop_color="gr
 
 def cbag(selection="(all)",quiet=1):
    s = str(selection)
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color("carbon","(elem C and ("+s+"))",quiet=quiet)
-
+   
 def cbac(selection="(all)",quiet=1):
    s = str(selection)
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color("cyan","(elem C and ("+s+"))",quiet=quiet)
+   
+def cbam(selection="(all)",quiet=1):
+   s = str(selection)   
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
+   cmd.color("lightmagenta","(elem C and ("+s+"))",quiet=quiet)
 
 def cbay(selection="(all)",quiet=1):
    s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color("yellow","(elem C and ("+s+"))",quiet=quiet)
 
 def cbas(selection="(all)",quiet=1):
    s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color("salmon","(elem C and ("+s+"))",quiet=quiet)
 
-def cbap(selection="(all)",quiet=1):
-   s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
-   cmd.color("purple","(elem C and ("+s+"))",quiet=quiet)
-
-def cbak(selection="(all)",quiet=1):
-   s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
-   cmd.color("pink","(elem C and ("+s+"))",quiet=quiet)
-
-def cbam(selection="(all)",quiet=1):
-   s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
-   cmd.color("lightmagenta","(elem C and ("+s+"))",quiet=quiet)
-
 def cbaw(selection="(all)",quiet=1):
    s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color("hydrogen","(elem C and ("+s+"))",quiet=quiet)
 
 def cbab(selection="(all)",quiet=1):
    s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color("slate","(elem C and ("+s+"))",quiet=quiet)
 
 def cbao(selection="(all)",quiet=1):
    s = str(selection)   
-   cmd.color("magenta","(("+s+"))",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color("brightorange","(elem C and ("+s+"))",quiet=quiet)
 
+def cbap(selection="(all)",quiet=1):
+   s = str(selection)   
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
+   cmd.color("purple","(elem C and ("+s+"))",quiet=quiet)
+
+def cbak(selection="(all)",quiet=1):
+   s = str(selection)   
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
+   cmd.color("pink","(elem C and ("+s+"))",quiet=quiet)
+
 def cnc(selection="(all)",quiet=1):
    s = str(selection)
-   cmd.color("magenta","(("+s+") and not elem c)",quiet=quiet)
-   cmd.color("oxygen","(elem O and ("+s+"))",quiet=quiet)
-   cmd.color("nitrogen","(elem N and ("+s+"))",quiet=quiet)
-   cmd.color("sulfur","(elem S and ("+s+"))",quiet=quiet)
-   cmd.color("hydrogen","(elem H and ("+s+"))",quiet=quiet)   
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
+
+def cba(color,selection="(all)",quiet=1):
+   s = str(selection)
+   cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
+   cmd.color(color,"(elem C and ("+s+"))",quiet=quiet)
+   cmd.color(color,s,flags=1,quiet=quiet)
 
 def performance(mode):
    mode = int(mode)
@@ -244,7 +208,7 @@ def performance(mode):
       cmd.set('sphere_quality',2)
       cmd.set('cartoon_sampling',14)
       cmd.set('ribbon_sampling',10)
-      cmd.set('transparency_mode',1)
+      cmd.set('transparency_mode',2)
       cmd.set('stick_ball',1)
       cmd.do("rebuild")
    elif mode==33:
@@ -255,7 +219,8 @@ def performance(mode):
       cmd.set('stick_quality',8)
       cmd.set('sphere_quality',1)
       cmd.set('cartoon_sampling',7)
-      cmd.set('transparency_mode',1)
+      cmd.set('ribbon_sampling',1)
+      cmd.set('transparency_mode',2)
       cmd.set('stick_ball',0)
       cmd.do("rebuild")
    elif mode==66: # good perfomance
@@ -266,8 +231,9 @@ def performance(mode):
       cmd.set('stick_quality',8)
       cmd.set('sphere_quality',1)
       cmd.set('cartoon_sampling',6)
+      cmd.set('ribbon_sampling',1)
       cmd.set('transparency_mode',2)
-      cmd.set('stick_balls',0.0)
+      cmd.set('stick_ball',0.0)
       cmd.do("rebuild")         
    else: # maximum performance
       cmd.set('line_smooth',0)
@@ -276,9 +242,10 @@ def performance(mode):
       cmd.set('surface_quality',-1) # new
       cmd.set('stick_quality',5)
       cmd.set('sphere_quality',0)
+      cmd.set('ribbon_sampling',1)
       cmd.set('cartoon_sampling',3)
       cmd.set('transparency_mode',0)
-      cmd.set('stick_balls',0.0)
+      cmd.set('stick_ball',0.0)
       cmd.do("rebuild")         
    
    
@@ -416,27 +383,27 @@ def ray_shadows(mode):
       cmd.set('spec_reflect',0.5)
       cmd.set('spec_power',60) # was 50
       cmd.set('ambient',0.12)
-      cmd.set('reflect',0.9) 
+      cmd.set('reflect',0.95) 
       cmd.set('direct',0.25) 
-      cmd.set('gamma',1.3) 
+      cmd.set('gamma',1.5) 
    elif mode=='heavy':
       cmd.set('reflect_power',1.0)
       cmd.set('spec_reflect',0.60) # was 0.53
       cmd.set('spec_power',90) # was 60
       cmd.set('ambient',0.03) # was 0.08
-      cmd.set('reflect',0.75) # was 0.70
+      cmd.set('reflect',0.9) # was 0.75
       cmd.set('direct',0.06)
       cmd.set('power',0.3)
-      cmd.set('gamma',1.5) # was 1.43
+      cmd.set('gamma',1.45) # was 1.5
    elif mode=='black': # best for light backgrounds
       cmd.set('power',0.3)
       cmd.set('spec_reflect',1.1)
       cmd.set('spec_power',90) 
       cmd.set('ambient',0.001)
-      cmd.set('reflect',0.8)
+      cmd.set('reflect',1.0)
       cmd.set('direct',0.00)
       cmd.set('reflect_power',1.7)
-      cmd.set('gamma',1.8) # was 1.55
+      cmd.set('gamma',1.7) # was 1.8
 
 def ff_copy(src,dst): # NOT THREAD SAFE
    pymol._rcopy = pymol.Scratch_Storage()
@@ -1014,3 +981,16 @@ def ss(selection="(name ca and alt '',A)",state=1): # NOT THREAD SAFE
 #   print conn_hash.keys()
    print " util.ss: assignment complete."
 
+def colors(scheme=""):
+   if scheme=="jmol":
+      cmd.set("auto_color",0)
+      cmd.set_color("hydrogen",[1.000,1.000,1.000])
+      cmd.set_color("carbon",[0.567,0.567,0.567])
+      cmd.set_color("nitrogen",[0.189,0.315,0.976])
+      cmd.set_color("oxygen",[1.000,0.051,0.051])
+      cmd.set_color("fluorine",[0.567,0.882,0.314])
+      cmd.set_color("sulfur",[1.000,1.000,0.189])
+      cmd.color("carbon","elem c")
+      cmd.recolor()
+      
+                    
