@@ -7,7 +7,7 @@
 #G* Please see the accompanying LICENSE file for further information. 
 #H* -------------------------------------------------------------------
 #I* Additional authors of this source file include:
-#-* 
+#-* Filipe Maia (slicing code)
 #-* 
 #-*
 #Z* -------------------------------------------------------------------
@@ -15,6 +15,7 @@
 if __name__=='pymol.viewing':
    
    import thread
+   import threading
    import string
    import types
    import traceback
@@ -29,12 +30,19 @@ if __name__=='pymol.viewing':
         _feedback,fb_module,fb_mask, \
         repres,repres_sc, is_string, is_list, \
         toggle_dict,toggle_sc,stereo_dict,stereo_sc, \
-        palette_dict ,palette_sc
-
+        palette_dict ,palette_sc, window_dict, window_sc, \
+        safe_list_eval, lock_without_glut
+   
    rep_list = [ "lines","sticks","spheres",
                 "dots","surface","mesh",
                 "nonbonded", "nb_spheres",
-                "cartoon","ribbon","labels"]
+                "cartoon","ribbon","labels","slice"]
+
+   scene_action_sc = Shortcut(['store','recall','clear','insert_before',
+                       'insert_after','next','previous',
+                       'start', 'update','rename'])
+   scene_action_dict = {}
+   scene_action_dict_sc = Shortcut([])
 
    view_sc = Shortcut(['store','recall','clear'])
    view_dict = {}
@@ -42,8 +50,11 @@ if __name__=='pymol.viewing':
 
    scene_dict = {}
    scene_dict_sc = Shortcut([])
+   scene_order = []
+   scene_counter = 1
+   scene_quit_on_action = ''
 
-   def zoom(selection="all",buffer=0.0,state=0,complete=0):
+   def zoom(selection="all",buffer=0.0,state=0,complete=0,animate=0):
       '''
 DESCRIPTION
 
@@ -93,12 +104,13 @@ SEE ALSO
       #   
       try:
          lock()   
-         r = _cmd.zoom(str(selection),float(buffer),int(state)-1,int(complete))
+         r = _cmd.zoom(str(selection),float(buffer),
+                       int(state)-1,int(complete),int(animate))
       finally:
          unlock()
       return r
 
-   def center(selection="all",state=0,origin=1):
+   def center(selection="all",state=0,origin=1,animate=0):
       '''
 DESCRIPTION
 
@@ -135,7 +147,7 @@ SEE ALSO
       #   
       try:
          lock()   
-         r = _cmd.center(str(selection),int(state)-1,int(origin))
+         r = _cmd.center(str(selection),int(state)-1,int(origin),int(animate))
       finally:
          unlock()
       return r
@@ -221,7 +233,7 @@ SEE ALSO
          if position==None: position=(0.0,0.0,0.0)
          else:
             if cmd.is_string(position):
-               position = eval(position)
+               position = safe_list_eval(position)
             selection = ''
          r = _cmd.origin(selection,str(object),
                          (float(position[0]),
@@ -232,7 +244,7 @@ SEE ALSO
          unlock()
       return r
 
-   def orient(selection="(all)",state=0):
+   def orient(selection="(all)",state=0,animate=0):
       '''
 DESCRIPTION
 
@@ -264,7 +276,7 @@ SEE ALSO
       #   
       try:
          lock()
-         r = _cmd.orient(selection,int(state)-1)
+         r = _cmd.orient(selection,int(state)-1,float(animate))
       finally:
          unlock()
       return r
@@ -436,9 +448,9 @@ DESCRIPTION
 
    The available representations are:
 
-      lines     spheres   mesh      ribbon     cartoon
-      sticks    dots      surface   labels     extent
-      nonbonded nb_spheres 
+      lines     spheres    mesh      ribbon     cartoon
+      sticks    dots       surface   labels     extent
+      nonbonded nb_spheres slice
 
 USAGE
 
@@ -459,7 +471,7 @@ EXAMPLES
 NOTES
 
    "selection" can be an object name
-   "show" alone will turn on lines for all bonds.
+   "show" alone will turn on lines and nonbonded for all bonds.
 
 SEE ALSO
 
@@ -469,7 +481,8 @@ SEE ALSO
       try:
          lock()
          if (representation=="") and (selection==""):
-            r = _cmd.showhide("(all)",repres['lines'],1); # show lines by default       
+            if _cmd.showhide("(all)",repres['lines'],1): # show lines by default
+               r = _cmd.showhide("(all)",repres['nonbonded'],2)
          elif (representation!="") and (selection!=""):
             rep = representation
             rep = repres_sc.auto_err(rep,'representation')
@@ -479,17 +492,91 @@ SEE ALSO
             #   
             r = _cmd.showhide(str(selection),int(repn),1);
          elif representation=='all':
-            r = _cmd.showhide("all",repres['lines'],1); # show lines by default 
+            if _cmd.showhide("all",repres['lines'],1): # show lines by default
+               r = _cmd.showhide("all",repres['nonbonded'], 1) # nonbonded
          elif (representation[0:1]=='(') or (string.find(representation,'/')>=0):
             # preprocess selection
             selection = selector.process(representation)
             #                  
-            r = _cmd.showhide(str(selection),repres['lines'],1);
+            if _cmd.showhide(str(selection),repres['lines'],1):
+               r = _cmd.showhide(str(selection),repres['nonbonded'],2);
+         else: # selection==""
+            rep = representation
+            rep = repres_sc.auto_err(rep,'representation')
+            repn = repres[rep]
+            r = _cmd.showhide("all",int(repn),1);
+      finally:
+         unlock()
+      return r
+
+   def as(representation="",selection=""):
+      '''
+DESCRIPTION
+
+   "as" turns on and off atom and bond representations.
+
+   The available representations are:
+
+      lines     spheres    mesh      ribbon     cartoon
+      sticks    dots       surface   labels     extent
+      nonbonded nb_spheres slice
+
+USAGE
+
+   as
+   as reprentation [,object]
+   as reprentation [,(selection)]
+   as (selection)
+
+PYMOL API
+
+   cmd.as( string representation="", string selection="" )
+
+EXAMPLES
+
+   as lines,(name ca or name c or name n)
+   as ribbon
+
+NOTES
+
+   "selection" can be an object name
+   "as" alone will turn on lines and nonbonded and hide everything else.
+   
+SEE ALSO
+
+   show, hide, enable, disable
+      '''
+      r=1
+      try:
+         lock()
+         if (representation=="") and (selection==""):
+            if _cmd.showhide(str(selection),-1,0):
+               if _cmd.showhide("(all)",repres['lines'],1):
+                  r = _cmd.showhide("(all)",repres['nonbonded'],1)
+         elif (representation!="") and (selection!=""):
+            rep = representation
+            rep = repres_sc.auto_err(rep,'representation')
+            repn = repres[rep]
+            # preprocess selection 
+            selection = selector.process(selection)
+            #
+            if _cmd.showhide(str(selection),-1,0):
+               r = _cmd.showhide(str(selection),int(repn),1)
+         elif representation=='all':
+            if _cmd.showhide(str(selection),-1,0):            
+               if _cmd.showhide("all",repres['lines'],1): # show lines by default
+                  r = _cmd.showhide("all",repres['nonbonded'],1) # show nonbonded by default
+         elif (representation[0:1]=='(') or (string.find(representation,'/')>=0):
+            # preprocess selection
+            selection = selector.process(representation)
+            if _cmd.showhide(str(selection),-1,0):
+               r = _cmd.showhide(str(selection),repres['lines'],1)
          else: # selection==""
             rep = representation
             rep = repres_sc.auto_err(rep,'representation')
             repn = repres[rep];
-            r = _cmd.showhide("all",int(repn),1);
+            if _cmd.showhide("all",-1,0):
+               r = _cmd.showhide("all",int(repn),1);
       finally:
          unlock()
       return r
@@ -639,7 +726,7 @@ SEE ALSO
       r = r[0:3]+r[4:7]+r[8:11]+r[16:25]
       return r
 
-   def set_view(view,quiet=1):
+   def set_view(view,quiet=1,animate=0):
       '''
 DESCRIPTION
 
@@ -659,7 +746,6 @@ SEE ALSO
 
    get_view
    '''
-
       r = None
       if cmd.is_string(view):
          try:
@@ -680,7 +766,7 @@ SEE ALSO
                0.0,0.0,0.0,1.0,
                float(view[ 9]),float(view[10]),float(view[11]),
                float(view[12]),float(view[13]),float(view[14]),
-               float(view[15]),float(view[16]),float(view[17])),quiet)
+               float(view[15]),float(view[16]),float(view[17])),quiet,float(animate))
          finally:
             unlock()
       return r
@@ -707,8 +793,8 @@ PYMOL API
 VIEWS
 
    Views F1 through F12 are automatically bound to function keys
-   provided that "set_key" hasn't been used to redefine the behaviour
-   of the respective key, and that a "scene" hasn't been defined for
+   provided that "set_key" has not been used to redefine the behaviour
+   of the respective key, and that a "scene" has not been defined for
    that key.
 
 EXAMPLES
@@ -737,10 +823,10 @@ SEE ALSO
          action = view_sc.auto_err(action,'action')
          if action=='recall':
             key = view_dict_sc.auto_err(key,'view')
-            set_view(view_dict[key])
+            set_view(view_dict[key],animate=-1)
             if _feedback(fb_module.scene,fb_mask.actions): # redundant
                print " view: \"%s\" recalled."%key
-         elif action=='store':
+         elif (action=='store') or (action=='update'):
             view_dict_sc.append(key)
             view_dict[key]=cmd.get_view(0)
             if _feedback(fb_module.scene,fb_mask.actions):
@@ -786,6 +872,14 @@ SEE ALSO
          unlock()
       return r
 
+   def set_colorection_name(dict,key,new_key):
+      try:
+         lock()         
+         r = _cmd.set_colorection_name(dict,key,new_key)
+      finally:
+         unlock()
+      return r
+
    def del_colorection(dict,key):
       try:
          lock()         
@@ -801,9 +895,60 @@ SEE ALSO
       finally:
          unlock()
       return cpy
+
+   scene_sort_dict = {
+      'F1' : 'F01',
+      'F2' : 'F02',
+      'F3' : 'F03',
+      'F4' : 'F04',
+      'F5' : 'F05',
+      'F6' : 'F06',
+      'F7' : 'F07',
+      'F8' : 'F08',
+      'F9' : 'F09',
+      'SHFT-F1' : 'SHFT-F01',
+      'SHFT-F2' : 'SHFT-F02',
+      'SHFT-F3' : 'SHFT-F03',
+      'SHFT-F4' : 'SHFT-F04',
+      'SHFT-F5' : 'SHFT-F05',
+      'SHFT-F6' : 'SHFT-F06',
+      'SHFT-F7' : 'SHFT-F07',
+      'SHFT-F8' : 'SHFT-F08',
+      'SHFT-F9' : 'SHFT-F09',
+      }
+
+   def _scene_get_unique_key():
+      global scene_dict,scene_order      
+      global scene_counter
+      keys = scene_dict.keys()
+      while 1:
+         key = "%03d"%scene_counter
+         if scene_dict.has_key(key):
+            scene_counter = scene_counter + 1
+         else:
+            break;
+      return key
    
-   def scene(key,action='recall',message=None,
-             view=1,color=1,active=1,rep=1,frame=1):
+   def _scene_validate_list():
+      global scene_dict,scene_order      
+      new_list = []
+      new_dict = {}
+      for a in scene_order:
+         if scene_dict.has_key(a) and not new_dict.has_key(a):
+            new_list.append(a)
+            new_dict[a] = 1
+      lst = map(lambda x:(scene_sort_dict.get(x,x),x), scene_dict.keys())
+      lst.sort()
+      lst = map(lambda x:x[1],lst)
+      for a in lst:
+         if not new_dict.has_key(a):
+            new_list.append(a)
+      scene_order = new_list
+      return scene_order
+
+   def scene(key='auto',action='recall',message=None,
+             view=1,color=1,active=1,rep=1,frame=1,animate=-1,
+             new_key=None, quiet=1):
       '''
 DESCRIPTION
 
@@ -814,11 +959,18 @@ DESCRIPTION
 
 USAGE
 
-   scene key [,action [,message [ ,view [,color [,active [,rep [,frame]]]]]]]
+   scene key [,action [,message [ ,view [,color [,active [,rep [,frame
+             [, animate, [, new_key ]]]]]]]]]
    scene *
 
-   key can be any string
-   action should be 'store' or 'recall' (default: 'recall')
+   key can be any string, or
+      'new' for an automatically numbered new scene, or
+      'auto' for the current scene (if one exists), or
+      '*' for all scenes ('clear' and 'recall' actions only)
+      
+   action should be 'store', 'recall', 'insert_after', 'insert_before',
+                    'next',  'previous', 'update', 'rename', or 'clear'
+                    (default: 'recall')
 
    view: 1 or 0 controls whether the view is stored
    color: 1 or 0 controls whether colors are stored
@@ -829,7 +981,8 @@ USAGE
 PYMOL API
 
    cmd.scene(string key,string action,string-or-list message,int view,
-             int color, int active, int rep, int frame)
+             int color, int active, int rep, int frame, int animate,
+             string new_key)
 
 EXAMPLES
 
@@ -839,6 +992,8 @@ EXAMPLES
    scene F1
    scene F2
 
+   scene F3, rename, new_key=F5
+
 NOTES
 
    Scenes F1 through F12 are automatically bound to function keys
@@ -860,20 +1015,33 @@ DEVELOPMENT TO DO
       which are enabled/disabled differentially.
       
       '''
-      global scene_dict,scene_dict_sc
+      global scene_dict,scene_dict_sc,scene_order
+      global scene_quit_on_action
 
+      view = int(view)
+      rep = int(rep)
+      color = int(color)
+      active = int(active)
+      frame = int(frame)
+      quiet = int(quiet)
+      if animate<0:
+         if int(cmd.get_setting_legacy("scene_animation"))!=0:
+            animate = cmd.get_setting_legacy("scene_animation_duration")
       try:
-         view = int(view)
-         rep = int(rep)
-         color = int(color)
-         active = int(active)
-         frame = int(frame)
          lock() # manipulating global data, so need lock
-         
+         if key=='auto':
+            action = scene_action_sc.auto_err(action,'action')
+            if action=='recall':
+               action='next'
+            if action=='start':
+               lst = _scene_validate_list()
+               if len(lst):
+                  key = lst[0]
+                  action='recall'
+                  
          if key=='*':
-            action = view_sc.auto_err(action,'action')
+            action = scene_action_sc.auto_err(action,'action')
             if action=='clear':
-
                for key in scene_dict.keys():
                   # free selections
                   list = scene_dict[key]
@@ -884,29 +1052,73 @@ DEVELOPMENT TO DO
                   name = "_scene_"+key+"_*"
                   cmd.delete(name)
                scene_dict = {}
-               scene_dict_sc = Shortcut(scene_dict.keys())                        
+               scene_dict_sc = Shortcut(scene_dict.keys())
+               scene_order = []
             else:
                print " scene: stored scenes:"
-               lst = scene_dict.keys()
-               lst.sort()
+               lst = _scene_validate_list()
                parsing.dump_str_list(lst)
          else:
-            action = view_sc.auto_err(action,'action')
+            action = scene_action_sc.auto_err(action,'action')
+            
+            if action=='insert_before':
+               key = _scene_get_unique_key()
+               cur_scene = setting.get("scene_current_name")
+               if cur_scene in scene_order:
+                  ix = scene_order.index(cur_scene)
+               else:
+                  ix = 0
+               scene_order.insert(ix,key)
+               action='store'
+            elif action=='rename':
+               if not scene_dict.has_key(key):
+                  print "Error: scene '%s' not found."%key
+               elif new_key==None:
+                  print "Error: must provide the 'new_key' argument"
+               else:
+                  new_scene_order = []
+                  for a in scene_order:
+                     if a==key:
+                        new_scene_order.append(new_key)
+                     else:
+                        new_scene_order.append(a)
+                  scene_order=new_scene_order
+                  scene_dict[new_key] = scene_dict[key]
+                  del scene_dict[key]
+                  for rep_name in rep_list:
+                     name = "_scene_"+key+"_"+rep_name
+                     new_name = "_scene_"+new_key+"_"+rep_name
+                     cmd.set_name(name,new_name)
+                  list = scene_dict[new_key]
+                  if len(list)>3:
+                     cmd.set_colorection_name(list[3],key,new_key)
+                  print" scene: '%s' renamed to '%s'."%(key,new_key)
+                  scene_dict_sc = Shortcut(scene_dict.keys())                  
+            elif action=='insert_after':
+               key = _scene_get_unique_key()            
+               cur_scene = setting.get("scene_current_name")
+               if cur_scene in scene_order:
+                  ix = scene_order.index(cur_scene) + 1
+               else:
+                  ix = len(scene_order)
+               scene_order.insert(ix,key)
+               action='store'
+               
             if action=='recall':
                cmd.set("scenes_changed",1,quiet=1);
                key = scene_dict_sc.auto_err(key,'scene')
+               cmd.set('scene_current_name', key, quiet=1)               
                list = scene_dict[key]
                ll = len(list)
-               if (ll>0) and (view):
-                  if list[0]!=None:
-                     set_view(list[0])
                if (ll>1) and (active):
                   if list[1]!=None:
                      cmd.disable()
+                     cmd.deselect()
                      cmd.set_vis(list[1])
                if (ll>2) and (frame):
                   if list[2]!=None:
-                     cmd.frame(list[2])
+                     if not cmd.get_movie_playing(): # don't set frame when movie is already playing
+                        cmd.frame(list[2])
                if (ll>3) and (color):
                   if list[3]!=None:
                      cmd.set_colorection(list[3],key)
@@ -947,19 +1159,32 @@ DEVELOPMENT TO DO
                         mess_flag = 1
                if replace_flag and not mess_flag:
                   cmd.wizard()
-               if _feedback(fb_module.scene,fb_mask.actions): # redundant
+               if (ll>0) and (view):
+                  if list[0]!=None:
+                     set_view(list[0],quiet,animate)
+               if not quiet and _feedback(fb_module.scene,fb_mask.actions): # redundant
                   print " scene: \"%s\" recalled."%key
-            elif action=='store':
+            elif (action=='store') or (action=='update'):
+               if key =='new':
+                  key=_scene_get_unique_key()               
+               if key =='auto':
+                  key = setting.get("scene_current_name")
+                  if key=='':
+                     key=_scene_get_unique_key()
                if not scene_dict.has_key(key):
                   scene_dict_sc.append(key)
                else: # get rid of existing one (if exists)
                   list = scene_dict[key]
+                  if (action=='update') and (message==None) and len(list)>5:
+                     message = scene_dict[key][5]
                   if len(list)>3:
                      colorection = list[3]
                      if colorection!=None:
                         cmd.del_colorection(colorection,key) # important -- free RAM
                   name = "_scene_"+key+"_*"
                   cmd.delete(name)
+               if key not in scene_order:
+                  scene_order.append(key)
                entry = []
                if view:
                   entry.append(cmd.get_view(0))
@@ -996,10 +1221,12 @@ DEVELOPMENT TO DO
                if _feedback(fb_module.scene,fb_mask.actions):
                   print " scene: scene stored as \"%s\"."%key
                cmd.set("scenes_changed",1,quiet=1);
+               cmd.set('scene_current_name',key,quiet=1)
             elif action=='clear':
+               if key=='auto':
+                  key = setting.get("scene_current_name")
                key = scene_dict_sc.auto_err(key,'view')
                if scene_dict.has_key(key):
-
                   list = scene_dict[key]
                   if len(list)>3:
                      colorection = list[3]
@@ -1013,6 +1240,65 @@ DEVELOPMENT TO DO
                   scene_dict_sc = Shortcut(scene_dict.keys())            
                   if _feedback(fb_module.scene,fb_mask.actions):
                      print " scene: '%s' deleted."%key
+            elif action=='next':
+               lst = _scene_validate_list()
+               cur_scene = setting.get('scene_current_name',quiet=1)
+               if cur_scene in lst:
+                  ix = lst.index(cur_scene) + 1
+               else:
+                  ix = 0
+                  animate = 0
+                  if ((scene_quit_on_action==action) and
+                      (setting.get("presentation")=="on") and 
+                      (setting.get("presentation_auto_quit")=="on")):
+                     cmd.quit()
+               if ix<len(lst):
+                  scene_name = lst[ix]
+                  cmd.set('scene_current_name', scene_name, quiet=1)
+                  scene(scene_name,'recall',animate=animate)
+               elif setting.get("scene_loop")=="on": # loop back to the beginning
+                  if len(lst):
+                     scene_name = lst[0]
+                     cmd.set('scene_current_name', scene_name, quiet=1)
+                     scene(scene_name,'recall',animate=animate)
+               else: # otherwise put up blank screen
+                  cmd.set('scene_current_name','',quiet=1)
+                  if ((setting.get("presentation")=="on") and 
+                      (setting.get("presentation_auto_quit")=="on")):
+                     scene_quit_on_action = action
+                  if len(lst):
+                     cmd.disable() # just hide everything
+                     cmd.wizard()
+                     
+            elif action=='previous':
+               lst = _scene_validate_list()            
+               cur_scene = setting.get('scene_current_name',quiet=1)
+               if cur_scene in lst:
+                  ix = lst.index(cur_scene) - 1
+               else:
+                  ix = len(lst)-1
+                  animate = 0
+                  if ((scene_quit_on_action==action) and
+                      (setting.get("presentation")=="on") and 
+                      (setting.get("presentation_auto_quit")=="on")):
+                     cmd.quit()
+               if ix>=0:
+                  scene_name = lst[ix]
+                  scene(scene_name,'recall',animate=animate)
+               elif setting.get("scene_loop")=="on": # loop back to the beginning
+                  print setting.get("scene_loop")
+                  if len(lst):
+                     scene_name = lst[-1]
+                     cmd.set('scene_current_name', scene_name, quiet=1)
+                     scene(scene_name,'recall',animate=animate)
+               else: # otherwise put up blank screen
+                  cmd.set('scene_current_name','',quiet=1)
+                  if ((setting.get("presentation")=="on") and 
+                      (setting.get("presentation_auto_quit")=="on")):
+                     scene_quit_on_action = action
+                  if len(lst):
+                     cmd.disable() # just hide everything
+                     cmd.wizard()
       finally:
          unlock()
          
@@ -1036,13 +1322,18 @@ DEVELOPMENT TO DO
 
    def session_save_scenes(session):
       session['scene_dict']=copy.deepcopy(scene_dict)
+      session['scene_order']=copy.deepcopy(scene_order)
       return 1
 
    def session_restore_scenes(session):
-      global scene_dict,scene_dict_sc
+      global scene_dict,scene_dict_sc,scene_order
       if session.has_key('scene_dict'):
-         scene_dict=copy.deepcopy(session['scene_dict'])
+         scene_dict = copy.deepcopy(session['scene_dict'])
          scene_dict_sc = Shortcut(scene_dict.keys())
+      if session.has_key('scene_order'):
+         scene_order = copy.deepcopy(session['scene_order'])
+      else:
+         scene_order = []
       return 1
 
    if session_restore_scenes not in pymol._session_restore_tasks:
@@ -1131,7 +1422,7 @@ SEE ALSO
       '''
 DESCRIPTION
 
-   "full_screen" enables or disables PyMOL's full screen mode.  This
+   "full_screen" enables or disables PyMOL\'s full screen mode.  This
    does not work well on all platforms.  
 
 USAGE
@@ -1150,7 +1441,7 @@ USAGE
       else:
          try:
             lock()
-            r = _cmd.do("_cmd.full_screen(%d)"%int(toggle),0)
+            r = cmd._do("_cmd.full_screen(%d)"%int(toggle))
          finally:
             unlock()
       return r
@@ -1218,7 +1509,34 @@ EXAMPLES
          unlock()   
       return r
 
+   def window(action='show'):
+      '''
+DESCRIPTION
 
+   "window" controls the visibility of PyMOL\'s output window
+
+USAGE
+
+   window action
+
+PYMOL API
+
+   cmd.window(string action)
+
+   action = \'show\' or \'hide\'
+   
+      '''
+      action = window_sc.auto_err(action,'action')
+      action = window_dict[str(action)]
+
+      r=1
+      try:
+         lock()
+         r = _cmd.window(action)
+      finally:
+         unlock()
+      return r
+      
    def viewport(width=-1,height=-1):
       '''
 DESCRIPTION
@@ -1280,7 +1598,8 @@ PYMOL API
       'oval'        : 3,
       'tube'        : 4,
       'arrow'       : 5,
-      'dumbbell'    : 6,   
+      'dumbbell'    : 6,
+      'putty'       : 7,
    }
 
    cartoon_sc = Shortcut(cartoon_dict.keys())
@@ -1324,7 +1643,20 @@ NOTES
          unlock()
       return r
 
-   def ray(width=0,height=0,renderer=-1,angle=0.0,shift=0.0,quiet=1):
+   def _ray(width,height,renderer,angle,shift,quiet):
+      try:
+         lock_without_glut()
+         try:
+            _cmd.set_busy(1)
+            r = _cmd.render(int(width),int(height),
+                            int(renderer),float(angle),float(shift),int(quiet))
+         finally:
+            _cmd.set_busy(0)
+      finally:
+         unlock()
+      return r
+   
+   def ray(width=0,height=0,renderer=-1,angle=0.0,shift=0.0,quiet=1,async=0):
       '''
 DESCRIPTION
 
@@ -1363,12 +1695,16 @@ SEE ALSO
    ray-tracing engine.
 
       '''
-      try:
-         lock()   
-         r = _cmd.render(int(width),int(height),
-                         int(renderer),float(angle),float(shift),int(quiet))
-      finally:
-         unlock()
+      arg_tup = (int(width),int(height),
+                 int(renderer),float(angle),
+                 float(shift),int(quiet))
+      if not async:
+         r = apply(_ray, arg_tup)
+      else:
+         render_thread = threading.Thread(target=_ray, args=arg_tup)
+         render_thread.setDaemon(1)
+         render_thread.start()
+         r = 1
       return r
 
    def refresh():
@@ -1395,7 +1731,7 @@ SEE ALSO
       else:
          try:
             lock()
-            r = _cmd.do("cmd._refresh()",0)
+            r = cmd._do("cmd._refresh()")
          finally:
             unlock()
       return r
@@ -1517,7 +1853,7 @@ SEE ALSO
       finally:
          unlock()
 
-   def color(color,selection="(all)",quiet=1):
+   def color(color,selection="(all)",quiet=1,flags=0):
       '''
 DESCRIPTION
 
@@ -1543,7 +1879,7 @@ EXAMPLES
       #
       try:
          lock()
-         r = _cmd.color(str(color),str(selection),0,int(quiet))
+         r = _cmd.color(str(color),str(selection),int(flags),int(quiet))
       finally:
          unlock()
       if not r:
@@ -1615,7 +1951,7 @@ EXAMPLES
       '''
       r = 1
       if cmd.is_string(rgb):
-         rgb = eval(rgb)
+         rgb = safe_list_eval(rgb)
       if not (isinstance(rgb,types.ListType) or isinstance(rgb,types.TupleType)):
          print "Error: color specification must be a list such as [ 1.0, 0.0, 0.0 ]"
       elif len(rgb)!=3:
@@ -1633,6 +1969,189 @@ EXAMPLES
             unlock()
       return r
 
+   unused = """
+   def slice_lock(name,state=-1):
+      '''
+DESCRIPTION
+
+   "slice_lock" imprisons the slice exactly between the 2 clipping planes.
+
+
+USAGE
+
+   slice_lock slice [, state ]
+
+   slice = the name of the slice object you want to lock.
+
+   state = the state of the slice you want to lock (default=-1)
+
+
+NOTES
+
+   state > 0: specific state
+   state = 0: all states
+   state = -1: current state
+
+
+SEE ALSO
+
+   slice_unlock
+      '''
+      # preprocess selection
+      selection = selector.process(name)
+      #   
+      try:
+         lock()   
+         r = _cmd.slice_setlock(str(selection),int(state)-1,1)
+      finally:
+         unlock()
+      return r
+
+   def slice_unlock(name,state=-1):
+      '''
+DESCRIPTION
+
+   "slice_unlock" allows the slice to freely move on the 3D space.
+
+
+USAGE
+
+   slice_unlock slice [, state ]
+
+   slice = the name of the slice object you want to unlock.
+
+   state = the state of the slice you want to unlock (default=-1)
+
+
+NOTES
+
+   state > 0: specific state
+   state = 0: all states
+   state = -1: current state
+
+
+SEE ALSO
+
+   slice_lock
+      '''
+      # preprocess selection
+      selection = selector.process(name)
+      #   
+      try:
+         lock()   
+         r = _cmd.slice_setlock(str(selection),int(state)-1,0)
+      finally:
+         unlock()
+      return r
+
+
+   def rgbfunction(name,function="traditional",state=-1):
+      '''
+DESCRIPTION
+
+   "rgbfunction" defines the fuction that is used to map a color to a
+   value
+
+USAGE
+
+   rgbfunction name, [ function ]
+
+   name = the name of the object from which you wish to change the
+   rgb color map function
+
+   function = the mapping function you want to use (default=traditional)
+
+   state = the state of the slice you want to unlock (default=-1)
+
+
+NOTES:
+
+   Possible functions:
+   
+   traditional
+   sludge
+   ocean
+   hot
+   grayable
+   rainbow
+   afmhot
+   grayscale
+
+
+   state > 0: specific state
+   state = 0: all states
+   state = -1: current state
+
+
+   
+EXAMPLES 
+
+   rgbfunction slice,ocean
+      '''
+
+      # Evil literal constants alert!! It would be nice to be able to use
+      # the same defines that are specified in ObjectSlice.c
+      # but i have not idea on how to do it
+      r = 1
+      try:
+
+         lock()
+         if (cmp(function.lower(),"traditional") == 0):
+            r = _cmd.rgbfunction(str(name),1,int(state)-1)
+         elif (cmp(function.lower(),"sludge") == 0):
+            r = _cmd.rgbfunction(str(name),2,int(state)-1)
+         elif (cmp(function.lower(),"ocean") == 0):
+            r = _cmd.rgbfunction(str(name),3,int(state)-1)
+         elif (cmp(function.lower(),"hot") == 0):
+            r = _cmd.rgbfunction(str(name),4,int(state)-1)
+         elif (cmp(function.lower(),"grayable") == 0):
+            r = _cmd.rgbfunction(str(name),5,int(state)-1)
+         elif (cmp(function.lower(),"rainbow") == 0):
+            r = _cmd.rgbfunction(str(name),6,int(state)-1)
+         elif (cmp(function.lower(),"afmhot") == 0):
+            r = _cmd.rgbfunction(str(name),7,int(state)-1)
+         elif (cmp(function.lower(),"grayscale") == 0):
+            r = _cmd.rgbfunction(str(name),8,int(state)-1)
+         else:
+            print "Error: Invalid function name."
+      finally:
+         unlock()
+      return r
+
+   def slice_heightmap(name,state=-1):
+      '''
+DESCRIPTION
+
+   "slice_heightmap" toogles the heightmap representation.
+
+
+USAGE
+
+   heightmap slice [, state ]
+
+   slice = the name of the slice.
+
+   state = the state of the slice (default=-1)
+
+
+NOTES
+
+   state > 0: specific state
+   state = 0: all states
+   state = -1: current state
+
+      '''
+      # preprocess selection
+      selection = selector.process(name)
+      #   
+      try:
+        lock()   
+        r = _cmd.slice_heightmap(str(selection),int(state)-1)
+      finally:
+         unlock()
+      return r
+
+"""
 
 # Aliases for Mother England.
 
@@ -1642,3 +2161,5 @@ EXAMPLES
    recolour = recolor
    
 
+   import setting
+   
