@@ -6,7 +6,6 @@ from glob import glob
 
 import threading
 import time
-import whrandom
 from pymol import cmd
 import sys, os, os.path
 
@@ -23,6 +22,8 @@ def load():
    for file in list:
       cmd.delete('pdb')
       cmd.load(file,'pdb')
+      cmd.set_title('pdb',1,os.path.split(file)[-1])
+      cmd.rewind()
       cmd.orient('pdb')
       cmd.refresh()
       cmd.hide()
